#include "ovcontrol_2d.h"

#include "2d/plot_2dplot.h"
#include "qwt_plot_layout.h"
#include <QMouseEvent>
#include "widgets/twodoverlaybuttons.h"
#include "widgets/colorrect2.h"
#include "2d/plot_2dpure.h"
#include <QSettings>
#include <QGraphicsOpacityEffect>
#include <QPropertyAnimation>
#include "core/plot_core.h"
#include "qwt_plot_grid.h"

namespace plot {

#ifdef _ADROID
ovcontrol_2d::ovcontrol_2d(TwoDOverlayButtons *overlay, bool vertical)
#else
ovcontrol_2d::ovcontrol_2d(TwoDOverlayButtons *overlay)
#endif
{
    this->overlay = overlay;
    connect(overlay, SIGNAL(replacedGUIPlotResized(QSize)), this, SLOT(plotResized(QSize)));
    connect(overlay, SIGNAL(replacedGUIPlotSetVisible(bool)), this, SLOT(plotSetVisible(bool)));
    connect(overlay, SIGNAL(onBackClicked()), this, SLOT(onBackClicked()));
    
#ifdef _ADROID
    this->vertical = vertical;
#endif
    
    {
        #ifdef _ADROID
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
#else
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
#endif
        showHelpButton = settings.value("show_help_buttons", true).toBool();
    }
    
    colorSettings = new ovMenu_2d_colorSettings(overlay);
    #ifndef _ADROID
    filterSettings = new ovMenu_2d_filterSettings(overlay);
#else
    colorSettings->vertical = vertical;
#endif
    
    //---------------4D buttons
    B4DSameXYPoints = new TwoStateButton(":/plot/Pall_Zoom_Of.png", ":/plot/Pall_Zoom_On.png");
    B4DSameXYPoints->setToolTip(QObject::tr("Same X\\Y points as scan"));
    B4DSameXYPoints->setFlat(true);
    overlay->setButtonStyle(B4DSameXYPoints);
    B4DSameXYPoints->hide();
    
    B4DSendToData = new QPushButton();
    B4DSendToData->hide();
    B4DSendToData->setFlat(true);
    overlay->setButtonStyle(B4DSendToData);
    B4DSendToData->setIcon(makeHugeIcon(":/ScanPlot/attach4DPlot2D.png"));
    B4DSendToData->setToolTip(QObject::tr("Send to data"));
    
    B4DClearSpectrum = new QPushButton();
    B4DClearSpectrum->hide();
    B4DClearSpectrum->setFlat(true);
    overlay->setButtonStyle(B4DClearSpectrum);
    B4DClearSpectrum->setIcon(makeHugeIcon(":/CCD/mclear.png"));
    B4DClearSpectrum->setToolTip(QObject::tr("Clear spectrum"));
    
    B4DSectionPointSpectr = new TwoStateButton(":/CCD/point_spect_off.png", ":/plot/point.png");
    B4DSectionPointSpectr->setToolTip(QObject::tr("Switch between section and point spectra"));
    B4DSectionPointSpectr->setFlat(true);
    overlay->setButtonStyle(B4DSectionPointSpectr);
    B4DSectionPointSpectr->hide();
    connect(B4DSectionPointSpectr, SIGNAL(state_changed(bool)), this, SLOT(on4DSwitchSpectra_clicked(bool)));
    
    B4DSelectCapturedSpectr = new QPushButton();
    B4DSelectCapturedSpectr->hide();
    B4DSelectCapturedSpectr->setCheckable(true);
    B4DSelectCapturedSpectr->setFlat(true);
    overlay->setButtonStyle(B4DSelectCapturedSpectr);
    B4DSelectCapturedSpectr->setIcon(makeHugeIcon(":/CCD/Sp_part.png"));
    B4DSelectCapturedSpectr->setToolTip(QObject::tr("Select captured specter part"));
    connect(B4DSelectCapturedSpectr, SIGNAL(toggled(bool)), this, SLOT(on4DSelectCapturedSpectr_clicked(bool)));
    
    B4DSelectConvolutionFunc = new QPushButton();
    B4DSelectConvolutionFunc->hide();
    B4DSelectConvolutionFunc->setCheckable(true);
    B4DSelectConvolutionFunc->setFlat(true);
    overlay->setButtonStyle(B4DSelectConvolutionFunc);
    B4DSelectConvolutionFunc->setIcon(makeHugeIcon(":/CCD/Func_range.png"));
    B4DSelectConvolutionFunc->setToolTip(QObject::tr("Select convolution function range"));
    connect(B4DSelectConvolutionFunc, SIGNAL(toggled(bool)), this, SLOT(on4DSelectConvolutionFunc_clicked(bool)));
    //-----------------
    
    //----------------- 2D buttons
    
    B2DMarkers = new QPushButton();
    B2DMarkers->hide();
    B2DMarkers->setCheckable(true);
    B2DMarkers->setFlat(true);
    overlay->setButtonStyle(B2DMarkers);
    B2DMarkers->setIcon(makeHugeIcon(":/ScanPlot/markersPlotOff2D.png"));
    B2DMarkers->setToolTip(QObject::tr("Markers"));
    connect(B2DMarkers, SIGNAL(toggled(bool)), this, SLOT(on2DMarkers_clicked(bool)));
    
    B2DGrid = new TwoStateButton(":/ScanPlot/gridPlot2D.png", ":/ScanPlot/gridPlot2D.png");
    B2DGrid->setToolTip(QObject::tr("Show grid"));
    B2DGrid->setFlat(true);
    overlay->setButtonStyle(B2DGrid);
    B2DGrid->hide();
    connect(B2DGrid, SIGNAL(state_changed(bool)), this, SLOT(on2DGrid_changed(bool)));
    
    B2DRestoreScale = new QPushButton();
    B2DRestoreScale->hide();
    B2DRestoreScale->setFlat(true);
    overlay->setButtonStyle(B2DRestoreScale);
    B2DRestoreScale->setIcon(makeHugeIcon(":/plot/restore_scale.png"));
    B2DRestoreScale->setToolTip(QObject::tr("Restore scale"));
    connect(B2DRestoreScale, SIGNAL(released()), this, SLOT(on2DRestoreScale_clicked()));
    
    B2DSaveImage = new QPushButton();
    B2DSaveImage->hide();
    B2DSaveImage->setFlat(true);
    overlay->setButtonStyle(B2DSaveImage);
    B2DSaveImage->setIcon(makeHugeIcon(":/plot/picturePlot2D.png"));
    B2DSaveImage->setToolTip(QObject::tr("Save image"));
    connect(B2DSaveImage, SIGNAL(released()), this, SLOT(on2DSaveImage_clicked()));
    
    B2DSendToData = new QPushButton();
    B2DSendToData->hide();
    B2DSendToData->setFlat(true);
    overlay->setButtonStyle(B2DSendToData);
    B2DSendToData->setIcon(makeHugeIcon(":/plot/attachPlot2D.png"));
    B2DSendToData->setToolTip(QObject::tr("Send to data"));
    connect(B2DSendToData, SIGNAL(released()), this, SLOT(on2DSendToData_clicked()));
    
    B2DFilter = new TwoStateButton(":/ScanPlot/filterPlot2D.png", ":/ScanPlot/filterPlot2D.png");
    B2DFilter->setToolTip(QObject::tr("enable filtering"));
    B2DFilter->setFlat(true);
    overlay->setButtonStyle(B2DFilter);
    B2DFilter->hide();
    connect(B2DFilter, SIGNAL(state_changed(bool)), this, SLOT(on2DFilter_clicked(bool)));
    
#ifndef _ADROID
    BFiltersSettings = new QPushButton();
    BFiltersSettings->setFlat(true);
    BFiltersSettings->setIcon(makeHugeIcon(":/MainToolbar/tools.png"));
    BFiltersSettings->setToolTip(QObject::tr("Filter settings"));
    BFiltersSettings->setParent(B2DFilter);
    {
        QString style = "";
        
        style += "QPushButton {background-color: rgba(255,255,255,0);";
        style += "border: 0px solid;}";
        style += "QToolTip { color: black; background-color: #FFFFE1; border: 1px solid black; }";
        style += "QPushButton:pressed{ background-color: lightgray; }";
        
        BFiltersSettings->setStyleSheet(style);
    }
    connect(BFiltersSettings, SIGNAL(released()), filterSettings, SLOT(filterSettingsClicked()));
#endif
    
    BOldFilter = new QPushButton();
    BOldFilter->hide();
    BOldFilter->setFlat(true);
    overlay->setButtonStyle(BOldFilter);
    BOldFilter->setIcon(makeHugeIcon(":/plot/math.png"));
    BOldFilter->setToolTip(QObject::tr("Filters"));
    MMathProcessing = new QMenu;
    BOldFilter->setMenu(MMathProcessing);
    AGFitOrders = new QActionGroup(this);
    const QString FitOrders[] = {tr("Fit. None"), tr("Fit. 0 order"), tr("Fit. 1 order"), tr("Fit. 2 order"),
                                 tr("Fit. 3 order"), tr("Fit. 4 order"), tr("Fit. 5 order"), tr("Manual fit")};
    int i = 0;
    for (auto &fitOrder:FitOrders)
    {
        AGFitOrders->addAction(MMathProcessing->addAction(fitOrder));
        AGFitOrders->actions()[i]->setCheckable(true);
        i++;
    }
    AGFitOrders->actions()[0]->setChecked(true);
    connect(AGFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitOrder_changed()));
    
    BFitData = new QPushButton();
    BFitData->hide();
    BFitData->setFlat(true);
    overlay->setButtonStyle(BFitData);
    BFitData->setIcon(makeHugeIcon(":/ScanPlot/restore1x12D.png"));
    BFitData->setToolTip(QObject::tr("Fit data size"));
    connect(BFitData, SIGNAL(clicked(bool)), this, SLOT(onFitDataClicked()));
    
    helpButton = new QPushButton();
    helpButton->hide();
    helpButton->setFlat(true);
    overlay->setButtonStyle(helpButton);
    helpButton->setIcon(makeHugeIcon(":/plot/helpBtn.png"));
    helpButton->setToolTip(QObject::tr("Show help"));
    connect(helpButton, SIGNAL(released()), this, SLOT(onHelpBtn_clicked()));
    
    BShowLineMenu = new QPushButton();
    BShowLineMenu->hide();
    BShowLineMenu->setCheckable(true);
    BShowLineMenu->setFlat(true);
    overlay->setButtonStyle(BShowLineMenu);
    BShowLineMenu->setIcon(makeHugeIcon(":/plot/line2DPlotSettings.png"));
    BShowLineMenu->setToolTip(QObject::tr("Show line settings"));
    connect(BShowLineMenu, SIGNAL(clicked(bool)), this, SLOT(onBShowLineMenuClicked()));
    
    overlay->addRightButton(BShowLineMenu);
    overlay->addRightButton(B2DSendToData);
    overlay->addRightButton(B2DSaveImage);
    overlay->addRightButton(B2DRestoreScale);
    overlay->addRightButton(B2DGrid);
    overlay->addRightButton(B2DMarkers);
    overlay->addRightButton(B2DFilter);
    
    helpScrollArea = new QScrollArea();
    helpScrollArea->hide();
    helpScrollArea->setStyleSheet("QSplitter {border: 40px solid black; border-style: outset; border-width: 40px; border-radius: 10px; border-color: #B4B4B4; background-color: #E3E3EC;}");
}

void ovcontrol_2d::set2DPlot(plot::T2dPlot *pl)
{
    parentPlot = pl;
    
    overlay->setButtonActive(B2DGrid, pl->Pure2dPlot->PlGrid->isVisible());
    B2DGrid->change_state_man(pl->Pure2dPlot->PlGrid->isVisible());
    overlay->setButtonActive(B2DFilter, parentPlot->Pure2dPlot->isFiltering);
    B2DFilter->change_state_man(parentPlot->Pure2dPlot->isFiltering);
    helpScrollArea->setParent(parentPlot);
    
    parentPlot->FillCBChooseActiveCurve_overlay();
    
    colorSettings->set2DPlot(pl);
    #ifndef _ADROID
    filterSettings->set2DPlot(pl);
#endif
    
#ifdef _ADROID
    if (vertical)
    {
        const QString FitOrders[] = {tr("Fit. None"), tr("Fit. 0 order"), tr("Fit. 1 order"), tr("Fit. 2 order"),
                                     tr("Fit. 3 order"), tr("Fit. 4 order"), tr("Fit. 5 order"), tr("Manual fit")};
        
        vMenuFilters = new verticalQMenu(parentPlot);
        vFilters = new verticalQPushButton(vMenuFilters, parentPlot);
        vFilters->hide();
        vFilters->setFlat(true);
        vFilters->setIcon(makeHugeIcon(":/plot/math.png"));
        rotareButtonIcon(vFilters, 90);
        vFilters->setIconSize(vFilters->size());
        int i = 0;
        for (auto &fitOrder:FitOrders)
        {
            auto item = vMenuFilters->addMenuItem(fitOrder, false);
            item->checkable = true;
            if (i == 0)
                item->checked = true;
            connect(item, SIGNAL(clicked(int)), this, SLOT(setFilterVertical(int)));
            i++;
        }
        overlay->setButtonStyle(vFilters, false);
    }
#endif
}

void ovcontrol_2d::on_FitOrder_changed()
{
    int id = FindCheckedItemIn(AGFitOrders);
    parentPlot->PlotControl->AGFitOrders->actions()[id]->setChecked(true);
    parentPlot->PlotControl->AGFitOrders->actions()[id]->trigger();
}
void ovcontrol_2d::onBShowLineMenuClicked()
{
    overlay->visMode = TwoDOverlayButtons::svm_color_settings;
    parentPlot->on_CBChooseActiveCurve_activated(0);
    overlay->showBack = true;
    overlay->setVisibleGUI2D(true);
}
void ovcontrol_2d::onFitDataClicked()
{
    parentPlot->PlotControl->BAutoScale->toggle();
}

void ovcontrol_2d::setShowingSendToDataButton(bool showing)
{
    if (showing)
    {
        overlay->addRightButton(B2DSendToData, 0);
        if (overlay->constantShowingMenu)
        {
            B2DSendToData->setParent(parentPlot);
            B2DSendToData->setVisible(true);
        }
    }
    else
    {
        B2DSendToData->setVisible(false);
        overlay->removeButton(B2DSendToData);
    }
    hideToDataButton = !showing;
    overlay->plot2DResized();
}
void ovcontrol_2d::on4DSelectCapturedSpectr_clicked(bool pressed)
{
    if (FourDScan == "")
        return;
    
    if (pressed)
        overlay->visMode = TwoDOverlayButtons::svm_select_captured_spectr;
    else
        overlay->visMode = TwoDOverlayButtons::svm_none;
    overlay->waitingEnterButton = pressed;
    overlay->showBack = pressed;
    overlay->setVisibleGUI2D(true);
}
void ovcontrol_2d::on4DSelectConvolutionFunc_clicked(bool pressed)
{
    if (FourDScan == "")
        return;
    
    if (pressed)
        overlay->visMode = TwoDOverlayButtons::svm_select_convolution_func;
    else
        overlay->visMode = TwoDOverlayButtons::svm_none;
    overlay->waitingEnterButton = pressed;
    overlay->showBack = pressed;
    overlay->setVisibleGUI2D(true);
}
void ovcontrol_2d::on4DSwitchSpectra_clicked(bool pressed)
{
    if (FourDScan == "")
    {
        B4DSectionPointSpectr->change_state_man(!pressed);
        return;
    }
}
void ovcontrol_2d::on2DMarkers_clicked(bool pressed)
{
    parentPlot->PlotControl->BMarkers->click();
    if (pressed)
    {
        overlay->visMode = TwoDOverlayButtons::svm_markers;
        overlay->showBack = true;
    }
    overlay->setVisibleGUI2D(true);
}
void ovcontrol_2d::on2DGrid_changed(bool pressed)
{
    overlay->setButtonActive(B2DGrid, pressed);
    parentPlot->PlotControl->BGrid->setChecked(pressed);
}
void ovcontrol_2d::on2DRestoreScale_clicked()
{
    parentPlot->Refresh();
    bool autoScaleEnabled = parentPlot->PlotControl->BAutoScale->isChecked();
#ifdef _ADROID
    autoScaleEnabled = autoScaleEnabled || vertical;
#endif
    parentPlot->Pure2dPlot->SetAutoScaleEnabled(autoScaleEnabled, true);
    parentPlot->PlotControl->BRestoreScale[0]->click();
}
void ovcontrol_2d::on2DSaveImage_clicked()
{
    overlay->blockShowing2DPlotButtons = true;
    overlay->setVisibleGUI2D(false);
    parentPlot->PlotControl->BSavePlotPixmap->click();
    overlay->blockShowing2DPlotButtons = false;
    overlay->setVisibleGUI2D(true);
}
void ovcontrol_2d::on2DSendToData_clicked()
{
    parentPlot->PlotControl->BSave->click();
}

void ovcontrol_2d::on2DFilter_clicked(bool pressed)
{
    overlay->setButtonActive(B2DFilter, pressed);
    parentPlot->setFiltering(pressed);
    if (parentPlot->depent2dPlot)
        parentPlot->depent2dPlot->setFiltering(pressed);
    else if (parentPlot->master2dPlot)
        parentPlot->master2dPlot->setFiltering(pressed);
#ifdef _ADROID
    if (vertical)
        parentPlot->Refresh();
#else
    auto filterSetSize = B2DFilter->size().height()/2;
    BFiltersSettings->setFixedSize(filterSetSize, filterSetSize);
    BFiltersSettings->setIconSize(QSize(filterSetSize, filterSetSize));
#endif
}

void ovcontrol_2d::set4DScan(const QString &scan)
{
    FourDScan = scan;
    
    for (int i=0; i<overlay->leftBtnsList.size(); i++)
        overlay->removeButton(overlay->leftBtnsList.at(i));
    
    if (FourDScan != "")
    {
        overlay->addLeftButton(B4DSendToData);
        overlay->addLeftButton(B4DClearSpectrum);
        overlay->addLeftButton(B4DSectionPointSpectr);
    }
    
    if (FourDScan == "jmscan")
    {
        overlay->removeButton(B4DSelectCapturedSpectr);
        overlay->removeButton(B4DSelectConvolutionFunc);
        overlay->addLeftButton(B4DSameXYPoints);
    }
    if (FourDScan == "ccdscan")
    {
        overlay->removeButton(B4DSameXYPoints);
        overlay->addLeftButton(B4DSelectCapturedSpectr);
        overlay->addLeftButton(B4DSelectConvolutionFunc);
    }
}

void ovcontrol_2d::setVisibleDataFitSizeButton(bool visible)
{
    dataFitSizeVisible = visible;
    if (visible)
        overlay->addRightButton(BFitData, 3);
    else
        overlay->removeButton(BFitData);
}

void ovcontrol_2d::removeFilterButton()
{
    hideFilterButton = true;
    isOldFiltersVisible = false;
    B2DFilter->setVisible(false);
    BOldFilter->setVisible(false);
    overlay->removeButton(B2DFilter);
    overlay->removeButton(BOldFilter);
    overlay->plot2DResized();
#ifndef _ADROID
    filterSettings->applyFilter2DSettings();
#endif
}

void ovcontrol_2d::setVisibleOldFilters(bool visible)
{
    if (isOldFiltersVisible == visible)
        return;
    
    hideFilterButton = !visible;
    isOldFiltersVisible = visible;
    if (visible)
    {
        parentPlot->Pure2dPlot->isFiltering = true;
        overlay->addRightButton(BOldFilter);
        overlay->removeButton(B2DFilter);
        B2DFilter->setVisible(false);
    }
    else
    {
        overlay->removeButton(BOldFilter);
        overlay->addRightButton(B2DFilter);
    }
    overlay->plot2DResized();
}

void ovcontrol_2d::resetOldFilters()
{
    AGFitOrders->actions()[0]->setChecked(true);
    AGFitOrders->actions()[0]->trigger();
    
}

#ifdef _ADROID
void ovcontrol_2d::setFilterVertical(int id)
{
    AGFitOrders->actions()[id]->setChecked(true);
    AGFitOrders->actions()[id]->trigger();
}
#endif

void ovcontrol_2d::onHelpBtn_clicked()
{
    overlay->visMode = TwoDOverlayButtons::svm_help;
    visModeHelpIsShowing = true;
    overlay->showBack = true;
    overlay->setVisibleGUI2D(true);
    parentPlot->Pure2dPlot->blockWheelZooming = true;
}

void ovcontrol_2d::setHelpImage(const QString path)
{
    QPixmap img = QPixmap(path);
    auto lab = new QLabel();
    lab->setPixmap(img);
    helpScrollArea->setWidget(lab);
    helpImgWidth = img.width() + 17;
    helpSetted = true;
    //allWidgets.append(helpScrollArea);
}

void ovcontrol_2d::plotResized(QSize newSize)
{
    helpScrollArea->move(overlay->buttonRect.x(),
                         overlay->B2DBack->pos().y() + overlay->B2DBack->height());
    int helpScrollAreaWidth = newSize.width() - overlay->buttonRect.x() - overlay->lastUpButtonSize*overlay->mulBtn;
    helpScrollAreaWidth = qMin(helpScrollAreaWidth, helpImgWidth);
    helpScrollArea->setFixedSize(helpScrollAreaWidth,
                                 (overlay->buttonRect.y() + overlay->buttonRect.height()) - overlay->B2DBack->pos().y() - overlay->B2DBack->height()*1.3);
    
#ifndef _ADROID
    B2DFilter->change_state_man(parentPlot->Pure2dPlot->isFiltering);
    overlay->setButtonActive(B2DFilter, parentPlot->Pure2dPlot->isFiltering);
#endif
    {

#ifndef _ADROID
        auto filterSetSize = B2DFilter->size().height()/2;
        BFiltersSettings->setFixedSize(filterSetSize, filterSetSize);
        BFiltersSettings->setIconSize(QSize(filterSetSize, filterSetSize));
        BFiltersSettings->move(filterSetSize, 0);
#endif
    }
}

void ovcontrol_2d::plotSetVisible(bool visible)
{
    bool visibleHelp = visible && (overlay->visMode == TwoDOverlayButtons::svm_help) && visModeHelpIsShowing;
    helpScrollArea->setVisible(visibleHelp);
    
    bool showBtns = visible && (overlay->visMode == TwoDOverlayButtons::svm_none);
    overlay->setButtonVisible(helpButton, showBtns && showHelpButton && helpSetted);
    
    //B2DFilter->setVisible(!hideFilterButton && visible);
    overlay->setButtonVisible(B2DSendToData, !hideToDataButton && showBtns);
}

void ovcontrol_2d::onBackClicked()
{
    if (B2DMarkers->isChecked())
        B2DMarkers->setChecked(false);
    else if (B4DSelectCapturedSpectr->isChecked() || B4DSelectConvolutionFunc->isChecked())
    {
        overlay->visMode = TwoDOverlayButtons::svm_none;
        B4DSelectCapturedSpectr->setChecked(false);
        B4DSelectConvolutionFunc->setChecked(false);
        emit overlay->on_BCancelApply_clickedSignal();
    }
    else if (BShowLineMenu->isChecked())
    {
        bool autoScale = parentPlot->Pure2dPlot->IsAutoScaleEnabled();
        BShowLineMenu->setChecked(false);
        if (!parentPlot->Pure2dPlot->singleAutoScale)
            parentPlot->Pure2dPlot->SetAutoScaleEnabled(autoScale, false);
    }
    else if (visModeHelpIsShowing)
    {
        visModeHelpIsShowing = false;
        parentPlot->Pure2dPlot->blockWheelZooming = false;
    }
}

}
