#ifndef OVCONTROL_2D_H
#define OVCONTROL_2D_H

#include <QPushButton>
#include <QActionGroup>
#include <QScrollArea>
#include <QLabel>
#include <QComboBox>
#ifdef _ADROID
#include "widgets/verticalqmenu.h"
#include "widgets/verticalqpushbutton.h"
#include "widgets/verticallabel.h"
#include "widgets/verticalqcombobox.h"
#else
#include "widgets/ovmenu_2d_filtersettings.h"
#endif
#include "widgets/twodoverlaybuttons.h"
#include "widgets/ovmenu_2d_colorsettings.h"

class TwoStateButton;
class colorRect2;

namespace plot {

class ovcontrol_2d : public QObject
{
    Q_OBJECT
public:
#ifdef _ADROID
    ovcontrol_2d(TwoDOverlayButtons *overlay, bool vertical);
#else
    ovcontrol_2d(TwoDOverlayButtons *overlay);
#endif
    
    void setHelpImage(const QString path);
    void resetOldFilters();
    void set4DScan(const QString &scan);
    void setVisibleOldFilters(bool visible);
    void setVisibleDataFitSizeButton(bool visible);
    void removeFilterButton();
    void set2DPlot(plot::T2dPlot *pl);
    
    bool isOldFiltersVisible = false;
    
    //2d buttons
    //4d
    TwoStateButton *B4DSameXYPoints;
    QPushButton *B4DSendToData;
    QPushButton *B4DClearSpectrum;
    TwoStateButton *B4DSectionPointSpectr;
    
    QPushButton *B4DSelectCapturedSpectr;
    QPushButton *B4DSelectConvolutionFunc;
    //2d
    QPushButton *B2DMarkers;
    TwoStateButton *B2DGrid;
    QPushButton *B2DRestoreScale;
    QPushButton *B2DSaveImage;
    QPushButton *B2DSendToData;
    TwoStateButton *B2DFilter;
#ifndef _ADROID
    QPushButton *BFiltersSettings;
    ovMenu_2d_filterSettings *filterSettings;
#endif
    QPushButton *BOldFilter;
    QPushButton *BFitData;
    QMenu *MMathProcessing;
    QActionGroup *AGFitOrders;
    QPushButton *BShowLineMenu;
    
    bool hideFilterButton = false;
    
    QPushButton *helpButton;
    bool showHelpButton;
    int helpImgWidth = 1;
    
    ovMenu_2d_colorSettings *colorSettings;

#ifdef _ADROID
    bool vertical = false;
    
    verticalQPushButton *vFilters;
    verticalQMenu *vMenuFilters;
#endif
    
public slots:
    void onBShowLineMenuClicked();
    void on4DSwitchSpectra_clicked(bool pressed);
    void on4DSelectCapturedSpectr_clicked(bool pressed);
    void on4DSelectConvolutionFunc_clicked(bool pressed);
    void on2DMarkers_clicked(bool pressed);
    void on2DGrid_changed(bool pressed);
    void on2DRestoreScale_clicked();
    void on2DSaveImage_clicked();
    void on2DSendToData_clicked();
    void on2DFilter_clicked(bool pressed);
    void setShowingSendToDataButton(bool showing);
    void onFitDataClicked();
    void onHelpBtn_clicked();
    void on_FitOrder_changed();
#ifdef _ADROID
    void setFilterVertical(int);
#endif
    
private slots:
    void plotResized(QSize newSize);
    void plotSetVisible(bool visible);
    void onBackClicked();
    
private:
    TwoDOverlayButtons *overlay;
    T2dPlot *parentPlot;
    
    QString FourDScan = "";
    bool hideToDataButton = false;
    bool dataFitSizeVisible = false;
    
    QScrollArea *helpScrollArea;
    QImage helpImage;
    bool helpSetted = false; //Does help image is setted? If not - helpButton will not be visible
    bool visModeHelpIsShowing = false;
    
};

}

#endif // OVCONTROL_2D_H
