#ifndef OVCONTROL_3D_H
#define OVCONTROL_3D_H

#include <QObject>
#include <QPushButton>
#include <QLabel>
#include <QComboBox>
#include <QDoubleSpinBox>
#include <QActionGroup>
#include "core/plot_core.h"
#include <QCheckBox>
#include <QScrollArea>
#include "../../src/gui/edit.hpp"
#include <QGraphicsOpacityEffect>
#include <QPropertyAnimation>
#include "../widgets/threedoverlaybuttons.h"
#include "../widgets/colorrect2.h"

namespace plot {

#ifndef _ADROID
class ovMenu_3d_FilterSettings;
#endif
class ovmenu_3d_palettes;
class ThreeDOverlayButtons;

class ovControl_3d : public QObject
{
    Q_OBJECT
public:
    ovControl_3d(ThreeDOverlayButtons *overlay);
    
    void ActivateRatisMode();
    void setVisibleFilterButton(bool visible);
    void setVisibleOldFilters(bool visible);
    void resetOldFilters();
    void setVisiblePointSelection(bool visible);
    void setHelpImage(const QString path);
    void setAttributes(plot::TMetaStorage *attributes);
    
    ovmenu_3d_palettes *palettesMenu;
    
    QPushButton *BPointSelection;
    QPushButton *BMarkers;
    TwoStateButton *BFilters;
    #ifndef _ADROID
    ovMenu_3d_FilterSettings *filterSettingsMenu;
    QPushButton *BFiltersSettings;
#endif
    QPushButton *BAreaZzoom;
    QPushButton *BPalette;
    QPushButton *BSaveImage;
    QPushButton *BOldFilter;
    QPushButton *BOldFilterAddit;
    QMenu *MMath2;
    bool hideFilterButton = false;
    QPushButton *helpButton;
    bool showHelpButton;
    int helpImgWidth = 1;
    plot::TMetaStorage *attributes = nullptr;
    bool helpSetted = false; //Does help image is setted? If not - helpButton will not be visible
    
public slots:
    void onPointSelection_changed();
    void on_Filter_changed(bool pressed);
    void onMarkers_changed();
    void colorPickClicked();
    void colorPickClicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void filterSettingsClicked();
    void on3DBAreaZoom_changed();
    void onSaveImage_clicked();
    void on_GwyXFit_triggered(bool Checked);
    void on_GwyYFit_triggered(bool Checked);
    void on_MeanValue_triggered(bool Checked);
    void on_MedianValue_triggered(bool Checked);
    void on_Gaussian_triggered(bool Checked);
    void on_Histogram_triggered(bool Checked);
    void on_FitXOrder_changed();
    void on_FitYOrder_changed();
    void on_SubtrOrder_changed();
    void onHelpBtn_clicked();
    
private slots:
    void setVisiblePlot(bool visible, plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void resized(QSize newSize);
    void ApplyEnterClicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void BackCancelClicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    
private:
    void InitFilters();
    void InitOldFilters();
    void CreateActionsAndMenus();
    
    ThreeDOverlayButtons *overlay;
    TPure3dPlot *parentPlot;
    T3dPlotControl *PlotControl;
    
    bool isOldFiltersVisible = false;
    QActionGroup *AGXFitOrders;
    QActionGroup *AGYFitOrders;
    QActionGroup *AGSubtr;
    QMenu *MMathProcessing;
    
    QScrollArea *helpScrollArea;
};

}

#endif // OVCONTROL_3D_H
