#include "ovmenu_2d_colorsettings.h"
#include "2d/plot_2dplot.h"
#include "qwt_plot_layout.h"
#include <QMouseEvent>
#include "widgets/twodoverlaybuttons.h"
#include "widgets/colorrect2.h"
#include "2d/plot_2dpure.h"
#include <QSettings>
#include <QGraphicsOpacityEffect>
#include <QPropertyAnimation>
#include "2d/ovcontrol_2d.h"
#include <QPainter>

namespace plot {

ovMenu_2d_colorSettings::ovMenu_2d_colorSettings(TwoDOverlayButtons *overlay) : colorRect2(QColor(240, 240, 240))
{
    this->overlay = overlay;
    this->hide();
    
    connect(overlay, SIGNAL(replacedGUIPlotResized(QSize)), this, SLOT(plotResized(QSize)));
    connect(overlay, SIGNAL(replacedGUIPlotSetVisible(bool)), this, SLOT(plotSetVisible(bool)));
    #ifdef _ADROID
    connect(overlay, SIGNAL(eventSignal(QEvent*)), this, SLOT(eventSlot(QEvent*)));
#endif
    
    style = "color: #8C8C8C;";
    CurveQColours[0] = Qt::black;
    CurveQColours[1] = Qt::red;
    CurveQColours[2] = Qt::green;
    CurveQColours[3] = Qt::blue;
    CurveQColours[4] = Qt::magenta;
    CurveQColours[5] = Qt::cyan;
    CurveQColours[6] = Qt::black;
    
    //--
    
    LLineSelector = new QLabel(tr("Current line:"));
    LLineSelector->hide();
    
    LLineColor = new QLabel(tr("Line color:"));
    LLineColor->hide();
    
    LLineWidth = new QLabel(tr("Line width:"));
    LLineWidth->hide();
    
    LFont = new QLabel(tr("Font size:"));
    LFont->hide();
    
    QString cbStyle = "QComboBox {border-width:0;}\
            QComboBox:!on {" + style + "}\
            QComboBox:pressed {border-width:1;}";
    cbStyle = "";
    
    CBLineSelector = new QComboBox;
    CBLineSelector->hide();
    CBLineSelector->setStyleSheet(cbStyle);
    CBLineSelector->addItem("All");
    
    CBLineColor = new QComboBox;
    CBLineColor->hide();
    CBLineColor->setStyleSheet(cbStyle);
    const char *CurveColours[] = {"Black", "Red", "Green", "Blue", "Magenta", "Cyan", "Other..."};
    for (int i=0; i<7; i++)
    {
        auto icon = getColoredIcon(CurveQColours[i], 190, 50);
        
        CBLineColor->addItem(icon, "");
        CBLineColor->setItemData(i, CurveQColours[i]);
    }
    CBLineColor->setCurrentIndex(0);
    connect(CBLineColor, SIGNAL(currentIndexChanged(int)), this, SLOT(onCBLineColor_changed(int)));
    
    CBLineWidth = new QComboBox;
    CBLineWidth->hide();
    CBLineWidth->setStyleSheet(cbStyle);
    const int CurveThickness[] = {1, 2, 3, 4, 5, 6, 8, 10, 15, 20};
    for (int i=0; i<10; i++)
    {
        auto icon = getColoredIcon(Qt::black, 100, CurveThickness[i] * 5);
        CBLineWidth->addItem(icon, QString::number(CurveThickness[i]), CurveThickness[i]);
    }
    CBLineWidth->setCurrentIndex(0);
    connect(CBLineWidth, SIGNAL(currentIndexChanged(int)), this, SLOT(onCbLineWidth_changed(int)));
    
    CBFont = new QComboBox;
    CBFont->hide();
    CBFont->setStyleSheet(cbStyle);
    CBFont->addItem("6", 6);
    CBFont->addItem("8", 8);
    CBFont->addItem("10", 10);
    for (int i = 0; i < 11; i++) {
        CBFont->addItem(QString::number(12 + 4*i), 12 + 4*i);
    }
    CBFont->setCurrentIndex(3);
    connect(CBFont, SIGNAL(currentIndexChanged(int)), this, SLOT(fontChanged(int)));
    
    this->setStyleSheet("QSplitter {border: 1px solid #B4B4B4; border-style: outset; border-width: 2px; border-radius: 10px; border-color: #B4B4B4; background-color: #E3E3EC;}");
    
    lineChangersListLabels << LLineSelector << LLineColor << LLineWidth << LFont;
    lineChangersListComboBox << CBLineSelector << CBLineColor << CBLineWidth << CBFont;
    
    QString style = "color: black; font: 17px;";
    for (auto lab:lineChangersListLabels)
    {
        lab->setStyleSheet(style);
    }
}

QIcon ovMenu_2d_colorSettings::getColoredIcon(QColor color, int width, int height)
{
    QPixmap px = QPixmap(width, height);
    QPainter *paint = new QPainter(&px);
    paint->setBrush(QBrush(color));
    paint->setPen(color);
    paint->drawRect(0, 0, px.width(), px.height());
    delete paint;
    return QIcon(px);
}

#ifdef _ADROID
void ovMenu_2d_colorSettings::eventSlot(QEvent *Event)
{
    if (Event->type() == QEvent::MouseButtonPress)
    {
        if (vCBLineSelector->vMenu->isVisible())
            vCBLineSelector->event(Event);
        if (vCBLineColor->vMenu->isVisible())
            vCBLineColor->event(Event);
        if (vCBLineWidth->vMenu->isVisible())
            vCBLineWidth->event(Event);
        if (vCBFont->vMenu->isVisible())
            vCBFont->event(Event);
    }
}
#endif

void ovMenu_2d_colorSettings::set2DPlot(plot::T2dPlot *pl)
{
    parentPlot = pl;
    this->setParent(pl);
    connect(CBLineSelector, SIGNAL(highlighted(int)), pl, SLOT(on_CBChooseActiveCurve_highlighted(int)));
    connect(CBLineSelector, SIGNAL(activated(int)), pl, SLOT(on_CBChooseActiveCurve_activated(int)));
    for (auto lab:lineChangersListLabels)
    {
        lab->setParent(pl);
    }
    for (auto w:lineChangersListComboBox)
    {
        w->setParent(pl);
    }
    
    #ifdef _ADROID
    if (vertical)
    {
        vLLineSelector = new VerticalLabel(tr("Current line:"));
        vLLineSelector->hide();
        
        vLLineColor = new VerticalLabel(tr("Line color:"));
        vLLineColor->hide();
        
        vLLineWidth = new VerticalLabel(tr("Line width:"));
        vLLineWidth->hide();
        
        vLFont = new VerticalLabel(tr("Font size:"));
        vLFont->hide();
        
        vCBLineSelector = new verticalQComboBox();
        vCBLineSelector->hide();
        vCBLineSelector->addItem("All", QVariant());
        vCBLineSelector->setCurrentIndex(0);
        
        vCBLineColor = new verticalQComboBox();
        vCBLineColor->hide();
        
        const char *CurveColours[] = {"Black", "Red", "Green", "Blue", "Magenta", "Cyan", "Other..."};
        for (int i=0; i<7; i++)
        {
            auto icon = getColoredIcon(CurveQColours[i]);
            auto pixmap = icon.pixmap(100, 70);
            vCBLineColor->addItem(CurveColours[i], QBrush(CurveQColours[i]));
        }
        vCBLineColor->setCurrentIndex(0);
        connect(vCBLineColor, SIGNAL(currentIndexChanged(int)), this, SLOT(onCBLineColor_changed(int)));
        
        vCBLineWidth = new verticalQComboBox();
        vCBLineWidth->hide();
        const int CurveThickness[] = {1, 2, 3, 4, 5, 6, 8, 10, 15, 20};
        for (int i=0; i<10; i++)
        {
            auto icon = getColoredIcon(Qt::black, CurveThickness[i] * 5);
            auto pixmap = icon.pixmap(100, CurveThickness[i] * 5);
            vCBLineWidth->addItem(QString::number(CurveThickness[i]), CurveThickness[i]);
        }
        vCBLineWidth->setCurrentIndex(0);
        connect(vCBLineWidth, SIGNAL(currentIndexChanged(int)), this, SLOT(onCbLineWidth_changed(int)));
        
        vCBFont = new verticalQComboBox();
        vCBFont->hide();
        vCBFont->addItem("6", 6);
        vCBFont->addItem("8", 8);
        vCBFont->addItem("10", 10);
        for (int i = 0; i < 11; i++) {
            vCBFont->addItem(QString::number(12 + 4*i), 12 + 4*i);
        }
        vCBFont->setCurrentIndex(3);
        connect(vCBFont, SIGNAL(currentIndexChanged(int)), this, SLOT(fontChanged(int)));
        lineChangersListLabels.clear();
        lineChangersListComboBox.clear();
        lineChangersListLabels << vLLineSelector << vLLineColor << vLLineWidth << vLFont;
        lineChangersListComboBox << vCBLineSelector << vCBLineColor << vCBLineWidth << vCBFont;
    }
#endif
}

void ovMenu_2d_colorSettings::onCBLineColor_changed(int n)
{
    bool autoScale = parentPlot->Pure2dPlot->IsAutoScaleEnabled();
    parentPlot->Pure2dPlot->SetAutoScaleEnabled(false, false);
    parentPlot->PlotControl->AGCurveColours->actions()[n]->setChecked(true);
    parentPlot->PlotControl->AGCurveColours->actions()[n]->trigger();
    if (!parentPlot->Pure2dPlot->singleAutoScale)
        parentPlot->Pure2dPlot->SetAutoScaleEnabled(autoScale, false);
    int line = parentPlot->Pure2dPlot->ActiveCurve;
    if (line >= 0)
    {
        CBLineColor->setItemData(n, parentPlot->lineColors[line]);
        CBLineColor->setItemIcon(n, getColoredIcon(parentPlot->lineColors[line], 190, 50));
    }
//    for (int i=0; i<7; i++)
//        CBLineColor->setItemData(i, QBrush(CurveQColours[i]), Qt::TextColorRole);
}
void ovMenu_2d_colorSettings::onCbLineWidth_changed(int n)
{
    bool autoScale = parentPlot->Pure2dPlot->IsAutoScaleEnabled();
    parentPlot->Pure2dPlot->SetAutoScaleEnabled(false, false);
    parentPlot->PlotControl->AGCurveThickness->actions()[n]->setChecked(true);
    parentPlot->PlotControl->AGCurveThickness->actions()[n]->trigger();
    if (!parentPlot->Pure2dPlot->singleAutoScale)
        parentPlot->Pure2dPlot->SetAutoScaleEnabled(autoScale, false);
}

void ovMenu_2d_colorSettings::fontChanged(int n)
{
    bool autoScale = parentPlot->Pure2dPlot->IsAutoScaleEnabled();
    parentPlot->Pure2dPlot->SetAutoScaleEnabled(false, false);
    parentPlot->PlotControl->AGFontSizes->actions().at(n)->trigger();
    if (!parentPlot->Pure2dPlot->singleAutoScale)
        parentPlot->Pure2dPlot->SetAutoScaleEnabled(autoScale, false);
}

void ovMenu_2d_colorSettings::plotResized(QSize newSize)
{
    int cbSize = (newSize.height() - overlay->buttonRect.y())*0.05*1.14;
    cbSize = overlay->lastUpButtonSize;
    if (cbSize < 0) cbSize = 0;
    if (cbSize > 50) cbSize = 50;
    auto font = QFont(LFont->font().family(), cbSize*0.45);
    font.setBold(true);

#ifdef _ADROID
    if (vertical)
    {
        overlay->control->vFilters->setIconSize(QSize(overlay->lastUpButtonSize, overlay->lastUpButtonSize));
        int maxHeightCB = 0;
        for (int i=0; i<lineChangersListComboBox.size(); i++)
        {
            int t = lineChangersListComboBox.at(i)->sizeHint().height();
            if (t > maxHeightCB)
                maxHeightCB = t;
        }
        
        for (int i=0; i<lineChangersListComboBox.size(); i++)
        {
            auto item = lineChangersListComboBox.at(i);
            item->setFixedSize(cbSize, maxHeightCB);
        }
        
        int maxRightY = 0;
        int cbsWidth = 0;
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            int t = lineChangersListLabels.at(i)->height() + lineChangersListComboBox.at(i)->height();
            if (t > maxRightY)
                maxRightY = t;
            cbsWidth += lineChangersListComboBox.at(i)->width();
        }
        
        this->setFixedSize(cbsWidth + 20*2, maxRightY + 20*2);
        int x = overlay->buttonRect.x() + overlay->buttonRect.width() - this->width() - 20;
        int y = overlay->buttonRect.y() + overlay->buttonRect.height() - this->height() - 200;
        this->move(x, y - 20);
        x += cbsWidth;
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            lineChangersListLabels.at(i)->move(x, y);
            lineChangersListComboBox.at(i)->move(x, y + maxRightY - lineChangersListComboBox.at(i)->height());
            x -= lineChangersListComboBox.at(i)->width();
        }
    }
    else
#endif
    {     
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            lineChangersListLabels.at(i)->setScaledContents(true);
            //lineChangersListLabels.at(i)->setFont(font);
            lineChangersListLabels.at(i)->adjustSize();
        }
        int maxWidthCB = 0;
        for (int i=0; i<lineChangersListComboBox.size(); i++)
        {
            int t = lineChangersListComboBox.at(i)->sizeHint().width();
            lineChangersListComboBox.at(i)->setFixedWidth(t);
            lineChangersListComboBox.at(i)->setFixedHeight(lineChangersListComboBox.at(i)->sizeHint().height());
            if (t > maxWidthCB)
                maxWidthCB = t;
        }
        
        for (int i=0; i<lineChangersListComboBox.size(); i++)
        {
            lineChangersListComboBox.at(i)->setFixedWidth(maxWidthCB);
            //auto item = lineChangersListComboBox.at(i);
            //auto view = item;
            //view->setFont(font);
            //item->setFixedSize(maxWidthCB, cbSize);
            //item->setFont(font);
        }
        
//        int maxRightX = 0;
//        int cbsHeight = 0;
//        for (int i=0; i<lineChangersListLabels.size(); i++)
//        {
//            int t = lineChangersListLabels.at(i)->width()*1.3 + lineChangersListComboBox.at(i)->sizeHint().width();
//            if (t > maxRightX)
//                maxRightX = t;
//            cbsHeight += lineChangersListComboBox.at(i)->sizeHint().height();
//        }
        
//        int x = newSize.width()*0.5 - maxRightX*0.5;
//        int y = newSize.height()*0.5 - cbsHeight*0.5;
        
//        this->setFixedSize(maxRightX + 20*2, cbsHeight + 20*2);
        
//        //x = leftX*1.3;
//        x = (overlay->buttonRect.x() + overlay->buttonRect.width()) - this->width();
//        y = overlay->buttonRect.y()*1.9;
        
        
//        this->move(x - 20, y - 20);
//        for (int i=0; i<lineChangersListLabels.size(); i++)
//        {
//            lineChangersListLabels.at(i)->move(x, y + lineChangersListLabels.at(i)->height()*0.2);
//            lineChangersListComboBox.at(i)->move(x + maxRightX - lineChangersListComboBox.at(i)->sizeHint().width(), y);
//            y += lineChangersListComboBox.at(i)->sizeHint().height();
//        }
        
        const int edge = 20;
        const int distanceBetween = 10;
        int maxRightX = 0;
        int cbsHeight = 0;
        int maxLeftWidth = 0;
        int maxRightWidth = 0;
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            int widthLab = lineChangersListLabels.at(i)->minimumSizeHint().width();
            if (widthLab > maxLeftWidth)
                maxLeftWidth = widthLab;
            int widthRight = lineChangersListComboBox.at(i)->minimumSizeHint().width();
            if (widthRight > maxRightWidth)
                maxRightWidth = widthRight;
            cbsHeight += lineChangersListComboBox.at(i)->minimumSizeHint().height();
            cbsHeight += distanceBetween;
        }
        maxRightX = maxLeftWidth + edge + maxRightWidth;
        
        int backWidth = maxRightX + edge*1.5;
        int backHeight = cbsHeight + edge*1.5;
        this->setFixedSize(backWidth, backHeight);
        int x = (overlay->buttonRect.x() + overlay->buttonRect.width()) - this->width() - 50 - edge;
        int y = overlay->buttonRect.y()*1.9;
        this->move(x, y);
        x += edge;
        y += edge;
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            int rWidth = qMin(maxRightWidth, lineChangersListComboBox.at(i)->width());
            
            lineChangersListLabels.at(i)->move(x, y + lineChangersListLabels.at(i)->minimumSizeHint().height()*0.2);
            lineChangersListComboBox.at(i)->move(x + maxRightX - rWidth - edge*0.5, y);
            y += (lineChangersListLabels.at(i)->minimumSizeHint().height() < lineChangersListComboBox.at(i)->minimumSizeHint().height() ?
                      lineChangersListComboBox.at(i)->minimumSizeHint().height():lineChangersListLabels.at(i)->minimumSizeHint().height());
            y += distanceBetween;
        }
    }
}

void ovMenu_2d_colorSettings::plotSetVisible(bool visible)
{
    for (int i=0; i<lineChangersListLabels.size(); i++)
    {
        lineChangersListLabels.at(i)->setParent(parentPlot);
        lineChangersListComboBox.at(i)->setParent(parentPlot);
    }
    
    visible = visible && (overlay->visMode == TwoDOverlayButtons::svm_color_settings);
    this->setVisible(visible);
    for (int i=0; i<lineChangersListLabels.size(); i++)
    {
        lineChangersListLabels.at(i)->setVisible(visible);
        lineChangersListComboBox.at(i)->setVisible(visible);
    }
}

}
