#include "ovmenu_2d_filtersettings.h"

#include "2d/plot_2dplot.h"
#include "qwt_plot_layout.h"
#include <QMouseEvent>
#include "widgets/twodoverlaybuttons.h"
#include "widgets/colorrect2.h"
#include "2d/plot_2dpure.h"
#include <QSettings>
#include <QGraphicsOpacityEffect>
#include <QPropertyAnimation>
#include "2d/ovcontrol_2d.h"

namespace plot {

ovMenu_2d_filterSettings::ovMenu_2d_filterSettings(TwoDOverlayButtons *overlay) : colorRect2(QColor(240, 240, 240))
{
    this->overlay = overlay;
    this->hide();
    
    connect(overlay, SIGNAL(replacedGUIPlotResized(QSize)), this, SLOT(plotResized(QSize)));
    connect(overlay, SIGNAL(replacedGUIPlotSetVisible(bool)), this, SLOT(plotSetVisible(bool)));
    connect(overlay, SIGNAL(onBackClicked()), this, SLOT(onBackClicked()));
    
    auto labFilter = new QLabel(tr("Filter"));
    fontFamily = labFilter->font().family();
    labFilter->hide();
    
#ifndef _ADROID
    //----------Filter 2D settings
    const QString FitOrders2D[] = {tr("Fit. None"), tr("Fit. 0 order"), tr("Fit. 1 order"), tr("Fit. 2 order"),
                                   tr("Fit. 3 order"), tr("Fit. 4 order"), tr("Fit. 5 order"), tr("Manual fit")};
    cb2DFilter = new QComboBox();
    cb2DFilter->hide();
    
    for (auto &fitOrder:FitOrders2D)
    {
        cb2DFilter->addItem(fitOrder);
    }
    
    //loading filter settings
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        settings.beginGroup("2D_plot_filter");
        cb2DFilter->setCurrentIndex(settings.value("filter", 0).toInt());
        settings.endGroup();
    }
    connect(cb2DFilter, SIGNAL(currentIndexChanged(int)), this, SLOT(saveScan2DFilterSettings()));
#endif
    
    filterSettingsLabels << labFilter;
    filterSettingsRightPart << cb2DFilter;
    
    this->setStyleSheet("QSplitter {border: 40px solid black; border-style: outset; border-width: 40px; border-radius: 10px; border-color: #B4B4B4; background-color: #E3E3EC;}");
    
    QString style = "color: black; font: bold 17px;";
    for (auto lab:filterSettingsLabels)
    {
        lab->setStyleSheet(style);
    }

}

#ifndef _ADROID
void ovMenu_2d_filterSettings::saveScan2DFilterSettings()
{
    QSettings settings("Param/scan.conf", QSettings::IniFormat);
    settings.beginGroup("2D_plot_filter");
    settings.setValue("filter", cb2DFilter->currentIndex());
    settings.endGroup();
    
    applyFilter2DSettings();
}

void ovMenu_2d_filterSettings::applyFilter2DSettings()
{
    int fitOrder = cb2DFilter->currentIndex();
    if (overlay->control->hideFilterButton)
        fitOrder = 0;
    if (parentPlot && parentPlot->PlotControl && parentPlot->PlotControl->AGFitOrders &&
        fitOrder < parentPlot->PlotControl->AGFitOrders->actions().count())
    {
        parentPlot->PlotControl->AGFitOrders->actions()[fitOrder]->setChecked(true);
        parentPlot->PlotControl->AGFitOrders->actions()[fitOrder]->trigger();
        
        parentPlot->Refresh();
    }
    
    emit changedFilterSettings(fitOrder);
}

void ovMenu_2d_filterSettings::filterSettingsClicked()
{
    overlay->visMode = TwoDOverlayButtons::svm_filter_settings;
    showingfilterSettings = true;
    overlay->showBack = true;
    overlay->setVisibleGUI2D(true);
}
#endif

void ovMenu_2d_filterSettings::set2DPlot(plot::T2dPlot *pl)
{
    parentPlot = pl;
    this->setParent(pl);
    saveScan2DFilterSettings();
#ifndef _ADROID
    applyFilter2DSettings();
#endif
    for (auto lab:filterSettingsLabels)
    {
        lab->setParent(pl);
    }
    for (auto w:filterSettingsRightPart)
    {
        w->setParent(pl);
    }
}

void ovMenu_2d_filterSettings::plotResized(QSize newSize)
{
    int cbSize = (newSize.height() - overlay->buttonRect.y())*0.05*1.14;
    cbSize = overlay->lastUpButtonSize;
    if (cbSize < 0) cbSize = 0;
    if (cbSize > 50) cbSize = 50;
    auto font = QFont(fontFamily, cbSize*0.45);
    font.setBold(true);
    
    for (int i=0; i<filterSettingsLabels.size(); i++)
    {
        filterSettingsLabels.at(i)->setScaledContents(true);
        filterSettingsLabels.at(i)->setFont(font);
        filterSettingsLabels.at(i)->adjustSize();
    }
    int maxWidthCB = 0;
    for (int i=0; i<filterSettingsRightPart.size(); i++)
    {
        int t = filterSettingsRightPart.at(i)->minimumSizeHint().width();
        if (t > maxWidthCB)
            maxWidthCB = t;
    }
    
    cb2DFilter->setMaximumHeight(cb2DFilter->minimumSizeHint().height());
    cb2DFilter->setMaximumWidth(cb2DFilter->minimumSizeHint().width());
    
    int maxRightX = 0;
    int cbsHeight = 0;
    for (int i=0; i<filterSettingsLabels.size(); i++)
    {
        int t = filterSettingsLabels.at(i)->width() + filterSettingsRightPart.at(i)->width();
        if (t > maxRightX)
            maxRightX = t;
        cbsHeight += filterSettingsRightPart.at(i)->height();
    }
    
    int x = newSize.width()*0.5 - maxRightX*0.5;
    int y = newSize.height()*0.5 - cbsHeight*0.5;
    
    this->setFixedSize(maxRightX + 20*2, cbsHeight + 20*1.7);
    
    //x = leftX*1.3;
    x = (overlay->buttonRect.x() + overlay->buttonRect.width()) - this->width();
    y = overlay->buttonRect.y()*2;
    
    this->move(x - 20, y - 20);
    for (int i=0; i<filterSettingsLabels.size(); i++)
    {
        filterSettingsLabels.at(i)->move(x, y + filterSettingsLabels.at(i)->height()*0.2);
        filterSettingsRightPart.at(i)->move(x + maxRightX - filterSettingsRightPart.at(i)->width() + 5, y);
        y += filterSettingsRightPart.at(i)->height();
    }
}

void ovMenu_2d_filterSettings::plotSetVisible(bool visible)
{
#ifndef _ADROID
    for (int i=0; i<filterSettingsLabels.size(); i++)
    {
        filterSettingsLabels.at(i)->setParent(parentPlot);
        filterSettingsRightPart.at(i)->setParent(parentPlot);
    }
    
    visible = (overlay->visMode == TwoDOverlayButtons::svm_filter_settings);
    this->setVisible(visible);
    for (int i=0; i<filterSettingsLabels.size(); i++)
    {
        filterSettingsLabels.at(i)->setVisible(visible);
        filterSettingsRightPart.at(i)->setVisible(visible);
    }
#endif
}

void ovMenu_2d_filterSettings::onBackClicked()
{
#ifndef _ADROID
    showingfilterSettings = false;
#endif
}

}
