#include "ovmenu_3d_palettes.h"
#include "3d/plot_3dplot.h"
#include "3d/plot_3dpure.h"
#include <QAbstractItemView>
#include "qwt_plot_layout.h"

namespace plot {

ovmenu_3d_palettes::ovmenu_3d_palettes(ThreeDOverlayButtons *overlay) : QWidget()
{
    this->overlay = overlay;
    connect(overlay, &ThreeDOverlayButtons::GUIPlotSetVisible, this, &ovmenu_3d_palettes::setVisiblePlot);
    connect(overlay, &ThreeDOverlayButtons::GUIPlotResized, this, &ovmenu_3d_palettes::resized);
    
    LFont = new QLabel(tr("Font size:"));
    LFont->setStyleSheet("QLabel { color: rgb(100,100,100)}");
    LFont->hide();

    CBFont = new QComboBox;
    CBFont->hide();
    CBFont->setStyleSheet("QComboBox {border-width:0; background-color:rgba(255,255,255,100);}\
                          QComboBox:pressed { background-color:rgba(255,255,255,100); border-width:1;}");
    CBFont->addItem("6", 6);
    CBFont->addItem("8", 8);
    CBFont->addItem("10", 10);
    for (int i = 0; i < 11; i++) {
        CBFont->addItem(QString::number(12 + 4*i), 12 + 4*i);
    }
    CBFont->setCurrentIndex(3);
    connect(CBFont, SIGNAL(currentIndexChanged(int)), this, SLOT(fontChanged(int)));
    
    colorChooser = new plot_colorChooser(this);
    colorChooser->hide();
    connect(colorChooser, SIGNAL(colorChoosed(QColor, int)), this, SLOT(PaletteChoosed(QColor, int)));
    
    LFont->setParent(overlay->parentPlot);
    CBFont->setParent(overlay->parentPlot);
    colorChooser->setParent(overlay->parentPlot);
}

void ovmenu_3d_palettes::setVisiblePlot(bool visible, plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode)
{
    bool showPalettes = (visibilityMode == plot::ThreeDOverlayButtons::svm_palettes);
    LFont->setVisible(showPalettes);
    CBFont->setVisible(showPalettes);
    colorChooser->setVisible(showPalettes);
}

void ovmenu_3d_palettes::resized(QSize newSize)
{
    int downYH = overlay->parentPlot->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
    int colorW = overlay->buttonRect.width();
    int colorH = overlay->buttonRect.height();
    int colorSz = (colorW < colorH) ? colorW:colorH;
    colorChooser->updateRectSizes();
    colorChooser->setFixedSize(colorSz*0.4, colorSz*0.4);
    colorChooser->move(
                        overlay->buttonRect.x() + colorW*0.5 - colorChooser->width()*0.5,
                        overlay->buttonRect.y() + (colorH - overlay->buttonRect.y() - downYH)*0.5 - colorChooser->height()*0.5);

    int cbSize = (newSize.height() - overlay->buttonRect.y())*0.05*1.14;
    if (cbSize < 1) cbSize = 1;
    if (cbSize > 9999) cbSize = 9999;
    auto font = QFont(LFont->font().family(), cbSize*0.45);
    font.setBold(true);
    LFont->setScaledContents(true);
    LFont->setFont(font);
    LFont->adjustSize();
    int cbfontSize = colorChooser->width() - LFont->width()*1.15;
    if (cbfontSize < 1) cbfontSize = 1;
    CBFont->setFixedSize(cbfontSize, cbSize);
    CBFont->setFont(font);
    auto view = CBFont->view();
    view->setFont(font);

    int y = colorChooser->y() + colorChooser->height();

    LFont->move(colorChooser->x(), y + LFont->height()*0.2);
    CBFont->move(LFont->x() + LFont->width()*1.05, y);
    
    colorChooser->installEventFilter(overlay);
    
    LFont->setParent(overlay->parentPlot);
    CBFont->setParent(overlay->parentPlot);
    colorChooser->setParent(overlay->parentPlot);
}

void ovmenu_3d_palettes::fontChanged(int n)
{
    overlay->PlotControl->AGFontSizes->actions().at(n)->trigger();
}

void ovmenu_3d_palettes::PaletteChoosed(QColor color, int pal)
{
    overlay->parentPlot->PaletteChoosed(color, pal);
    overlay->BBackButton->click();
}

}
