#include <QObject>
#include <QWidget>
#include "widgets/threedoverlaybuttons.h"
#include "3d/plot_3dplot.h"
#include "3d/plot_3dpure.h"
#include "../../src/gui/controls.h"
#include "qwt_plot_layout.h"
#include "core/plot_core.h"
#include <QAbstractItemView>
#include <QTimer>
#include "../3d/ovcontrol_3d.h"

namespace plot {

int ThreeDOverlayButtons::fadingAnimationMS = 1000;

ThreeDOverlayButtons::ThreeDOverlayButtons() : QWidget(nullptr)
{
    QSettings settings2("Param/gui.conf", QSettings::IniFormat);
    useBlockResize = settings2.value("gui/Ratis_xyz_stage_mode", false).toBool();

    installEventFilter(this);

    BBackButton = new QPushButton();
    BBackButton->hide();
    BBackButton->setFlat(true);
    setButtonStyle(BBackButton);
    BBackButton->setIcon(makeHugeIcon(":/ScanPlot/backPlot3D.png"));
    BBackButton->setToolTip(QObject::tr("Cancel"));
    BBackButton->setShortcut(QKeySequence(Qt::Key_Escape));
    connect(BBackButton, SIGNAL(released()), this, SLOT(onBack_clicked()));

    BApplyButton = new QPushButton();
    BApplyButton->hide();
    BApplyButton->setFlat(true);
    setButtonStyle(BApplyButton);
    BApplyButton->setIcon(makeHugeIcon(":/ScanPlot/applyPlot.png"));
    BApplyButton->setToolTip(QObject::tr("Apply"));
    //BApplyButton->setShortcut(QKeySequence(Qt::Key_Enter));
    //connect(BApplyButton, SIGNAL(released()), this, SLOT(bApply_clicked()));
    connect(BApplyButton, &QPushButton::clicked, this, &ThreeDOverlayButtons::bApply_clicked);
    
    //--------------------------------------------------------------------------------

    allBtnsList << BBackButton << BApplyButton << rightBtnsList << upBtnsList;;
    for (auto w : allBtnsList)
        allWidgets << w;
}



void ThreeDOverlayButtons::setPlot(TPure3dPlot *plot)
{
    parentPlot = plot;
    PlotControl = plot->Child->PlotControl;
    control = new ovControl_3d(this);
    
    plot->installEventFilter(this);

    connect(PlotControl->BCancel, SIGNAL(clicked()), BBackButton, SLOT(click()));
    connect(PlotControl->BApply, SIGNAL(clicked()), BApplyButton, SLOT(click()));
    
    connect(parentPlot, SIGNAL(onResized(QSize)),
            this, SLOT(ChannelResized(QSize)));
    connect(parentPlot, SIGNAL(MouseEventSignal(QEvent*)),
            this, SLOT(MouseEvent(QEvent*)));
    
    for (auto w : allBtnsList)
        w->setParent(parentPlot);
    updateResize();
}

void ThreeDOverlayButtons::setButtonVisible(QWidget *btn, bool visible)
{
    if (fadingAnimationMS == 0)
    {
        btn->setVisible(visible);
        return;
    }
    if (btn->isHidden() || !aniMapState.value(btn))
    {
        if (visible)
        {
            btn->setVisible(true);
            fadeIn(btn);
        }
    }
    else
    {
        if (!visible)
            fadeOut(btn);
    }
}

void ThreeDOverlayButtons::scanButtonsSetVisibleGUI(bool visible)
{
    if (!enabled || visibilityMode == svm_nothing_show)
        visible = false;

    if (setActivePlotByMouseMove)
    {
        if (visible)
            parentPlot->Child->SetActivePlot(parentPlot->N);
    }

    if (visible && blockShowing3DPlotButtons)
        return;

    blockLeaveEvent = true;

    if (visible)
    {
        updateResize();
    }
    
    bool showBtns = visible && (visibilityMode == svm_none);
    for (int i=0; i<rightBtnsList.size(); i++)
    {
        setButtonVisible(rightBtnsList[i], showBtns);
    }
    for (int i=0; i<upBtnsList.size(); i++)
    {
        setButtonVisible(upBtnsList[i], showBtns);
    }
    
    setButtonVisible(BBackButton, visible && showingBackButtonPlot && visibilityMode != svm_none);
    setButtonVisible(BApplyButton, visible && showingApplyButtonPlot && visibilityMode != svm_none);
    
    emit GUIPlotSetVisible(visible, visibilityMode);
    blockLeaveEvent = false;
}

void ThreeDOverlayButtons::updateResize()
{
    ChannelResized(parentPlot->size());
}

void ThreeDOverlayButtons::ChannelResized(QSize newSize)
{
    if (useBlockResize && blockResizeTimes > 0)
        {blockResizeTimes--; return;}

    auto plotLayout = parentPlot->QPlot->plotLayout();
    int leftX = plotLayout->scaleRect(QwtPlot::yLeft).width()*1.15;
    int width = newSize.width() - leftX;
    if (width <= 0) width = 1;
    int height = newSize.height();
    if (height <= 0) height = 1;
    int minBtnsDistBetw = width*0.007;
    if (minBtnsDistBetw == 0)
        minBtnsDistBetw = 1;
    int upY = plotLayout->titleRect().height() * 1.35;
    int upNum = upBtnsList.size() + ((upBtnsList.size() >= 6) ? 0 : 1);
    UpButtonSize = width/upNum - (upNum)*minBtnsDistBetw;
    //UpButtonSize *= 1.6;//btnSizeMultip;
    btnsDistBetw = (width - UpButtonSize*upNum)/(upNum-1);
    btnsDistBetw *= 0.95;
    if (UpButtonSize < 0) UpButtonSize = 1;
    if (UpButtonSize > 150) UpButtonSize = 150;
    
    buttonRect.setWidth(width);
    buttonRect.setHeight(height - upY - plotLayout->scaleRect(QwtPlot::xBottom).height());
    buttonRect.setX(leftX);
    buttonRect.setY(upY);

    bool longVertical = false;
    bool longHorizontal = false;
    bool smallWidth = false;
    if (newSize.width() > height*1.3)
        longHorizontal = true;
    if (height > newSize.width()*1.3)
        longVertical = true;
    if (height > newSize.width()*1.6)
        smallWidth = true;

    if (longVertical || longHorizontal)
    {
        UpButtonSize = 60;
    }

    if (width < UpButtonSize*upBtnsList.size())
        UpButtonSize = minBtnsSize;

    if (longHorizontal)
        btnsDistBetw = UpButtonSize*0.3;
    if (longVertical && !smallWidth)
    {
        btnsDistBetw = (width - UpButtonSize*upNum)/(upNum-1);
        btnsDistBetw *= 0.95;
    }

    BBackButton->setFixedSize(UpButtonSize, UpButtonSize);
    BBackButton->setIconSize(QSize(UpButtonSize, UpButtonSize));
    BApplyButton->setFixedSize(UpButtonSize, UpButtonSize);
    BApplyButton->setIconSize(QSize(UpButtonSize, UpButtonSize));

    int x = leftX;
    int y = upY;

    BBackButton->move(leftX, upY);
    BApplyButton->move(leftX + UpButtonSize + btnsDistBetw, upY);

    if (replacedGUIButtons)
    {
        emit replacedGUIPlotResized(newSize);
        return;
    }

    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList[i]->setFixedSize(UpButtonSize, UpButtonSize);
        allBtnsList[i]->setIconSize(QSize(UpButtonSize, UpButtonSize));
    }

    for (int i=0; i<upBtnsList.size(); i++)
    {
        upBtnsList[i]->move(x, y);
        if (smallWidth)
            y += UpButtonSize + btnsDistBetw;
        else
            x += UpButtonSize + btnsDistBetw;
    }

    y = upY;
    if (upBtnsList.size() == 6 && !smallWidth)
    {
        y += UpButtonSize + btnsDistBetw;
        x = upBtnsList.last()->x();
    }
    if (smallWidth)
        x = newSize.width() - UpButtonSize - btnsDistBetw;

    for (int i=0; i<rightBtnsList.size(); i++)
    {
        rightBtnsList[i]->move(x, y);
        if (longHorizontal)
            x += UpButtonSize + btnsDistBetw;
        else
            y += UpButtonSize + btnsDistBetw;
    }
    
    emit GUIPlotResized(newSize);
}

void ThreeDOverlayButtons::setVisibleMenu(bool visible)
{
    if (replacedGUIButtons)
        emit replacedGUIPlotSetVisible(visible);
    else
        scanButtonsSetVisibleGUI(visible);
}

void ThreeDOverlayButtons::endFadingHide()
{
    QPropertyAnimation *a = static_cast<QPropertyAnimation*>(sender());
    
    QWidget *w = static_cast<QWidget*>(aniMap.key(a));
    if (!aniMapState.value(w))
        w->hide();
}

void ThreeDOverlayButtons::setVisibleIfUnderMouse()
{
    if (constantShowingMenu)
        setVisibleMenu(true);
    else
        setVisibleMenu(isPlotContainMouse());
}

bool ThreeDOverlayButtons::isPlotContainMouse()
{
    auto rect = parentPlot->rect();
    auto pos = parentPlot->mapFromGlobal(QCursor::pos());
    return rect.contains(pos);
}

QPropertyAnimation *ThreeDOverlayButtons::createAnimationToObject(QWidget *w)
{
    QGraphicsOpacityEffect *eff = new QGraphicsOpacityEffect(this);
    w->setGraphicsEffect(eff);
    QPropertyAnimation *a = new QPropertyAnimation(eff, "opacity");
    a->setDuration(fadingAnimationMS);
    
    aniMap.insert(w, a);
    return a;
}

void ThreeDOverlayButtons::fadeIn(QWidget *w)
{
    auto a = aniMap.value(w);
    
    if (!w->graphicsEffect() || !a)
    {
        a = createAnimationToObject(w);
    }
    
    aniMapState.insert(w, true);
    disconnect(a, 0, 0, 0);
    if (a->Running)
    {
        a->stop();
    }
    a->setDuration(fadingAnimationMS);
    a->setStartValue(0);
    a->setEndValue(1);
    a->start();
}

void ThreeDOverlayButtons::fadeOut(QWidget *w)
{
    aniMapState.insert(w, false);
    if (w->isHidden())
        return;
    
    auto a = aniMap.value(w);
    
    if (!w->graphicsEffect() || !a)
        a = createAnimationToObject(w);
    
    disconnect(a, 0, 0, 0);
    if (a->Running)
        a->stop();
    a->setDuration(fadingAnimationMS);
    a->setStartValue(1);
    a->setEndValue(0);
    a->start();
    connect(a, SIGNAL(finished()), this, SLOT(endFadingHide()));
}

void ThreeDOverlayButtons::MouseEvent(QEvent* event)
{
    if (constantShowingMenu)
        return;

    if (event->type() == QEvent::MouseButtonRelease)
    {
//        if (CBFont->isVisible())
//        {
//            BBackButton->click();
//        }
        #ifndef _ADROID
        auto mouse = static_cast<QMouseEvent*>(event);
        if (mouse->button() == Qt::RightButton)
        {
            hidedRMB = !hidedRMB;
            setVisibleMenu(!hidedRMB);
        }
#endif
    }

    if (event->type() == QEvent::Enter)
    {
        #ifndef _ADROID
        if (!hidedRMB)
#endif
            setVisibleMenu(true);
    }
    else if (event->type() == QEvent::Leave && !blockLeaveEvent)
    {
        auto rect = parentPlot->rect();
        if (setActivePlotByMouseMove && parentPlot->Child->ActivePlot != parentPlot->N)
        {
            event->ignore();
            return;
        }

        auto pos = parentPlot->mapFromGlobal(QCursor::pos());
        if (rect.contains(pos))
        {
            event->ignore();
            return;
        }

        int boundH = rect.height()*0.01;
        int boundW = rect.width()*0.01;
        QRect smallerRect(rect.left()+boundW, rect.top()+boundH,
			rect.width()-boundW*2, rect.height()-boundH*2);
        bool visible = smallerRect.contains(pos);
        #ifndef _ADROID
        if (!visible)
            hidedRMB = false;
#endif
        setVisibleMenu(visible);
    }
}
void ThreeDOverlayButtons::onBack_clicked()
{
    parentPlot->Child->SetActivePlot(parentPlot->N);
    auto visMode = visibilityMode;
    blockResizeTimes = 20;
    
    setBackButtonVisible(false);
    setApplyButtonVisible(false);

    visibilityMode = svm_none;
    emit BackCancelClicked(visMode);

    setVisibleIfUnderMouse();
}

void ThreeDOverlayButtons::bApply_clicked()
{
    parentPlot->Child->SetActivePlot(parentPlot->N);
    auto visMode = visibilityMode;
    
    setBackButtonVisible(false);
    setApplyButtonVisible(false);
    visibilityMode = svm_none;

    emit ApplyEnterClicked(visMode);

    setVisibleIfUnderMouse();
}

void ThreeDOverlayButtons::resetBackBtnWainting()
{
    if (visibilityMode != svm_none)
        BBackButton->click();
    setBackButtonVisible(false);
    visibilityMode = svm_none;
}

bool ThreeDOverlayButtons::eventFilter(QObject *Object, QEvent *Event)
{
    if (Event->type() == QEvent::KeyPress)
    {
        auto key = ((QKeyEvent *)Event)->key();
        bool enterKey = (key == Qt::Key_Return || key == Qt::Key_Enter);
        bool escKey = key == Qt::Key_Escape;
        if (escKey)
        {
            if (BBackButton->isVisible())
                BBackButton->click();
        }
        if (enterKey)
        {
            if (BApplyButton->isVisible())
                BApplyButton->click();
        }
//        if (enterKey || escKey)
//        {
//            if (CBFont->isVisible())
//            {
//                BBackButton->click();
//            }
//        }
    }

    return false;
}

bool ThreeDOverlayButtons::event(QEvent *Event)
{
    eventFilter(nullptr, Event);

    return QWidget::event(Event);
}

void ThreeDOverlayButtons::addRightButton(QPushButton *btn, int n)
{
    btn->setParent(parentPlot);
    if (!rightBtnsList.contains(btn))
    {
        if (n == -1 || n >= rightBtnsList.size())
            rightBtnsList.push_back(btn);
        else
            rightBtnsList.insert(n, btn);
    }
    if (!allBtnsList.contains(btn))
        allBtnsList.push_back(btn);
    if (!allWidgets.contains(btn))
        allWidgets.append(btn);
}

void ThreeDOverlayButtons::addUpButton(QPushButton *btn, int n)
{
    btn->setParent(parentPlot);
    if (!upBtnsList.contains(btn))
    {
        if (n == -1 || n >= upBtnsList.size())
            upBtnsList.push_back(btn);
        else
            upBtnsList.insert(n, btn);
    }
    if (!allBtnsList.contains(btn))
        allBtnsList.push_back(btn);
    if (!allWidgets.contains(btn))
        allWidgets.append(btn);
}

void ThreeDOverlayButtons::removeButton(QPushButton *btn)
{
    upBtnsList.removeOne(btn);
    rightBtnsList.removeOne(btn);
    allBtnsList.removeOne(btn);
    allWidgets.removeOne(btn);
}


void ThreeDOverlayButtons::setButtonActive(QPushButton *btn, bool active)
{
    auto sz = btn->iconSize();
    setButtonStyle(btn, active);
    btn->setIconSize(sz);
}
void ThreeDOverlayButtons::setButtonStyle(QPushButton *btn, bool activated)
{
    QString borderColor = activated ? "red" : "white";
    QString style = "";

    style += "QPushButton {background-color: rgba(255,255,255,10);";
    style += "border: 2px solid " + borderColor +";}";
    style += "QToolTip { color: black; background-color: #FFFFE1; border: 1px solid black; }";
    style += "QPushButton:pressed{ background-color: lightgray; }";
    //style += ":disabled { color: #121212; background-color: #121212 }";

    btn->setStyleSheet( style );
}

void ThreeDOverlayButtons::setComboBoxStyle(QComboBox *cb)
{
    cb->setStyleSheet("QComboBox { border-width:0; background-color:rgba(255,255,255,100); }\
               QComboBox:pressed { background-color:rgba(255,255,255,100); border-width:1; }");
}

void ThreeDOverlayButtons::setDoubleSpinBoxStyle(QDoubleSpinBox *dsp)
{
    QString str = "";
    str += "QDoubleSpinBox {border-width:0; background-color:rgba(255,255,255,100);";
    str += "selection-color: yellow;selection-background-color: blue;alternate-background-color: yellow;";
    str += "}";
    str += "QDoubleSpinBox:edit { background: red; background-color:rgba(255,255,255,100);}";
    dsp->setStyleSheet(str);
}

void ThreeDOverlayButtons::setVisibilityMode(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode)
{
    this->visibilityMode = visibilityMode;
}

void ThreeDOverlayButtons::setBackButtonVisible(bool visible)
{
    //BBackButton->setVisible(visible);
    
    showingBackButtonPlot = visible;
}

void ThreeDOverlayButtons::setApplyButtonVisible(bool visible)
{
    //BApplyButton->setVisible(visible);
    
    showingApplyButtonPlot = visible;
}

}
