#ifndef THREEDOVERLAYBUTTONS2_H
#define THREEDOVERLAYBUTTONS2_H

#include <QObject>
#include <QPushButton>
#include <QLabel>
#include <QComboBox>
#include <QDoubleSpinBox>
#include <QActionGroup>
#include "core/plot_core.h"
#include "colorrect2.h"
#include <QCheckBox>
#include <QScrollArea>
#include "plot_colorchooser.h"
#include "../../src/gui/edit.hpp"
#include <QGraphicsOpacityEffect>
#include <QPropertyAnimation>

class TwoStateButton;

namespace plot {

class T3dPlot;
class T3dPlotControl;
class TPure3dPlot;
class ovControl_3d;

class ThreeDOverlayButtons : public QWidget
{
    Q_OBJECT
public:
    ThreeDOverlayButtons();

    enum SpecialVisibilityMode
    {
        svm_none,
        svm_nothing_show,
        svm_point_selection,
        svm_markers,
        svm_area_selection,
        svm_palettes,
        svm_range_selection,
#ifndef _OLD_EXTRA_CONTROL
        svm_extra_control,
#endif
        svm_image_crop,
        svm_filter_settings,
        svm_help,
        svm_laser_etta,
    };
    
    void setBackButtonVisible(bool visible);
    void setApplyButtonVisible(bool visible);
    void setVisibilityMode(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void setPlot(TPure3dPlot *plot);
    void addRightButton(QPushButton *btn, int n = -1);
    void addUpButton(QPushButton *btn, int n = -1);
    void removeButton(QPushButton *btn);
    void resetBackBtnWainting();
    static void setButtonStyle(QPushButton *btn, bool activated = false);
    static void setComboBoxStyle(QComboBox *cb);
    static void setDoubleSpinBoxStyle(QDoubleSpinBox *dsp);
    bool event(QEvent *Event) override;
    bool eventFilter(QObject *Object, QEvent *Event) override;
    static void setButtonActive(QPushButton *btn, bool active);
    void setVisibleMenu(bool visible);
    bool isPlotContainMouse();
    void setVisibleIfUnderMouse();
    void setButtonVisible(QWidget *btn, bool visible);
    void updateResize();

    ovControl_3d *control;
    bool constantShowingMenu = false;
    float btnSizeMultip = 1.0;
    TPure3dPlot *parentPlot;
    T3dPlotControl *PlotControl;
    bool enabled = true;
    int blockResizeTimes = 0; //HACK 
    bool useBlockResize = false; //HACK 
    bool showingBackButtonPlot = false;
    bool showingApplyButtonPlot = false;
    bool replacedGUIButtons = false;
    SpecialVisibilityMode visibilityMode = svm_none;
    int minBtnsSize = 25;
    QPushButton *BBackButton;
    QPushButton *BApplyButton;
    int UpButtonSize;
    int btnsDistBetw;
    QRect buttonRect;
    bool setActivePlotByMouseMove = true;
    bool blockShowing3DPlotButtons = false;
    QList<QPushButton*> allBtnsList;
    QList<QWidget*> allWidgets;
    QMap<QObject*, QPropertyAnimation*> aniMap;
    QMap<QObject*, bool> aniMapState;

signals:
    void replacedGUIPlotResized(QSize newSize);
    void replacedGUIPlotSetVisible(bool visible);
    void replacedGUIPlotCalculateVisible(bool visible);
    void GUIPlotResized(QSize newSize);
    void GUIPlotSetVisible(bool visible, plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void BackCancelClicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void ApplyEnterClicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);

public slots:
    void onBack_clicked();
    void bApply_clicked();
    void ChannelResized(QSize newSize);
    void MouseEvent(QEvent* event);
    
private slots:
    void endFadingHide();
    void fadeIn(QWidget *w);
    void fadeOut(QWidget *w);
    
private:
    QPropertyAnimation *createAnimationToObject(QWidget *w);
    void scanButtonsSetVisibleGUI(bool visible);
    #ifndef _ADROID
    void ChannelResized_filterSettingsMenu(QSize newSize);
#endif
    
    bool blockLeaveEvent = false;
    #ifndef _ADROID
    bool hidedRMB = false; //Hided wia Right Mouse Button
#endif
    
    QList<QPushButton*> upBtnsList;
    QList<QPushButton*> rightBtnsList;

public:
    static int fadingAnimationMS;
};

}

#endif // THREEDOVERLAYBUTTONS2_H
