#include "2d/plot_2dplot.h"

#include "qwt_plot_layout.h"
#include <QMouseEvent>
#include "widgets/twodoverlaybuttons.h"
#include "widgets/colorrect2.h"
#include "2d/plot_2dpure.h"
#include <QSettings>
#include <QGraphicsOpacityEffect>
#include <QPropertyAnimation>
#include "2d/ovcontrol_2d.h"

namespace plot
{

int TwoDOverlayButtons::fadingAnimationMS = 1000;

#ifdef _ADROID
TwoDOverlayButtons::TwoDOverlayButtons(bool vertical) : QWidget(nullptr)
#else
TwoDOverlayButtons::TwoDOverlayButtons() : QWidget(nullptr)
#endif
{
    B2DBack = new QPushButton();
    B2DBack->hide();
    B2DBack->setFlat(true);
    setButtonStyle(B2DBack);
    B2DBack->setIcon(makeHugeIcon(":/ScanPlot/backPlot2D.png"));
    B2DBack->setToolTip(QObject::tr("Back"));
    B2DBack->setShortcut(QKeySequence(Qt::Key_Escape));
    connect(B2DBack, SIGNAL(released()), this, SLOT(on2DBack()));
    
    allBtnsList << B2DBack;
    for (auto w : allBtnsList)
        allWidgets.append(w);
        
#ifdef _ADROID
    control = new ovcontrol_2d(this, vertical);
#else
    control = new ovcontrol_2d(this);
#endif
}

void TwoDOverlayButtons::setVisibleGUI2D(bool visible)
{
    if (!enabled)
        visible = false;

    if (visible && blockShowing2DPlotButtons)
        return;

    for (int i=0; i<allBtnsList.size(); i++)
        allBtnsList.at(i)->setParent(plot);

    if (plot == nullptr)
        visible = false;
    else
        plot2DResized();
    
    setButtonVisible(B2DBack, visible && showBack);

//    if (visible &&
//            (plot->PlotControl->BMarkers->isChecked() ||
//                control->B4DSelectCapturedSpectr->isChecked() ||
//                control->B4DSelectConvolutionFunc->isChecked() ||
//                control->BShowLineMenu->isChecked() ||
//             showBack/* ||
//             B2DBack->isVisible()*/))
//    {
//        visible = false;
//    }
    
    bool visibleBtns = visible && (visMode == svm_none);

    for (int i=0; i<leftBtnsList.size(); i++)
        setButtonVisible(leftBtnsList.at(i), visibleBtns);

    for (int i=0; i<rightBtnsList.size(); i++)
        setButtonVisible(rightBtnsList.at(i), visibleBtns);

    emit replacedGUIPlotSetVisible(visible);
    lastVisibleState = visible;
}
void TwoDOverlayButtons::plot2DResized()
{
    plot2DResized(plot->Pure2dPlot->size(), plot->Pure2dPlot);
}
void TwoDOverlayButtons::plot2DResized(QSize newSize, plot::TPure2dPlot* plotPure)
{
    bool horiz = !plotPure->vertical;
    int leftX = plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::yLeft).width()*1.15;
    int rightX = plotPure->QPlot->width() - plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::yRight).width()*1.15;
    int upY = plotPure->QPlot->plotLayout()->titleRect().height()*1.5;
    if (upY == 0)
        upY = 13*3;
    int downY = newSize.height() - plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
    int lessSz = newSize.width() < newSize.height() ? newSize.width():newSize.height();
    int BtnsSize = (lessSz - leftX)*0.2;
    int UpButtonSize = BtnsSize*0.8;
    if (UpButtonSize < minimumButtonSize) UpButtonSize = minimumButtonSize;
    if (UpButtonSize > maximumButtonSize) UpButtonSize = maximumButtonSize;
    UpButtonSize *= btnSizeMultip;
    lastUpButtonSize = UpButtonSize;
    
    buttonRect.setX(leftX);
    buttonRect.setY(upY);
    buttonRect.setWidth(rightX - leftX);
    buttonRect.setHeight(downY - upY);
    
    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList.at(i)->setFixedSize(UpButtonSize, UpButtonSize);
        allBtnsList.at(i)->setIconSize(QSize(UpButtonSize, UpButtonSize));
    }

    mulBtn = 0.05;

    if (horiz)
    {
        int x = leftX;
        int y = upY;
        int leftMaxX = x;
        for (int i=0; i<leftBtnsList.size(); i++)
        {
            leftBtnsList[i]->move(x, y);
            if (isLeftButtonsVertical)
            {
                y += UpButtonSize + UpButtonSize*mulBtn;
                if (y + UpButtonSize > downY - UpButtonSize*mulBtn)
                {
                    x += UpButtonSize + UpButtonSize*mulBtn;
                    y = upY;
                }
            }
            else
                x += UpButtonSize + UpButtonSize*mulBtn;

            if (x > leftMaxX)
                leftMaxX = x;
        }

        if (verticalPlacingRightButtons)
        {
            int yStart = newSize.height() - plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
            y = upY - (UpButtonSize + UpButtonSize*mulBtn);
            x = newSize.width() - UpButtonSize - UpButtonSize*mulBtn;
            for (int i=0; i<rightBtnsList.size(); i++)
            {
                y += UpButtonSize + UpButtonSize*mulBtn;
                if (y > yStart - (UpButtonSize + UpButtonSize*mulBtn))
                {
                    y = upY;
                    x -= UpButtonSize + UpButtonSize*mulBtn;
                }
                rightBtnsList[i]->move(x, y);
            }
        }
        else
        {
            leftMaxX += UpButtonSize + UpButtonSize*mulBtn;
            if (leftBtnsList.size() == 0)
                leftMaxX = leftX;
            x = newSize.width();
            y = upY;
            for (int i=0; i<rightBtnsList.size(); i++)
            {
                x -= UpButtonSize + UpButtonSize*mulBtn;
                if (x <= leftMaxX)
                {
                    x = newSize.width() - UpButtonSize - UpButtonSize*mulBtn;
                    y += UpButtonSize + UpButtonSize*mulBtn;
                }
                rightBtnsList[i]->move(x, y);
            }
        }
    }
    else
    {
        int x = newSize.width() - UpButtonSize - UpButtonSize*mulBtn;

        int y = upY;
        for (int i=0; i<leftBtnsList.size(); i++)
        {
            leftBtnsList[i]->move(x, y);
            y += UpButtonSize + UpButtonSize*mulBtn;
        }
        int lastUpbtnY = y;

        int yStart = newSize.height() - plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
        y = yStart;
        for (int i=0; i<rightBtnsList.size(); i++)
        {
            y -= UpButtonSize + UpButtonSize*mulBtn;
            if (lastUpbtnY > y)
            {
                y = yStart - (UpButtonSize + UpButtonSize*mulBtn);
                x -= UpButtonSize + UpButtonSize*mulBtn;
            }
            rightBtnsList[i]->move(x, y);
        }
    }

    B2DBack->setFixedSize(UpButtonSize, UpButtonSize);
    B2DBack->setIconSize(QSize(UpButtonSize, UpButtonSize));
    B2DBack->move(leftX, upY);

    emit replacedGUIPlotResized(newSize);
}
void TwoDOverlayButtons::plot2DMouseEvent(QMouseEvent* mouse, plot::TPure2dPlot* plotPure)
{
    if (constantShowingMenu)
        return;
    
    if (mouse->type() == QEvent::MouseButtonRelease)
    {
        #ifndef _ADROID
        if (mouse->button() == Qt::RightButton)
        {
            hidedRMB = !hidedRMB;
            setVisibleGUI2D(!hidedRMB);
        }
#endif
    }

    if (mouse->type() == QEvent::Enter /*|| mouse->type() == QEvent::MouseMove*/)
    {
#ifndef _ADROID
        if (!hidedRMB && !lastVisibleState)
#endif
            setVisibleGUI2D(true);
    }
    else if (mouse->type() == QEvent::Leave)
    {
		auto rect = plotPure->rect();
        int boundH = rect.height()*0.01;
        int boundW = rect.width()*0.01;
        QRect smallerRect(rect.left()+boundW, rect.top()+boundH,
			rect.width()-boundW*2, rect.height()-boundH*2);
        
        auto pos = plotPure->mapFromGlobal(QCursor::pos());
        if (rect.contains(pos))
        {
            mouse->ignore();
            return;
        }
        
        bool visible = smallerRect.contains(pos);
        
#ifndef _ADROID
        if (!visible)
            hidedRMB = false;
#endif
        setVisibleGUI2D(visible);
    }
//    else if (mouse->type() == QEvent::Resize && plotPure->vertical)
//    {
//        setVisibleGUI2D(true);
//        setVisibleGUI2D(false);
//    }
}


void TwoDOverlayButtons::on2DBack()
{
    visMode = svm_none;
    showBack = false;
    emit onBackClicked();
    setVisibleGUI2D(true);
}


void TwoDOverlayButtons::set2DPlot(plot::T2dPlot *pl)
{
    plot = pl;
    control->set2DPlot(pl);
}



bool TwoDOverlayButtons::event(QEvent *Event)
{
    if (Event->type() == QEvent::KeyPress)
    {
        if (((QKeyEvent *)Event)->key() == Qt::Key_Return ||
            ((QKeyEvent *)Event)->key() == Qt::Key_Enter)
        {
            if (waitingEnterButton)
            {
                waitingEnterButton = false;
                plot->PlotControl->BApply->click();
                plot2DResized();
            }
        }
    }
    
    emit eventSignal(Event);

    return QWidget::event(Event);
}

bool TwoDOverlayButtons::eventFilter(QObject *Obj, QEvent *Event)
{
    if (Event->type() == QEvent::KeyPress)
    {
        if (((QKeyEvent *)Event)->key() == Qt::Key_Return ||
            ((QKeyEvent *)Event)->key() == Qt::Key_Enter)
        {
            if (waitingEnterButton)
            {
                waitingEnterButton = false;
                plot->PlotControl->BApply->click();
                plot2DResized();
            }
        }
    }

    return QObject::eventFilter(Obj, Event);
}

void TwoDOverlayButtons::addLeftButton(QPushButton *btn)
{
    if (!leftBtnsList.contains(btn))
        leftBtnsList.push_back(btn);
    if (!allBtnsList.contains(btn))
        allBtnsList.push_back(btn);
    if (!allWidgets.contains(btn))
        allWidgets.push_back(btn);
}

void TwoDOverlayButtons::addRightButton(QPushButton *btn, int i)
{
    if (!rightBtnsList.contains(btn))
    {
        if (i == -1)
            rightBtnsList.push_back(btn);
        else
            rightBtnsList.insert(i, btn);
    }
    if (!allBtnsList.contains(btn))
    {
        allBtnsList.push_back(btn);
    }
    if (!allWidgets.contains(btn))
        allWidgets.push_back(btn);
}

void TwoDOverlayButtons::removeButton(QPushButton *btn)
{
    btn->setVisible(false);
    allBtnsList.removeOne(btn);
    leftBtnsList.removeOne(btn);
    rightBtnsList.removeOne(btn);
    allWidgets.removeOne(btn);
}


void TwoDOverlayButtons::setButtonActive(QPushButton *btn, bool active)
{
    auto sz = btn->iconSize();
    setButtonStyle(btn, active);
    btn->setIconSize(sz);
}
void TwoDOverlayButtons::setButtonStyle(QPushButton *btn, bool activated)
{
    QString borderColor = activated ? "red" : "#9F9F9F";

    QString style = "";

    style += "QPushButton {background-color: rgba(255,255,255,10);";
    style += "border: 2px solid " + borderColor +";}";
    style += "QToolTip { color: black; background-color: #FFFFE1; border: 1px solid black; }";
    style += "QPushButton:pressed{ background-color: lightgray; }";

    btn->setStyleSheet(style);
}

void TwoDOverlayButtons::fadeIn(QWidget *w)
{
    auto a = aniMap.value(w);
    
    if (!w->graphicsEffect() || !a)
        a = createAnimationToObject(w);
    
    aniMapState.insert(w, true);
    disconnect(a, SIGNAL(finished()), this, SLOT(endFadingHide()));
    if (a->Running)
        a->stop();
    a->setDuration(fadingAnimationMS);
    a->setStartValue(0);
    a->setEndValue(1);
    a->start();
}

void TwoDOverlayButtons::fadeOut(QWidget *w)
{
    aniMapState.insert(w, false);
    if (w->isHidden())
        return;
    
    auto a = aniMap.value(w);
    
    if (!w->graphicsEffect() || !a)
        a = createAnimationToObject(w);
    
    disconnect(a, SIGNAL(finished()), this, SLOT(endFadingHide()));
    if (a->Running)
        a->stop();
    a->setDuration(fadingAnimationMS);
    a->setStartValue(1);
    a->setEndValue(0);
    a->start();
    connect(a, SIGNAL(finished()), this, SLOT(endFadingHide()));
}

void TwoDOverlayButtons::endFadingHide()
{
    QPropertyAnimation *a = static_cast<QPropertyAnimation*>(sender());
    
    QWidget *w = static_cast<QWidget*>(aniMap.key(a));
    if (!aniMapState.value(w))
        w->hide();
}

QPropertyAnimation *TwoDOverlayButtons::createAnimationToObject(QWidget *w)
{
    QGraphicsOpacityEffect *eff = new QGraphicsOpacityEffect(this);
    w->setGraphicsEffect(eff);
    QPropertyAnimation *a = new QPropertyAnimation(eff, "opacity");
    a->setDuration(fadingAnimationMS);
    
    aniMap.insert(w, a);
    return a;
}

void TwoDOverlayButtons::setButtonVisible(QWidget *btn, bool visible)
{
    if (fadingAnimationMS == 0)
    {
        btn->setVisible(visible);
        return;
    }
    if (btn->isHidden() || !aniMapState.value(btn))
    {
        if (visible)
        {
            btn->setVisible(true);
            fadeIn(btn);
        }
    }
    else
    {
        if (!visible)
            fadeOut(btn);
    }
}

}



