#ifndef TWODOVERLAYBUTTONS2_H
#define TWODOVERLAYBUTTONS2_H

#include <QObject>
#include <QPushButton>
#include <QActionGroup>
#include <QScrollArea>
#include <QLabel>
#include <QComboBox>
#include <QGraphicsOpacityEffect>
#include <QPropertyAnimation>
#ifdef _ADROID
#include "widgets/verticalqmenu.h"
#include "widgets/verticalqpushbutton.h"
#include "widgets/verticallabel.h"
#include "widgets/verticalqcombobox.h"
#endif

class TwoStateButton;
class colorRect2;

namespace plot {

class T2dPlot;
class TPure2dPlot;
class ovcontrol_2d;

class TwoDOverlayButtons : public QWidget
{
public:
    enum SpecialVisibilityMode
    {
        svm_none,
        svm_nothing_show,
        svm_filter_settings,
        svm_color_settings,
        svm_help,
        svm_markers,
        svm_select_captured_spectr,
        svm_select_convolution_func,
        svm_func_interval,
        svm_cross_section,
        svm_focus_mode,
        };
    
    Q_OBJECT
public: signals:
    void onBackClicked();
    void on_BCancelApply_clickedSignal();
    void on_KeyEnterPressed();
    void replacedGUIPlotResized(QSize newSize);
    void replacedGUIPlotSetVisible(bool visible);
    void replacedGUIShowBackBtn();
    void eventSignal(QEvent *Event);

public:
    #ifdef _ADROID
    explicit TwoDOverlayButtons(bool vertical);
#else
    explicit TwoDOverlayButtons();
#endif
    
    void set2DPlot(T2dPlot *pl);
    void removeButton(QPushButton *btn);
    void addLeftButton(QPushButton *btn);
    void addRightButton(QPushButton *btn, int i=-1);
    static void setButtonStyle(QPushButton *btn, bool activated = false);
    static void setButtonActive(QPushButton *btn, bool active);
    bool event(QEvent *Event) override;
    
    QPropertyAnimation *createAnimationToObject(QWidget *w);
    void setButtonVisible(QWidget *btn, bool visible);
    
    #ifndef _ADROID
    bool hidedRMB = false; //Hided wia Right Mouse Button
#endif
    
    SpecialVisibilityMode visMode = svm_none;
    T2dPlot *plot = nullptr;
    ovcontrol_2d *control = nullptr;
    QPushButton *B2DBack;
    QRect buttonRect;
    double mulBtn;
    bool showBack = false;
    bool enabled = true;
    bool verticalPlacingRightButtons = false;
    bool lastVisibleState = false;
    
    bool constantShowingMenu = false;
    float btnSizeMultip = 1.0;

    bool blockShowing2DPlotButtons = false;
    bool waitingEnterButton = false;
    bool isLeftButtonsVertical = false;
    
    int minimumButtonSize = 55;
    int maximumButtonSize = 100;
    int lastUpButtonSize = 1;

    QList<QPushButton*> allBtnsList;
    QList<QWidget*> allWidgets;
    QList<QPushButton*> leftBtnsList;
    QList<QPushButton*> rightBtnsList;
    QMap<QObject*, QPropertyAnimation*> aniMap;
    QMap<QObject*, bool> aniMapState;

public slots:
    void setVisibleGUI2D(bool visible);
    void plot2DResized();
    void plot2DResized(QSize newSize, plot::TPure2dPlot* plotPure);
    void plot2DMouseEvent(QMouseEvent* mouse, plot::TPure2dPlot* plotPure);
    void on2DBack();
    
private slots:
    void fadeIn(QWidget *w);
    void fadeOut(QWidget *w);
    void endFadingHide();
    
private:
    bool eventFilter(QObject *Obj, QEvent *Event) override;
    
public:
    static int fadingAnimationMS;
};

}

#endif // TWODOVERLAYBUTTONS2_H
