#ifndef IEXTERNALSCAN_H
#define IEXTERNALSCAN_H

#include <plot.h>
#include "controls.h"
#include "edit.hpp"
#include <defs.h>
#include "tools.hpp"
#include <QPointer>
#include <QElapsedTimer>
#include "USBVideo/videoplayer.h"
#include "gui/rectslider.h"
#include <QPointer>
#include "status.hpp"
#include "gui/t3dchooser.h"
#include "gui/threestatebutton.h"
#include "gui/devicescheme.h"
#include "gui/hoverbutton.h"
#include "gui/manystatebutton.h"
#include "options/options.h"
#include "gui/swaptwostatebutton.h"
#include "plot/3d/plot_3dplot.h"
#include "plot/widgets/threedoverlaybuttons.h"

class IExternalScan : public QObject
{
    Q_OBJECT

public:
    IExternalScan() {
        displayPlot = nullptr;
        nx = ny = dir = start_point = dir_stpoint = 0;
        started = false;
        plot::ResizeOwner(Output3d, 1);
        plot::ResizeOwner(Section, 1);
        Section[0]->Pen.setColor(Qt::red);
    }
    ~IExternalScan() override {}
    void SetDirStPoint(int _dir, int _stpoint) { dir = _dir; start_point = _stpoint; }
    void SetStPointDirection(int value) {
        switch(value) {
        case spTopLeftAxisX: SetDirStPoint(Horiz, TopLeft); break;
        case spTopRightAxisX: SetDirStPoint(Horiz, TopRight); break;
        case spBottomRightAxisX: SetDirStPoint(Horiz, BottomRight); break;
        case spBottomLeftAxisX: SetDirStPoint(Horiz, BottomLeft); break;
        case spTopLeftAxisY: SetDirStPoint(Vert, TopLeft); break;
        case spTopRightAxisY: SetDirStPoint(Vert, TopRight); break;
        case spBottomRightAxisY: SetDirStPoint(Vert, BottomRight); break;
        case spBottomLeftAxisY: SetDirStPoint(Vert, BottomLeft); break;
        }
    }
    enum TDirections { Vert, Horiz };
    enum TStartPoints { BottomLeft, BottomRight, TopLeft, TopRight };

    virtual bool init(int _nx, int _ny, int _dir_stpoint, Range _sx, Range _sy) {
        nx = _nx;
        ny = _ny;
        dir_stpoint = _dir_stpoint;
        SetStPointDirection(dir_stpoint);
        sx = _sx;
        sy = _sy;
        started = true;

        clear();

        Output3d.at(0)->XLimits = sx;
        Output3d.at(0)->YLimits = sy;
        Output3d.at(0)->setSize(nx, ny);

        foreach (QWidget *wid, disableWidgets)
            wid->setEnabled(false);

        return true;
    }
    virtual void finish() {
        started = false;
        foreach (QWidget *wid, disableWidgets)
            wid->setEnabled(true);
    }
    virtual void save() { }
    virtual void clear() {
        lastError.clear();
        Output3d.at(0)->clear();
        Section[0]->clear();
    }
    virtual int getDataChannel() { return srcCCD; }
    virtual void updateData() { emit updated(); }
    virtual void setPointSpection(bool pressed) {}
    virtual void sl_Spectrum_range_select(bool pressed) {} //todo remove it from this
    virtual void sl_Func_range_select(bool pressed) {} //todo remove it from this
    virtual void setSelectionRange(double Min, double Max) {}
    virtual void delayInit() {} //that function executing when showing GUI
    virtual int get2DPlotFilter() {return 0;}
    
    virtual QVector<QWidget *> *createInterfaceWidget(QVBoxLayout**, QVBoxLayout*) { return nullptr; }

    virtual bool acquireNextPoint() = 0;
signals:
    void acquired();
    void updated();
    void setSectionEnabled(bool);

public:
    int nx, ny, dir, start_point, dir_stpoint;
    Range sx, sy;
    bool started;
    plot::T3dPlot *displayPlot;

    QString name = "";

    //data interface
    QString lastError;
    plot::TCurvesDataOwner Section;
    plot::T3dDataOwner Output3d;
    plot::TMetaStorage attributes;
    QVector<QWidget *> disableWidgets;
};

#endif // IEXTERNALSCAN_H
