#ifndef ARS3000_H
#define ARS3000_H

#include <QObject>
#include <QWidget>
#include <QGroupBox>
#include "motors.hpp"
#include <QRadioButton>
#include <QCheckBox>
#include <QQueue>
#include "controls.h"

class ARS3000 : public QGroupBox
{
    Q_OBJECT
public:
    static bool init();
    static bool setSlitWidth(int idx, double width); //mm
    static bool setLaser(int idx);
    static bool isReady();
    static bool isMotorRoleARS3000(Motors::MotorRoles role);
    
public: signals:
    void motorStopped(int motor, Motors::MotorRoles role);
    void confLineChanged(Motors::MotorRoles role, int confLine);
    void motorsUpdated();
    
private:
    static double ars_slit_pos[2];
    static double ars_laser_pos;
    
public:
    static const int numMotorRoles = 10;
    
    ARS3000();
    void setMotors(Motors *m);
    
    void setActiveMotor(int num, Motors::MotorRoles role);
    void setSelectedConfLine(int motor, int confLine);
    void addMotorToStartQueue(Motors::MotorRoles role);
    void startMotor(bool start, Motors::MotorRoles role);
    void stopAllMotors();
    void fillComboBoxConfLines(Motors::MotorRoles role, QComboBox *cb);
    void updateMotors();
    QList<QString> getConfLines(Motors::MotorRoles role);
    
    Edit *objsl_stepValue[2];
    Edit *objsl_speedValue[2];
    
public slots:
    void startMotorWithConfLine(Motors::MotorRoles role, int confLine);
    
private:
    struct motorQueueStruct
    {
        Motors::MotorRoles role;
        int dist;
    };
    
private:
    void initGUI();
    void loadSettings();
    
    QString title;
    
    int motorCurrentStages[numMotorRoles];
    
    Motors::MotorRoles currentMotorRole;
    int currentMotorNum = 0;
    static const int ARS3000_maxNumConfLines = 10;
    TwoStateButton *ARS3000_start;
    int ARS3000_limitter[Motors::numMaxMotors];
    int ARS3000_numConfLines[Motors::numMaxMotors];
    QString ARS3000_confLinesText[Motors::numMaxMotors][ARS3000_maxNumConfLines];
    int ARS3000_confLinesValues[Motors::numMaxMotors][ARS3000_maxNumConfLines];
    int ARS3000_selectedConfLines[Motors::numMaxMotors];
    
    QCheckBox *ARS3000_limitterGui;
    Edit *ARS3000_numConfLinesGui;
    QTableWidget *ARS3000_TableConf;
    QRadioButton *ARS3000_radioSelectedConfLine[ARS3000_maxNumConfLines];
    QLineEdit *ARS3000_textboxConfLine[ARS3000_maxNumConfLines];
    Edit *ARS3000_confLinesValuesGui[ARS3000_maxNumConfLines];
    
    QGroupBox *objsl_wid[2];
    
    QQueue<motorQueueStruct> motorQueue;
    
private slots:
    void motorEvent(int motor, bool atEnd, bool active);
    void numLinesChanged();
    void saveSettings();
    void confLineChangedSlot();
    void start(bool);
    void updateQueue();
};

#endif // ARS3000_H
