#include "ccdplot.h"
#include "gui/status.hpp"
#include "flowlayout.hpp"
#include <qwt_thermo.h>

#include "modules.hpp"
#include "oscilloscope.h"
#include "gui/ttabwidget.hpp"
#include "mainwindow.h"

#include "ccdmatrix.h"
#include "i_spectr.hpp"
#include "hardware/coboltcontroller.hpp"
#include "hardware/nspectr_proxy.h"
#include "qwt/qwt_plot_layout.h"
#include "scanplot.h"
#include "singleapp.h"
#include <QScrollArea>
#include <QComboBox>
#include <QCheckBox>
#include <QGroupBox>
#include <QLabel>
#include <QTabWidget>
#include <QTextEdit>
#include <QRadioButton>
#include <QTimer>
#include <QSettings>
#include <QCloseEvent>
#include <QProgressBar>
#include <QFormLayout>
#include <QWidgetAction>
#include "widgets/threedoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"
#include "3d/ovcontrol_3d.h"
#include "2d/ovcontrol_2d.h"
#include "ars3000.h"
#include "xyzcalibrations.hpp"
#include "dacscontrol.h"

bool CCDPlot::eventFilter(QObject *Obj, QEvent *Event)
{
    if (Event->type() == plot::TScalesChangedEvent::id && setReadMode->currentIndex() == 2) {
        sl_set_new_3D_Limits(((plot::TScalesChangedEvent *) Event)->XLimits, ((plot::TScalesChangedEvent *) Event)->YLimits);
        ccdMatrix->sl_set_new_3D_Limits(((plot::TScalesChangedEvent *) Event)->XLimits, ((plot::TScalesChangedEvent *) Event)->YLimits);
    }
    if (Event->type() == QEvent::KeyPress)
    {
        if (((QKeyEvent *)Event)->key() == Qt::Key_Return || ((QKeyEvent *)Event)->key() == Qt::Key_Enter)
        {
            if (((plot::TCommonPlotControl*)MyPl->ThreeDPlot->ControlLayout->itemAt(1)->widget())->BApply->isVisible())
                ((plot::TCommonPlotControl*)MyPl->ThreeDPlot->ControlLayout->itemAt(1)->widget())->BApply->animateClick();
            if (MyPl->TwoDPlot->Pure2dPlot->FitOrder == -2)
                MyPl->TwoDPlot->PlotControl->BApply->click();
            if (MyPl->ThreeDPlot->TwoDPlot->Pure2dPlot->FitOrder == -2)
                MyPl->ThreeDPlot->TwoDPlot->PlotControl->BApply->click();
        }
        if (MyPl->ThreeDPlot->ThreeDPlots[0]->overlayButtons)
            MyPl->ThreeDPlot->ThreeDPlots[0]->overlayButtons->event(Event);
    }
    return QWidget::eventFilter(Obj, Event);
}

CCDPlot::CCDPlot(QWidget *parent) : QWidget(parent)
{
    #ifdef _ENABLE_NST_LASERS
    {
        QSettings settings("Param/laser.conf", QSettings::IniFormat);
        
        if (settings.value("nstLaserAlpha", false).toBool())
            nstLaser_list.append(new nstLasers(nstLasers::pidAlpha));
        if (settings.value("nstLaserBeta", false).toBool())
            nstLaser_list.append(new nstLasers(nstLasers::pidBeta));
        if (settings.value("nstLaserGamma", false).toBool())
            nstLaser_list.append(new nstLasers(nstLasers::pidGamma));
        if (settings.value("nstLaserDelta", false).toBool())
            nstLaser_list.append(new nstLasers(nstLasers::pidDelta));
        if (settings.value("nstLaserEtta", false).toBool())
            nstLaser_list.append(new nstLasers(nstLasers::pidEtta));
    }
    #endif
    
    {
        QSettings settings("Param/analog.conf", QSettings::IniFormat);
        ars3000PMTHVUmax[0] = settings_double(settings, "pmt/hv1_umax", 1000);
        ars3000PMTHVUmax[1] = settings_double(settings, "pmt/hv2_umax", 1000);
        ars3000PMTDacMinMax[0][0] = settings_double(settings, "pmt/dacaux1_min", -10.);
        ars3000PMTDacMinMax[0][1] = settings_double(settings, "pmt/dacaux1_max", 10.);
        ars3000PMTDacMinMax[1][0] = settings_double(settings, "pmt/dacaux2_min", -10.);
        ars3000PMTDacMinMax[1][1] = settings_double(settings, "pmt/dacaux2_max", 10.);
    }
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    showRightMenu = settingsGui.value("gui/show_spectr_right_menu", true).toBool();
    AutoSend = false;
    kinSerDACinitialValue = 0.;
    isInterrupted = false;
    Nspec_Initalized=false;
    nspectr_mutex = nullptr;
    ccdWidget = nullptr;
    ScanCCDMode = false;
    ccdMatrix = CCDMatrix::GetIt();
    ccdMatrix->curveCalib = &curveCalib;
    PMTPlot = nullptr;

    Nx=1024;
    Ny=256;

    deviceStatus = new StatusWidget1(tr("CCD"));
    deviceStatus->label->setToolTip(tr("Andor"));
#ifdef USE_CCD_EMULATOR
    {
        QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
        bool useCCDEmulator = settings.value("ccd", false).toBool();
        
        if (useCCDEmulator)
            deviceStatus->setEmulator(true);
    }
#endif

    deviceStatus2 = new StatusWidget1(tr("Optics"));

    acqUpd = new QTimer(this);
    temperatureUpd = new QTimer(this);
    refresh_limit_timer.invalidate();

    connect(acqUpd, SIGNAL(timeout()), SLOT(updateSpecBar()));
    connect(temperatureUpd, SIGNAL(timeout()), SLOT(slot_updateTemperature()));

    fillLaserConf();

    setWindowTitle(tr("CCD - NSpec"));

    createPlot();
    plot::ResizeOwner(curveData, 1);
    MyPl->TwoDPlot->Pure2dPlot->SourceData = plot::getUnLock(curveData);
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->UseMathOnly = true;
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->clearSouceData();
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SourceData = *MyPl->TwoDPlot->Pure2dPlot->cloneSourceData();
    MyPl->TwoDPlot->overlayButtons->control->setVisibleOldFilters(true);
    MyPl->TwoDPlot->overlayButtons->isLeftButtonsVertical = true;
    MyPl->ThreeDPlot->ThreeDPlots[0]->overlayButtons->control->setVisibleFilterButton(false);
    MyPl->ThreeDPlot->ThreeDPlots[0]->overlayButtons->control->setVisibleOldFilters(true);
    MyPl->TwoDPlot->showPlotControl = showRightMenu;
    MyPl->ThreeDPlot->TwoDPlot->showPlotControl = showRightMenu;
    MyPl->ThreeDPlot->TwoDPlotVertical->showPlotControl = showRightMenu;
    MyPl->ThreeDPlot->showPlotControl = showRightMenu;
    if (!showRightMenu)
    {
        MyPl->TwoDPlot->PlotControl->hide();
        MyPl->ThreeDPlot->TwoDPlot->PlotControl->hide();
        MyPl->ThreeDPlot->TwoDPlotVertical->PlotControl->hide();
        MyPl->ThreeDPlot->PlotControl->hide();
    }

    MyPl->TwoDPlot->overlayButtons->control->setHelpImage(":/helpManuall/spectroscopy2D.png");
    if (MyPl->TwoDPlot->overlayButtons->control->showHelpButton)
        MyPl->TwoDPlot->overlayButtons->addRightButton(MyPl->TwoDPlot->overlayButtons->control->helpButton, 0);

    plot::ResizeOwner(imgData, 1);
    imgData.at(0)->setSize(Nx, Ny);

    createButtons();
    createCCDAtt();
    createLayout();
    
    {
        bool show;
        QSettings settings("Param/laser.conf", QSettings::IniFormat);
        settings.beginGroup("list");
        show = settings.allKeys().size() != (settings.allKeys().contains("dummy") ? 1 : 0);
        foreach (QString key, settings.allKeys()) {
            if (key == "dummy")
                continue;

            CBPreMonochWl->addItem(key, settings.value(key, -1).toInt());
        }
        settings.endGroup();

        CBPreMonochWl->setCurrentIndex(-1);
        //CBPreMonochWl->setVisible(show);
        if (show)
            connect(CBPreMonochWl, SIGNAL(currentIndexChanged(int)), SLOT(monoch_laser_wl()));
    }
    
    load_data();

    monoch_status_update(false);

    QMetaObject::connectSlotsByName(this);

    installEventFilter(this);

    connect(MyPl->TwoDPlot->overlayButtons, SIGNAL(replacedGUIPlotResized(QSize)), this, SLOT(gui2DPlotResized(QSize)));
    connect(MyPl->TwoDPlot->overlayButtons, SIGNAL(replacedGUIPlotSetVisible(bool)), this, SLOT(gui2DPlotSetVisible(bool)));
    connect(oscPlot->overlayButtons, SIGNAL(replacedGUIPlotSetVisible(bool)), this, SLOT(gui2DPlotSetVisible(bool)));

    connect(base_line_manual, SIGNAL(clicked()),MyPl->TwoDPlot, SLOT(StartMCCurveConstructing()));
    connect(base_line_manual, SIGNAL(clicked()),MyPl->ThreeDPlot->TwoDPlotVertical, SLOT(StartMCCurveConstructing()));
    connect(MyPl->TwoDPlot->Pure2dPlot, SIGNAL(MathUpdated()), this, SLOT(Send2dPlotData()));
    connect(MyPl->TwoDPlot, SIGNAL(ManualConstructingDone()), this, SLOT(SetManualBaseLine()));
    connect(MyPl->ThreeDPlot->TwoDPlotVertical, SIGNAL(ManualConstructingDone()), this, SLOT(SetManualBaseLine()));
    connect(MyPl, SIGNAL(AddDataToContainer(plot::T3dDataOwner,plot::TMetaStorage)), SLOT(AddDataToContainer_slot(plot::T3dDataOwner,plot::TMetaStorage)));
    connect(MyPl, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), SLOT(AddDataToContainer_slot(plot::TCurvesDataOwner,plot::TMetaStorage)));

    //ccdmatrix connections  //TODO module connected()?
    ccdMatrix->setOutput(&imgData.at(0)->Array);

    connect(ccdMatrix, SIGNAL(sig_Update()), SLOT(sl_Plot_Refresh()), Qt::QueuedConnection);
    connect(ccdMatrix, SIGNAL(new2dData(QVector<double>,QVector<double>)), SLOT(new2dData(QVector<double>,QVector<double>)), Qt::QueuedConnection);
    connect(ccdMatrix, SIGNAL(sigUpdateNxNy()), SLOT(UpdateNxNy()));
    connect(ccdMatrix, SIGNAL(ccd_finished()), SLOT(sl_aq_but_off()));
    connect(ccdMatrix, SIGNAL(sig_cur_kin_ser_num(int)), SLOT(sl_set_current_kin_num(int)));
    connect(ccdMatrix, SIGNAL(sig_cur_ac_num(int)), SLOT(sl_set_current_acc_num(int)));

    connect(lineSetTemp, SIGNAL(newValue(int)), ccdMatrix, SLOT(s_setTemp(int)));
    connect(lineExpTime, SIGNAL(newValue(double)), ccdMatrix, SLOT(sl_set_Exp_time(double)));
    connect(setAcqMode, SIGNAL(currentIndexChanged(int)), ccdMatrix, SLOT(sl_set_acqMode(int)));
    connect(setReadMode, SIGNAL(currentIndexChanged(int)), ccdMatrix, SLOT(sl_set_readMode(int)));

    connect(lineNumAcc, SIGNAL(newValue(int)), ccdMatrix, SLOT(sl_set_Acc_num(int)));
    connect(lineAccCycleTime, SIGNAL(newValue(double)), ccdMatrix, SLOT(sl_set_Acc_time(double)));
    connect(Num_Kin_ser, SIGNAL(newValue(int)), ccdMatrix, SLOT(sl_set_Kin_ser_num(int)));
    connect(Time_Kin_ser, SIGNAL(newValue(double)), ccdMatrix, SLOT(sl_set_Kin_ser_time(double)));

    connect(setTriggerMode, SIGNAL(currentIndexChanged(int)), ccdMatrix, SLOT(sl_set_Triger(int)));
    connect(lineCenterRowTrack, SIGNAL(newValue(int)), ccdMatrix, SLOT(sl_set_Center_row(int)));
    connect(lineHeighTrack, SIGNAL(newValue(int)), ccdMatrix, SLOT(sl_set_Height(int)));

    connect(One_Cont, SIGNAL(state_changed(bool)), ccdMatrix, SLOT(s_isContScan(bool)));
    connect(Cut_Pikes, SIGNAL(state_changed(bool)), ccdMatrix, SLOT(sl_cut_pikes(bool)));
    connect(base_line_save, SIGNAL(clicked()), ccdMatrix, SLOT(sl_save_base_line()));
    connect(Base_line, SIGNAL(state_changed(bool)), ccdMatrix, SLOT(sl_set_baseline_on_off(bool)));

    {//ccd settings
        QSettings settings("Param/ccd.conf", QSettings::IniFormat);

        if (settings.value("use_ormins", false).toBool())
            deviceStatus->label->setToolTip(tr("Ormins"));
        lineExpTime->setValue(settings.value("exposure_time", 0.1).toDouble());
        ccdMatrix->m_exposureTime = lineExpTime->value();
        ccdMatrix->m_update_exposure = true;
    }

    devStatusSync = new QTimer(this);
    devStatusSync->setSingleShot(true);

    prevDevStatus = -1;
    syncDevStatus = EDno;
    setDeviceStatus();
    setDeviceStatus2(EDno);

    connect(ccdMatrix, SIGNAL(updateCCDStatus(int)), this, SLOT(setDeviceStatus_handler(int)), Qt::QueuedConnection);
    connect(ccdMatrix, SIGNAL(monoch_status(bool)), SLOT(monoch_status_update(bool)), Qt::QueuedConnection);
    connect(devStatusSync, SIGNAL(timeout()), this, SLOT(setDeviceStatus()));


    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    bool b1on = settings.value("ccd/intensity_calib_on_off", false).toBool();
    bool b2on = settings.value("ccd/intensity_calib_pixel_on_off", false).toBool();
    QString name1 = settings.value("ccd/intensity_calib_name", QString()).toString();
    QString name2 = settings.value("ccd/intensity_calib_pixel_name", QString()).toString();
    if (!name1.isEmpty()) intensity_calib_open_wl(name1);
    if (!name2.isEmpty()) intensity_calib_open_px(name2);
    calibEn->setChecked(b1on);
    pixelCalibEn->setChecked(b2on);
}

void CCDPlot::AddDataToContainer_slot(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    attributes.insert("Name", MyPl->TwoDPlot->Pure2dPlot->Title);
    attributes.insert("Source", "CCD");

    auto osc = Oscilloscope::GetIt();
    if (osc) {
        attributes.insert("Stage X, um", osc->DacsWidget->X2->value());
        attributes.insert("Stage Y, um", osc->DacsWidget->Y2->value());
    }
    attributes.insert("CCD NX", Nx);
    attributes.insert("CCD NY", Ny);
    attributes.insert("CCD Central Wavelength, nm", ccdMatrix->Cent_WL);
    attributes.insert("CCD Dispersion", ccdMatrix->Disp);
    attributes.insert("CCD Central Pixel", lineCentralPx->value());
    attributes.insert("Exposure time", lineExpTime->value());
    if (ccdMatrix->m_spectrInterface)
        attributes.insert("Grating", CBgrating_select->currentText());
    if (Nspec_Initalized && nspectr_mutex)
        attributes.insert("ND filter", NDEdit->value());

    foreach (CoboltGui *it, cobolt_gui_list)
        attributes.insert("Laser Power, mW - " + it->windowTitle(), it->cpower->text());


    attributes.unite(curveCalib);

    emit AddDataToContainer(src, attributes);
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        mob->setPropIdVar(DataID::addedScanDataToContainer_2d_spectr, QVariant::fromValue(attributes));
    }
}

void CCDPlot::AddDataToContainer_slot(plot::T3dDataOwner src, plot::TMetaStorage attributes)
{
    attributes.insert("Name", MyPl->ThreeDPlot->ThreeDPlots.at(0)->Title);
    attributes.insert("Source", "CCD");

    emit AddDataToContainer(src, attributes);
}

CCDPlot::~CCDPlot()
{
    foreach (CoboltGui *it, cobolt_gui_list)
        delete it;
    cobolt_gui_list.clear();
}

void CCDPlot::closeEvent(QCloseEvent *Event)
{

}

void CCDPlot::slot_updateTemperature()
{
    int Temper = ccdMatrix->getCameraTemperature();
    if (Temper > 15) Termometer->setFillBrush(Qt::red);
    else if (Temper >= 0) {
        Termometer->setFillBrush(Qt::green);
        temp_pic->setPixmap(QPixmap(":/CCD/plus.png"));
    } else {
        Termometer->setFillBrush(Qt::blue);
        temp_pic->setPixmap(QPixmap(":/CCD/minus.png"));
    }
    Termometer->setValue(Temper);
    lb_temp->setText(QString::number(Temper));
}

void CCDPlot::UpdateNxNy()
{
    Nx = ccdMatrix->Nx;
    Ny = ccdMatrix->Ny;

    imgData.at(0)->setSize(Nx, Ny);
}
void CCDPlot::sl_set_new_3D_Limits(const Range &/*XLimits*/, const Range &YLimits)
{
    double up_limit;
    if (YLimits[1] > (Ny-1))
        up_limit = Ny-1;
    else
        up_limit = YLimits[1];

    int cent_row = (int)(0.5*(up_limit + YLimits[0]));
    int track_height = (int)(up_limit - YLimits[0] - 1);
    if (track_height < 3) track_height = 3;

    lineCenterRowTrack->setValue(cent_row);
    lineHeighTrack->setValue(track_height);
    ccdMatrix->m_centralRow = cent_row;
    ccdMatrix->m_trackHeight = track_height;
}
void CCDPlot::new2dData(QVector<double> ydata, QVector<double> xdata)
{
    if (spec_bar != nullptr) {
        auto bar = spec_bar->bar;
        bar->setUpdatesEnabled(false);
        bar->setMaximum(ccdMatrix->m_exposureTime*1000);
        bar->reset();
        bar->setUpdatesEnabled(true);
    }

    curveData[0]->XSeq = xdata;
    curveData[0]->YSeq = ydata;

    if (!refresh_limit_timer.isValid() || refresh_limit_timer.elapsed() > 100) { //10 refreshes per second max
        refresh_limit_timer.start();

        MyPl->Refresh();

        if (PMTPlot && PMTPlot->isVisibleTo(this)) {
            PMTPlot->hide();
            MyPl->show();
        }
    }
}

void CCDPlot::sl_Plot_Refresh()
{
    if (spec_bar != nullptr) {
		auto bar = spec_bar->bar;
		bar->setUpdatesEnabled(false);
        bar->setMaximum(ccdMatrix->m_exposureTime*1000);
        bar->reset();
        bar->setUpdatesEnabled(true);
    }

    if (!refresh_limit_timer.isValid() || refresh_limit_timer.elapsed() > 100) { //10 refreshes per second max
        refresh_limit_timer.start();

        auto img = imgData.at(0);
        img->XLimits[0] = ccdMatrix->imageX_min;
        img->XLimits[1] = ccdMatrix->imageX_max;
        img->YLimits[0] = ccdMatrix->imageY_min;
        img->YLimits[1] = ccdMatrix->imageY_max;
        img->NXElems = ccdMatrix->imageX_max - ccdMatrix->imageX_min + 1;
        img->NYElems = ccdMatrix->imageY_max - ccdMatrix->imageY_min + 1;
        img->UpdateDxDy();
        img->resetMathLims();

        MyPl->ThreeDPlot->SetPlotsXScales(ccdMatrix->imageX_min, ccdMatrix->imageX_max);
        MyPl->ThreeDPlot->SetPlotsYScales(ccdMatrix->imageY_min, ccdMatrix->imageY_max);
        MyPl->ThreeDPlot->ThreeDPlots[0]->SourceData = plot::getUnLock(imgData);

        MyPl->Refresh();

        if (PMTPlot && PMTPlot->isVisibleTo(this)) {
            PMTPlot->hide();
            MyPl->show();
        }
    }
}
void CCDPlot::sl_set_current_acc_num(int num)
{
    lineNumAcc->blockSignals(true);
    lineNumAcc->setValue(num);
    lineNumAcc->blockSignals(false);
}
void CCDPlot::sl_set_current_kin_num(int num)
{
    Num_Kin_ser->blockSignals(true);
    Num_Kin_ser->setValue(num);
    Num_Kin_ser->blockSignals(false);

    if (setReadMode->currentIndex() != 2) {
        bool last = num >= ccdMatrix->Kin_ser_number;
        if (!last) {
            curveData.prepend(QSharedPointer<plot::TCurveData>(new plot::TCurveData));
            curveData[0]->Pen.setColor(QColor((int)((double)qrand() / (double)RAND_MAX * 240. ), (int)((double)qrand() / (double)RAND_MAX * 240. ), (int)((double)qrand() / (double)RAND_MAX * 240. )));
            MyPl->TwoDPlot->Pure2dPlot->SourceData = plot::getUnLock(curveData);
            MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->clearSouceData();
            MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SourceData = *MyPl->TwoDPlot->Pure2dPlot->cloneSourceData();
        }

        if (kinSerDACscan->currentIndex() > 0) {
            auto osc = Oscilloscope::GetIt();
            if (!osc)
                return;
            DoubleChanger *dc = osc->DacsWidget->get_dacs_channel(kinSerDACscan);
            if (num == 1)
                kinSerDACinitialValue = dc->value();
            if (last)
                dc->setValue(kinSerDACinitialValue);
            else
                dc->setValue(dc->value() + kinSerDACstep->value());
            dc->setNewValue(true);
        }
    }
}
void CCDPlot::enable_options_rm(int mode)
{
    box_singleTrack->setVisible(mode == 1);
    BPanoram->setEnabled(mode != 2);
    Base_line->setEnabled(mode != 2);
    base_line_save->setEnabled(mode != 2);
    base_line_manual->setEnabled(mode != 2);

    if (mode == 2) {
        MyPl->SwitchPlots(plot::TPlot::_3d);
        MyPl->ThreeDPlot->SetXYDescAndSIUnit(QObject::tr("pixels"), QString());
        MyPl->ThreeDPlot->SetZDescAndSIUnit(QObject::tr("Counts"), QString());

        static bool first = true;
        if (first) {
            first = false;
            MyPl->ThreeDPlot->SetDefaultXLimits(ccdMatrix->imageX_min, ccdMatrix->imageX_max);
            MyPl->ThreeDPlot->SetDefaultYLimits(ccdMatrix->imageY_min, ccdMatrix->imageY_max);
        }
        MyPl->ThreeDPlot->SetPlotsXScales(ccdMatrix->imageX_min, ccdMatrix->imageX_max);
        MyPl->ThreeDPlot->SetPlotsYScales(ccdMatrix->imageY_min, ccdMatrix->imageY_max);
        MyPl->ThreeDPlot->SetTitle(tr("CCD Image"));

        MyPl->ThreeDPlot->ThreeDPlots[0]->ResizeQPlot();
        sl_Plot_Refresh();

        if (setAcqMode->currentIndex() == 2)
            setAcqMode->setCurrentIndex(0);
        setAcqMode->removeItem(2);

        if (BPanoram->isOn)
            BPanoram->change_state_man_with_sig(false);
        if (Base_line->isOn)
            Base_line->change_state_man_with_sig(false);
    } else {
        MyPl->SwitchPlots(plot::TPlot::_2d);
        MyPl->TwoDPlot->Pure2dPlot->SetDefaultYLimits(0, 65535);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetDefaultYLimits(0, 65535);
        sl_set_X_scale(currentRelativeScale);

        if (mode == 1)
        {
            MyPl->TwoDPlot->SetTitle(tr("CCD Single Track Curve"));
            MyPl->ThreeDPlot->TwoDPlotVertical->SetTitle(tr("CCD Single Track Curve"));
        }
        else
        {
            MyPl->TwoDPlot->SetTitle(tr("CCD Full V Binning Curve"));
            MyPl->ThreeDPlot->TwoDPlotVertical->SetTitle(tr("CCD Full V Binning Curve"));
        }

        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
            mob->setPropIdVar(DataID::PlotTitle2d_Spectr, MyPl->TwoDPlot->Pure2dPlot->Title);

        if (setAcqMode->count() != 3)
            setAcqMode->addItem(tr("Kinetic Series"));
    }

    if (PMTPlot && PMTPlot->isVisibleTo(this)) {
        PMTPlot->hide();
        MyPl->show();
    }
}
void CCDPlot::enable_options_acm(int mode)
{
    box_accumulation->setVisible(mode != 0);
    Kin_series->setVisible(mode == 2);

    auto osc = Oscilloscope::GetIt();
    if (mode == 2 && osc) {
        kinSerDACscan->clear();
        kinSerDACscan->addItem(tr("None"), -1);
        osc->DacsWidget->fill_dacs_channel_CB(kinSerDACscan);
    }
}

void CCDPlot::s_triggerMode(int mode)
{
    lineAccCycleTime->setEnabled(mode == 0);
    Time_Kin_ser->setEnabled(mode == 0);
}


void CCDPlot::sl_calib_change(bool value)
{
    bool auto_spectr = value && ccdMatrix->m_spectrInterface;
    lineDispersion->setVisible(!auto_spectr);
    lb_dispersion->setVisible(!auto_spectr);
    lineCentrWL->setVisible(!auto_spectr);
    lb_centralWL->setVisible(!auto_spectr);
    ccdMatrix->sl_set_Auto_calib(auto_spectr);

    if (auto_spectr) {
        lb_auto->setText(tr("Status: auto"));
    } else {
        lb_auto->setText(tr("Status: manual"));
        s_apply_calib_Settings();
        AutoCalibration->change_state_man(false);
    }
}
void CCDPlot::sl_set_X_scale(int sc)
{
    if (setReadMode->currentIndex() == 2)
        return;

    AGScales->actions()[sc]->trigger();
    AGScales->actions()[sc]->setChecked(true);
    sl_set_X_scale();
}
void CCDPlot::sl_set_X_scale()
{
    if (setReadMode->currentIndex() == 2)
        return;

    int sc = 0;
    if (AGScales->actions()[1]->isChecked())
        sc = 1;
    if (AGScales->actions()[2]->isChecked())
        sc = 2;
    currentRelativeScale = sc;

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::spectrSetRelativeScales, sc);

    QString icons[3] = {":/CCD/ccdRelativeScale.png", ":/CCD/ccdRelativeScaleNM.png", ":/CCD/ccdRelativeScaleCM.png"};
    BRelativeScales->setIcon(makeHugeIcon(icons[sc]));

    switch (sc) {
    case 0:
        MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
        MyPl->TwoDPlot->Pure2dPlot->SetAllXScales(0,ccdMatrix->Nx_real);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllXScales(0,ccdMatrix->Nx_real);
        lineLaserWl->hide();
        lb_laser->hide();
        break;
    case 1:
        MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("WL"), QObject::tr("m"));
        MyPl->TwoDPlot->Pure2dPlot->SetAllXScales(0,2000);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("WL"), QObject::tr("m"));
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllXScales(0,2000);
        lineLaserWl->hide();
        lb_laser->hide();
        break;
    case 2:
        MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("Raman Shift, 1/cm"));
        MyPl->TwoDPlot->Pure2dPlot->SetAllXScales(-10000,10000);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("Raman Shift, 1/cm"));
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllXScales(-10000,10000);
        lineLaserWl->show();
        lb_laser->show();
        break;
    }

    s_apply_calib_Settings();
}

void CCDPlot::s_apply_calib_Settings()
{
    save_data();
    ccdMatrix->s_setXAxisScale(currentRelativeScale, lineLaserWl->value(), lineCentralPx->value(),
                                                     lineCentrWL->value(), lineDispersion->value());

    if (ccdMatrix->Scale != currentRelativeScale)
        for (int i = 0; i < curveData.size(); i++)
            set_spectra_calib(*curveData[i].data(), curveCalib, currentRelativeScale);
    MyPl->Refresh();
}

void CCDPlot::load_data()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);

    int scale_ind = settings.value("scale_ind", 0).toInt();
    lineLaserWl->setValue(settings.value("laser_wl", 532).toDouble());
    lineCentrWL->setValue(settings.value("central_wl", 532).toDouble());
    lineDispersion->setValue(settings.value("dispersion", 10).toDouble());
    lineCentralPx->setValue(settings.value("central_px", 512).toDouble());

    sl_set_X_scale(scale_ind);
}

void CCDPlot::save_data()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);

    settings.setValue("scale_ind", currentRelativeScale);
    settings.setValue("laser_wl", lineLaserWl->text());
    settings.setValue("central_wl", lineCentrWL->text());
    settings.setValue("central_px", lineCentralPx->text());
    settings.setValue("dispersion", lineDispersion->text());
}


void CCDPlot::startAcquisition(bool value)
{
    foreach (QWidget *wid, ccdControlWidgets)
        wid->setEnabled(!value && !ScanCCDMode);

    One_Cont->setVisible(!value);
    if (value) {
        MyPl->HideSelectionIfAny();
        if (MyPl->TwoDPlot->PlotControl->BMarkers->isChecked())
            MyPl->TwoDPlot->PlotControl->BMarkers->click();

        QColor color = Qt::black;
        if (curveData.size() == 1)
            color = curveData[0]->Pen.color();

        plot::ResizeOwner(curveData, 1);
        curveData[0]->Pen.setColor(color);
        MyPl->TwoDPlot->Pure2dPlot->SourceData = plot::getUnLock(curveData);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->clearSouceData();
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SourceData = *MyPl->TwoDPlot->Pure2dPlot->cloneSourceData();

        ccdMatrix->startAcquisition();

        //if (ccdMatrix->m_exposureTime*1000 >= 100) {
        if (!spec_bar) {
            spec_bar = new StatusBar(tr("CCD exposure"), ccdMatrix->m_exposureTime*1000);
            spec_bar->bar->setFormat(tr("%p% of %m ms"));
            connect(spec_bar, SIGNAL(canceled()), m_BStart, SLOT(click()));
            spec_bar->add();
        } else {
            auto bar = spec_bar->bar;
            bar->setUpdatesEnabled(false);
            bar->setMaximum(ccdMatrix->m_exposureTime*1000);
            bar->reset();
            bar->setUpdatesEnabled(true);
        }
        spec_bar->cancel->setEnabled(m_BStart->isEnabled());

        acqUpd->start(100);
    } else {
        ccdMatrix->s_abortAcq();

        acqUpd->stop();
        if (!ScanCCDMode)
            destructStatus(spec_bar);

        static bool first_spectra = true; //guard auto-send filler data started in main.c
        if (!first_spectra && AutoSend && MyPl->CurrentPlotType == plot::TPlot::_2d && !ScanCCDMode) {
            MyPl->TwoDPlot->on_SyncTimer_timeout();
            MyPl->TwoDPlot->PlotControl->BSave->animateClick();
        }
        first_spectra = false;


        lineNumAcc->blockSignals(true);
        lineNumAcc->setValue(ccdMatrix->numOfAccumulations);
        lineNumAcc->blockSignals(false);
        Num_Kin_ser->blockSignals(true);
        Num_Kin_ser->setValue(ccdMatrix->Kin_ser_number);
        Num_Kin_ser->blockSignals(false);

        if (ccdMatrix->m_spectrInterface && ccdMatrix->panoram) //update central wl
            lineCentralWlMonoch->setValue(ccdMatrix->m_spectrInterface->getWl());
    }
    
    static QTime lastTime = QTime::currentTime();
    static const int minWaitSec = 1;
    if (lastTime.secsTo(QTime::currentTime()) < minWaitSec)
        return;
    
    lastTime = QTime::currentTime();
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::spectrStartOnOff, value, true);
}

void CCDPlot::updateSpecBar()
{
    if (spec_bar != nullptr && ccdMatrix->m_ccdStart) {
        spec_bar->setValue(spec_bar->bar->value() + 100);
    }
}

void CCDPlot::sl_aq_but_off()
{
    if (m_BStart->isOn)
        m_BStart->change_state_man_with_sig(false);
}


void CCDPlot::onPanoram(bool val)
{
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(BPanoram, val);
    BPanoram->setIconSize(QSize(BPanoram->iconWidth, BPanoram->iconWidth));
#ifdef USE_CCD_EMULATOR
    QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
    bool useCCDEmulator = settings.value("ccd", false).toBool();
    
    if (!useCCDEmulator)
#endif
    if (val && !ccdMatrix->m_spectrInterface) {
        new Notification(tr("Panoram") + ": " + tr("Initialize optics before use."), Notification::Warn, 10);
        doBeep();

        BPanoram->change_state_man(false);
        return;
    }
    ccdMatrix->panoram = val;

    //manual calib not applicable
    if (val && BNLCFile->isChecked())
        BNLCAuto->click();
    BNLCFile->setEnabled(!val);
    
    panoramControl->setEnabled(val);
    if (One_Cont->isOn)
        One_Cont->change_state_man_with_sig(false);
    One_Cont->setEnabled(!val);
}
void CCDPlot::onPanoramMin(double val)
{
    if (val >= ccdMatrix->panoram_max) {
        val = ccdMatrix->panoram_max - 10;
        PanoramMin->setValue(val);
    }
    ccdMatrix->panoram_min = val;
}
void CCDPlot::onPanoramMax(double val)
{
    if (val <= ccdMatrix->panoram_min) {
        val = ccdMatrix->panoram_min + 10;
        PanoramMax->setValue(val);
    }
    ccdMatrix->panoram_max = val;
}

#include "math/plot_MathProcessing.h"

void CCDPlot::createPlot()
{
    MyPl = new plot::TPlot(this, this);

    MyPl->ThreeDPlot->TwoDPlot->Pure2dPlot->SetAutoLegendEnabled(false);
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->UseMathOnly = true;
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAutoLegendEnabled(false);

    MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
    MyPl->TwoDPlot->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(QObject::tr("Counts"));
    MyPl->TwoDPlot->SetTitle(tr("CCD Full V Binning Curve"));
    MyPl->TwoDPlot->Pure2dPlot->SetAllXScales(0, Nx - 1);
    MyPl->TwoDPlot->Pure2dPlot->SetAllYScales(0, 65535);


    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(QObject::tr("Counts"));
    MyPl->ThreeDPlot->TwoDPlotVertical->SetTitle(tr("CCD Full V Binning Curve"));
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllXScales(0, Nx - 1);
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllYScales(0, 65535);
    
    connect(MyPl->TwoDPlot->overlayButtons->control->AGFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(filter2DChanged()));
    
    plot::TwoDOverlayButtons *overlayBtns = new plot::TwoDOverlayButtons();
    MyPl->addWidget(oscPlot = new plot::T2dPlot(this, false, MyPl, false, overlayBtns));
    oscPlot->showPlotControl = false;
    oscPlot->Init();
    oscPlot->PlotControl->hide();
    overlayBtns->set2DPlot(oscPlot);
    overlayBtns->control->setShowingSendToDataButton(false);
    overlayBtns->control->removeFilterButton();
    overlayBtns->control->B2DGrid->change_state_man_with_sig(true);
}

void CCDPlot::filter2DChanged()
{
    int id = FindCheckedItemIn(MyPl->TwoDPlot->overlayButtons->control->AGFitOrders);
    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        twodPlotFilter = id;
        if (!scan->twodPlotFilterScanMode)
        {
            scan->Plot->TwoDPlot->PlotControl->AGFitOrders->actions()[id]->setChecked(true);
            scan->Plot->TwoDPlot->PlotControl->AGFitOrders->actions()[id]->trigger();
            scan->Plot->TwoDPlot->Refresh();
            scan->Plot->TwoDPlotVertical->Refresh();
        }
    }
}

QList<QWidget *> CCDPlot::getDeviceStatusList()
{
    QList<QWidget *> list;

    list << (QWidget *)2; //group id
    list << deviceStatus;
    list << deviceStatus2;

    if (cobolt_gui_list.size()) {
        list << (QWidget *)3; //group id
        list << new QLabel(QMainWindow::tr("Lasers:"));
    }
    for (int i = 0; i < cobolt_gui_list.size(); i++)
        list << cobolt_gui_list.at(i)->deviceStatus;

    return list;
}


void CCDPlot::setDeviceStatus2(int idx)
{
    if (!deviceStatus2)
        return;

    auto status = deviceStatus2->status;
    switch (idx) {
    case EDno:
        status->set(StatusWidget::Cross, Qt::red);
        status->setToolTip(tr("No device"));
        break;
    case EDok:
        status->set(StatusWidget::Circle, Qt::green);
        status->setToolTip(tr("Connected"));
        break;
    case EDplay:
        status->set(StatusWidget::Play, Qt::green);
        status->setToolTip(tr("Busy"));
        break;
    }
}

void CCDPlot::setDeviceStatus_handler(int idx)
{
    syncDevStatus = idx;
    if (!devStatusSync->isActive())
        devStatusSync->start(100);
}
void CCDPlot::setDeviceStatus()
{
    if (!deviceStatus || prevDevStatus == syncDevStatus)
        return;

    prevDevStatus = syncDevStatus;
    bool st = (syncDevStatus >= EDok);

    auto status = deviceStatus->status;
    switch (syncDevStatus) {
    case EDno:
        status->set(StatusWidget::Cross, Qt::red);
        status->setToolTip(tr("No device"));
        break;
    case EDok:
        status->set(StatusWidget::Circle, Qt::green);
        status->setToolTip(tr("Connected"));
        break;
    case EDplay:
        status->set(StatusWidget::Play, Qt::green);
        status->setToolTip(tr("Acquiring"));
        break;
    }

    //ccdsett->setEnabled(st);
    //m_BStart->setEnabled(st);

    bool st1 = st && ccdMatrix->hasTemperature;

    bool temp_upd = temperature_widget->isVisibleTo(this) != st1;
    temperature_widget->setVisible(st1);
    sendTemp->setVisible(st1);
    lineSetTemp->setVisible(st1);

    if (!st1) {
        temperatureUpd->stop();
    } else {
        if (!temperatureUpd->isActive())
            temperatureUpd->start(1000);
    }

    void *prev = ccdWidget;
    if (!st && ccdWidget) {
        delete ccdWidget;
        ccdWidget = nullptr;
    }

    if (st && !ccdWidget) {
        ccdWidget = ccdMatrix->getCCDInterfaceWidget();
        if (ccdWidget)
        {
            ccdWidget->setVisible(false);
            layoutccd->insertWidget(1, ccdWidget);
        }
    }

    if (prev != ccdWidget || temp_upd) {
        LeftScroll->setWidget(LeftScroll->takeWidget()); //reset sizeHint
        LeftScroll->updateGeometry(); //notify layout
    }
}

void CCDPlot::monoch_status_update(bool st)
{
    setDeviceStatus2(st ? EDok : EDplay);

    for (auto wid : monochromatorWidgets)
        wid->setEnabled(st);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrEnabled, st);
    NDEdit->setEnabled(st && Nspec_Initalized);
    linePreCentralWlMonoch->setEnabled(st && Nspec_Initalized);
    CBPreMonochWl->setEnabled(st && Nspec_Initalized);

    if (!st)
        return;

    if (Nspec_Initalized && nspectr_mutex) {
        QMutexLocker lck(nspectr_mutex);
        NDEdit->setValue(NProxy_GetExitNDFilter());
        linePreCentralWlMonoch->setValue(NProxy_GetWL());
    }

    if (CBgrating_select->isEnabled()) {
        CBgrating_select->blockSignals(true);
        int index = ccdMatrix->m_spectrInterface->getActiveGrating();
        CBgrating_select->setCurrentIndex(index);
        if (mob && mob->need_send())
            mob->setPropIdVar(DataID::setMonochrGrating, index);
        CBgrating_select->blockSignals(false);
    }

    lineSlitInMonoch->setValue(ccdMatrix->m_spectrInterface->getSlitWidth(SPECTR_PORT_IN));
    lineSlitOutMonoch->setValue(ccdMatrix->m_spectrInterface->getSlitWidth(SPECTR_PORT_OUT));
    lineCentralWlMonoch->setValue(ccdMatrix->m_spectrInterface->getWl());
    CBexitport->blockSignals(true);
    int index = ccdMatrix->m_spectrInterface->getPortPosition(SPECTR_PORT_OUT);
    CBexitport->setCurrentIndex(index);
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrExitMirror, index);
    CBexitport->blockSignals(false);
    
    CBInputMirror->blockSignals(true);
    index = ccdMatrix->m_spectrInterface->getPortPosition(SPECTR_PORT_IN);
    CBInputMirror->setCurrentIndex(index);
    CBInputMirror->blockSignals(false);
}
void CCDPlot::monoch_laser_wl()
{
    int idx = CBPreMonochWl->currentIndex();
    if (idx == -1)
        return;

    QSettings settings("Param/laser.conf", QSettings::IniFormat);
    settings.setValue("index", idx);

    if (settings.value("set_wl", false).toBool()) {
        if (idx < 0 || idx >= laser_list_wavelength.size()) {
            qDebug() << "wl list fail";
        } else {
            lineLaserWl->setValue(laser_list_wavelength.at(idx));
            lineLaserWl->setNewValue(true);
            linePreCentralWlMonoch->setValue(laser_list_wavelength.at(idx));
            linePreCentralWlMonoch->setNewValue(true);
        }
    }

    if (nspectr_mutex) {
        QMutexLocker lck(nspectr_mutex);

        if (settings.value("set_ef", false).toBool() &&
                idx >= 0 && idx < laser_list_edgefilter_idx.size())
            NProxy_SetEdgeFilterPos(laser_list_edgefilter_idx.at(idx));

        if (settings.value("set_be", false).toBool() &&
                idx >= 0 && idx < laser_list_beamexpander_wl_idx.size() && idx < laser_list_beamexpander_diam_idx.size())
            NProxy_SetBeamExpanderPos(laser_list_beamexpander_wl_idx.at(idx),
                                      laser_list_beamexpander_diam_idx.at(idx));

        if (settings.value("set_mxyz", false).toBool() &&
                idx >= 0 && idx < laser_list_monoch_x.size() && idx < laser_list_monoch_y.size() && idx < laser_list_monoch_z.size())
            NProxy_SetMonochromatorLensPos(laser_list_monoch_z.at(idx),
                                           laser_list_monoch_x.at(idx),
                                           laser_list_monoch_y.at(idx), true);

        if (settings.value("set_cxyz", false).toBool()  &&
                idx >= 0 && idx < laser_list_confocal_pmt_x.size() && idx < laser_list_confocal_pmt_y.size() && idx < laser_list_confocal_pmt_z.size())
            NProxy_SetConfocalLensPos(laser_list_confocal_pmt_z.at(idx),
                                      laser_list_confocal_pmt_x.at(idx),
                                      laser_list_confocal_pmt_y.at(idx), true);
    }

    int laser_idx = CBPreMonochWl->itemData(CBPreMonochWl->currentIndex()).toInt();
    for (int i = 0; i < cobolt_gui_list.size(); i++) {
        auto coboltGui = cobolt_gui_list.at(i);
        if ((!(i == laser_idx) != coboltGui->Laser_onoff->isOn) &&
                coboltGui->isEnabled())
            coboltGui->Laser_onoff->change_state_man_with_sig(!(i == laser_idx));
    }
}
void CCDPlot::nd_filter(double idx)
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_ars", true).toBool()) {
        ARS3000::setSlitWidth(1, NDEdit->value());
    } else if (nspectr_mutex) {
        QMutexLocker lck(nspectr_mutex);
        NProxy_SetExitNDFilter(idx, FALSE);
    }
}
void CCDPlot::monoch_pre_wl()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_ars", true).toBool()) {
        ARS3000::setLaser(CBPreMonochWl->currentIndex());
    } else if (nspectr_mutex) {
        QMutexLocker lck(nspectr_mutex);
        NProxy_SetWL(linePreCentralWlMonoch->value(), true);
    }
}
void CCDPlot::monoch_grating()
{
    if (!ccdMatrix->m_spectrInterface)
        return;

    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    settings.setValue("central_px_gr_"+QString::number(ccdMatrix->m_spectrInterface->getActiveGrating()), lineCentralPx->text());

    ccdMatrix->m_spectrInterface->setActiveGrating(CBgrating_select->currentIndex());

    lineCentralPx->setValue(settings.value("central_px_gr_"+QString::number(CBgrating_select->currentIndex()), lineCentralPx->value()).toDouble());
    lineCentralPx->setNewValue(true);
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrGrating, CBgrating_select->currentIndex());
}
void CCDPlot::monoch_wl()
{
    ccdMatrix->m_spectrInterface->setWl(lineCentralWlMonoch->value(), central_wl_reset->isChecked());
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrWaveLength, lineCentralWlMonoch->value());
}
void CCDPlot::monoch_slit_in()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_ars", true).toBool()) {
        ARS3000::setSlitWidth(0, lineSlitInMonoch->value() * 1e-3);
    }
    ccdMatrix->m_spectrInterface->setSlitWidth(SPECTR_PORT_IN, lineSlitInMonoch->value());
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrEntrance, lineSlitInMonoch->value());
}
void CCDPlot::monoch_slit_out()
{
    ccdMatrix->m_spectrInterface->setSlitWidth(SPECTR_PORT_OUT, lineSlitOutMonoch->value());
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrExisSlit, lineSlitOutMonoch->value());
}
void CCDPlot::monoch_Input_mirror()
{
    ccdMatrix->m_spectrInterface->setPortPosition(SPECTR_PORT_IN, CBInputMirror->currentIndex());
}
void CCDPlot::monoch_mirror()
{
    ccdMatrix->m_spectrInterface->setPortPosition(SPECTR_PORT_OUT, CBexitport->currentIndex());
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrExitMirror, CBexitport->currentIndex());
}

void CCDPlot::autoScaleClicked()
{
    MyPl->TwoDPlot->Pure2dPlot->SetAutoScaleEnabled(plotAutoScale->isChecked(), true);
    MyPl->TwoDPlot->overlayButtons->setButtonActive(plotAutoScale, plotAutoScale->isChecked());
}

void CCDPlot::focusChanged(double value)
{
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::spectrSetFocus, value);
}

void CCDPlot::showAdditCCDSettings(bool show)
{
    ccdWidget->setVisible(show);
    ccdAdditSetttings->setChecked(show);
}

void CCDPlot::Send2dPlotData()
{
    send2dPlotSpectr(MyPl->TwoDPlot);
}

void CCDPlot::send2dPlotSpectr(const plot::T2dPlot *plot2d)
{
    auto mob = MobileServer::GetIt();
    if (!mob)
        return;
    if (!mob->need_send())
        return;
    
    static QTime lastTime = QTime::currentTime();
    static const int minWaitSec = 1;
    if (lastTime.secsTo(QTime::currentTime()) < minWaitSec)
        return;
    
    lastTime = QTime::currentTime();
    
    if (mob->any_need_data("need_2dPlotSpectr"))
    {
        mob->setPropIdVar(DataID::PlotData2dInfo_Spectr, QVariant::fromValue(*((plot::TInfoFor2dPlot *)plot2d->Pure2dPlot)), true);

        plot::TCurvesDataOwner data = plot::getLock(plot2d->Pure2dPlot->SourceData);
        for (int i = 0; i < data.size(); i++)
            if (data.at(i) && data.at(i)->size())
            {
                auto dt = *data.at(i).data();
                mob->setPropIdVar(DataID::PlotData2d_Spectr, QVariant::fromValue(dt), true);
            }
    }
}

void CCDPlot::oneContChanged()
{
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::spectrSetInfMode, One_Cont->isOn);
}
