#include "ccdplot.h"
#include "gui/status.hpp"
#include "flowlayout.hpp"
#include <qwt_thermo.h>

#include "modules.hpp"
#include "oscilloscope.h"
#include "gui/ttabwidget.hpp"
#include "mainwindow.h"

#include "ccdmatrix.h"
#include "i_spectr.hpp"
#include "hardware/coboltcontroller.hpp"
#include "hardware/nspectr_proxy.h"
#include "qwt/qwt_plot_layout.h"
#include "scanplot.h"

#include <QScrollArea>
#include <QComboBox>
#include <QCheckBox>
#include <QGroupBox>
#include <QLabel>
#include <QTabWidget>
#include <QTextEdit>
#include <QRadioButton>
#include <QTimer>
#include <QSettings>
#include <QCloseEvent>
#include <QProgressBar>
#include <QFormLayout>
#include <QWidgetAction>
#include "widgets/threedoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"
#include "3d/ovcontrol_3d.h"
#include "2d/ovcontrol_2d.h"
#include "ars3000.h"

void CCDPlot::createButtons()
{
    sendTemp = new QLabel(tr("Set Temperature, ")+QChar(176)+tr("C"));
    
    ccdAdditSetttings = new TwoStateButton2(":/ScanPlot/lowVoltageMode.png",":/ScanPlot/lowVoltageMode.png", 40);
    ccdAdditSetttings->setCheckable(true);
    ccdAdditSetttings->setIconSize(QSize(40, 40));
    connect(ccdAdditSetttings, SIGNAL(state_changed(bool)), this, SLOT(showAdditCCDSettings(bool)));
    
    m_BStart = new TwoStateButton(":/CCD/ccdstart.png",":stop.png");
    m_BStart->setToolTip(tr("Start/stop Acquisition"));
    m_BStart->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(m_BStart);
    MyPl->TwoDPlot->overlayButtons->allWidgets.append(m_BStart);
    connect(m_BStart, SIGNAL(state_changed(bool)), this, SLOT(startAcquisition(bool)));
    
    m_3DBStart = new TwoStateButton(":start.png",":stop.png");
    m_3DBStart->setToolTip(tr("Start/stop Acquisition"));
    m_3DBStart->setFlat(true);
    plot::ThreeDOverlayButtons::setButtonStyle(m_3DBStart);
    connect(m_BStart, SIGNAL(state_changed(bool)), m_3DBStart, SLOT(change_state_man(bool)));
    connect(m_3DBStart, SIGNAL(state_changed(bool)), m_BStart, SLOT(change_state_man_with_sig(bool)));
    
    One_Cont= new TwoStateButton(":/CCD/oneshoot.png",":/CCD/continum.png");
    One_Cont->setParent(m_BStart);
    One_Cont->setFlat(true);
    One_Cont->setToolTip(tr("single/non stop Acquisition"));
    
    B3DOne_Cont= new TwoStateButton(":/CCD/oneshoot.png",":/CCD/continum.png");
    B3DOne_Cont->setFlat(true);
    plot::ThreeDOverlayButtons::setButtonStyle(B3DOne_Cont);
    B3DOne_Cont->setToolTip(tr("single/non stop Acquisition"));
    connect(One_Cont, SIGNAL(state_changed_man(bool)), this, SLOT(oneContChanged()));
    connect(One_Cont, SIGNAL(state_changed(bool)), B3DOne_Cont, SLOT(change_state_man(bool)));
    connect(B3DOne_Cont, SIGNAL(state_changed(bool)), One_Cont, SLOT(change_state_man_with_sig(bool)));
    
    Monochr = new QPushButton;
    Monochr->setIcon(makeHugeIcon(":/CCD/monochromator.png"));
    Monochr->setToolTip(tr("Run Centaur instrument control"));
    connect(Monochr, SIGNAL(clicked()), this, SLOT(sl_show_Monochromator()));
    Monochr->hide();
    
    B3DMonochr = new QPushButton;
    B3DMonochr->setIcon(makeHugeIcon(":/CCD/monochromator.png"));
    B3DMonochr->setToolTip(tr("Run Centaur instrument control"));
    B3DMonochr->setFlat(true);
    plot::ThreeDOverlayButtons::setButtonStyle(B3DMonochr);
    connect(B3DMonochr, SIGNAL(clicked()), this, SLOT(sl_show_Monochromator()));
    
    BPanoram = new TwoStateButton2(":/CCD/Pspecoff.png",":/CCD/Pspecoff.png", 100);
    BPanoram->setToolTip(tr("Toggle panoram mode"));
    BPanoram->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(BPanoram);
    MyPl->TwoDPlot->overlayButtons->allWidgets.append(BPanoram);
    connect(BPanoram, SIGNAL(state_changed(bool)), this, SLOT(onPanoram(bool)));
    
    Cut_Pikes = new TwoStateButton(":/CCD/cut_pikes_off.png",":/CCD/cut_pikes_off.png");
    Cut_Pikes->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(Cut_Pikes);
    MyPl->TwoDPlot->overlayButtons->allWidgets.append(Cut_Pikes);
    Cut_Pikes->setToolTip(tr("Cut spikes on/off"));
    
    B3DCut_Pikes = new TwoStateButton(":/CCD/cut_pikes_off.png",":/CCD/cut_pikes_off.png");
    B3DCut_Pikes->setFlat(true);
    plot::ThreeDOverlayButtons::setButtonStyle(B3DCut_Pikes);
    B3DCut_Pikes->setToolTip(tr("Cut spikes on/off"));
    connect(Cut_Pikes, SIGNAL(state_changed(bool)), B3DCut_Pikes, SLOT(change_state_man(bool)));
    connect(B3DCut_Pikes, SIGNAL(state_changed(bool)), Cut_Pikes, SLOT(change_state_man_with_sig(bool)));
    connect(Cut_Pikes, SIGNAL(state_changed(bool)), this, SLOT(cutSpikesChanged(bool)));
    
    Base_line = new TwoStateButton(":/CCD/base_line_off.png",":/CCD/base_line_off.png");
    Base_line->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(Base_line);
    MyPl->TwoDPlot->overlayButtons->allWidgets.append(Base_line);
    Base_line->setToolTip(tr("Base line substract on/off"));
    connect(Base_line, SIGNAL(state_changed(bool)), this, SLOT(baselineChanged(bool)));
    
    base_line_save = new QPushButton;
    base_line_save->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(base_line_save);
    MyPl->TwoDPlot->overlayButtons->allWidgets.append(base_line_save);
    base_line_save->setIcon(makeHugeIcon(":/CCD/base_line_save.png"));
    base_line_save->setToolTip(tr("Store current line as baseline"));
    
    base_line_manual = new QPushButton();
    base_line_manual->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(base_line_manual);
    MyPl->TwoDPlot->overlayButtons->allWidgets.append(base_line_manual);
    base_line_manual->setIcon(makeHugeIcon(":/CCD/base_manual.png"));
    base_line_manual->setToolTip(tr("Create manual baseline"));
    
    BPMTScan = new QPushButton;
    BPMTScan->setIcon(makeHugeIcon(":/CCD/Sp_part.png"));
    BPMTScan->setToolTip(tr("Scan PMT spectra"));
    connect(BPMTScan, SIGNAL(clicked(bool)), this, SLOT(PMTScan()));
    
    plotFocus[0] = new QPushButton;
    plotFocus[0]->setIcon(makeHugeIcon(":/plot/focusBtn.png"));
    plotFocus[0]->setToolTip(tr("Show focus settings"));
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(plotFocus[0]);
    connect(plotFocus[0], SIGNAL(clicked(bool)), this, SLOT(showPlotFocusClicked()));
    
    plotFocus[1] = new QPushButton;
    plotFocus[1]->setIcon(makeHugeIcon(":/plot/focusBtn.png"));
    plotFocus[1]->setToolTip(tr("Show focus settings"));
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(plotFocus[1]);
    connect(plotFocus[1], SIGNAL(clicked(bool)), this, SLOT(showPlotFocusClicked()));
    
    plotAutoScale = new QPushButton;
    plotAutoScale->setCheckable(true);
    plotAutoScale->setIcon(makeHugeIcon(":/plot/allow_deny_autoscale.png"));
    plotAutoScale->setToolTip(tr("Set autoscale"));
    plotAutoScale->setChecked(MyPl->TwoDPlot->Pure2dPlot->IsAutoScaleEnabled());
    MyPl->TwoDPlot->overlayButtons->setButtonActive(plotAutoScale, MyPl->TwoDPlot->Pure2dPlot->IsAutoScaleEnabled());
    connect(plotAutoScale, SIGNAL(clicked(bool)), this, SLOT(autoScaleClicked()));
    
    BRelativeScales = new QPushButton();
    BRelativeScales->hide();
    BRelativeScales->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(BRelativeScales);
    MyPl->TwoDPlot->overlayButtons->allWidgets.append(BRelativeScales);
    BRelativeScales->setIcon(makeHugeIcon(":/CCD/ccdRelativeScale.png"));
    BRelativeScales->setToolTip(QObject::tr("Relative scales"));
    MenuRelativeScale = new QMenu;
    BRelativeScales->setMenu(MenuRelativeScale);
    AGScales = new QActionGroup(this);
    QString Scales[3] = {tr("Pixels"), tr("Wavelength"), tr("Raman shift")};
    for (int i = 0; i < 3; i++)
    {
        AGScales->addAction(MenuRelativeScale->addAction(Scales[i]));
        AGScales->actions()[i]->setCheckable(true);
    }
    AGScales->actions()[0]->setChecked(true);
    connect(AGScales, SIGNAL(triggered(QAction*)), this, SLOT(sl_set_X_scale()));
    
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (!settings.value("use_pmt", false).toBool())
        BPMTScan->hide();
}

void CCDPlot::createCCDAtt()
{
    Termometer = new QwtThermo();
    Termometer->setOrientation(Qt::Horizontal, QwtThermo::BottomScale);
    Termometer->setFillBrush(Qt::blue);
    Termometer->setRange(-100, 30, 0);
    
    setAcqMode = new QComboBox;
    ccdControlWidgets << setAcqMode;
    setAcqMode->addItem(tr("Single Scan"));
    setAcqMode->addItem(tr("Accumulate"));
    setAcqMode->addItem(tr("Kinetic Series"));
    connect(setAcqMode, SIGNAL(currentIndexChanged(int)), SLOT(enable_options_acm(int)));
    
    setReadMode = new QComboBox;
    ccdControlWidgets << setReadMode;
    setReadMode->addItem(tr("Full Vertical Binning"));
    setReadMode->addItem(tr("Single-Track"));
    setReadMode->addItem(tr("Image"));
    connect(setReadMode, SIGNAL(currentIndexChanged(int)), SLOT(enable_options_rm(int)));
    
    labExpTime = new QLabel(tr("Exposition time") + ", " + tr("s"));
    
    lineExpTime = new Edit(0.001, 1000);
    lineExpTime->setDecimals(3);
    lineExpTime->setSingleStep(pow(10., -2));
    lineExpTime->setValue(0.1);
    
    lineSetTemp = new Edit(-100, 100, 0);
    lineSetTemp->setValue(20);
    
    lb_temp = new QLabel("0");
    lb_temp->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    lb_temp->setFrameStyle(QFrame::Panel);
    
    temp_pic = new QLabel;
    temp_pic->setPixmap(QPixmap(":/CCD/plus.png"));
    temp_pic->setMaximumSize(40,40);
    
    lineCenterRowTrack = new Edit(0, 10000);
    lineCenterRowTrack->setValue(128);
    ccdControlWidgets << lineCenterRowTrack;
    
    lineHeighTrack = new Edit(0, 1000);
    lineHeighTrack->setValue(10);
    ccdControlWidgets << lineHeighTrack;
    
    lineNumAcc = new Edit(1, 1000);
    lineNumAcc->setImmediateMode();
    ccdControlWidgets << lineNumAcc;
    
    lineAccCycleTime = new Edit(0, 10000, 2);
    
    //--------------Kinetic series-----------------------------------------------
    Num_Kin_ser = new Edit(1, 1000);
    Num_Kin_ser->setImmediateMode();
    ccdControlWidgets << Num_Kin_ser;
    
    Time_Kin_ser = new Edit(0, 1000, 2);
    
    kinSerDACscan = new QComboBox;
    ccdControlWidgets << kinSerDACscan;
    kinSerDACscan->addItem(tr("None"), -1);
    kinSerDACstep = new Edit(0, 10000, 2);
    kinSerDACstep->setFixedWidth(47);
    
    
    lb_auto = new QLabel(tr("Status: manual"));
    
    setTriggerMode = new QComboBox;
    ccdControlWidgets << setTriggerMode;
    setTriggerMode->addItem(tr("Internal"));
    setTriggerMode->addItem(tr("External"));
    connect(setTriggerMode, SIGNAL(currentIndexChanged(int)), SLOT(s_triggerMode(int)));
    
    AutoCalibration = new TwoStateButton(":/off.png",":/onn.png");
    connect(AutoCalibration, SIGNAL(state_changed(bool)), this, SLOT(sl_calib_change(bool)));
    AutoCalibration->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    
    CBgrating_select = new QComboBox;
    CBgrating_select->addItem(tr("Default grating"));
    connect(CBgrating_select, SIGNAL(currentIndexChanged(int)), SLOT(monoch_grating()));
    
    CBPreMonochWl = new QComboBox;
    
    CBexitport = new QComboBox;
    CBexitport->addItem(tr("Axial"));
    CBexitport->addItem(tr("Lateral"));
    connect(CBexitport, SIGNAL(currentIndexChanged(int)), SLOT(monoch_mirror()));
    
    CBInputMirror = new QComboBox;
    CBInputMirror->addItem(tr("Direct"));
    CBInputMirror->addItem(tr("Side"));
    connect(CBInputMirror, SIGNAL(currentIndexChanged(int)), SLOT(monoch_Input_mirror()));
    
    lb_centralWL = new QLabel(tr("Central WL (nm)"));
    lineCentrWL = new Edit(0, 10000, 2);
    lineCentrWL->setValue(488);
    connect(lineCentrWL, SIGNAL(newValue()), this, SLOT(s_apply_calib_Settings()));
    
    lb_dispersion = new QLabel(tr("Dispersion (nm/mm)"));
    lineDispersion = new Edit(0, 10000, 3);
    lineDispersion->setValue(10);
    connect(lineDispersion, SIGNAL(newValue()), this, SLOT(s_apply_calib_Settings()));
    
    lineCentralPx = new Edit(0, 4000);
    lineCentralPx->setValue(512);
    connect(lineCentralPx, SIGNAL(newValue()), this, SLOT(s_apply_calib_Settings()));
    
    labCentralWlMonoch = new QLabel(tr("Wavelength (nm)"));
    labGrating_select = new QLabel(tr("Grating"));
    labSlitInMonoch = new QLabel(tr("Entrance slit (nm)"));
    labSlitOutMonoch = new QLabel(tr("Exit slit (nm)"));
    labExitport = new QLabel(tr("Exit mirror"));
    labInputMirror = new QLabel(tr("Input mirror"));
    
    lb_laser = new QLabel(tr("Laser") + ", " + tr("nm"));
    lb_laser->hide();
    
    lineLaserWl = new Edit(0, 10000, 2);
    lineLaserWl->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    lineLaserWl->setValue(488);
    lineLaserWl->hide();
    connect(lineLaserWl, SIGNAL(newValue()), this, SLOT(s_apply_calib_Settings()));
    
    lineCentralWlMonoch = new Edit(0, 10000, 2);
    lineCentralWlMonoch->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    lineCentralWlMonoch->setValue(488);
    connect(lineCentralWlMonoch, SIGNAL(newValue()), this, SLOT(monoch_wl()));
    
    NDEdit = new Edit(0, 4, 1);
    connect (NDEdit, SIGNAL(newValue(double)), this, SLOT(nd_filter(double)));
    
    linePreCentralWlMonoch = new Edit(0, 10000, 2);
    linePreCentralWlMonoch->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    linePreCentralWlMonoch->setValue(532);
    connect(linePreCentralWlMonoch, SIGNAL(newValue()), this, SLOT(monoch_pre_wl()));
    
    lineSlitInMonoch = new Edit(0, 10000, 1);
    lineSlitInMonoch->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    lineSlitInMonoch->setValue(100);
    connect(lineSlitInMonoch, SIGNAL(newValue()), this, SLOT(monoch_slit_in()));
    
    lineSlitOutMonoch = new Edit(0, 10000, 1);
    lineSlitOutMonoch->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    lineSlitOutMonoch->setValue(100);
    connect(lineSlitOutMonoch, SIGNAL(newValue()), this, SLOT(monoch_slit_out()));
    
    central_wl_reset = new QCheckBox(tr("Central WL reset"));
    
    zPos = new DoubleChanger(0, 10, 4);
    
    calibLbl = new QLabel("-");
    pixelCalibLbl = new QLabel("-");
    calibLbl->setWordWrap(true);
    pixelCalibLbl->setWordWrap(true);
    
    calibOpn = new QPushButton(tr("Open"));
    pixelCalibOpn = new QPushButton(tr("Open"));
    connect(calibOpn, SIGNAL(clicked()), SLOT(intensity_calib_open_wl()));
    connect(pixelCalibOpn, SIGNAL(clicked()), SLOT(intensity_calib_open_px()));
    
    calibEn = new QCheckBox(tr("Enable WL Intensity calib"));
    pixelCalibEn = new QCheckBox(tr("Enable pixel Intensity calib"));
    connect(calibEn, SIGNAL(toggled(bool)), SLOT(intensity_calib_enabled()));
    connect(pixelCalibEn, SIGNAL(toggled(bool)), SLOT(intensity_calib_enabled()));
    
    box_Zpos = new QGroupBox(tr("Objective Z, ")+tr("mkm"));
    QHBoxLayout *layout_Zpos = new QHBoxLayout(box_Zpos);
    layout_Zpos->addWidget(zPos, 1);
    box_Zpos->setParent(MyPl->TwoDPlot);
    box_Zpos->setVisible(false);
    
    auto color = QColor(Qt::blue);
    hbdMotorZFocus = new HoverButtonsDirectionsHalf(color, "F");
    hbdMotorZFocus->hbEllipse->fontColor = color;
    
    hbdMotorObjMPMT = new HoverButtonsDirectionsFull(color, "M");
    hbdMotorObjMPMT->hbEllipse->fontColor = color;
    
    hbdMotorZFocus->setParent(MyPl->TwoDPlot);
    hbdMotorObjMPMT->setParent(MyPl->TwoDPlot);
    hbdMotorZFocus->setVisible(false);
    hbdMotorObjMPMT->setVisible(false);
    
    connect(hbdMotorObjMPMT->hbYTop, SIGNAL(holdingPressed(bool)), this, SLOT(ars3000MotorMPMT_pressed(bool)));
    connect(hbdMotorObjMPMT->hbYBottom, SIGNAL(holdingPressed(bool)), this, SLOT(ars3000MotorMPMT_pressed(bool)));   
    connect(hbdMotorObjMPMT->hbXLeft, SIGNAL(holdingPressed(bool)), this, SLOT(ars3000MotorMPMT_pressed(bool)));
    connect(hbdMotorObjMPMT->hbXRight, SIGNAL(holdingPressed(bool)), this, SLOT(ars3000MotorMPMT_pressed(bool)));    
    
    Cut_Pikes->setParent(MyPl->TwoDPlot);
    Base_line->setParent(MyPl->TwoDPlot);
    base_line_save->setParent(MyPl->TwoDPlot);
    BPanoram->setParent(MyPl->TwoDPlot);
    Cut_Pikes->setVisible(false);
    Base_line->setVisible(false);
    base_line_save->setVisible(false);
    BPanoram->setVisible(false);
    
    monochromatorWidgets << central_wl_reset
                         << labExitport << CBexitport
                         << labGrating_select << CBgrating_select
                         << labCentralWlMonoch << lineCentralWlMonoch
                         << labSlitInMonoch << lineSlitInMonoch
                         << labSlitOutMonoch << lineSlitOutMonoch;
    
#ifdef USE_SHAMROCK
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_shamrock", false).toBool())
    {
        monochromatorWidgets << labInputMirror << CBInputMirror;
    }
    else
#endif
    {
        labInputMirror->hide();
        CBInputMirror->hide();
    }
}

void CCDPlot::createLayout_ccd(QTabWidget *tab)
{
    QWidget *ccdsett = new QWidget();
    tab->addTab(ccdsett, tr("CCD") + "                   ");
    layoutccd = new QVBoxLayout(ccdsett);
    
    temperature_widget = new QWidget();
    QHBoxLayout *temperature_layout = new QHBoxLayout(temperature_widget);
    temperature_layout->addWidget(ccdAdditSetttings);
    temperature_layout->addWidget(Termometer, 1);
    temperature_layout->addWidget(temp_pic);
    temperature_layout->addWidget(lb_temp);
    layoutccd->addWidget(temperature_widget);
    
    ccdAdditSetttings->setFixedSize(temp_pic->height(), temp_pic->height());
    
    createLayout_ccd_otherTable();
    createLayout_ccd_accumulation();
    createLayout_ccd_kinetic();
    createLayout_ccd_singleTrack();
    
    QHBoxLayout *accumKinLay = new QHBoxLayout();
    layoutccd->addLayout(accumKinLay);
    accumKinLay->addWidget(box_accumulation);
    accumKinLay->addWidget(Kin_series);
    
    layoutccd->addWidget(box_singleTrack);
}

void CCDPlot::createLayout_ccd_otherTable()
{
    auto box = new QGroupBox();
    auto lay = new QHBoxLayout(box);
    QTableWidget *TableMotors = new QTableWidget(4, 2);
    TableMotors->verticalHeader()->setVisible(false);
    TableMotors->horizontalHeader()->setVisible(false);
    lay->addWidget(TableMotors);
    layoutccd->addWidget(box, 0, Qt::AlignTop);
    
    auto labAcqu = new QLabel(tr("Acquisition mode"));
    auto labReadMode = new QLabel(tr("Read mode"));
    
    TableMotors->setDragEnabled(false);
    TableMotors->setFocusPolicy(Qt::NoFocus);
    TableMotors->horizontalHeader()->setClickable(false);
    TableMotors->horizontalHeader()->setMovable(false);
    TableMotors->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
    TableMotors->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
    TableMotors->setSortingEnabled(false);
    TableMotors->setAlternatingRowColors(true);
    TableMotors->verticalScrollBar()->setVisible(false);
    TableMotors->verticalScrollBar()->setEnabled(false);
    TableMotors->verticalScrollBar()->setDisabled(true);
    TableMotors->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    
    TableMotors->setCellWidget(0, 0, sendTemp);
    TableMotors->setCellWidget(0, 1, lineSetTemp);
    TableMotors->setCellWidget(1, 0, labExpTime);
    TableMotors->setCellWidget(1, 1, lineExpTime);
    TableMotors->setCellWidget(2, 0, labAcqu);
    TableMotors->setCellWidget(2, 1, setAcqMode);
    TableMotors->setCellWidget(3, 0, labReadMode);
    TableMotors->setCellWidget(3, 1, setReadMode);
    
    sendTemp->setText(" " + sendTemp->text());
    labExpTime->setText(" " + labExpTime->text());
    labAcqu->setText(" " + labAcqu->text());
    labReadMode->setText(" " + labReadMode->text());
    
    int minHeight = 0;
    for (int i=0; i<TableMotors->rowCount(); i++)
        minHeight += TableMotors->cellWidget(i, 1)->minimumSizeHint().height();
    minHeight += 5;
    TableMotors->setFixedHeight(minHeight);
}

void CCDPlot::createLayout_ccd_accumulation()
{
    box_accumulation = new QGroupBox(tr("Accumulation options"));
    box_accumulation->hide();
    QHBoxLayout *accumHLay = new QHBoxLayout(box_accumulation);
    QVBoxLayout **accumVLays = new QVBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        accumVLays[i] = new QVBoxLayout();
        accumHLay->addLayout(accumVLays[i]);
    }
    accumVLays[0]->addWidget(new QLabel(tr("Count")));
    accumVLays[1]->addWidget(lineNumAcc);
    
    accumVLays[0]->addWidget(new QLabel(tr("Delay") + ", " + tr("s")));
    accumVLays[1]->addWidget(lineAccCycleTime);
    
    accumVLays[0]->addWidget(new QLabel(tr("Trigger mode")));
    accumVLays[1]->addWidget(setTriggerMode);
}

void CCDPlot::createLayout_ccd_kinetic()
{
    Kin_series = new QGroupBox(tr("Kinetic Series options"));
    Kin_series->hide();
    QHBoxLayout *kineticHLay = new QHBoxLayout(Kin_series);
    QVBoxLayout **kineticVLays = new QVBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        kineticVLays[i] = new QVBoxLayout();
        kineticHLay->addLayout(kineticVLays[i]);
    }
    kineticVLays[0]->addWidget(new QLabel(tr("Count")));
    kineticVLays[1]->addWidget(Num_Kin_ser);
    
    kineticVLays[0]->addWidget(new QLabel(tr("Delay") + ", " + tr("s")));
    kineticVLays[1]->addWidget(Time_Kin_ser);
    
    QHBoxLayout *kinserdacl = new QHBoxLayout;
    kinserdacl->addWidget(kinSerDACscan);
    kinserdacl->addWidget(kinSerDACstep);
    kineticVLays[0]->addWidget(new QLabel(tr("DAC step")));
    kineticVLays[1]->addLayout(kinserdacl);
}

void CCDPlot::createLayout_ccd_singleTrack()
{
    box_singleTrack = new QGroupBox(tr("Single-track options"));
    box_singleTrack->hide();
    QHBoxLayout *singleTrackHLay = new QHBoxLayout(box_singleTrack);
    QVBoxLayout **singleTrackVLays = new QVBoxLayout*[4];
    for (int i=0; i<4; i++)
    {
        singleTrackVLays[i] = new QVBoxLayout();
        singleTrackHLay->addLayout(singleTrackVLays[i]);
    }
    
    singleTrackVLays[0]->addWidget(new QLabel(tr("Center row")));
    singleTrackVLays[1]->addWidget(lineCenterRowTrack);
    
    singleTrackVLays[2]->addWidget(new QLabel(tr("Height")));
    singleTrackVLays[3]->addWidget(lineHeighTrack);
}

void CCDPlot::createLayout_PMT(QTabWidget *tab)
{
    QWidget *pmtWid = new QWidget();
    tab->addTab(pmtWid, tr("PMT") + "                   ");
    auto pmtlay = new QVBoxLayout(pmtWid);
    
    auto hv1lay = new QHBoxLayout();
    pmtlay->addLayout(hv1lay);
    ars3000PMTHVlab[0] = new QLabel(tr("PMT HV1, V"));
    hv1lay->addWidget(ars3000PMTHVlab[0]);
    ars3000PMTHV[0] = new DoubleChanger(0, ars3000PMTHVUmax[0]);
    hv1lay->addWidget(ars3000PMTHV[0]);
    connect(ars3000PMTHV[0], SIGNAL(newValue(double)), this, SLOT(ars3000PMTHV1_changed(double)));
    
    auto hv2lay = new QHBoxLayout();
    pmtlay->addLayout(hv2lay);
    ars3000PMTHVlab[1] = new QLabel(tr("PMT HV2, V"));
    hv2lay->addWidget(ars3000PMTHVlab[1]);
    ars3000PMTHV[1] = new DoubleChanger(0, ars3000PMTHVUmax[1]);
    hv2lay->addWidget(ars3000PMTHV[1]);
    connect(ars3000PMTHV[1], SIGNAL(newValue(double)), this, SLOT(ars3000PMTHV2_changed(double)));
}

void CCDPlot::createLayout_monochromator(QTabWidget *tab)
{
    auto wid = new QWidget();
    tab->addTab(wid, tr("Monochromator"));
    auto layMonochromator = new QHBoxLayout(wid);
    
    
    QVBoxLayout *leftMonochrLay = new QVBoxLayout();
    layMonochromator->addLayout(leftMonochrLay);
    
    leftMonochrLay->addWidget(central_wl_reset);
    
    QHBoxLayout *leftHMonochr = new QHBoxLayout();
    leftMonochrLay->addLayout(leftHMonochr);
    QVBoxLayout **leftVMonochr = new QVBoxLayout*[2];
    
    for (int i=0; i<2; i++)
    {
        leftVMonochr[i] = new QVBoxLayout();
        leftHMonochr->addLayout(leftVMonochr[i]);
    }
    
    leftVMonochr[0]->addWidget(labCentralWlMonoch);
    leftVMonochr[1]->addWidget(lineCentralWlMonoch);
    
    leftVMonochr[0]->addWidget(labGrating_select);
    leftVMonochr[1]->addWidget(CBgrating_select);
    
    leftVMonochr[0]->addWidget(labSlitInMonoch);
    leftVMonochr[1]->addWidget(lineSlitInMonoch);
    
    leftVMonochr[0]->addWidget(labSlitOutMonoch);
    leftVMonochr[1]->addWidget(lineSlitOutMonoch);
    
    leftVMonochr[0]->addWidget(labExitport);
    leftVMonochr[1]->addWidget(CBexitport);
    
    leftVMonochr[0]->addWidget(labInputMirror);
    leftVMonochr[1]->addWidget(CBInputMirror);
    
    QHBoxLayout *leftMonochrBtnsLay = new QHBoxLayout();
    leftMonochrLay->addLayout(leftMonochrBtnsLay, 0);
    leftMonochrBtnsLay->setAlignment(Qt::AlignLeft);
    
    leftMonochrBtnsLay->addWidget(Monochr, 0, Qt::AlignLeft);
    Monochr->setFixedSize(40, 40);
    
    createLayout_monochromator_panoram(leftMonochrLay);
    
    TTabWidget *calib_tabs = new TTabWidget();
    
    layMonochromator->addWidget(calib_tabs);
    
    createLayout_monochromator_tab_calibration(calib_tabs);
    createLayout_monochromator_tab_intensity(calib_tabs);
    createLayout_monochromator_tab_nonlinear(calib_tabs);
}

void CCDPlot::createLayout_monochromator_tab_calibration(TTabWidget *calib_tabs)
{
    QWidget *calibrationsWid = new QWidget(calib_tabs);
    QHBoxLayout *calibrationLay = new QHBoxLayout(calibrationsWid);
    calib_tabs->addTab(calibrationsWid, tr("Calibrations"));
    
    QVBoxLayout **calibrationLays = new QVBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        calibrationLays[i] = new QVBoxLayout();
        calibrationLay->addLayout(calibrationLays[i]);
    }
    
    calibrationLays[0]->addWidget(lb_auto, 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(AutoCalibration, 0, Qt::AlignVCenter);
    
    calibrationLays[0]->addWidget(new QLabel(tr("Central Pixel")), 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(lineCentralPx, 0, Qt::AlignVCenter);
    
    calibrationLays[0]->addWidget(lb_laser, 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(lineLaserWl, 0, Qt::AlignVCenter);
    
    calibrationLays[0]->addWidget(lb_centralWL, 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(lineCentrWL, 0, Qt::AlignVCenter);
    
    calibrationLays[0]->addWidget(lb_dispersion, 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(lineDispersion, 0, Qt::AlignVCenter);
    
    int maxEditorWidth = 70;
    lineCentralPx->setFixedWidth(maxEditorWidth);
    lineLaserWl->setFixedWidth(maxEditorWidth);
    lineCentrWL->setFixedWidth(maxEditorWidth);
    lineDispersion->setFixedWidth(maxEditorWidth);
}

void CCDPlot::createLayout_monochromator_tab_intensity(TTabWidget *calib_tabs)
{
    QWidget *intensityWid = new QWidget(calib_tabs);
    QFormLayout *layout_calibration2 = makeFormLayout(intensityWid);
    calib_tabs->addTab(intensityWid, tr("Intensity"));
    
    layout_calibration2->addRow(calibEn);
    layout_calibration2->addRow(calibOpn, calibLbl);
    layout_calibration2->addRow(pixelCalibEn);
    layout_calibration2->addRow(pixelCalibOpn, pixelCalibLbl);
}

void CCDPlot::createLayout_monochromator_tab_nonlinear(TTabWidget *calib_tabs)
{
    QWidget *nonlinearWid = new QWidget(calib_tabs);
    QGridLayout *layout_calibration3 = new QGridLayout(nonlinearWid);
    calib_tabs->addTab(nonlinearWid, tr("Nonlinear"));
    
    layout_calibration3->setAlignment(Qt::AlignTop);
    layout_calibration3->addWidget(BNLCNone = new QRadioButton(tr("Disabled")), 0, 0, 1, 2);
    layout_calibration3->addWidget(BNLCAuto = new QRadioButton(tr("Auto - device")), 1, 0, 1, 2);
    layout_calibration3->addWidget(BNLCFormula = new QRadioButton(tr("Formula")), 2, 0, 1, 2);
    layout_calibration3->addWidget(BNLCFile = new QRadioButton(tr("Manual list")), 3, 0, 1, 2);
    BNLCAuto->setChecked(true);
    
    QLabel *NLCStatus;
    layout_calibration3->addWidget(BNLCCfg = new QPushButton(tr("Edit")), 5, 0);
    layout_calibration3->addWidget(NLCStatus = new QLabel("-"), 5, 1);
    BNLCCfg->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    BNLCCfg->setEnabled(false);
    connect(NLCEdit, SIGNAL(newStatus(QString)), NLCStatus, SLOT(setText(QString)));
    connect(BNLCCfg, SIGNAL(clicked()), NLCEdit, SLOT(show()));
    connect(BNLCCfg, SIGNAL(clicked()), NLCEdit, SLOT(raise()));
    
    connect(BNLCNone, SIGNAL(toggled(bool)), NLCEdit, SLOT(hide()));
    connect(BNLCAuto, SIGNAL(toggled(bool)), NLCEdit, SLOT(hide()));
    
    connect(BNLCNone, SIGNAL(toggled(bool)), NLCEdit, SLOT(setMode()));
    connect(BNLCAuto, SIGNAL(toggled(bool)), NLCEdit, SLOT(setMode()));
    connect(BNLCFormula, SIGNAL(toggled(bool)), NLCEdit, SLOT(setMode()));
    connect(BNLCFile, SIGNAL(toggled(bool)), NLCEdit, SLOT(setMode()));
    
    connect(BNLCFormula, SIGNAL(toggled(bool)), NLCEdit, SLOT(show()));
    connect(BNLCFile, SIGNAL(toggled(bool)), NLCEdit, SLOT(show()));
    connect(BNLCFormula, SIGNAL(toggled(bool)), NLCEdit, SLOT(raise()));
    connect(BNLCFile, SIGNAL(toggled(bool)), NLCEdit, SLOT(raise()));
}

void CCDPlot::createLayout_monochromator_panoram(QVBoxLayout *leftMonochrLay)
{
    PanoramMin = new Edit(200, 800, 1);
    PanoramMin->setValue(200);
    connect(PanoramMin, SIGNAL(newValue(double)), this, SLOT(onPanoramMin(double)));
    
    PanoramMax = new Edit(200, 800, 1);
    PanoramMax->setValue(800);
    connect(PanoramMax, SIGNAL(newValue(double)), this, SLOT(onPanoramMax(double)));
    
    panoramControl = new QGroupBox(tr("Panoram Control"));
    panoramControl->setDisabled(true);
    QFormLayout *layoutPanoram = makeFormLayout(panoramControl);
    
    leftMonochrLay->addWidget(panoramControl);
    layoutPanoram->addRow(tr("From, nm"), PanoramMin);
    layoutPanoram->addRow(tr("To, nm"), PanoramMax);
}

void CCDPlot::createLayout_ARS3000(QTabWidget *tab)
{
    auto wid = new QWidget();
    tab->addTab(wid, tr("Confocal unit"));
    auto mainLay = new QVBoxLayout(wid);
    
    {
    auto gbConf = new QGroupBox(tr("Configuration"));
    mainLay->addWidget(gbConf);
    auto hlayGbConf = new QHBoxLayout(gbConf);
    ars3000RemoveLinePreset = new QPushButton("-");
    ars3000AddLinePreset = new QPushButton("+");
    ars3000PrefLinesChooser = new QComboBox();
    ars3000StartAllMotorBtn = new TwoStateButton(":/USBVideo/CameraStart.png",":/USBVideo/CameraStop.png");
    connect(ars3000StartAllMotorBtn, SIGNAL(state_changed(bool)), this, SLOT(ars3000StartAllMotors(bool)));
    connect(ars3000RemoveLinePreset, SIGNAL(clicked(bool)), this, SLOT(ars3000RemoveCurrentConfLine()));
    connect(ars3000AddLinePreset, SIGNAL(clicked(bool)), this, SLOT(ars3000AddNewConfLine()));
    connect(ars3000PrefLinesChooser, SIGNAL(currentTextChanged(const QString &)), this, SLOT(ars3000SavePreferences()));
    
    ars3000RemoveLinePreset->setFixedWidth(ars3000RemoveLinePreset->sizeHint().height());
    ars3000AddLinePreset->setFixedWidth(ars3000AddLinePreset->sizeHint().height());
    ars3000PrefLinesChooser->setEditable(true);
    ars3000PrefLinesChooser->setInsertPolicy(QComboBox::InsertAtCurrent);
    
    QString styleBtn = "font: bold; font-size: 300 ptx;";
    ars3000AddLinePreset->setStyleSheet(styleBtn);
    ars3000RemoveLinePreset->setStyleSheet(styleBtn);
    
    ars3000RemoveLinePreset->setToolTip(tr("Remove current configuration"));
    ars3000AddLinePreset->setToolTip(tr("Add new configuration"));
    ars3000StartAllMotorBtn->setToolTip(tr("Start all ARS3000 motors in order, with current configuration"));
    
    hlayGbConf->addWidget(ars3000AddLinePreset, 0, Qt::AlignLeft);
    hlayGbConf->addWidget(ars3000RemoveLinePreset, 0, Qt::AlignLeft);
    hlayGbConf->addWidget(ars3000PrefLinesChooser, 1);
    hlayGbConf->addWidget(ars3000StartAllMotorBtn);
    }
    
    ars3000LoadPreferences();
    
    auto widButtons = new QWidget();
    widButtons->setContentsMargins(0,0,0,0);
    mainLay->addWidget(widButtons);
    auto hlayButtons = new QHBoxLayout(widButtons);
    hlayButtons->setContentsMargins(0,0,0,0);
    auto vlaysButtons = new QVBoxLayout*[4];
    for (int i=0; i<4; i++)
    {
        vlaysButtons[i] = new QVBoxLayout();
        vlaysButtons[i]->setContentsMargins(0,0,0,0);
        hlayButtons->addLayout(vlaysButtons[i]);
    }
    
    QString imgs[numARS3000Motors];
    imgs[0] = ":/ars3000/ars3000_ndfilter.png";
    imgs[1] = ":/ars3000/ars3000_beamexp.png";
    imgs[2] = ":/ars3000/ars3000_edgefilter.png";
    imgs[3] = ":/ars3000/ars3000_slitmonochr.png";
    imgs[4] = ":/ars3000/ars3000_slitpmt.png";
    imgs[5] = ":/ars3000/ars3000_microsctrin.png";
    imgs[6] = ":/ars3000/ars3000_objmonochr.png";
    imgs[7] = ":/ars3000/ars3000_objpmt.png";
    
    for (int i=0, b=0, c=0; i<numARS3000Motors; i++, b++, c++)
    {
        if (b == 4)
            b = 0;
        
        int btnSz = 95;
        auto btn = new TwoStateButton2(imgs[i], imgs[i], btnSz);
        btn->setToolTip(Motors::axes_names[Motors::NDFilter + i]);
        btn->setStyleSheet(HIDING_MENU_INDICATOR_STYLESHEET);
        vlaysButtons[b]->addWidget(btn, 1, Qt::AlignCenter);
        btn->setFixedSize(btnSz, btnSz);
        btn->setDisabled(true);
        
        ars3000ConfUnitButtons << btn;
        
        auto menu = new QMenu();
        connect(menu, SIGNAL(aboutToShow()), this, SLOT(ars3000BtnMenuAboutToShow()));
        connect(menu, SIGNAL(triggered(QAction*)), this, SLOT(ars3000ConfLineSelected(QAction*)));
        btn->setMenu(menu);
        
        int stSz = 20;
        ars3000MotorStatuses[c] = new StatusWidget(stSz);
        ars3000MotorStatuses[c]->set(StatusWidget::Square, Qt::gray);
        ars3000MotorStatuses[c]->setParent(btn);
        int x = btnSz - stSz;
        ars3000MotorStatuses[c]->move(x, 0);
        
        if (c >= (Motors::ObjXSlMonochr - Motors::NDFilter))
        {
            c++;
            ars3000MotorStatuses[c] = new StatusWidget(stSz);
            ars3000MotorStatuses[c]->set(StatusWidget::Square, Qt::gray);
            ars3000MotorStatuses[c]->setParent(btn);
            int x = btnSz - stSz;
            ars3000MotorStatuses[c]->move(x, stSz);
        }
    }
    
    ars3000ConfUnitButtons.at(6)->setToolTip(tr("Obj XY Sl Monochr."));
    ars3000ConfUnitButtons.last()->setToolTip(tr("Obj XY Sl PMT"));
}

#ifdef _ENABLE_NST_LASERS
void CCDPlot::createLayout_nstLaser(QVBoxLayout *leftLay)
{
    if (nstLaser_list.size() != 0)
    {
        QGroupBox *box_laser = new QGroupBox(tr("NST lasers"));
        leftLay->addWidget(box_laser, 0, Qt::AlignBottom);
        QVBoxLayout *mainVLayLaser = new QVBoxLayout(box_laser);
        
        QTabWidget *laserTabWidget = new QTabWidget(box_laser);
        mainVLayLaser->addWidget(laserTabWidget);
        
        foreach (nstLasers *laser, nstLaser_list)
        {
            laserTabWidget->addTab(laser, laser->windowTitle());
        }
    }
}
#endif

void CCDPlot::createLayout_laser(QVBoxLayout *leftLay)
{
    QSettings settings("Param/laser.conf", QSettings::IniFormat);
    
    bool useCoboltLaser = settings.value("useCoboltLasers", false).toBool();
    if (!useCoboltLaser)
        return;
    
    QGroupBox *box_laser = new QGroupBox(tr("Laser control"));
    leftLay->addWidget(box_laser, 0, Qt::AlignBottom);
    QVBoxLayout *mainVLayLaser = new QVBoxLayout(box_laser);
    
    if (cobolt_gui_list.size() != 0) {
        QTabWidget *laserTabWidget = new QTabWidget(box_laser);
        mainVLayLaser->addWidget(laserTabWidget);
        
        foreach (CoboltGui *it, cobolt_gui_list)
            laserTabWidget->addTab(it, it->windowTitle());
    }
    QHBoxLayout *mainHLayLaser = new QHBoxLayout();
    QVBoxLayout **VLayLaser = new QVBoxLayout*[4];
    mainVLayLaser->addLayout(mainHLayLaser);
    for (int i=0; i<4; i++)
    {
        VLayLaser[i] = new QVBoxLayout();
        mainHLayLaser->addLayout(VLayLaser[i]);
    }
    {
        QSettings settings("Param/ccd.conf", QSettings::IniFormat);
        
        //VLayLaser[0]->addWidget(new QLabel(tr("ND Filter")));
        //VLayLaser[1]->addWidget(NDEdit);
        NDEdit->hide();
        //        if (!settings.value("use_ars", true).toBool()) {
        //            laser_l->addRow(tr("Pre-monoch wl"), linePreCentralWlMonoch);
        //        }
    }
    //VLayLaser[2]->addWidget(new QLabel(tr("Optics configuration")));
    //VLayLaser[3]->addWidget(CBPreMonochWl);
    CBPreMonochWl->hide();
}

void CCDPlot::createLayout()
{
    QWidget *w = this;
    
    QHBoxLayout *MainLayout = new QHBoxLayout(this);
    MainLayout->setContentsMargins(9, 9, 3, 9);
    
    LeftScroll = new TScrollArea(w);
    //LeftScroll->setMinimumWidth(500);
    
    MyPl->TwoDPlot->layout()->setContentsMargins(0, 0, 6, 0);
    MainLayout->addWidget(LeftScroll);
    MainLayout->addWidget(MyPl, 1);
    
    QWidget *LeftWid = new QWidget(LeftScroll);
    LeftScroll->setWidget(LeftWid);
    QVBoxLayout *leftLay = new QVBoxLayout(LeftWid);
    //leftLay->setSpacing(3);
    //leftLay->setContentsMargins(0, 6, 6, 6);
    
    QVBoxLayout *layCCDMono = new QVBoxLayout(LeftWid);
    leftLay->addLayout(layCCDMono);
    
    NLCEdit = new CCDNLCEditor(this);
    
    auto ccdGB = new QGroupBox(tr("Detectors"));
    auto ccdgbLay = new QHBoxLayout(ccdGB);
    ccdPMTTab = new QTabWidget();
    ccdPMTTab->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    ccdgbLay->addWidget(ccdPMTTab);
    connect(ccdPMTTab, SIGNAL(currentChanged(int)), this, SLOT(ccdPMTTab_tabChanged(int)));
    layCCDMono->addWidget(ccdGB);
    
    createLayout_ccd(ccdPMTTab);
    createLayout_PMT(ccdPMTTab);
    
    auto monoGB = new QGroupBox(tr("Opto mechanics"));
    auto monogbLay = new QHBoxLayout(monoGB);
    monochARSTab = new QTabWidget();
    monogbLay->addWidget(monochARSTab, 0, Qt::AlignTop);
    createLayout_monochromator(monochARSTab);
    createLayout_ARS3000(monochARSTab);
    #ifdef _ENABLE_NST_LASERS
    createLayout_nstLaser(leftLay);
    #endif
    createLayout_laser(leftLay);
    layCCDMono->addWidget(monoGB);
    
    MyPl->ThreeDPlot->ThreeDPlots[0]->overlayButtons->addUpButton(m_3DBStart);
    MyPl->ThreeDPlot->ThreeDPlots[0]->overlayButtons->addUpButton(B3DOne_Cont);
    MyPl->ThreeDPlot->ThreeDPlots[0]->overlayButtons->addUpButton(B3DCut_Pikes);
    MyPl->ThreeDPlot->ThreeDPlots[0]->overlayButtons->addUpButton(B3DMonochr);
    
    oscPlot->overlayButtons->addRightButton(plotFocus[1], 0);
    
    MyPl->TwoDPlot->overlayButtons->addLeftButton(m_BStart);
    MyPl->TwoDPlot->overlayButtons->addRightButton(plotFocus[0], 1);
    MyPl->TwoDPlot->overlayButtons->addRightButton(plotAutoScale, 1);
    MyPl->TwoDPlot->overlayButtons->addRightButton(BRelativeScales);
    
    plotFocus[0]->hide();
    plotFocus[1]->hide();
}

void CCDPlot::cutSpikesChanged(bool on)
{
    plot::TwoDOverlayButtons::setButtonActive(Cut_Pikes, on);
    plot::ThreeDOverlayButtons::setButtonActive(B3DCut_Pikes, on);
}

void CCDPlot::baselineChanged(bool on)
{
    plot::TwoDOverlayButtons::setButtonActive(Base_line, on);
}

void CCDPlot::gui2DPlotSetVisible(bool visible)
{
    plot::TwoDOverlayButtons *overlay;
    if (ccdPMTTab->currentIndex() == 0)
        overlay = MyPl->TwoDPlot->overlayButtons;
    else
        overlay = oscPlot->overlayButtons;
    
    if (showPlotFocus && !overlay->showBack)
        showPlotFocus = false;
    bool visibleBtns = visible && (overlay->visMode == plot::TwoDOverlayButtons::svm_none);
    
    if (showPlotFocus)
    {
        visible &= overlay->B2DBack->isVisible();
        MyPl->TwoDPlot->overlayButtons->setButtonVisible(Cut_Pikes, false);
        MyPl->TwoDPlot->overlayButtons->setButtonVisible(Base_line, false);
        MyPl->TwoDPlot->overlayButtons->setButtonVisible(base_line_save, false);
        MyPl->TwoDPlot->overlayButtons->setButtonVisible(BPanoram, false);
        
        box_Zpos->setVisible(visible);
        hbdMotorZFocus->setVisible(visible);
        hbdMotorObjMPMT->setVisible(visible);
    }
    else
    {
        box_Zpos->setVisible(false);
        hbdMotorZFocus->setVisible(false);
        hbdMotorObjMPMT->setVisible(false);
        MyPl->TwoDPlot->overlayButtons->setButtonVisible(Cut_Pikes, visibleBtns);
        MyPl->TwoDPlot->overlayButtons->setButtonVisible(Base_line, visibleBtns);
        MyPl->TwoDPlot->overlayButtons->setButtonVisible(base_line_save, visibleBtns);
        MyPl->TwoDPlot->overlayButtons->setButtonVisible(BPanoram, visibleBtns);
    }
}

void CCDPlot::gui2DPlotResized(QSize size)
{
    auto plot = MyPl->TwoDPlot;
    auto overlay = plot->overlayButtons;
    int leftX = overlay->buttonRect.x();
    int upY = MyPl->TwoDPlot->Pure2dPlot->QPlot->plotLayout()->titleRect().height();
    int upYPerc = upY*0.25;
    upY += upYPerc;
    int cbSize = 60;
    
    One_Cont->setParent(m_BStart);
    int miniBtnSize = m_BStart->width()*0.43;
    One_Cont->setFixedSize(miniBtnSize, miniBtnSize);
    One_Cont->move(m_BStart->width()-miniBtnSize, m_BStart->width()-miniBtnSize);
    
    box_Zpos->setFixedWidth(size.width()*0.4);
    box_Zpos->move(size.width() - 14 - box_Zpos->width(), upY + 10);
    auto fontSmall = QFont(zPos->font().family(), cbSize*0.15);
    zPos->setFont(fontSmall);
    zPos->edit->setFont(fontSmall);
    zPos->slider->setFont(fontSmall);
    box_Zpos->setFixedHeight(90);
    
    auto fontBig = QFont(zPos->font().family(), cbSize*0.45);
    hbdMotorZFocus->setFont(fontBig);
    hbdMotorObjMPMT->setFont(fontBig);
    
    int motorBtnsSize = cbSize*1.5;
    hbdMotorZFocus->setSize(motorBtnsSize);
    hbdMotorObjMPMT->setSize(motorBtnsSize);
    
    int btnX = size.width() - motorBtnsSize*1.5;
    int btnY = size.height()*0.5 - (motorBtnsSize*3 + 5*2)*0.5;
    hbdMotorZFocus->move(btnX, btnY);
    hbdMotorObjMPMT->move(leftX, btnY);
    
    int UpButtonSize = MyPl->TwoDPlot->overlayButtons->lastUpButtonSize;
    Cut_Pikes->setFixedSize(UpButtonSize, UpButtonSize);
    Cut_Pikes->setIconSize(QSize(UpButtonSize, UpButtonSize));
    Base_line->setFixedSize(UpButtonSize, UpButtonSize);
    Base_line->setIconSize(QSize(UpButtonSize, UpButtonSize));
    base_line_save->setFixedSize(UpButtonSize, UpButtonSize);
    base_line_save->setIconSize(QSize(UpButtonSize, UpButtonSize));
    BPanoram->setFixedSize(UpButtonSize, UpButtonSize);
    BPanoram->setIconSize(QSize(UpButtonSize, UpButtonSize));
    BPanoram->iconWidth = UpButtonSize;
    int distBetween = UpButtonSize*0.05;
    int x = m_BStart->x();
    int y = m_BStart->y() + UpButtonSize*2 + distBetween;
    Cut_Pikes->move(x, y);
    y += UpButtonSize + distBetween;
    Base_line->move(x, y);
    y += UpButtonSize + distBetween;
    base_line_save->move(x, y);
    y += UpButtonSize + distBetween;
    BPanoram->move(x, y);
}

void CCDPlot::showPlotFocusClicked()
{
    plot::TwoDOverlayButtons *overlay;
    if (ccdPMTTab->currentIndex() == 0)
        overlay = MyPl->TwoDPlot->overlayButtons;
    else
        overlay = oscPlot->overlayButtons;
    
    overlay->showBack = true;
    showPlotFocus = true;
    overlay->visMode = plot::TwoDOverlayButtons::svm_focus_mode;
    overlay->setVisibleGUI2D(true);
}

void CCDPlot::ARS3000MotorsUpdated()
{
    updateARS3000Motors();
}

void CCDPlot::updateARS3000Motors(Motors* m)
{
    if (!m)
    {
        m = Motors::GetIt();
        if (!m) return;
    }
    
    auto role = Motors::NDFilter;
    for (auto btn : ars3000ConfUnitButtons)
    {
        bool isObjMotor = role == Motors::ObjXSlMonochr ||
                          role == Motors::ObjXSlPMT;
        bool exist = m->isMotorExist(role);
        if (isObjMotor)
        {
            role = (Motors::MotorRoles)((int)(role) + 1);
            exist &= m->isMotorExist(role);
        }
        btn->setEnabled(exist);
        if (!exist)
            btn->setStyleSheet(HIDING_MENU_INDICATOR_STYLESHEET);
        role = (Motors::MotorRoles)((int)(role) + 1);
        btn->restoreSize();
    }
}
