#include "dacscontrol.h"

#include "oscilloscope.h"

#include <QSettings>
#include <QMessageBox>
#include <QCloseEvent>
#include <QHBoxLayout>
#include <QStackedLayout>
#include <QCheckBox>
#include <QTimer>
#include "remote_nst_dataid.hpp"
#include "gui/status.hpp"
#include "autocalib.hpp"
#include <QDebug>
#include <QtGlobal>
#include "modules.hpp"
#include "scanplot.h"
#include <qwt_plot_canvas.h>
#include "mobile/remote_mobile.hpp"
#include <QVariant>
#include "hardware/eg3000_calibrations.h"
#include "hardware/EG3000.h"
#include "spmtunning.h"
#include "options_window.h"
#include "qwt_plot_layout.h"
#include "2d/plot_2dplot.h"
#include "2d/plot_2dpure.h"
#include "datalistoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"
#include <i_eg3000.h>

#include <nstslider.h>
DacsControl::DacsControl(QWidget *parent) : QWidget(parent)
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    bool RatisMode = settingsGui.value("gui/Ratis_xyz_stage_mode", false).toBool();
    for (int i=0; i<10; i++)
    {
        dcCurrentDacChannel[i] = nullptr;
        timerMoveStarted[i] = false;
    }

    Xsp = Ysp = Zsp = 0;
    Xsp2 = Ysp2 = Zsp2 = 0;
    Zsp3 = 0;

    QSettings settings("Param/analog.conf", QSettings::IniFormat);
    double a1 = settings_double(settings, "dac/v1_min", -10.);
    double b1 = settings_double(settings, "dac/v1_max", 10.);
    double a2 = settings_double(settings, "dac/v2_min", -10.);
    double b2 = settings_double(settings, "dac/v2_max", 10.);
    double a3 = settings_double(settings, "dac/v3hr_min", -10.);
    double b3 = settings_double(settings, "dac/v3hr_max", 10.);
    double a4 = settings_double(settings, "dac/v4hr_min", -10.);
    double b4 = settings_double(settings, "dac/v4hr_max", 10.);

    Extra_Dac_1 = new DoubleChanger(a1, b1, 3);
    Extra_Dac_2 = new DoubleChanger(a2, b2, 3);
    {
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (!settings.value("use_pmt", false).toBool()) {
        Extra_Dac_3 = new DoubleChanger(a3, b3, 3);
        Extra_Dac_4 = new DoubleChanger(a4, b4, 3);
    } else {
        Extra_Dac_3 = new DoubleChanger(0, b3/10, 3);
        Extra_Dac_4 = new DoubleChanger(0, b4/10, 3);
    }
    }
    Z = new DoubleChanger(0, 50, 4);
    X = new DoubleChanger(0, 150, 4);
    Y = new DoubleChanger(0, 150, 4);
    Z2 = new DoubleChanger(0, 50, 4);
    X2 = new DoubleChanger(0, 150, 4);
    Y2 = new DoubleChanger(0, 150, 4);
    Z3 = new DoubleChanger(0, 50, 4);
    setPoint = new DoubleChanger(0, 50, 4);
    Zobj = new DoubleChanger(0, 50, 4);

    Extra_Dac_1->setProperty("id", DataID::SPM_DAC1);
    Extra_Dac_2->setProperty("id", DataID::SPM_DAC2);
    Extra_Dac_3->setProperty("id", DataID::SPM_DAC3);
    Extra_Dac_4->setProperty("id", DataID::SPM_DAC4);
    X->setProperty("id", DataID::HV_DAC_X);
    Y->setProperty("id", DataID::HV_DAC_Y);
    Z->setProperty("id", DataID::HV_DAC_Z);
    X2->setProperty("id", DataID::HV_DAC_X2);
    Y2->setProperty("id", DataID::HV_DAC_Y2);
    Z2->setProperty("id", DataID::HV_DAC_Z2);
    Z3->setProperty("id", DataID::HV_DAC_Z3);
    Zobj->setProperty("id", DataID::HV_DAC_Zobj);
    
    DacsChannel = new QComboBox;
    connect(DacsChannel, SIGNAL(currentIndexChanged(int)), this, SLOT(s_dacs_channel(int)));

    StartMoveSignal = new TwoStateButton(":start.png",":stop.png");
    connect(StartMoveSignal, SIGNAL(state_changed(bool)), this, SLOT(startSignalMoving(bool)));

    QHBoxLayout *layout_Dacs = new QHBoxLayout(this);
    layout_Dacs->setSpacing(6);
    layout_Dacs->setContentsMargins(0, 0, 0, 0);

    box_Ex_Dac1 = new QGroupBox(this);
    layout_Ex_Dac = new QStackedLayout(box_Ex_Dac1);

    QGroupBox* box_Ex_Dac2 = new QGroupBox(tr("DACS Channel"), this);
    QVBoxLayout *layout_Ex_VeDac22 = new QVBoxLayout(box_Ex_Dac2);
    QHBoxLayout *layout_Ex_Dac22 = new QHBoxLayout();
    layout_Ex_Dac22->setContentsMargins(6, 3, 6, 3);

    QHBoxLayout *layPeriod = new QHBoxLayout();
    layPeriod->addWidget(new QLabel(tr("T,ms:")), 0, Qt::AlignLeft);
    layPeriod->addWidget(timerPeriod = new Edit(1, 10000), 0, Qt::AlignLeft);
    layPeriod->addWidget(new QLabel(tr("A,%:")), 0, Qt::AlignRight);
    layPeriod->addWidget(procentPeriod = new Edit(1, 100), 0, Qt::AlignRight);
    procentPeriod->setValue(settings_double(settings, "dac/percentPeriodMoveSensor", 100));
    timerPeriod->setValue(settings_double(settings, "dac/timerPeriodMoveSensor", 1000));
    connect(procentPeriod, SIGNAL(newValue(int)), this, SLOT(timerPeriodNewValue(int)));
    connect(timerPeriod, SIGNAL(newValue(int)), this, SLOT(timerPeriodNewValue(int)));

    layout_Dacs->addWidget(box_Ex_Dac1);
    layout_Dacs->addWidget(box_Ex_Dac2);

    layout_Ex_Dac22->addWidget(DacsChannel);
    layout_Ex_Dac22->addWidget(StartMoveSignal);

    layout_Ex_VeDac22->addLayout(layout_Ex_Dac22);
    layout_Ex_VeDac22->addLayout(layPeriod);

    QString v_unit = QObject::tr("V");
    QString um_unit = tr("micro") + QObject::tr("m");

    QSettings settings1("Param/gui.conf", QSettings::IniFormat);
    QSettings settings2("Param/controller.conf", QSettings::IniFormat);
    if (!RatisMode)
    {
        if (!settings2.value("use_eg4000", true).toBool()) {
            insertChannel(tr("DAC 1/PMT"), v_unit, Extra_Dac_1);
            insertChannel(tr("DAC 2"), v_unit, Extra_Dac_2);
        }
        {
            QSettings settings("Param/ccd.conf", QSettings::IniFormat);

            if (!settings.value("use_pmt", false).toBool()) {
                insertChannel(tr("DAC AUX 1"), v_unit, Extra_Dac_3);
                insertChannel(tr("DAC AUX 2"), v_unit, Extra_Dac_4);
            } else {
                insertChannel(tr("PMT 1(INT)"), v_unit, Extra_Dac_3);
                insertChannel(tr("PMT 2(EXT)"), v_unit, Extra_Dac_4);
            }
        }
    }
    if (settings1.value("display/oscilloscope/head", true).toBool()) {
        insertChannel(tr("X head"), um_unit, X);
        insertChannel(tr("Y head"), um_unit, Y);
        insertChannel(tr("Z head"), um_unit, Z);
    }
    if (settings1.value("display/oscilloscope/stage", true).toBool()) {
        insertChannel(tr("X stage"), um_unit, X2);
        insertChannel(tr("Y stage"), um_unit, Y2);
        insertChannel(tr("Z stage"), um_unit, Z2);
    }
    if (settings1.value("display/oscilloscope/z3", false).toBool())
        insertChannel(tr("Z3"), um_unit, Z3);
    
    insertChannel(tr("Set point"), v_unit, setPoint);

    DacsChannel->setCurrentIndex(0);


    connect(Extra_Dac_1, SIGNAL(newValue(double)), this, SLOT(set_Ex_dac1(double)));
    connect(Extra_Dac_2, SIGNAL(newValue(double)), this, SLOT(set_Ex_dac2(double)));
    connect(Extra_Dac_3, SIGNAL(newValue(double)), this, SLOT(set_Ex_dac3(double)));
    connect(Extra_Dac_4, SIGNAL(newValue(double)), this, SLOT(set_Ex_dac4(double)));

    connect(X, SIGNAL(newValue(double)), this, SLOT(Go_to_X(double)));
    connect(Y, SIGNAL(newValue(double)), this, SLOT(Go_to_Y(double)));
    connect(Z, SIGNAL(newValue(double)), this, SLOT(Go_to_Z(double)));
    connect(X2, SIGNAL(newValue(double)), this, SLOT(Go_to_X2(double)));
    connect(Y2, SIGNAL(newValue(double)), this, SLOT(Go_to_Y2(double)));
    connect(Z2, SIGNAL(newValue(double)), this, SLOT(Go_to_Z2(double)));
    connect(Z3, SIGNAL(newValue(double)), this, SLOT(Go_to_Z3(double)));

    connect(this, SIGNAL(xUpdate(double)), X, SLOT(setValue(double)));
    connect(this, SIGNAL(x2Update(double)), X2, SLOT(setValue(double)));
    connect(this, SIGNAL(yUpdate(double)), Y, SLOT(setValue(double)));
    connect(this, SIGNAL(y2Update(double)), Y2, SLOT(setValue(double)));
    connect(this, SIGNAL(zUpdate(double)), Z, SLOT(setValue(double)));
    connect(this, SIGNAL(z2Update(double)), Z2, SLOT(setValue(double)));
    connect(this, SIGNAL(z3Update(double)), Z3, SLOT(setValue(double)));

    //zobj target
    bool useZ3 = settings1.value("display/oscilloscope/z3", false).toBool();
    DoubleChanger *zobj = useZ3 ? Z3 : Z2;

    linkDoubleChangers(zobj, Zobj);
    connect(zobj->slider, SIGNAL(valueChanged(double)), Zobj, SLOT(setValue(double))); //sync value change on setValue
}

void DacsControl::timerPeriodNewValue(int val)
{
    QSettings settings("Param/analog.conf", QSettings::IniFormat);
    settings.setValue("dac/percentPeriodMoveSensor", procentPeriod->value());
    settings.setValue("dac/timerPeriodMoveSensor", timerPeriod->value());

    for (int i=0; i<dacs.count(); i++)
    {
        sendSignalMoving(i, timerMoveStarted[i]);
        
        while (sendDACJumping)
        {
            SleepExport::msleep_gui(10);
        }
    }
}

void DacsControl::startSignalMoving(bool st)
{
    sendSignalMoving(currentChannel, st);
}

void DacsControl::sendSignalMoving(int channel, bool start)
{
    DACChannels dacChannel;
    
    //todo refactor it. (Oplkill)
    auto dc = dacs.at(channel);
    if (dc == X)
        dacChannel = dc_axisX;
    else if (dc == Y)
        dacChannel = dc_axisY;
    else if (dc == Z)
        dacChannel = dc_axisZ;
    
    else if (dc == X2)
        dacChannel = dc_axisX2;
    else if (dc == Y2)
        dacChannel = dc_axisY2;
    else if (dc == Z2)
        dacChannel = dc_axisZ2;
    
    else if (dc == Z3)
        dacChannel = dc_axisZ3;
    
    else if (dc == Extra_Dac_1)
        dacChannel = dc_dac1;
    else if (dc == Extra_Dac_2)
        dacChannel = dc_dac2;
    else if (dc == Extra_Dac_3)
        dacChannel = dc_dacAUX1;
    else if (dc == Extra_Dac_4)
        dacChannel = dc_dacAUX2;
    else if (dc == setPoint)
        dacChannel = dc_setPoint;
    else
        dacChannel = dc_vTip;
    
    if (dacChannel == dc_setPoint)
    {
        if (start)
            DACJumpingStartSetPointValue = dc->value();
        else
        {
            QTimer *tmr = new QTimer(this); // Создаем объект класса QTimer и передаем адрес переменной
            tmr->setInterval(1000); // Задаем интервал таймера
            tmr->setSingleShot(true);
            connect(tmr, SIGNAL(timeout()), this, SLOT(dacJumpReturnStartValueSetPoint())); // Подключаем сигнал таймера к нашему слоту
            tmr->start(); // Запускаем таймер
        }
    }
    
    timerMoveStarted[channel] = start;
    
    float proc = 100.0 - procentPeriod->value();
    proc /= 2;
    proc /= 100.0;
    
    dacSendData.DACIndex = dacChannel;
    dacSendData.MSPeriod = timerPeriod->value();
    dacSendData.StartJumping = start;
    dacSendData.BoundJump = proc;
    dacSendData.direction = 0;
            
    sendDACJumping = true;
}

void DacsControl::dacJumpReturnStartValueSetPoint()
{
    setPoint->setValue(DACJumpingStartSetPointValue);
    setPoint->setNewValue();
    setPoint->slider->setValue(DACJumpingStartSetPointValue);
    setPoint->slider->setNewValue();
}

void DacsControl::insertChannel(QString ch_name, QString unit, DoubleChanger *dc, int idx)
{
    if (idx < 0)
        idx = dacs.size();

    dacs.insert(idx, dc);
    units.insert(idx, unit);

    dc->setContentsMargins(6, 3, 6, 3);
    layout_Ex_Dac->addWidget(dc);

    DacsChannel->insertItem(idx, ch_name, QVariant::fromValue((void *)dc));
}

void DacsControl::sl_xyz(double x, double y, double z)
{
    X->setMaximum(x);
    Y->setMaximum(y);
    Z->setMaximum(z);
}
void DacsControl::sl_xyz2(double x, double y, double z)
{
    X2->setMaximum(x);
    Y2->setMaximum(y);
    Z2->setMaximum(z);
    //todo REFACTOR. move that code away.
    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        scan->deviceSchemeWid->ZPiezoPosEdit[0]->setMaximum(x);
        scan->deviceSchemeWid->ZPiezoPosEdit[1]->setMaximum(y);
        scan->deviceSchemeWid->ZPiezoPosEdit[2]->setMaximum(z);
    }
}
void DacsControl::sl_xyz3(double, double, double z)
{
    Z3->setMaximum(z);
}

void DacsControl::zPiezoPosX_X2_newValue(double value)
{
    X2->setValue(value);
    X2->setNewValue();
}

void DacsControl::zPiezoPosY_Y2_newValue(double value)
{
    Y2->setValue(value);
    Y2->setNewValue();
}

void DacsControl::zPiezoPosZ_Z2_newValue(double value)
{
    Z2->setValue(value);
    Z2->setNewValue();
}

void DacsControl::go_to_center(int scanner)
{
    if (scanner == 0) {
        X->setValue(X->edit->maximum() / 2);
        Y->setValue(Y->edit->maximum() / 2);
        Z->setValue(Z->edit->maximum() / 2);
        X->setNewValue();
        Y->setNewValue();
        Z->setNewValue();

    }
    if (scanner == 1) {
        X2->setValue(X2->edit->maximum() / 2);
        Y2->setValue(Y2->edit->maximum() / 2);
        Z2->setValue(Z2->edit->maximum() / 2);
        X2->setNewValue();
        Y2->setNewValue();
        Z2->setNewValue();
        //todo REFACTOR. move that code away.
        auto scan = ScanPlot::GetIt();
        if (scan)
        {
            scan->deviceSchemeWid->ZPiezoPosEdit[0]->setValue(X2->edit->maximum() / 2);
            scan->deviceSchemeWid->ZPiezoPosEdit[1]->setValue(Y2->edit->maximum() / 2);
            scan->deviceSchemeWid->ZPiezoPosEdit[2]->setValue(Z2->edit->maximum() / 2);
            scan->deviceSchemeWid->ZPiezoPosEdit[0]->setNewValue();
            scan->deviceSchemeWid->ZPiezoPosEdit[1]->setNewValue();
            scan->deviceSchemeWid->ZPiezoPosEdit[2]->setNewValue();
        }
    }
    if (scanner == 2) {
        Z3->setValue(Z3->edit->maximum() / 2);
        Z3->setNewValue();
    }
}


DoubleChanger *DacsControl::get_dacs_channel(QComboBox *CB)
{
    DoubleChanger *dc = (DoubleChanger *)CB->itemData(CB->currentIndex()).value<void *>();
    return dc;
}
void DacsControl::fill_dacs_channel_CB(QComboBox *CB)
{
    for (int i = 0; i < DacsChannel->count(); i++)
        CB->addItem(DacsChannel->itemText(i), DacsChannel->itemData(i));
}
void DacsControl::s_dacs_channel(int ch)
{
    DoubleChanger *dc = (DoubleChanger *)DacsChannel->itemData(ch).value<void *>();
    ch = dacs.indexOf(dc);
    if (ch < 0)
        return;

    layout_Ex_Dac->setCurrentWidget(dc);
    box_Ex_Dac1->setTitle(DacsChannel->currentText() + ", " + units.at(ch));
    dcCurrentDacChannel[ch] = dc;
    StartMoveSignal->change_state_man(timerMoveStarted[ch]);
    currentChannel = ch;
}


void DacsControl::Go_to_XY(double Xsp_n, double Ysp_n)
{
    Xsp = Xsp_n;
    Ysp = Ysp_n;

    emit xUpdate(Xsp);
    emit yUpdate(Ysp);
    sendSP[0] = true;
}
void DacsControl::Go_to_XY2(double Xsp_n, double Ysp_n)
{
    Xsp2 = Xsp_n;
    Ysp2 = Ysp_n;

    emit x2Update(Xsp2);
    emit y2Update(Ysp2);
    sendSP[1] = true;
}
void DacsControl::Go_to_X(double Xsp_n)
{
    Xsp = Xsp_n;
    emit xUpdate(Xsp);
    sendSP[0] = true;
}
void DacsControl::Go_to_X2(double Xsp_n)
{
    Xsp2 = Xsp_n;
    emit x2Update(Xsp2);
    sendSP[1] = true;
}
void DacsControl::Go_to_Y(double Ysp_n)
{
    Ysp = Ysp_n;
    emit yUpdate(Ysp);
    sendSP[0] = true;
}
void DacsControl::Go_to_Y2(double Ysp_n)
{
    Ysp2 = Ysp_n;
    emit y2Update(Ysp2);
    sendSP[1] = true;
}
void DacsControl::Go_to_Z(double Zsp_n)
{
    Zsp = Zsp_n;
    emit zUpdate(Zsp);
    sendSP[0] = true;
}
void DacsControl::Go_to_Z2(double Zsp_n)
{
    Zsp2 = Zsp_n;
    emit z2Update(Zsp2);
    sendSP[1] = true;
}
void DacsControl::Go_to_Z3(double Zsp_n)
{
    Zsp3 = Zsp_n;
    emit z3Update(Zsp3);
    sendSP[2] = true;
}
void DacsControl::set_Ex_dac1(double value)
{
    value = analog_calib(value, "dac/v1");
    Ex_dac[0] = value;
    sendExDAC[0] = true;
}
void DacsControl::set_Ex_dac2(double value)
{
    value = analog_calib(value, "dac/v2");
    Ex_dac[1] = value;
    sendExDAC[1] = true;
}
void DacsControl::set_Ex_dac3(double value)
{
    value = analog_calib(value, "dac/v3hr");
    Ex_dac[2] = value;
    sendExDAC[2] = true;
}
void DacsControl::set_Ex_dac4(double value)
{
    value = analog_calib(value, "dac/v4hr");
    Ex_dac[3] = value;
    sendExDAC[3] = true;
}
bool DacsControl::sendData(const void *handle)
{
    bool wrote = false;

    if (sendDACJumping.claimMessage()) {
        wrote = true;
        eg3000_send_packet(handle, ADR_CPU, CPU_DAC_JUMP_CHANNELS, &dacSendData, sizeof(EG_DacJumpChannel));
    }
    
    if (sendSP[0].claimMessage()) {
        wrote = true;
        eg3000_set_point_xyz((void *)handle, 0, Xsp, Ysp, Zsp);
    }
    if (sendSP[1].claimMessage()) {
        wrote = true;
        eg3000_set_point_xyz((void *)handle, 1, Xsp2, Ysp2, Zsp2);
    }
    if (sendSP[2].claimMessage()) {
        wrote = true;
        eg3000_set_point_xyz((void *)handle, 2, 0, 0, Zsp3);
    }

    if (sendExDAC[0].claimMessage()) {
        wrote = true;
        eg3000_set_extra_DAC1((void *)handle, Ex_dac[0]);
    }
    if (sendExDAC[1].claimMessage()) {
        wrote = true;
        eg3000_set_extra_DAC2((void *)handle, Ex_dac[1]);
    }
    if (sendExDAC[2].claimMessage()) {
        wrote = true;
        eg3000_set_extra_DAC3((void *)handle, Ex_dac[2]);
    }
    if (sendExDAC[3].claimMessage()) {
        wrote = true;
        eg3000_set_extra_DAC4((void *)handle, Ex_dac[3]);
    }

    return wrote;
}
void DacsControl::resetWriter()
{
    for (int i = 0; i < 3; i++)
        sendSP[i] = false;
    for (int i = 0; i < 4; i++)
        sendExDAC[i] = true;
    
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    if (settings.value("display/oscilloscope/stage", true).toBool())
    {
        int idXstage = dacs.indexOf(X2);
        sendSignalMoving(idXstage, false);
    }
}
