#ifndef DACSCONTROL_H
#define DACSCONTROL_H

#include <QWidget>
#include <QElapsedTimer>
#include <QPointer>
#include "controls.h"
#include "edit.hpp"
#include "plot.h"
#ifndef _ADROID
#include "eg3000/defs.h"
#endif
#include "tools.hpp"

class QStackedLayout;
class DacsControl: public QWidget, public IWriter
{
    Q_OBJECT

private slots:
    void s_dacs_channel(int ch);
    void startSignalMoving(bool st);
    void timerPeriodNewValue(int);
    void sl_xyz(double, double, double);
    void sl_xyz2(double, double, double);
    void sl_xyz3(double, double, double);
    
    void Go_to_XY(double, double);
    void Go_to_XY2(double, double);
    void Go_to_Z(double);
    void Go_to_Z2(double);
    void Go_to_Z3(double);
    void Go_to_X(double);
    void Go_to_X2(double);
    void Go_to_Y(double);
    void Go_to_Y2(double);
    
    void set_Ex_dac1(double);
    void set_Ex_dac2(double);
    void set_Ex_dac3(double);
    void set_Ex_dac4(double);
    
    //todo temp things, delete it when SPM GUI will be removed
    void zPiezoPosX_X2_newValue(double value);
    void zPiezoPosY_Y2_newValue(double value);
    void zPiezoPosZ_Z2_newValue(double value);
    //---------------
    
    void dacJumpReturnStartValueSetPoint();
    
public slots:
    DoubleChanger *get_dacs_channel(QComboBox *CB);
    void fill_dacs_channel_CB(QComboBox *CB);
    void go_to_center(int scanner);

private: signals:
    void xUpdate(double);
    void x2Update(double);
    void yUpdate(double);
    void y2Update(double);
    void zUpdate(double);
    void z2Update(double);
    void z3Update(double);

private:
    TMessageFlag sendDACJumping;
    EG_DacJumpChannel dacSendData;
    TMessageFlag sendExDAC[4];
    double Ex_dac[4];
    DoubleChanger *dcCurrentDacChannel[10];
    bool timerMoveStarted[10];
    TwoStateButton *StartMoveSignal;
    int currentChannel = 0;
    Edit *timerPeriod;
    Edit *procentPeriod;
    QGroupBox *box_Ex_Dac1;
    QStackedLayout *layout_Ex_Dac;
    QVector<DoubleChanger *> dacs;
    QVector<QString> units;
    
    double DACJumpingStartSetPointValue;
    
public:
    DacsControl(QWidget *parent = nullptr);
    bool sendData(const void *handle) override;
    void resetWriter() override;
    //takes ownership of dc
    void insertChannel(QString ch_name, QString unit, DoubleChanger *dc, int idx = -1);
    
    TMessageFlag sendSP[3];
    double Xsp, Ysp, Zsp;
    double Xsp2, Ysp2, Zsp2;
    double Zsp3;
    
    QComboBox *DacsChannel;
    DoubleChanger *Extra_Dac_1;
    DoubleChanger *Extra_Dac_2;
    DoubleChanger *Extra_Dac_3;
    DoubleChanger *Extra_Dac_4;
    
    DoubleChanger *X;
    DoubleChanger *Y;
    DoubleChanger *Z;
    DoubleChanger *X2;
    DoubleChanger *Y2;
    DoubleChanger *Z2;
    DoubleChanger *Z3;
    DoubleChanger *Zobj;
    DoubleChanger *setPoint;
    
private:
    void sendSignalMoving(int channel, bool start);
};

#endif // DACSCONTROL_H
