#include "hoverbuttonsdirections.h"

HoverButtonsDirectionsFull::HoverButtonsDirectionsFull(QColor color, QString centerText)
{
    hbXLeft = new HoverButton(HoverButton::LeftArrow, color);
    hbXRight = new HoverButton(HoverButton::RightArrow, color);
    hbYTop = new HoverButton(HoverButton::TopArrow, color);
    hbYBottom = new HoverButton(HoverButton::BottomArrow, color);
    hbEllipse = new HoverButton(HoverButton::Circle, color);
    
    hbEllipse->text = centerText;
    
    hbXLeft->setParent(this);
    hbXRight->setParent(this);
    hbYTop->setParent(this);
    hbYBottom->setParent(this);
    hbEllipse->setParent(this);
}

void HoverButtonsDirectionsFull::setSize(int sz)
{
    int halfSz = sz*0.5;
    int fullSize = sz*3;
    this->setFixedSize(fullSize, fullSize);
    
    hbXLeft->setFixedSize(sz, sz);
    hbXRight->setFixedSize(sz, sz);
    hbYTop->setFixedSize(sz, sz);
    hbYBottom->setFixedSize(sz, sz);
    hbEllipse->setFixedSize(sz, sz);
    
    QPoint center(fullSize*0.5, fullSize*0.5);
    hbXLeft->move(center.x() - halfSz - sz,
                  center.y() - halfSz);
    hbXRight->move(center.x() + halfSz,
                   center.y() - halfSz);
    hbYTop->move(center.x() - halfSz,
                 center.y() - halfSz - sz);
    hbYBottom->move(center.x() - halfSz,
                    center.y() + halfSz);
    hbEllipse->move(center.x() - halfSz,
                    center.y() - halfSz);
}

void HoverButtonsDirectionsFull::setFont(QFont font)
{
    hbEllipse->font = font;
}

HoverButtonsDirectionsHalf::HoverButtonsDirectionsHalf(QColor color, QString centerText)
{
    hbYTop = new HoverButton(HoverButton::TopArrow, color);
    hbYBottom = new HoverButton(HoverButton::BottomArrow, color);
    hbEllipse = new HoverButton(HoverButton::Circle, color);
    
    hbEllipse->text = centerText;
    
    hbYTop->setParent(this);
    hbYBottom->setParent(this);
    hbEllipse->setParent(this);
}

void HoverButtonsDirectionsHalf::setSize(int sz)
{
    int halfSz = sz*0.5;
    int fullSize = sz*3;
    this->setFixedSize(sz, fullSize);
    
    hbYTop->setFixedSize(sz, sz);
    hbYBottom->setFixedSize(sz, sz);
    hbEllipse->setFixedSize(sz, sz);
    
    QPoint center(0, fullSize*0.5);
    hbYTop->move(0,
                 center.y() - halfSz - sz);
    hbYBottom->move(0,
                    center.y() + halfSz);
    hbEllipse->move(0,
                    center.y() - halfSz);
}

void HoverButtonsDirectionsHalf::setFont(QFont font)
{
    hbEllipse->font = font;
}
