#include "abstractinoutlines.h"

#include <QPainter>
#include <QPaintEngine>
#include <QPoint>

AbstractInOutLines::AbstractInOutLines(uint numOut, uint numIn) : countOut(numOut), countIn(numIn)
{
    backgroundColor = QColor(226, 226, 245);//212

    absPos = new QPoint();
    if (countOut > 0)
    {
        connectOutId = new uint[countOut];
        for (uint i=0; i<countOut; i++)
        {
            ptsOut.push_back(new QPoint());
            absOut.push_back(nullptr);
            connectOutId[i] = 0;
            cornersOut.push_back(new QVector<QPoint*>());
        }
    }
    else
    {
        connectOutId = nullptr;
    }

    for (uint i=0; i<countIn; i++)
    {
        ptsIn.push_back(new QPoint());
        absIn.push_back(nullptr);
    }
}

void AbstractInOutLines::connectAbs(int mineOutId, AbstractInOutLines *other, int otherInId)
{
    absOut[mineOutId] = other;
    connectOutId[mineOutId] = otherInId;
    other->absIn[otherInId] = this;
}

void AbstractInOutLines::setPtsInXY(int id, int x, int y)
{
    ptsIn[id]->setX(x);
    ptsIn[id]->setY(y);
}

void AbstractInOutLines::setPtsOutXY(int id, int x, int y)
{
    ptsOut[id]->setX(x);
    ptsOut[id]->setY(y);
}

void AbstractInOutLines::setPosXY(int x, int y)
{
    absPos->setX(x);
    absPos->setY(y);
    recalcPoints();
}

void AbstractInOutLines::addCorner(int outId, int x, int y)
{
    cornersOut[outId]->append(new QPoint(x, y));
}

void AbstractInOutLines::setShadow(QWidget *wid, int w, const QColor &color)
{
    auto *effect = new QGraphicsDropShadowEffect;
    effect->setBlurRadius(w);
    effect->setXOffset(w);
    effect->setYOffset(w);
    effect->setColor(color);
    wid->setGraphicsEffect(effect);
}

AbstractInOutLines::~AbstractInOutLines()
{
    for (auto &pts : ptsOut) {
            delete pts;
    }
    ptsOut.clear();
    for (auto &pts : ptsIn) {
            delete pts;
    }
    ptsIn.clear();

    for (auto &vec : cornersOut) {
        for (auto &point : *vec) {
            delete point;
        }
        delete vec;
    }
    cornersOut.clear();

    absOut.clear();
    absIn.clear();
    delete absPos;
    delete[] connectOutId;
}

void AbstractInOutLines::drawArrow
    (QPainter *painter, int x, int y, int arrowWidth, int arrowHeight,
     int length, enumDirection dir, bool doubleArrow)
{
    enumDirection oppositDir;
    QPoint lineStart(x, y);
    QPoint lineEnd;
    QPoint *lineArrowsPeaks[2];
    QPoint lineStartOffseted[2];
    lineStartOffseted[0] = lineStartOffseted[1] = lineStart;
    lineArrowsPeaks[0] = new QPoint(lineStart.x(), lineStart.y());
    lineArrowsPeaks[1] = new QPoint(lineStart.x(), lineStart.y());
    int halfWidthArrow = arrowWidth*0.5;
    
    getArrowCoords(lineArrowsPeaks, halfWidthArrow, arrowHeight, dir);
    
    switch (dir)
    {
    case ed_up:
        lineEnd.setX(x);
        lineEnd.setY(y + length);
        
        lineStartOffseted[1].setX(lineStartOffseted[1].x() + 1);
        
        oppositDir = ed_down;
        break;
        
    case ed_down:
        lineEnd.setX(x);
        lineEnd.setY(y - length);
        
        lineStartOffseted[1].setX(lineStartOffseted[1].x() + 1);
        
        oppositDir = ed_up;
        break;
        
    case ed_left:
        lineEnd.setX(x + length);
        lineEnd.setY(y);
        
        lineStartOffseted[1].setY(lineStartOffseted[1].y() + 1);
        
        oppositDir = ed_right;
        break;
        
    case ed_right:
        lineEnd.setX(x - length);
        lineEnd.setY(y);
        
        lineStartOffseted[1].setY(lineStartOffseted[1].y() + 1);
        
        oppositDir = ed_left;
        break;
    }
    
    painter->drawLine(lineStart, lineEnd);
    painter->drawLine(lineStartOffseted[0], *lineArrowsPeaks[0]);
    painter->drawLine(lineStartOffseted[1], *lineArrowsPeaks[1]);
    
    if (doubleArrow)
    {
        QPoint *lineArrowsPeaks[2];
        lineArrowsPeaks[0] = new QPoint(lineEnd.x(), lineEnd.y());
        lineArrowsPeaks[1] = new QPoint(lineEnd.x(), lineEnd.y());
        
        getArrowCoords(lineArrowsPeaks, halfWidthArrow, arrowHeight, oppositDir);
        
        auto copyOffseted = lineStartOffseted[1];
        lineStartOffseted[1] = lineEnd;
        if (copyOffseted.x() > lineStart.x())
            lineStartOffseted[1].setX(lineStartOffseted[1].x() + 1);
        else
            lineStartOffseted[1].setY(lineStartOffseted[1].y() + 1);
        
        painter->drawLine(lineEnd, *lineArrowsPeaks[0]);
        painter->drawLine(lineStartOffseted[1], *lineArrowsPeaks[1]);
    }
}

void AbstractInOutLines::getArrowCoords(QPoint *peak[2], int halfWidthArrow, int arrowHeight, enumDirection dir)
{
    switch (dir)
    {
    case ed_up:
        peak[0]->setX(peak[0]->x() - halfWidthArrow);
        peak[0]->setY(peak[0]->y() + arrowHeight);
        peak[1]->setX(peak[1]->x() + halfWidthArrow);
        peak[1]->setY(peak[1]->y() + arrowHeight);
        break;
        
    case ed_down:
        peak[0]->setX(peak[0]->x() - halfWidthArrow);
        peak[0]->setY(peak[0]->y() - arrowHeight);
        peak[1]->setX(peak[1]->x() + halfWidthArrow);
        peak[1]->setY(peak[1]->y() - arrowHeight);
        break;
        
    case ed_left:
        peak[0]->setX(peak[0]->x() + arrowHeight);
        peak[0]->setY(peak[0]->y() - halfWidthArrow);
        peak[1]->setX(peak[1]->x() + arrowHeight);
        peak[1]->setY(peak[1]->y() + halfWidthArrow);
        break;
        
    case ed_right:
        peak[0]->setX(peak[0]->x() - arrowHeight);
        peak[0]->setY(peak[0]->y() - halfWidthArrow);
        peak[1]->setX(peak[1]->x() - arrowHeight);
        peak[1]->setY(peak[1]->y() + halfWidthArrow);
        break;
    }
}
