#include "schactivebox.h"
#include <QPainter>
#include <QMouseEvent>
#include <QIcon>

schActiveBox::schActiveBox(QWidget *parent) :
    QWidget(parent), AbstractInOutLines(0, 1)
{
    turnOn = false;
    hovered = false;
    initPositions(0, 0);
    
    setMouseTracking(true);
}

void schActiveBox::recalcPoints()
{
    initPositions(width(), height());
}

void schActiveBox::initPositions(int w, int h)
{
    rect.setX(0);
    rect.setY(0);
    rect.setWidth(w);
    rect.setHeight(h);
}

void schActiveBox::setState(bool on)
{
    setState_man(on);
    emit changedState(turnOn);
}

void schActiveBox::setState_man(bool on)
{
    turnOn = on;
    update();
}

bool schActiveBox::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();
    
    if (isBoxEnabled)
    {
        if (e->type() == QEvent::Leave)
        {
            hovered = false;
            update();
        }
        else if (e->type() == QEvent::MouseMove)
        {
            auto me = dynamic_cast<QMouseEvent *>(e);
            hovered = rect.contains(me->pos());
            
            update();
        }
        if (e->type() == QEvent::MouseButtonRelease && hovered)
        {
            setState(!turnOn);
        }
    }
    
    return QWidget::event(e);
}

void schActiveBox::paintEvent(QPaintEvent *)
{
    QPainter painter(this);
    
    initPositions(width(), height());
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(QColor(outterLineColor));
    painter.setBrush(QBrush(Qt::black));
    
    if (isBoxEnabled)
    {
        if (turnOn)
            painter.setBrush(QBrush(onColor));
        else
            painter.setBrush(QBrush(offColor));
    }
    else
        painter.setBrush(QBrush(disabledColor));
    painter.drawRect(rect);
    
    if (pixmapImage)
        painter.drawPixmap(rect, *pixmapImage);
    
    //Hovered
    painter.setOpacity(0.5);
    painter.setBrush(QBrush(Qt::blue));
    if (hovered)
        painter.drawRect(rect);
}
