#include "schsamplecontrolplatform.h"

#include <QtEvents>
#include <QGestureEvent>
#include <QPainter>
#include <QDebug>
#include <QtGlobal>
#include <QTimer>
#include <QLabel>
#include <QPaintEngine>
#include <QLine>
#include "tools.hpp"

schSampleControlPlatform::schSampleControlPlatform(QWidget *parent) : QWidget(parent), AbstractInOutLines (2, 0)
{
    labelUp = new QLabel(tr("Sample fix"), this);
    labelDown = new QLabel(tr("Holder move"), this);
    
    initPositions(0, 0);
}

void schSampleControlPlatform::initPositions(int w, int h)
{
    int sampleYOffset = 5;
    int sampleUpXOffset = w*0.05;
    int sampleHeight = h*0.05;
    sample.clear();
    sample.append(QPoint(sampleUpXOffset, sampleYOffset));
    sample.append(QPoint(w, sampleYOffset));
    sample.append(QPoint(w - sampleUpXOffset, sampleYOffset + sampleHeight));
    sample.append(QPoint(0, sampleYOffset + sampleHeight));
    
    int distBetweenSampleBox = 20;
    int boxXOffset = 10;
    boxRect.setX(boxXOffset);
    boxRect.setY(sampleYOffset + sampleHeight + distBetweenSampleBox);
    boxRect.setWidth(w - 2*boxXOffset);
    boxRect.setHeight(h - boxRect.y());
    
    int smallBoxesHeight = boxRect.height() * 0.3;
    int smallBoxesInnerXOffset = boxRect.width() * 0.1;
    int smallBoxesXOffset = boxRect.x() + smallBoxesInnerXOffset;
    int smallBoxesWidth = boxRect.width() - 2*smallBoxesInnerXOffset;
    
    boxUp.setX(smallBoxesXOffset);
    boxUp.setY(boxRect.y());
    boxUp.setWidth(smallBoxesWidth);
    boxUp.setHeight(smallBoxesHeight);
    
    boxDown.setX(smallBoxesXOffset);
    boxDown.setY(boxRect.y() + boxRect.height() - smallBoxesHeight);
    boxDown.setWidth(smallBoxesWidth);
    boxDown.setHeight(smallBoxesHeight);
}

bool schSampleControlPlatform::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();
    
    return QWidget::event(e);
}

void schSampleControlPlatform::paintEvent(QPaintEvent *)
{
    QPainter painter(this);
    
    int w = width();
    int h = height();
    
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(QColor(outterLineColor));
    
    //sample
    painter.setBrush(QBrush(sampleColor));
    painter.drawPolygon(sample);
    
    //box
    painter.setBrush(QBrush(tableColor));
    painter.drawRect(boxRect);
    
    painter.setBrush(QBrush(boxesColor));
    painter.setPen(QColor(Qt::black));
    
    //Box Up
    painter.drawRect(boxUp);
    
    //Box Down
    painter.drawRect(boxDown);
    
    //tubes
    painter.setRenderHint(QPainter::Antialiasing, false);
    painter.setBrush(QBrush(Qt::white));
    for (int i=0; i<2; i++)
    {
        int x = ptsOut[i]->x();
        int y = ptsOut[i]->y() - tubeWidth*0.5 - 1;
        painter.setPen(QColor(Qt::white));
        painter.drawRect(x, y,
                         w - x, tubeWidth);
        
        painter.setPen(QColor(Qt::black));
        painter.drawLine(x, y,
                         w, y);
        y = ptsOut[i]->y() + tubeWidth*0.5 - 1;
        painter.drawLine(x, y,
                         w, y);
    }
    
    if (isSampleClosingToTable)
    {
        painter.setPen(QColor(sampleClosingColor));
        painter.setRenderHint(QPainter::Antialiasing, true);
        
        int arrowWidth = 8;
        int arrowHeight = 4;
        int offset = 3;
        int distArrows = 30;
        int x = arrowWidth;
        int y = boxRect.y() - offset;
        int length = y - sample.last().y() - offset;
        while (x < w)
        {
            drawArrow(&painter, x, y, arrowWidth, arrowHeight, length, ed_down);
            x += distArrows;
        }
    }
}

void schSampleControlPlatform::recalcPoints()
{
    initPositions(width(), height());
    
    setPtsOutXY(0, boxUp.right(), boxUp.y() + boxUp.height()*0.5);
    setPtsOutXY(1, boxDown.right(), boxDown.y() + boxDown.height()*0.5);
    
    labelUp->move(boxUp.x() + boxUp.width()*0.5 - labelUp->fontMetrics().width(labelUp->text())*0.5,
                  boxUp.y() + boxUp.height()*0.5 - labelUp->fontMetrics().height()*0.5);
    labelDown->move(boxDown.x() + boxDown.width()*0.5 - labelDown->fontMetrics().width(labelDown->text())*0.5,
                    boxDown.y() + boxDown.height()*0.5 - labelDown->fontMetrics().height()*0.5);
}
