#include "schtubelocker.h"
#include <QPainterPath>

schTubeLocker::schTubeLocker(QWidget *parent) :
    QWidget(parent), AbstractInOutLines(1, 1)
{
    turnOn = false;
    hovered = false;
    initPositions(0, 0);
    
    setMouseTracking(true);
}

void schTubeLocker::recalcPoints()
{
    int w = width();
    int h = height();
    initPositions(w, h);
    
    setPtsInXY(0, 0, centerY);
    setPtsOutXY(0, w, centerY);
}

void schTubeLocker::initPositions(int w, int h)
{
    centerY = h*0.5;
    centerX = w*0.5;
    
    QPoint endPointUp;
    endPointUp.setX(centerX);
    if (turnOn)
        endPointUp.setY(centerY - tubeWidth*0.5);
    else
        endPointUp.setY(centerY);
        
    polyTriangle[0].clear();
    polyTriangle[0].append(QPoint(0, 0));
    polyTriangle[0].append(QPoint(w, 0));
    polyTriangle[0].append(endPointUp);
    
    QPoint endPointDown;
    endPointDown.setX(centerX);
    if (turnOn)
        endPointDown.setY(centerY + tubeWidth*0.5);
    else
        endPointDown.setY(centerY);
    polyTriangle[1].clear();
    polyTriangle[1].append(QPoint(0, h));
    polyTriangle[1].append(QPoint(w, h));
    polyTriangle[1].append(endPointDown);
}

void schTubeLocker::setState(bool on)
{
    setState_man(on);
    emit changedState(turnOn);
}

void schTubeLocker::setState_man(bool on)
{
    turnOn = on;
    recalcPoints();
    update();
}

bool schTubeLocker::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();
    
    if (e->type() == QEvent::Leave)
    {
        hovered = false;
        update();
    }
    else if (e->type() == QEvent::MouseMove)
    {
        auto me = dynamic_cast<QMouseEvent *>(e);
        hovered = true;
        
        update();
    }
    if (e->type() == QEvent::MouseButtonRelease && hovered)
    {
        setState(!turnOn);
    }
    
    return QWidget::event(e);
}

void schTubeLocker::paintEvent(QPaintEvent *)
{
    QPainter painter(this);
    
    initPositions(width(), height());
    
    painter.setBrush(QBrush(Qt::black));
    painter.setPen(QColor(Qt::black));
    
    //Tube
    painter.setRenderHint(QPainter::Antialiasing, false);
    painter.drawLine(0, centerY - tubeWidth*0.5, width(), centerY - tubeWidth*0.5);
    painter.drawLine(0, centerY + tubeWidth*0.5, width(), centerY + tubeWidth*0.5);
    
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(QColor(outterLineColor));
    if (turnOn)
        painter.setBrush(QBrush(onColor));
    else
        painter.setBrush(QBrush(offColor));
    
    //triangles
    painter.drawPolygon(polyTriangle[0]);
    painter.drawPolygon(polyTriangle[1]);
    
    //Hovered
    painter.setOpacity(0.5);
    painter.setBrush(QBrush(Qt::blue));
    if (hovered)
    {
        painter.drawPolygon(polyTriangle[0]);
        painter.drawPolygon(polyTriangle[1]);
    }
}
