#include "rectslsensitivityadditwindow.h"

rectSLSensitivityAdditWindow::rectSLSensitivityAdditWindow(RectSlider *parent)
{
    parentSlider = parent;
    parentSlider->doubleClickInRect = true;
    this->setParent(parentSlider);
    this->setVisible(false);
    connect(parentSlider, SIGNAL(doubleClickedInRect()), this, SLOT(rectDblClicked()));
    connect(this, SIGNAL(doubleClicked()), this, SLOT(rectDblClicked()));

    this->installEventFilter(this);

    initVariables();
    initLayout();
}

void rectSLSensitivityAdditWindow::initVariables()
{
    labFBP = new QLabel(tr("P:"));
    labFBI = new QLabel(tr("I:"));
    labZ = new QLabel(tr("Z:"));

    labFBP->installEventFilter(this);
    labFBI->installEventFilter(this);
    labZ->installEventFilter(this);

    edFBP = new Edit();
    edFBI = new Edit();
    cbZ = new QComboBox(this);

    cbZ->addItem("Head");
    cbZ->addItem("Stage");
}

void rectSLSensitivityAdditWindow::initLayout()
{
    QHBoxLayout *hlayMain = new QHBoxLayout(this);
    QVBoxLayout **vlayRow = new QVBoxLayout*[2];

    vlayRow[0] = new QVBoxLayout();
    vlayRow[1] = new QVBoxLayout();

    hlayMain->addLayout(vlayRow[0]);
    hlayMain->addLayout(vlayRow[1]);

    vlayRow[0]->addWidget(labFBP);
    vlayRow[1]->addWidget(edFBP);

    vlayRow[0]->addWidget(labFBI);
    vlayRow[1]->addWidget(edFBI);

    vlayRow[0]->addWidget(labZ);
    vlayRow[1]->addWidget(cbZ);
}

void rectSLSensitivityAdditWindow::rectDblClicked()
{
    bool showAdditWindow = this->isHidden();
    this->setVisible(showAdditWindow);
    if (!showAdditWindow)
        SleepExport::msleep_gui(100); //for preventing DoubleClick affecting on "parentSlider" too
    parentSlider->blockFullyInterface = showAdditWindow;
    parentSlider->update();
}

void rectSLSensitivityAdditWindow::addConnects(DoubleChanger *fbP, DoubleChanger *fbI, QComboBox *z)
{
    edFBP->setMinimum(fbP->minimum());
    edFBP->setMaximum(fbP->maximum());
    edFBP->setDecimals(fbP->edit->decimals());
    edFBP->setSingleStep(pow(10., -fbP->edit->decimals()));
    edFBP->setValue(fbP->value());
    connect(fbP, SIGNAL(newValue(double)), edFBP, SLOT(setValue(double)));
    connect(edFBP, SIGNAL(newValue(double)), fbP, SLOT(setValue(double)));
    connect(edFBP, SIGNAL(newValue()), fbP, SLOT(setNewValue()));

    edFBI->setMinimum(fbI->minimum());
    edFBI->setMaximum(fbI->maximum());
    edFBI->setDecimals(fbI->edit->decimals());
    edFBI->setSingleStep(pow(10., -fbI->edit->decimals()));
    edFBI->setValue(fbI->value());
    connect(fbI, SIGNAL(newValue(double)), edFBI, SLOT(setValue(double)));
    connect(edFBI, SIGNAL(newValue(double)), fbI, SLOT(setValue(double)));
    connect(edFBI, SIGNAL(newValue()), fbI, SLOT(setNewValue()));

    cbZ->setCurrentIndex(z->currentIndex());
    connect(cbZ, SIGNAL(currentIndexChanged(int)), z, SLOT(setCurrentIndex(int)));
    connect(z, SIGNAL(currentIndexChanged(int)), this, SLOT(setSielentComboBox(int)));
}

void rectSLSensitivityAdditWindow::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    painter.setRenderHint(QPainter::Antialiasing, true);

    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(Qt::white));
    painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
}

bool rectSLSensitivityAdditWindow::eventFilter(QObject *object, QEvent *event)
{
    if (event->type() == QEvent::MouseButtonDblClick)
    {
        emit doubleClicked();
        return true;
    }

    return QWidget::eventFilter(object, event);
}

void rectSLSensitivityAdditWindow::setSielentComboBox(int index)
{
    cbZ->blockSignals(true);
    cbZ->setCurrentIndex(index);
    cbZ->blockSignals(false);
}
