#include "rectslspeedadditwindow.h"

rectSLSpeedAdditWindow::rectSLSpeedAdditWindow(RectSlider *parent)
{
    parentSlider = parent;
    parentSlider->doubleClickInRect = true;
    this->setParent(parentSlider);
    this->setVisible(false);
    connect(parentSlider, SIGNAL(doubleClickedInRect()), this, SLOT(rectDblClicked()));
    connect(this, SIGNAL(doubleClicked()), this, SLOT(rectDblClicked()));

    this->installEventFilter(this);

    initVariables();
    initLayout();
}

void rectSLSpeedAdditWindow::initVariables()
{
    labSpeed = new QLabel(tr("Spd:") + "\n(" + tr("ln/sec") + ")");

    labSpeed->installEventFilter(this);

    edSpeed = new Edit();
}

void rectSLSpeedAdditWindow::initLayout()
{
    QHBoxLayout *hlayMain = new QHBoxLayout(this);
    QVBoxLayout **vlayRow = new QVBoxLayout*[2];

    vlayRow[0] = new QVBoxLayout();
    vlayRow[1] = new QVBoxLayout();

    hlayMain->addLayout(vlayRow[0]);
    hlayMain->addLayout(vlayRow[1]);

    vlayRow[0]->addWidget(labSpeed);
    vlayRow[1]->addWidget(edSpeed);
}

void rectSLSpeedAdditWindow::rectDblClicked()
{
    bool showAdditWindow = this->isHidden();
    this->setVisible(showAdditWindow);
    if (!showAdditWindow)
        SleepExport::msleep_gui(100); //for preventing DoubleClick affecting on "parentSlider" too
    parentSlider->blockFullyInterface = showAdditWindow;
    parentSlider->update();
}

void rectSLSpeedAdditWindow::addConnects()
{
    edSpeed->setMinimum(parentSlider->minValue);
    edSpeed->setMaximum(parentSlider->maxValue);
    edSpeed->setDecimals(parentSlider->numbersAfterPoint);
    edSpeed->setSingleStep(pow(10., -parentSlider->numbersAfterPoint));
    edSpeed->setValue(parentSlider->Value());
    connect(parentSlider, SIGNAL(valueChanged(double)), edSpeed, SLOT(setValue(double)));
    connect(edSpeed, SIGNAL(newValue(double)), parentSlider, SLOT(setValue(double)));
}

void rectSLSpeedAdditWindow::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    painter.setRenderHint(QPainter::Antialiasing, true);

    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(Qt::white));
    painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
}

bool rectSLSpeedAdditWindow::eventFilter(QObject *object, QEvent *event)
{
    if (event->type() == QEvent::MouseButtonDblClick)
    {
        emit doubleClicked();
        return true;
    }

    return QWidget::eventFilter(object, event);
}
