#include "rectsltipdriveadditwindow.h"

rectSLTipDriveAdditWindow::rectSLTipDriveAdditWindow(RectSlider *parent)
{
    parentSlider = parent;
    parentSlider->doubleClickInRect = true;
    this->setParent(parentSlider);
    this->setVisible(false);
    connect(parentSlider, SIGNAL(doubleClickedInRect()), this, SLOT(rectDblClicked()));
    connect(this, SIGNAL(doubleClicked()), this, SLOT(rectDblClicked()));

    this->installEventFilter(this);

    initVariables();
    initLayout();
}

void rectSLTipDriveAdditWindow::initVariables()
{
    labAmplitude = new QLabel(tr("Amp:") + "\n(" + tr("V") + ")");

    labAmplitude->installEventFilter(this);

    edAmplitude = new Edit();
}

void rectSLTipDriveAdditWindow::initLayout()
{
    QHBoxLayout *hlayMain = new QHBoxLayout(this);
    QVBoxLayout **vlayRow = new QVBoxLayout*[2];

    vlayRow[0] = new QVBoxLayout();
    vlayRow[1] = new QVBoxLayout();

    hlayMain->addLayout(vlayRow[0]);
    hlayMain->addLayout(vlayRow[1]);

    vlayRow[0]->addWidget(labAmplitude);
    vlayRow[1]->addWidget(edAmplitude);
}

void rectSLTipDriveAdditWindow::rectDblClicked()
{
    bool showAdditWindow = this->isHidden();
    this->setVisible(showAdditWindow);
    if (!showAdditWindow)
        SleepExport::msleep_gui(100); //for preventing DoubleClick affecting on "parentSlider" too
    parentSlider->blockFullyInterface = showAdditWindow;
    parentSlider->update();
}

void rectSLTipDriveAdditWindow::addConnects(DoubleChanger *amplitude)
{
    edAmplitude->setMinimum(amplitude->minimum());
    edAmplitude->setMaximum(amplitude->maximum());
    edAmplitude->setDecimals(amplitude->edit->decimals());
    edAmplitude->setSingleStep(pow(10., -amplitude->edit->decimals()));
    edAmplitude->setValue(amplitude->value());
    connect(amplitude, SIGNAL(newValue(double)), edAmplitude, SLOT(setValue(double)));
    connect(edAmplitude, SIGNAL(newValue(double)), amplitude, SLOT(setValue(double)));
    connect(edAmplitude, SIGNAL(newValue()), amplitude, SLOT(setNewValue()));
}

void rectSLTipDriveAdditWindow::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    painter.setRenderHint(QPainter::Antialiasing, true);

    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(Qt::white));
    painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
}

bool rectSLTipDriveAdditWindow::eventFilter(QObject *object, QEvent *event)
{
    if (event->type() == QEvent::MouseButtonDblClick)
    {
        emit doubleClicked();
        return true;
    }

    return QWidget::eventFilter(object, event);
}
