#include "rectsltippushadditwindow.h"

rectSLTipPushAdditWindow::rectSLTipPushAdditWindow(RectSlider *parent)
{
    parentSlider = parent;
    parentSlider->doubleClickInRect = true;
    this->setParent(parentSlider);
    this->setVisible(false);
    connect(parentSlider, SIGNAL(doubleClickedInRect()), this, SLOT(rectDblClicked()));
    connect(this, SIGNAL(doubleClicked()), this, SLOT(rectDblClicked()));

    this->installEventFilter(this);

    initVariables();
    initLayout();
}

void rectSLTipPushAdditWindow::initVariables()
{
    labSetPoint = new QLabel(tr("SP:") + "\n(" + tr("V") + ")");

    labSetPoint->installEventFilter(this);

    edSetPoint = new Edit();
}

void rectSLTipPushAdditWindow::initLayout()
{
    QHBoxLayout *hlayMain = new QHBoxLayout(this);
    QVBoxLayout **vlayRow = new QVBoxLayout*[2];

    vlayRow[0] = new QVBoxLayout();
    vlayRow[1] = new QVBoxLayout();

    hlayMain->addLayout(vlayRow[0]);
    hlayMain->addLayout(vlayRow[1]);

    vlayRow[0]->addWidget(labSetPoint);
    vlayRow[1]->addWidget(edSetPoint);
}

void rectSLTipPushAdditWindow::rectDblClicked()
{
    bool showAdditWindow = this->isHidden();
    this->setVisible(showAdditWindow);
    if (!showAdditWindow)
        SleepExport::msleep_gui(100); //for preventing DoubleClick affecting on "parentSlider" too
    parentSlider->blockFullyInterface = showAdditWindow;
    parentSlider->update();
}

void rectSLTipPushAdditWindow::addConnects(DoubleChanger *SP)
{
    edSetPoint->setMinimum(SP->minimum());
    edSetPoint->setMaximum(SP->maximum());
    edSetPoint->setDecimals(SP->edit->decimals());
    edSetPoint->setSingleStep(pow(10., -SP->edit->decimals()));
    edSetPoint->setValue(SP->value());
    connect(SP, SIGNAL(newValue(double)), edSetPoint, SLOT(setValue(double)));
    connect(edSetPoint, SIGNAL(newValue(double)), SP, SLOT(setValue(double)));
    connect(edSetPoint, SIGNAL(newValue()), SP, SLOT(setNewValue()));
    connect(edSetPoint, SIGNAL(editingFinished()), SP, SIGNAL(editingFinished()));
}

void rectSLTipPushAdditWindow::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    painter.setRenderHint(QPainter::Antialiasing, true);

    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(Qt::white));
    painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
}

bool rectSLTipPushAdditWindow::eventFilter(QObject *object, QEvent *event)
{
    if (event->type() == QEvent::MouseButtonDblClick)
    {
        emit doubleClicked();
        return true;
    }

    return QWidget::eventFilter(object, event);
}
