#include "samplecontrol.h"
#include "scheme/abstractinoutlines.h"
#include <QPaintDevice>
#include <QPainter>
#include "options/relayoptions.h"
#include "tools.hpp"
#include <QDebug>

SampleControl::SampleControl(QWidget *parent)
{
    mainWidget = this;
    mainWidget->setMaximumSize(503, 407);
    
    schPlatform = new schSampleControlPlatform(mainWidget);
    schPlatform->sampleColor = QColor(0, 178, 255);
    schPlatform->tableColor = schPlatform->backgroundColor;
    schPlatform->boxesColor = QColor(229, 230, 255);
    schTubeLockerVacuum = new schTubeLocker(mainWidget);
    schTubeLockerAir = new schTubeLocker(mainWidget);
    schMotorVacuum = new schActiveBox(mainWidget);
    schMotorAir = new schActiveBox(mainWidget);
    schMotorAir->isBoxEnabled = false;
    schFloor = new schActiveBox(mainWidget);
    schFloor->isBoxEnabled = false;
    schFloor->disabledColor = schFloor->backgroundColor;
    
    schTubeLockerVacuum->offColor = schTubeLockerAir->offColor =
        schMotorVacuum->offColor = flyingColor = QColor(244, 165, 165);
        
    schTubeLockerVacuum->onColor = schTubeLockerAir->onColor =
        schMotorVacuum->onColor = QColor(112, 192, 62, 170);
    
    schPlatform->sampleClosingColor = QColor(112, 192, 62);
    
    QColor outerColor = QColor(0, 171, 255);
    schPlatform->outterLineColor = outerColor;
    schTubeLockerVacuum->outterLineColor = outerColor;
    schTubeLockerAir->outterLineColor = outerColor;
    schMotorVacuum->outterLineColor = outerColor;
    schMotorAir->outterLineColor = outerColor;
    schFloor->outterLineColor = outerColor;
    
    schMotorAir->pixmapImage = schMotorVacuum->pixmapImage = new QPixmap(":/ScanPlot/engineBlack.png");
    
    elements << schPlatform << schTubeLockerVacuum << schTubeLockerAir
             << schMotorVacuum << schMotorAir << schFloor;
    
    motorAirLabel = new QLabel(tr("Move pump"), mainWidget);
    motorVacuumLabel = new QLabel(tr("Fix pump"), mainWidget);
    
    connectElements();
    rescale();
    setConnects();
}

void SampleControl::setConnects()
{
    connect(schTubeLockerVacuum, SIGNAL(changedState(bool)), this, SLOT(onTubeLockerVacuumChanged(bool)));
    connect(schTubeLockerAir, SIGNAL(changedState(bool)), this, SLOT(onTubeLockerAirChanged(bool)));
    connect(schMotorVacuum, SIGNAL(changedState(bool)), this, SLOT(onMotorVacuumChanged(bool)));
}

void SampleControl::rescale()
{
    const double recW = 400;
    const double recH = 400;
    
    const int tubeLockerWidth = 25;
    
    int w = width();
    int h = height();
    float scaleWidth = w/recW;
    float scaleHeight = h/recH;
    
    int widthBoundOffset = 10*scaleWidth;
    int MotorsSize = 80*scaleHeight;
    schPlatform->setFixedSize(200*scaleWidth, 200*scaleHeight);
    schTubeLockerVacuum->setFixedSize(tubeLockerWidth*scaleWidth, 60*scaleHeight);
    schMotorVacuum->setFixedSize(MotorsSize, MotorsSize);
    schTubeLockerAir->setFixedSize(tubeLockerWidth*scaleWidth, 60*scaleHeight);
    schMotorAir->setFixedSize(MotorsSize, MotorsSize);
    schFloor->setFixedSize(schPlatform->width(), 15*scaleHeight);
    tubeWidth = 10*scaleHeight;
    int floorOffsetY = 20;
    
    schPlatform->tubeWidth = tubeWidth;
    schPlatform->setPosXY(widthBoundOffset,
                          h*0.5 - schPlatform->height()*0.5 - (floorOffsetY + schFloor->height())*0.5);
    int xEndPlatform = schPlatform->absPos->x() + schPlatform->width();
    
    schFloor->setPosXY(schPlatform->absPos->x(),
                       schPlatform->absPos->y() + schPlatform->height() + floorOffsetY);
    
    int xMotors = w - widthBoundOffset - MotorsSize;
    int distBetweenPlatformMotor = xMotors - xEndPlatform;
    int tubeLockerOffsetX = distBetweenPlatformMotor*0.1;
    
    schTubeLockerVacuum->tubeWidth = tubeWidth;
    schTubeLockerVacuum->setPosXY(xEndPlatform + distBetweenPlatformMotor*0.5 - schTubeLockerVacuum->width()*0.5 - tubeLockerOffsetX,
                                  schPlatform->absPos->y() + schPlatform->ptsOut[0]->y() - schTubeLockerVacuum->height()*0.5);
    
    schMotorVacuum->ptsIn[0]->setY(schMotorVacuum->height()*0.5);
    schMotorVacuum->setPosXY(xMotors,
                             schTubeLockerVacuum->absPos->y() + schTubeLockerVacuum->ptsOut[0]->y() - schMotorVacuum->ptsIn[0]->y());
    
    schTubeLockerAir->tubeWidth = tubeWidth;
    schTubeLockerAir->setPosXY(xEndPlatform + distBetweenPlatformMotor*0.5 - schTubeLockerAir->width()*0.5 - tubeLockerOffsetX,
                               schPlatform->absPos->y() + schPlatform->ptsOut[1]->y() - schTubeLockerAir->height()*0.5);
    
    schMotorAir->ptsIn[0]->setY(schMotorAir->height()*0.5);
    schMotorAir->setPosXY(w - widthBoundOffset - schMotorAir->width(),
                          schTubeLockerAir->absPos->y() + schTubeLockerAir->ptsOut[0]->y() - schMotorAir->ptsIn[0]->y());
    
    schPlatform->move(*schPlatform->absPos);
    schFloor->move(*schFloor->absPos);
    schTubeLockerVacuum->move(*schTubeLockerVacuum->absPos);
    schMotorVacuum->move(*schMotorVacuum->absPos);
    schTubeLockerAir->move(*schTubeLockerAir->absPos);
    schMotorAir->move(*schMotorAir->absPos);
    
    motorVacuumLabel->move(schMotorVacuum->absPos->x(),
                        schMotorVacuum->absPos->y() - motorVacuumLabel->fontMetrics().height());
    motorAirLabel->move(schMotorAir->absPos->x(),
                        schMotorAir->absPos->y() - motorAirLabel->fontMetrics().height());
}

void SampleControl::setCorners()
{
    
}

void SampleControl::connectElements()
{
    schPlatform->connectAbs(0, schTubeLockerVacuum, 0);
    schPlatform->connectAbs(1, schTubeLockerAir, 0);
    
    schTubeLockerVacuum->connectAbs(0, schMotorVacuum, 0);
    schTubeLockerAir->connectAbs(0, schMotorAir, 0);
}

bool SampleControl::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        rescale();
    
    return QWidget::event(e);
}

void SampleControl::paintEvent(QPaintEvent *e)
{
    QPainter painter(mainWidget);
    //painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(QColor(Qt::black));
    
    for (auto &el : elements)
    {
        QPoint pos = *el->absPos;
        for (uint b=0; b<el->countOut; b++)
        {
            if (el->absOut[b] == nullptr)
                continue;
            
            QPoint prevPointUp = *el->ptsOut[b] + pos;
            prevPointUp.setY(prevPointUp.y() - tubeWidth*0.5);
            QPoint prevPointDown = *el->ptsOut[b] + pos;
            prevPointDown.setY(prevPointDown.y() + tubeWidth*0.5);
            for (auto point : *el->cornersOut[b])
            {
                painter.drawLine(prevPointUp, *point);
                prevPointUp = *point;
                painter.drawLine(prevPointDown, *point);
                prevPointDown = *point;
            }
            
            auto pos2 = *el->absOut[b]->absPos;
            auto pointUp = *el->absOut[b]->ptsIn[el->connectOutId[b]] + pos2;
            pointUp.setY(pointUp.y() - tubeWidth*0.5);
            painter.drawLine(prevPointUp, pointUp);
            auto pointDown = *el->absOut[b]->ptsIn[el->connectOutId[b]] + pos2;
            pointDown.setY(pointDown.y() + tubeWidth*0.5);
            painter.drawLine(prevPointDown, pointDown);
        }
    }
    
    int arrowWidth = 8;
    int arrowHeight = 4;
    int arrowOffset = 3;
    int distArrows = 30;
    
    painter.setRenderHint(QPainter::Antialiasing, true);
    if (schTubeLockerAir->turnOn)
    {
        painter.setPen(QColor(flyingColor));
        
        int x = schPlatform->absPos->x() + arrowWidth;
        int y = schFloor->absPos->y() - arrowOffset;
        int length = y - (schPlatform->absPos->y() + schPlatform->height()) - arrowOffset;
        int maxWidth = schPlatform->width();
        while (x < maxWidth + 7)
        {
            schFloor->drawArrow(&painter, x, y, arrowWidth, arrowHeight, length, ed_down, true);
            x += distArrows;
        }
    }
    
    QWidget::paintEvent(e);
}

void SampleControl::onTubeLockerVacuumChanged(bool on)
{
    auto relay = RelayOptions::GetIt();
    relay->SetRelePower(on, scr_tubeVacuum);
    
    schPlatform->isSampleClosingToTable = schTubeLockerVacuum->turnOn && schMotorVacuum->turnOn;
    schPlatform->update();
}

void SampleControl::onTubeLockerAirChanged(bool on)
{
    auto relay = RelayOptions::GetIt();
    relay->SetRelePower(on, scr_tubeAir);
    
    mainWidget->update();
    this->update();
}

void SampleControl::onMotorVacuumChanged(bool on)
{
    auto relay = RelayOptions::GetIt();
    relay->SetRelePower(on, scr_motorVacuum);
    
    schPlatform->isSampleClosingToTable = schTubeLockerVacuum->turnOn && schMotorVacuum->turnOn;
    schPlatform->update();
}

void SampleControl::changeStateAll(bool on)
{
    schTubeLockerVacuum->setState(on);
    schTubeLockerAir->setState(on);
    schMotorVacuum->setState(on);
    mainWidget->update();
    this->update();
}

void SampleControl::updateReles()
{
    schTubeLockerVacuum->setState_man(false);
    schTubeLockerAir->setState_man(false);
    schMotorVacuum->setState_man(false);
    schPlatform->isSampleClosingToTable = schTubeLockerVacuum->turnOn && schMotorVacuum->turnOn;
    schPlatform->update();
    mainWidget->update();
    this->update();
}
