#ifndef SAMPLECONTROL_H
#define SAMPLECONTROL_H

#include <QObject>
#include <QWidget>
#include "scheme/schsamplecontrolplatform.h"
#include "scheme/schtubelocker.h"
#include "scheme/schactivebox.h"
#include <QLabel>

class SampleControl : public QWidget
{
    Q_OBJECT
    
public:
    SampleControl(QWidget *parent = nullptr);
    
public slots:
    void changeStateAll(bool on);
    
protected:
    void paintEvent(QPaintEvent *event) override;
    bool event(QEvent *) override;
    
protected slots:
    void updateReles();
    
private slots:
    void onTubeLockerVacuumChanged(bool on);
    void onTubeLockerAirChanged(bool on);
    void onMotorVacuumChanged(bool on);
    
protected:
    QWidget *mainWidget;
    
private:
    enum SampleControlRoles
    {
        scr_tubeVacuum = 2,
        scr_motorVacuum = 3,
        scr_tubeAir = 4,
    };
    
    void rescale();
    void setCorners();
    void connectElements();
    void setConnects();
    
    schSampleControlPlatform *schPlatform;
    schTubeLocker *schTubeLockerVacuum;
    schTubeLocker *schTubeLockerAir;
    schActiveBox *schMotorVacuum;
    schActiveBox *schMotorAir;
    schActiveBox *schFloor;
    int tubeWidth = 5;
    QVector<AbstractInOutLines *> elements;
    QColor flyingColor;
    
    QLabel *motorVacuumLabel;
    QLabel *motorAirLabel;
};

#endif // SAMPLECONTROL_H


