#include "EG3000_Program.h"

#include <QFile>
#include <QIODevice>
#include <QDebug>
#include "EG3000.h"
#include <i_eg3000.h>

bool validateHex(QString filename)
{
    QFile file(filename);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "validateHex can't open file";
        return false;
    }
    
    bool end = false;
    int c = 0;
    while (!file.atEnd()) {
        c++;
        QByteArray line = file.readLine();
        line.chop(1); // windows, "\cr\lf\", \cr remove
        
        if (line.size() < 11) {
            qDebug() << "validateHex short line" << c;
            file.close();
            return false;
        }
        if (line.at(0) != ':') {
            qDebug() << "validateHex no ':'" << c;
            file.close();
            return false;
        }
        if ((line.size() - 11) != 2*((QString(line.at(1)).toInt(nullptr, 16) << 4) + QString(line.at(2)).toInt(nullptr, 16))) {
            qDebug() << "validateHex wrong data len" << c;
            file.close();
            return false;
        }
        if (line.at(7) != '0') {
            qDebug() << "validateHex wrong record type1" << c;
            file.close();
            return false;
        }
        if (line.at(8) != '0' && line.at(8) != '1' && line.at(8) != '2' &&
            line.at(8) != '3' && line.at(8) != '4' && line.at(8) != '5') {
            qDebug() << "validateHex wrong record type2" << c;
            file.close();
            return false;
        }
        if (line.at(8) != '0' &&
            (line.at(3) != '0' ||
             line.at(4) != '0' ||
             line.at(5) != '0' ||
             line.at(6) != '0')) {
            qDebug() << "validateHex address not 0" << c;
            file.close();
            return false;
        }
        
        if (line.at(8) == '1')
            end = true;
        if (end && !file.atEnd()) {
            qDebug() << "validateHex end record not last" << c;
            file.close();
            return false;
        }
        
        unsigned long checksum = 0;
        for (int i = 0; i < (line.size() - 1)/2; i++) {
            checksum += (QString(line.at(2*i+1)).toInt(nullptr, 16) << 4) + QString(line.at(2*i+2)).toInt(nullptr, 16);
        }
        if ((0xFF & checksum) != 0) {
            qDebug() << "validateHex wrong checksum" << c;
            file.close();
            return false;
        }
    }
    
    if (!end)
        qDebug() << "validateHex no eof record";
    
    return end;
}

unsigned long bufferHex(QString filename, unsigned char *buffer, unsigned int max_size)
{
    QFile file(filename);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "bufferHex can't open file";
        return 0;
    }
    
    memset(buffer, 0xFF, max_size);
    
    unsigned long max_addr = 0;
    unsigned long high_addr = 0;
    while (!file.atEnd()) {
        QByteArray line = file.readLine();
        line.chop(1); // windows, "\cr\lf\", \cr remove
        
        if (line.at(8) == '0') {
            if (high_addr < 0x1D00 || high_addr > 0x1D08)
                continue;
            
            unsigned long size = (QString(line.at(1)).toInt(nullptr, 16) << 4) + QString(line.at(2)).toInt(nullptr, 16);
            unsigned long addr = (((QString(line.at(3)).toInt(nullptr, 16) << 4) + QString(line.at(4)).toInt(nullptr, 16)) << 8) +
                                 (QString(line.at(5)).toInt(nullptr, 16) << 4) + QString(line.at(6)).toInt(nullptr, 16);
            addr += (high_addr - 0x1D00) << 16;
            
            for (unsigned int i = 0; i < size && addr + i < max_size; i++) {
                buffer[addr + i] = (QString(line.at(i*2+9)).toInt(nullptr, 16) << 4) + QString(line.at(i*2+10)).toInt(nullptr, 16);
            }
            
            if (addr + size >= max_size)
                qDebug() << "bufferHex overflow!" << addr + size << max_size;
            
            if (addr + size > max_addr)
                max_addr = addr + size;
        } else if (line.at(8) == '1') {
            break;
        } else if (line.at(8) == '4') {
            high_addr = (((QString(line.at(9)).toInt(nullptr, 16) << 4) + QString(line.at(10)).toInt(nullptr, 16)) << 8) +
                        (QString(line.at(11)).toInt(nullptr, 16) << 4) + QString(line.at(12)).toInt(nullptr, 16);
        }
    }
    
    return max_addr;
}

bool program_hex(QString filename, struct eg3000_read_handle *read_handle, void *devHandle)
{
    if (!validateHex(filename))
        return false;
    
    unsigned char *buf = new unsigned char[520000];
    unsigned size = 4 + ((~3) & bufferHex(filename, buf, 520000));
    
    bool ok = RET_TRUE == eg3000_bl_program(devHandle, buf, size, read_handle, 0);
    eg3000_bl_stats(devHandle);
    delete[] buf;
    
    return ok;
}

inline BYTE invert_byte(BYTE b)
{ //from SO sth
    b = (b & 0xF0) >> 4 | (b & 0x0F) << 4; //swap every 4 bits
    b = (b & 0xCC) >> 2 | (b & 0x33) << 2; //swap every 2 bits
    b = (b & 0xAA) >> 1 | (b & 0x55) << 1; //swap every 1 bits
    return b;
}

bool program_bin(QString filename, struct eg3000_read_handle *read_handle, void *devHandle)
{
    QFile bin(filename);
    bin.open(QIODevice::ReadOnly);
    QByteArray buf = bin.readAll();
    bin.close();
    
    if (filename.endsWith(".rpd")) //flash lsb first
        for (int i = 0; i < buf.size(); i++)
            buf[i] = invert_byte(buf.at(i));
    
    bool ok = RET_TRUE == eg3000_bl_program(devHandle, (BYTE *)buf.data(), buf.size(), read_handle, 1);
    //need?? eg3000_bl_stats(devHandle);
    return ok;
}
