#include "ccdplotmodule.h"

#include "scanplot.h"
#include "gui/nstslider.h"
#include "IExternalScan.h"
#include "widgets/twodoverlaybuttons.h"
#include "2d/ovcontrol_2d.h"
#include "modules/datamodule.h"
#include "modules/scanmodule.h"
#include "modules/oscilloscopemodule.h"
#include "modules/motorsmodule.h"
#include "dacscontrol.h"

void CCDPlotModule::connected(const QLatin1String name, I_MODULE *module)
{
    if (name == DataModule::name()) {
        DataPlot *m = (DataPlot *)module->basecast();
        
        connect(this, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)),
                m->Container, SLOT(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)));
        connect(this, SIGNAL(AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage)),
                m->Container, SLOT(AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage)));
    }
    if (name == ScanModule::name()) {
        ScanPlot *m = (ScanPlot *)module->basecast();
        
        IExternalScan *externalScan = getNewCCDIExtScan(this);
        m->addExternalScan(MainWindow::tr("CCD"), externalScan);
        if (auto data = DataPlot::GetIt())
            connect(externalScan, SIGNAL(AddDataToContainer(ccd::TSetOfSpectrum*,plot::TMetaStorage)),
                    data->Container, SLOT(AddDataToContainer(ccd::TSetOfSpectrum*,plot::TMetaStorage)));
        
        connect(hbdMotorZFocus->hbYTop, SIGNAL(holdingPressed(bool)), m, SLOT(motorZFocusUp(bool)));
        connect(hbdMotorZFocus->hbYBottom, SIGNAL(holdingPressed(bool)), m, SLOT(motorZFocusDown(bool)));
    }
    if (name == OscilloscopeModule::name()) {
        Oscilloscope *m = (Oscilloscope *)module->basecast();
        
        linkDoubleChangers(m->DacsWidget->Zobj, zPos);
        connect(m->DacsWidget->Zobj->slider, SIGNAL(valueChanged(double)), zPos, SLOT(setValue(double))); //sync value change on setValue
        QObject::connect(zPos, SIGNAL(newValue(double)), this, SLOT(focusChanged(double)));
        
        DACAUX1_changed(m->DacsWidget->Extra_Dac_3->value());
        DACAUX2_changed(m->DacsWidget->Extra_Dac_4->value());
        connect(m->DacsWidget->Extra_Dac_3, SIGNAL(newValue(double)), this, SLOT(DACAUX1_changed(double)));
        connect(m->DacsWidget->Extra_Dac_4, SIGNAL(newValue(double)), this, SLOT(DACAUX2_changed(double)));
        
        connect(m, SIGNAL(oscRefreshed()), this, SLOT(osc_refresh()));
    }
    if (name == MotorsModule::name()) {
        Motors *m = (Motors *)module->basecast();
        
        connect(m->ars3000, SIGNAL(motorStopped(int, Motors::MotorRoles)), this, SLOT(ars3000MotorStopped(int, Motors::MotorRoles)));
        connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), this, SLOT(motorEvent(int, bool, bool)));
        connect(m->ars3000, SIGNAL(motorsUpdated()), this, SLOT(ARS3000MotorsUpdated()));
        //ars3000ConfUnitButtons.first()->change_state_man_with_sig(true);
        updateARS3000Motors(m);
        connect(m->ars3000, SIGNAL(confLineChanged(Motors::MotorRoles, int)), this, SLOT(onARS3000ConfLineChanged(Motors::MotorRoles, int)));
    }
}

void CCDPlotModule::initialSend(MobileInstance *client)
{
    client->serv->setPropIdVarS(client->socket, DataID::spectrSetRelativeScales, currentRelativeScale);
    client->serv->setPropIdVarS(client->socket, DataID::spectrStartOnOff, m_BStart->isOn);
    client->serv->setPropIdVarS(client->socket, DataID::spectrSetInfMode, One_Cont->isOn);
    client->serv->setPropIdVarS(client->socket, DataID::spectrSetFocusMax, zPos->maximum());
    client->serv->setPropIdVarS(client->socket, DataID::spectrSetFocus, zPos->value());
    client->serv->setPropIdVarS(client->socket, DataID::spectrSetCutSpikes, Cut_Pikes->isOn);
    client->serv->setPropIdVarS(client->socket, DataID::spectrSetBaselineEnable, Base_line->isOn);
    client->serv->setPropIdVarS(client->socket, DataID::PlotTitle2d_Spectr, MyPl->TwoDPlot->Pure2dPlot->Title);
    
    client->serv->setPropIdVarS(client->socket, DataID::setMonochrWaveLength, lineCentralWlMonoch->value());
    QList<QVariant> list;
    for (int i=0; i<CBgrating_select->count(); i++)
        list << CBgrating_select->itemText(i);
    client->serv->setPropIdVarS(client->socket, DataID::setMonochrGratingList, list);
    client->serv->setPropIdVarS(client->socket, DataID::setMonochrGrating, CBgrating_select->currentIndex());
    client->serv->setPropIdVarS(client->socket, DataID::setMonochrEntrance, lineSlitInMonoch->value());
    client->serv->setPropIdVarS(client->socket, DataID::setMonochrExisSlit, lineSlitOutMonoch->value());
    client->serv->setPropIdVarS(client->socket, DataID::setMonochrExitMirror, CBexitport->currentIndex());
    client->serv->setPropIdVarS(client->socket, DataID::setMonochrEnabled, monochromatorWidgets.first()->isEnabled());
    
    client->serv->setPropIdVarS(client->socket, DataID::setCCDExposure, lineExpTime->value());
    client->serv->setPropIdVarS(client->socket, DataID::setCCDTemperature, lineSetTemp->value());
}

void CCDPlotModule::setValue(int id, QVariant val, QObject *sender)
{
    if (id == DataID::needPlot2D_Spectr)
        if (sender)
        {
            sender->setProperty(QString("need_2dPlotSpectr").toLocal8Bit().constData(), val);
            Send2dPlotData();
        }
    
    if (id == DataID::spectrStartOnOff)
    {
        bool state = val.toBool();
        m_BStart->change_state_man_with_sig(state);
    }
    
    if (id == DataID::spectrSetInfMode)
    {
        bool state = val.toBool();
        One_Cont->change_state_man_with_sig(state);
    }
    
    if (id == DataID::spectrSetRelativeScales)
    {
        int scale = val.toInt();
        AGScales->actions()[scale]->setChecked(true);
        AGScales->actions()[scale]->trigger();
    }
    
    if (id == DataID::spectrSetFocus)
    {
        double focus = val.toDouble();
        zPos->setValue(focus);
        zPos->setNewValue(true);
    }
    
    if (id == DataID::spectrSetCutSpikes)
    {
        bool state = val.toBool();
        Cut_Pikes->change_state_man_with_sig(state);
    }
    
    if (id == DataID::spectrStoreBaseline)
    {
        base_line_save->click();
    }
    
    if (id == DataID::spectrSetBaselineEnable)
    {
        bool state = val.toBool();
        Base_line->change_state_man_with_sig(state);
    }
    if (id == DataID::plot2DSaveToDataSpectr)
        MyPl->TwoDPlot->overlayButtons->control->B2DSendToData->click();
    
    if (id == DataID::setMonochrWaveLength)
    {
        lineCentralWlMonoch->setValue(val.toDouble());
        lineCentralWlMonoch->setNewValue();
    }
    if (id == DataID::setMonochrGrating)
    {
        CBgrating_select->setCurrentIndex(val.toInt());
    }
    if (id == DataID::setMonochrEntrance)
    {
        lineSlitInMonoch->setValue(val.toDouble());
        lineSlitInMonoch->setNewValue();
    }
    if (id == DataID::setMonochrExisSlit)
    {
        lineSlitOutMonoch->setValue(val.toDouble());
        lineSlitOutMonoch->setNewValue();
    }
    if (id == DataID::setMonochrExitMirror)
    {
        CBexitport->setCurrentIndex(val.toInt());
    }
    if (id == DataID::setCCDTemperature)
    {
        lineSetTemp->setValue(val.toDouble());
        lineSetTemp->setNewValue();
    }
    if (id == DataID::setCCDExposure)
    {
        lineExpTime->setValue(val.toDouble());
        lineExpTime->setNewValue();
    }
}
