#ifndef EG3000MODULE_H
#define EG3000MODULE_H

#include "hardware/EG3000.h"
#include "modules.hpp"
#include <QSettings>
#include "gui/status.hpp"

class EG3000Module : public EG3000_CONTROL, public I_GUI_MODULE_BASE
{
public:
    EG3000Module()
    {
#ifdef USE_EG3K_EMULATOR
        QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
        bool useUsbEmulator = settings.value("usb", false).toBool();
        
        if (useUsbEmulator) {
            SWusb1->setEmulator(true);
            return;
        }
#endif
        
        start();
    }
    static const QLatin1String name() { return QLatin1String("eg3000"); }
    void *basecast(const QLatin1String name) override {
        if (name == QLatin1String("gui"))
            return static_cast<I_GUI_MODULE_BASE *>(this);
        return name.latin1() != nullptr ? nullptr : static_cast<EG3000_CONTROL *>(this); }
    void connected(const QLatin1String name, I_MODULE *module) override;
    void setRunning(bool on) override {
        if (on) {
            m_waitMainInit = false;
#ifdef USE_EG3K_EMULATOR
            QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
            bool useUsbEmulator = settings.value("usb", false).toBool();
            
            if (useUsbEmulator)
                emit usbStatus(EDok);
#endif
        } else
            thread_close();
    }
    
    QList<QWidget *> getStatusWidgets() override {
        QList<QWidget *> l;
        l << (QWidget *)nullptr << SWusb1;
        return l;
    }
};


#endif // EG3000MODULE_H
