#ifndef MODULESINTERFACE_H
#define MODULESINTERFACE_H

#include <QString>
#include <QList>

class I_MODULE
{
public:
    I_MODULE() { }
    virtual ~I_MODULE();
    
    static const QLatin1String name() { return QLatin1String("noname"); }
    static const QList<QLatin1String> requires() { return QList<QLatin1String>(); }
    void *basecast(const char *basename = nullptr) { return basecast(QLatin1String(basename)); }
    virtual void *basecast(const QLatin1String /*basename*/) { return nullptr; }
    
    //guidelines:
    //connect from user module to common service provider module
    //connect from user module to hook/notify module
    //be wary of possible order dependencies
    virtual void connected(const QLatin1String /*name*/, I_MODULE * /*module*/) { } //callback after new module loaded
    virtual void disconnected(const QLatin1String /*name*/, I_MODULE * /*module*/) { } //callback before old module unloaded
    virtual void setRunning(bool /*on*/) { } //callback on start/stop
};

class QWidget;
class QPushButton;
class I_GUI_MODULE_BASE : public I_MODULE
{
public:
    virtual QPushButton *detachButton() { return nullptr; }
    virtual QPushButton *createPanelButton() { return nullptr; } //new QPB(...
    virtual QList<QWidget *> getStatusWidgets() { return QList<QWidget *>(); } //<100 = wanted group index
};

#define GUI_MODULE_CTOR(classname) classname(QWidget *parent = nullptr) : I_GUI_MODULE(parent)
template <class T>
class I_GUI_MODULE : public T, public I_GUI_MODULE_BASE
{
public:
    //If there error in new module - u forgotted to add 
    //"QWidget *parent = nullptr" in the class constructor
    I_GUI_MODULE(QWidget *parent = nullptr) : T(parent) { }
    void *basecast(const QLatin1String name) override {
        if (name.latin1() == nullptr)
            return static_cast<T *>(this);
        if (name == QLatin1String("gui"))
            return static_cast<I_GUI_MODULE_BASE *>(this);
        return nullptr;
    }
};

#endif // MODULESINTERFACE_H
