#include "oscilloscopemodule.h"

#include "hardware/EG3000.h"
#include "dataplot.h"
#include "modules/datamodule.h"
#include "modules/eg3000module.h"
#include "modules/scanmodule.h"
#include "modules/optionsmodule.h"
#include "xyzcalibrations.hpp"
#include "motors.hpp"
#include "dacscontrol.h"
#include "scanplot.h"
#include "xyzcalibrations.hpp"
#include "autocalib.hpp"

void OscilloscopeModule::connected(const QLatin1String name, I_MODULE *module)
{
    if (name == ScanModule::name()) {
        ScanPlot *m = (ScanPlot *)module->basecast();
        
        QSettings settings2("Param/gui.conf", QSettings::IniFormat);
        if (settings2.value("gui/Ratis_xyz_stage_mode", false).toBool())
        {
            m->Plot->ThreeDPlots[0]->setParent(this);
            CalibWidget->threeDPlot = m->Plot->ThreeDPlots[0];
            CalibWidget->threeDPlot->secondParent = nullptr;
            CalibWidget->threeDPlot->singleInTabPage = true;
            CalibWidget->threeDPlot->SetTitle("");
            CalibWidget->threeDPlotGB = new QGroupBox(tr("Scan plot"));
            QHBoxLayout *lay = new QHBoxLayout(CalibWidget->threeDPlotGB);
            lay->addWidget(CalibWidget->threeDPlot, 1);
            CalibWidget->threeDPlotGB->setMinimumWidth(440);
            CalibWidget->vertCalibLay->addWidget(CalibWidget->threeDPlotGB, 1);
            
            disconnect(CalibWidget->threeDPlot, SIGNAL(doubleClicked()), m->Plot, SLOT(plotDoubleClicked()));
            
            m->pointSync->setParent(this);
            CalibWidget->pointSync = m->pointSync;
            auto gb = new QGroupBox(tr("Synchronization"));
            auto lay2 = new QHBoxLayout(gb);
            CalibWidget->stageLayout->addWidget(gb);
            lay2->addWidget(CalibWidget->pointSync);
            
            CalibWidget->eventFilter(nullptr, new QEvent(QEvent::Resize));
            CalibWidget->threeDPlot->Child->on_BRestoreScale_clicked();
        }
    }
    if (name == EG3000Module::name()) {
        EG3000_CONTROL *m = (EG3000_CONTROL *)module->basecast();
        
        m->rThread.readers.add(this);
        m->rThread.readers.add(acf);
        m->wThread.writers.add(this);
        m->wThread.writers.add(DacsWidget);
        
        connect(&m->wThread, SIGNAL(started()), acf, SLOT(reset()));
        connect(&m->rThread, SIGNAL(capBits(int,long)), acf, SLOT(capBits(int,long)));
        
        connect(m, SIGNAL(device_status(int)), this, SLOT(device_status(int)));
        
        //calib
        m->rThread.readers.add(CalibWidget);
        m->wThread.writers.add(CalibWidget);
        
        connect(&m->rThread, SIGNAL(capBits(int,long)), CalibWidget, SLOT(capBits(int,long)));
        
        connect(CalibWidget, SIGNAL(program_controller(QString)), m, SLOT(program(QString)));
    }
    if (name == DataModule::name()) {
        DataPlot *m = (DataPlot *)module->basecast();
        connect(this, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)),
                m->Container, SLOT(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)));
    }
    if (name == OptionsModule::name()) {
        options *m = (options *)module->basecast();
        
        m->addNewTab(mainWidget, tabName, 4);
        btnShowHideRightMenuBar->setParent(m);
        connect(m, SIGNAL(tabChanged(const QString)), this, SLOT(onSettingsTabChanged(const QString)));
        m->additionLayOverTabs->addWidget(btnShowHideRightMenuBar, 0, Qt::AlignRight | Qt::AlignTop);
    }
}

void OscilloscopeModule::setRunning(bool on)
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        scan->sl_ScanCorrection();
        scan->setScanner(0);
    }
    
    if (on) {
        for (int i = 0; i < 4; i++)
            CBChannel[i]->setCurrentIndex(
                qMax(0, CBChannel[i]->findData(
                            settings.value("state/osc/ch" + QString::number(i), 0))));
        
        setInitialStateOscilloscope();
        
        DacsWidget->DacsChannel->setCurrentIndex(
            qMax(0, DacsWidget->DacsChannel->findText(
                        settings.value("state/osc/ch_control").toString())));
        
        CalibWidget->LVMLoad();
        CalibWidget->read_calib_par(CalibWidget->defaultFileName);
    } else {
        for (int i = 0; i < 4; i++)
            settings.setValue("state/osc/ch" + QString::number(i), CBChannel[i]->itemData(CBChannel[i]->currentIndex()));
        
        settings.setValue("state/osc/ch_control", DacsWidget->DacsChannel->currentText());
    }
}
