#include "scanmodule.h"

#include <QCheckBox>
#include <QTimer>
#include <QDebug>
#include <ttabwidget.hpp>
#include "options_window.h"
#include <QHBoxLayout>
#include "widgets/threedoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"
#include "IExternalScan.h"
#include "2d/ovcontrol_2d.h"
#include "ars3000.h"
#include "hardware/EG3000.h"
#include "spmtunning.h"
#include "controls.h"
#include "oscilloscope.h"
#include "nstslider.h"
#include "autocalib.hpp"
#include "gui/single/samplecontrol.h"
#include "pumpsstage.h"
#include "ccd/ccdplot.h"
#include "modules/videomodule.h"
#include "modules/ccdplotmodule.h"
#include "modules/datamodule.h"
#include "modules/eg3000module.h"
#include "modules/spmmodule.h"
#include "modules/samplecontrolmodule.h"
#include "modules/motorsmodule.h"
#include "modules/oscilloscopemodule.h"
#include "xyzcalibrations.hpp"
#include "dacscontrol.h"

void ScanModule::connected(const QLatin1String name, I_MODULE *module)
{
    if (name == EG3000Module::name()) {
        EG3000_CONTROL *m = (EG3000_CONTROL *)module->basecast();
        
        m->rThread.readers.add(this);
        m->wThread.writers.add(this);
        
        connect(&m->rThread, SIGNAL(capBits(int,long)), this, SLOT(capBits(int,long)));
    }
    if (name == DataModule::name()) {
        DataPlot *m = (DataPlot *)module->basecast();
        
        connect(m->Container, SIGNAL(restoreAttributes(plot::TMetaStorage)),
                this, SLOT(restoreAttributes(plot::TMetaStorage)));
        
        //TODO cant connect externalScans here, race. need modules for extscanmodules
        //for now connect individually
        /*foreach (IExternalScan *sc, externalScans)
            if (sc->metaObject()->indexOfSignal("AddDataToContainer(ccd::TSetOfSpectrum*,plot::TMetaStorage)") != -1)
                connect(sc, SIGNAL(AddDataToContainer(ccd::TSetOfSpectrum*,plot::TMetaStorage)),
                        m->Container, SLOT(AddDataToContainer(ccd::TSetOfSpectrum*,plot::TMetaStorage)));*/
        
        connect(this, SIGNAL(AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage)),
                m->Container, SLOT(AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage)));
        connect(this, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)),
                m->Container, SLOT(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)));
        connect(m->Container, SIGNAL(onAutoSaveEnded()), this, SLOT(endPartScan()));
    }
    if (name == SPMModule::name()) {
        SPMTunning *m = (SPMTunning *)module->basecast();
        
#ifdef _OLD_EXTRA_CONTROL
        if (SPMTunning::getLandingMode() != lm01 && SPMTunning::getLandingMode() != lm05)
            BMotorLevel->show();
#endif
        
        connect(m->DCfbPrec, SIGNAL(newValue()), this, SLOT(check_speed()));
        connect(m->DCfbHeight, SIGNAL(newValue()), this, SLOT(check_speed()));
        connect(m->BFeedbackOnOffMode, SIGNAL(state_changed(bool)), this, SLOT(check_speed()));
        
        connect(PBSpectr_start_stop, SIGNAL(state_changed(bool)), m, SLOT(s_spectr_start(bool)));
        connect(ev_Speed_spectr, SIGNAL(newValue(double)), m->ev_Speed_spectr, SLOT(setValue(double)));
        connect(CB_Points_spectr, SIGNAL(currentIndexChanged(int)), m->CB_Points_spectr, SLOT(setCurrentIndex(int)));
        connect(m->ev_Z1_spectr, SIGNAL(newValue(double)), ev_Z1_spectr, SLOT(setValue(double)));
        connect(ev_Z1_spectr, SIGNAL(newValue(double)), m, SLOT(s_set_Z1_spectr(double)));
        //connect(ev_Z1_spectr, SIGNAL(newValue(double)), m->ev_Z1_spectr, SLOT(setValue(double)));
        connect(ev_Z1_spectr, SIGNAL(editingFinished()), m, SLOT(sl_up_lines_on()));
        connect(ev_Z1_spectr, SIGNAL(editingStarted()), m, SLOT(sl_up_lines_of()));
        connect(m->NS_Z2_spectr, SIGNAL(newValue(double)), ev_Z2_spectr, SLOT(setValue(double)));
        connect(ev_Z2_spectr, SIGNAL(newValue(double)), m, SLOT(s_set_Z2_spectr(double)));
        connect(ev_Z2_spectr, SIGNAL(editingFinished()), m, SLOT(sl_up_lines_on()));
        connect(ev_Z2_spectr, SIGNAL(editingStarted()), m, SLOT(sl_up_lines_of()));
        for (int i=0; i<m->CB_Channel->count(); i++)
        {
            CB_Channel->addItem(m->CB_Channel->itemText(i));
        }
        connect(CB_Channel, SIGNAL(currentIndexChanged(int)), m->CB_Channel, SLOT(setCurrentIndex(int)));
        
        //TODO add tab from spm::connected?
        //todo Oplkill Infinity loop, fix it
        //connect(m->DCfbP, SIGNAL(newValue(double)), this, SLOT(DCfbPNewValue(double)));
        //linkDoubleChangers(m->DCfbP, DCfbPPerc, false);
        linkDoubleChangers(m->driveInt, driveInt, true);
        connect(m->driveInt->slider, SIGNAL(valueChanged(double)), driveInt, SLOT(setValue(double)));
        connect(deviceSchemeWid->setPointEdit, SIGNAL(valueChanged(double)), m, SLOT(setPoint_DCfbSetPoint_newValue(double)));
        
        connect(m->BFeedbackOnOff, SIGNAL(state_changed(bool)), this, SLOT(FeedBackChangedUpdateMag(bool)));
        connect(deviceSchemeWid->feedBack, SIGNAL(changedState(bool)), this, SLOT(FeedBackChangedUpdateMag(bool)));
        
        double maxTipDriveAmplitude = 3.0;
        {
            QSettings settings("Param/landing.conf", QSettings::IniFormat);
            maxTipDriveAmplitude = settings_double(settings, "max_tip_drive_amplitude", 3.0);
        }
        rectAmplitude->setValue_man((m->driveInt->value()/maxTipDriveAmplitude)*100.0);
        connect(m->driveInt, SIGNAL(newValue(double)), this, SLOT(tipDriveNewValue(double)));
        connect(rectAmplitude, SIGNAL(valueChanged(double)), this, SLOT(tipDriveNewValue_FromSlider(double)));
        
        connect(m->DCfbSetPoint, SIGNAL(editingFinished()), this, SLOT(DCfbSetPointChanged()));
        
        rectSlVelocity->setValue_man(m->DCfbP->value()*10.0);
        
        connect(spmMonitorFeedback, SIGNAL(state_changed(bool)), m, SLOT(feedBackClickedInScan(bool)));
        spmMonitorFeedback->change_state_man(m->BFeedbackOnOff->isOn);
        
        connect(deviceSchemeWid->oscilator->btnAutoFreqScan, SIGNAL(clicked()), m, SLOT(s_FreqScanStartAuto()));
        deviceSchemeWid->oscilator->btnFreqStartScan->setProperty("id", DataID::SPM_freq_scan);
        connect(deviceSchemeWid->oscilator->btnFreqStartScan, SIGNAL(state_changed(bool)), m, SLOT(s_FreqScanStart(bool)));
        fillCBchannels_(deviceSchemeWid->oscilator->cbChannel, false);
        deviceSchemeWid->oscilator->cbChannel->setCurrentIndex(m->CB_fr_Channel->findData(srcMag1));
        deviceSchemeWid->setPointEdit->setProperty("id", DataID::SPM_SetPoint);
        //connect(deviceSchemeWid->setPointEdit, SIGNAL(newValue()), m, SLOT(s_sendSP()));
        connect(deviceSchemeWid->psd, SIGNAL(ChangedValue(bool)), m->BLaserOnOff, SLOT(change_state_man_with_sig(bool)));
        connect(deviceSchemeWid->feedBack, SIGNAL(changedState(bool)), this, SLOT(feedBackClickedInSPM(bool)));
        connect(deviceSchemeWid->feedBack, SIGNAL(changedState(bool)), m, SLOT(s_setFBOnOff(bool)));
        connect(deviceSchemeWid->feedBack, SIGNAL(changedState(bool)), m, SLOT(Zpos_on_off(bool)));
        connect(deviceSchemeWid->oscilator->chbSecHarmonic, SIGNAL(toggled(bool)), m->capMode, SLOT(setChecked(bool)));
        connect(rectSlFbi->centralWidget, SIGNAL(clicked(bool)), deviceSchemeWid->feedBack, SLOT(setState(bool)));
        deviceSchemeWid->ZPiezoPosEdit[0]->setProperty("id", DataID::HV_DAC_X2);
        deviceSchemeWid->ZPiezoPosEdit[1]->setProperty("id", DataID::HV_DAC_Y2);
        deviceSchemeWid->ZPiezoPosEdit[2]->setProperty("id", DataID::HV_DAC_Z2);
        connect(deviceSchemeWid->PIDSwitch, SIGNAL(indexChanged(int)), m, SLOT(pidSwitch_newValue(int)));
        connect(deviceSchemeWid->plusSwitch, SIGNAL(indexChanged(int)), m, SLOT(plusSwitch_newValue(int)));
        connect(deviceSchemeWid->bias->bias, SIGNAL(valueChanged(double)), m, SLOT(edBias_Vtip_newValue(double)));
        connect(deviceSchemeWid->LFLSwitch, SIGNAL(indexChanged(int)), m, SLOT(lflSwitch_lflCurrent_newValue(int)));
        connect(deviceSchemeWid->DFLSwitch, SIGNAL(indexChanged(int)), m, SLOT(dflSwitch_extCurrent_newValue(int)));
        connect(deviceSchemeWid->pid->tsbFBSign, SIGNAL(state_changed(bool)), m->B_PID_sign, SLOT(change_state_man_with_sig(bool)));
        
        rectSLSensitivityAdditWindowWid->addConnects(m->DCfbP, m->DCfbI, m->CBFBScanner);
        rectSLTipPushAdditWindowWid->addConnects(m->DCfbSetPoint);
        rectSLTipDriveAdditWindowWid->addConnects(m->driveInt);
    }
    if (name == CCDPlotModule::name()) {
        CCDPlot *m = (CCDPlot *)module->basecast();
        
        #ifdef _ENABLE_NST_LASERS
        if (useNSTLaserEtta)
        {
            auto nstlaserEtta = m->nstLaser_list.last();
            VideoLaserEttaSetCurrent->setValue(nstlaserEtta->GetPowerPercent());
            connect(this, SIGNAL(nstLaserEttaCurrentChanged(double)), nstlaserEtta, SLOT(setPowerPercent(double)));
            connect(nstlaserEtta, SIGNAL(powerPercentChanged(double)), VideoLaserEttaSetCurrent, SLOT(setValue(double)));
            
            VideoLaserEttaTurnOn->change_state_man(nstlaserEtta->Laser_onoff->isOn);
            connect(VideoLaserEttaTurnOn, SIGNAL(state_changed(bool)), nstlaserEtta->Laser_onoff, SLOT(change_state_man_with_sig(bool)));
            connect(nstlaserEtta->Laser_onoff, SIGNAL(state_changed_man(bool)), VideoLaserEttaTurnOn, SLOT(change_state_man(bool)));
        }
        #endif
    }
    if (name == OscilloscopeModule::name()) {
        Oscilloscope *m = (Oscilloscope *)module->basecast();
        
        connect(m->CalibWidget, SIGNAL(sigScanError()), this, SLOT(s_scanError()));
        connect(this, SIGNAL(recalibrateSensors(int)), m->acf, SLOT(doXYcalib(int)));
        connect(m->acf, SIGNAL(calibDone(bool)), this, SLOT(recalibDone(bool)));
        
        connect(m->DacsWidget, SIGNAL(xUpdate(double)), this, SLOT(sl_setCurrentX(double)));
        connect(m->DacsWidget, SIGNAL(yUpdate(double)), this, SLOT(sl_setCurrentY(double)));
        connect(m->DacsWidget, SIGNAL(x2Update(double)), this, SLOT(sl_setCurrentX2(double)));
        connect(m->DacsWidget, SIGNAL(y2Update(double)), this, SLOT(sl_setCurrentY2(double)));
        
        connect(this, SIGNAL(sync_scan_x(double)), m->DacsWidget, SIGNAL(xUpdate(double)));
        connect(this, SIGNAL(sync_scan_y(double)), m->DacsWidget, SIGNAL(yUpdate(double)));
        connect(this, SIGNAL(sync_scan_x2(double)), m->DacsWidget, SIGNAL(x2Update(double)));
        connect(this, SIGNAL(sync_scan_y2(double)), m->DacsWidget, SIGNAL(y2Update(double)));
        
        connect(this, SIGNAL(sig_XYPosition(double, double)), m->DacsWidget, SLOT(Go_to_XY(double, double)));
        connect(this, SIGNAL(sig_XYPosition2(double, double)), m->DacsWidget, SLOT(Go_to_XY2(double, double)));
        
        connect(m->CalibWidget, SIGNAL(sig_XYZ_range(double,double,double)), this, SLOT(sl_set_XYZ_range(double, double, double)));
        connect(m->CalibWidget, SIGNAL(sig_XYZ_range2(double,double,double)), this, SLOT(sl_set_XYZ_range2(double, double, double)));
        
        connect(m->CalibWidget, SIGNAL(input_calib_updated()), this, SLOT(sl_CB_update()));
        connect(m->CalibWidget, SIGNAL(input_calib_updated()), this, SLOT(SetSectionUnits()));
        
        connect(deviceSchemeWid->ZPiezoPosEdit[0], SIGNAL(newValue(double)), m->DacsWidget, SLOT(Go_to_X2(double)));
        connect(deviceSchemeWid->ZPiezoPosEdit[1], SIGNAL(newValue(double)), m->DacsWidget, SLOT(Go_to_Y2(double)));
        connect(deviceSchemeWid->ZPiezoPosEdit[2], SIGNAL(newValue(double)), m->DacsWidget, SLOT(Go_to_Z2(double)));
        
        connect(deviceSchemeWid->ZPiezoPosEdit[0], SIGNAL(newValue(double)), m->DacsWidget, SLOT(zPiezoPosX_X2_newValue(double)));
        connect(deviceSchemeWid->ZPiezoPosEdit[1], SIGNAL(newValue(double)), m->DacsWidget, SLOT(zPiezoPosY_Y2_newValue(double)));
        connect(deviceSchemeWid->ZPiezoPosEdit[2], SIGNAL(newValue(double)), m->DacsWidget, SLOT(zPiezoPosZ_Z2_newValue(double)));
        
        connect(m->DacsWidget, SIGNAL(x2Update(double)), deviceSchemeWid->ZPiezoPosEdit[0], SLOT(setValue(double)));
        connect(m->DacsWidget, SIGNAL(y2Update(double)), deviceSchemeWid->ZPiezoPosEdit[1], SLOT(setValue(double)));
        connect(m->DacsWidget, SIGNAL(z2Update(double)), deviceSchemeWid->ZPiezoPosEdit[2], SLOT(setValue(double)));
        
        connect(m->CalibWidget->CBlvm[0], SIGNAL(toggled(bool)), this, SLOT(setLowVoltageMode(bool)));
        connect(m->CalibWidget->CBlvm[1], SIGNAL(toggled(bool)), this, SLOT(setLowVoltageMode(bool)));
        connect(m->CalibWidget->Scanner_type, SIGNAL(currentChanged(int)), this, SLOT(setScannerLowVoltageMode(int)));
    }
    if (name == VideoModule::name()) {
        VideoPlayer *v = (VideoPlayer *)module->basecast();
        
        CBChannel[videoChannelId]->setItemText(0, tr("VideoCam"));
        
        auto list = v->getVideoProfiles();
        for (int i = 0; i < list->count(); i++)
        {
            comboVideoProfile->addItem(list->at(i));
            videoChannelMagnification->addItem(list->at(i));
        }
        connect(comboVideoProfile, SIGNAL(currentIndexChanged(int)), v, SLOT(sl_select_config(int)));
        connect(videoChannelMagnification, SIGNAL(currentIndexChanged(int)), v, SLOT(sl_select_config(int)));
        //connect(v, SIGNAL(VideoProfileChanged(int)), this, SLOT(videoProfileChanged(int)));
        
        connect(BAutoFocus, SIGNAL(state_changed(bool)), this, SLOT(autoFocus_ChangedState(bool)));
        connect(BAutoFocus2, SIGNAL(state_changed(bool)), this, SLOT(autoFocus2_ChangedState(bool)));
        connect(videoChannelAutoFocusButtonPAF, SIGNAL(state_changed(bool)), this, SLOT(autoFocus2_ChangedState(bool)));
        connect(v, SIGNAL(AutoFocus2Ended(bool)), this, SLOT(autoFocus2Ended(bool)));
        
        connect(Plot->ThreeDPlots[videoChannelId]->overlayButtons, SIGNAL(replacedGUIPlotResized(QSize)),
                this, SLOT(videoChannelResized(QSize)));
        connect(Plot->ThreeDPlots[videoChannelId]->overlayButtons, SIGNAL(replacedGUIPlotSetVisible(bool)),
                this, SLOT(videoChannelSetVisibleGUI(bool)));
        onVideoCBChannelChanged(CBChannel[videoChannelId]->currentIndex());
        Plot->ThreeDPlots[videoChannelId]->overlayButtons->setVisibleMenu(false);
    }
    if (name == SampleControlModule::name()) {
        SampleControl *s = (SampleControl *)module->basecast();
        
        connect(sampleControlActivateBtn, SIGNAL(state_changed(bool)), s, SLOT(changeStateAll(bool)));
    }
    if (name == MotorsModule::name()) {
        Motors *m = (Motors *)module->basecast();
        
        connect(moveStagePumpsBtn, SIGNAL(state_changed(bool)), m->pumpStage, SLOT(startMoveStages(bool)));
        connect(this, SIGNAL(motorArs3000MoveToPosition(Motors::MotorRoles, int)), m->ars3000, SLOT(startMotorWithConfLine(Motors::MotorRoles, int)));
        connect(m->ars3000, SIGNAL(confLineChanged(Motors::MotorRoles, int)), this, SLOT(onARS3000ConfLineChanged(Motors::MotorRoles, int)));
    }
}

void ScanModule::initialSend(MobileInstance *client)
{
    QStringList chnames;
    QList<QVariant> chdata;
    for (int i = 0; i < CBChannel[1]->count(); i++) {
        chnames.append(CBChannel[1]->itemText(i));
        chdata.append(CBChannel[1]->itemData(i));
    }
    QList<QVariant> chIndexes;
    chIndexes << CBChannel[0]->currentIndex()
              << CBChannel[1]->currentIndex()
              << CBChannel[2]->currentIndex()
              << CBChannel[3]->currentIndex();
    client->serv->setPropIdVarS(client->socket, DataID::ChannelList,
                                QList<QVariant>() << chnames << QVariant(chdata) << QVariant(chIndexes) << srcCCD);
    
    client->serv->setPropIdVarS(client->socket, DataID::ScanNxNy, qSqrt(Nx_points * Ny_points));
    
    QList<QVariant> sects;
    sects << CBSec[0]->isChecked()
          << CBSec[1]->isChecked()
          << CBSec[2]->isChecked()
          << CBSec[3]->isChecked();
    client->serv->setPropIdVarS(client->socket, DataID::Channel_sectionsUpdate, QVariant(sects));
    client->serv->setPropIdVarS(client->socket, DataID::smartLandingChanged, smartLanding->getState());
    
    if (!m_BStart[0]->on_off) {
        Send2dPlotData();
        Send3dPlotData1();
        Send3dPlotData2();
        Send3dPlotData3();
        Send3dPlotData4();
    }
    
    updateVideoChannel();
}
void ScanModule::setValue(int id, QVariant val, QObject *sender)
{
    if (id == DataID::ActivePlot_)
    {
        int plotId = val.toInt();
        if (plotId >= 0 && plotId < numScanChannel)
            Plot->SetActivePlot(plotId);
    }
    
    if (id == DataID::plot3DCCDColorPick)
    {
        int NOfPalette = val.toInt();
        Plot->SetActivePlot(1);
        Plot->PlotControl->AGPalettes->actions()[NOfPalette]->trigger();
        Plot->PlotControl->AGPalettes->actions()[NOfPalette]->setChecked(true);
    }
    
    if (id == DataID::ScanChB)
    {
        if (val.toInt() == srcCCD && CBExternalMode->currentIndex() != 1)
            CBExternalMode->setCurrentIndex(1);
        else if (CBExternalMode->currentIndex() == 1)
            CBExternalMode->setCurrentIndex(0);
    }
    
    if (id == DataID::scan3DSave)
    {
        int plotId = val.toInt();
        
        plot::T3dDataOwner src;
        plot::TMetaStorage attributes;
        if (!Plot->SavePlot(plotId, &src, &attributes))
            return;
        
        FillContainer(plotId, &src, &attributes);
        
        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
        {
            QList<QVariant> list;
            list << plotId
                 << QVariant::fromValue(attributes);
            
            send3dPlot(PlotID::Scan1+plotId, Plot, plotId, false, true);
            mob->setPropIdVar(DataID::addedScanDataToContainer_3d, QVariant::fromValue(list));
        }
    }
    
    if (id == DataID::scanConvFuncRangeChanged)
    {
        auto rect = val.toRectF();
        double Min = rect.left();
        double Max = rect.right();
        
        currExtScan->setSelectionRange(Min, Max);
    }
    
    if (id >= DataID::needPlot3D_A && id <= DataID::needPlot3D_D)
        if (sender)
        {
            int numPlot = id - DataID::needPlot3D_A;
            QString property = "scan_plot_" + QString::number(numPlot);
            sender->setProperty(property.toLocal8Bit().constData(), val);
            if (val.toBool())
                send3dPlot(PlotID::Scan1+numPlot, Plot, numPlot, false, true);
        }
    
    if (id == DataID::VideoOn)
        BMakeScreen->change_state_man_with_sig(val.toBool());
    if (id == DataID::needPlot2D)
        if (sender)
        {
            sender->setProperty(QString("need_2dPlot").toLocal8Bit().constData(), val);
            send2dPlot(PlotID::ScanSection, Plot->TwoDPlot);
        }
    
    if (id == DataID::plot2DSaveToData)
        Plot->TwoDPlot->overlayButtons->control->B2DSendToData->click();
    
    if (id == DataID::plot3DChangeSize)
    {
        auto list = val.toList();
        int indexPoints = list[0].toInt();
        double deltaValue = list[1].toDouble();
        if (CBPointsOX->currentIndex() != indexPoints)
            CBPointsOX->setCurrentIndex(indexPoints);
        if (LEdDeltaX->isEnabled() && LEdDeltaX->value() != deltaValue)
            LEdDeltaX->setValue(deltaValue);
    }
    
    if (id == DataID::plot3DGetSizes)
    {
        auto plot = Plot->ThreeDPlots[0];
        QList<QVariant> list;
        list << CBPointsOX->currentIndex();
        list << LEdDeltaX->value();
        list << LEdDeltaX->isEnabled();
        list << plot->XYAxisUnit.Legend;
        list << plot->XYAxisUnit.Multiplier;
        
        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
            mob->setPropIdVar(DataID::plot3DChangeSize, list);
    }
    
    if (id == DataID::plot3DAreaSelected)
    {
        auto list = val.toList();
        double x1 = list[0].toDouble();
        double x2 = list[1].toDouble();
        double y1 = list[2].toDouble();
        double y2 = list[3].toDouble();
        double selSize = list[4].toDouble();
        
        Plot->SetPlotsXScales(x1, x2);
        Plot->SetPlotsYScales(y1, y2);
        LEdDeltaX->setValue(selSize);
        
        for (int i=0; i<Plot->ThreeDPlots.size(); i++)
            Plot->ThreeDPlots[i]->ResizeQPlot();
    }
    
    if (id == DataID::motorZFocusMove)
    {
        int speed = val.toInt();
        auto m = Motors::GetIt();
        auto v = VideoPlayer::GetIt();
        auto mZfocus = m->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
        if (speed == 1)
            m->goDownE(mZfocus, v->GetCurrentZFocusSpeed());
        else if (speed == -1)
            m->goUpE(mZfocus, v->GetCurrentZFocusSpeed());
        else
            m->goStop(mZfocus);
    }
    
    if (id == DataID::motorXStageMove)
    {
        int speed = val.toInt();
        auto m = Motors::GetIt();
        auto v = VideoPlayer::GetIt();
        auto mXStage = m->getRoleMotor(Motors::MotorRoles::RoleStageX);
        if (speed == 1)
            m->goDownE(mXStage, v->GetCurrentVxyPos());
        else if (speed == -1)
            m->goUpE(mXStage, v->GetCurrentVxyPos());
        else
            m->goStop(mXStage);
    }
    
    if (id == DataID::motorYStageMove)
    {
        int speed = val.toInt();
        auto m = Motors::GetIt();
        auto v = VideoPlayer::GetIt();
        auto mYStage = m->getRoleMotor(Motors::MotorRoles::RoleStageY);
        if (speed == 1)
            m->goDownE(mYStage, v->GetCurrentVxyPos());
        else if (speed == -1)
            m->goUpE(mYStage, v->GetCurrentVxyPos());
        else
            m->goStop(mYStage);
    }
    
    if (id == DataID::videoChangedMagnification)
    {
        int index = val.toInt();
        if (index >= 0 && index < videoChannelMagnification->count())
            videoChannelMagnification->setCurrentIndex(index);
        else
            updateVideoChannel();
    }
    
    if (id == DataID::plot3DRestoreScale)
    {
        Plot->PlotControl->BRestoreScale[0]->click();
    }
    
    if (id == DataID::videoMafPrecise)
    {
        bool on = val.toBool();
        if (on)
        {
            QTimer *timer = new QTimer();
            timer->setSingleShot(true);
            connect(timer, SIGNAL(timeout()), this, SLOT(startMAFPrcs()));
            timer->start(10); //starting in another thread
        }
        else
            videoChannelAutoFocusButtonPrec->change_state_man_with_sig(false);
    }
    
    if (id == DataID::videoMafFast)
    {
        bool on = val.toBool();
        if (on)
        {
            QTimer *timer = new QTimer();
            timer->setSingleShot(true);
            connect(timer, SIGNAL(timeout()), this, SLOT(startPAFFast()));
            timer->start(10); //starting in another thread
        }
        else
            videoChannelAutoFocusButtonFast->change_state_man_with_sig(false);
    }
    
    if (id == DataID::videoPaf)
    {
        bool on = val.toBool();
        if (on)
        {
            QTimer *timer = new QTimer();
            timer->setSingleShot(true);
            connect(timer, SIGNAL(timeout()), this, SLOT(startPAF()));
            timer->start(10); //starting in another thread
        }
        else
            videoChannelAutoFocusButtonPAF->change_state_man_with_sig(false);
    }
    
    if (id == DataID::videoFocusUpDown)
    {
        int newSt = val.toInt();
        if (newSt == -1)
        {
            auto mob = MobileServer::GetIt();
            if (mob && mob->need_send())
            {
                //mob->setPropIdVar(DataID::videoFocusUpDown, videoChannelAutoFocusUpDownBtn->state);
            }
        }
        else
        {
            //videoChannelAutoFocusUpDownBtn->change_state(newSt);
        }
    }
    
#ifdef _SCAN_SLICE
    if (id == DataID::plot3DSliceScales)
    {
        int plotId = val.toInt();
        auto plot = Plot->ThreeDPlots.at(plotId);
        if (plot->sliceBounds)
        {
            QList<QVariant> list;
            list << plotId;
            list << plot->sliceX.arr[0];
            list << plot->sliceX.arr[1];
            list << plot->sliceY.arr[0];
            list << plot->sliceY.arr[1];
            list << plot->XYAxisUnit.Multiplier;
            auto mob = MobileServer::GetIt();
            if (mob && mob->need_send())
                mob->setPropIdVar(DataID::plot3DSliceScales, list);
        }
    }
#endif
    
    if (id == DataID::scanFilterOnOff)
        if (sender)
        {
            auto list = val.toList();
            int plotId = list.at(0).toInt();
            if (list.size() == 1)
            {
                QList<QVariant> list;
                list << plotId
                     << Plot->ThreeDPlots[plotId]->isFiltering;
                auto mob = MobileServer::GetIt();
                if (mob && mob->need_send())
                    mob->setPropIdVar(DataID::scanFilterOnOff, list);
            }
            else
            {
                bool pressed = list.at(1).toInt();
                Plot->ThreeDPlots[plotId]->setFiltering(pressed);
            }
        }
    
    if (id == DataID::plot2DFilterOnOff)
        if (sender)
        {
            if (val.isNull())
            {
                auto mob = MobileServer::GetIt();
                if (mob && mob->need_send())
                    mob->setPropIdVar(DataID::plot2DFilterOnOff, Plot->TwoDPlot->Pure2dPlot->isFiltering);
            }
            else
            {
                bool pressed = val.toBool();
                Plot->TwoDPlot->overlayButtons->control->B2DFilter->blockSignals(true);
                Plot->TwoDPlot->overlayButtons->control->B2DFilter->change_state_man(pressed);
                Plot->TwoDPlot->setFiltering(pressed);
                Plot->TwoDPlot->overlayButtons->setButtonActive(Plot->TwoDPlot->overlayButtons->control->B2DFilter, pressed);
                Plot->TwoDPlot->overlayButtons->control->B2DFilter->blockSignals(false);
            }
        }
    
    if (id == DataID::scanDirectionChanged)
        if (sender)
        {
            m_BStart[0]->changeDirection(static_cast<swapDir::Direction>(val.toInt()), true);
            for (auto btn:m_BStart)
                btn->changeDirection(static_cast<swapDir::Direction>(val.toInt()), false);
        }
    
    if (id == DataID::rectSpeed)
    {
        rectSlFbp->setValue(val.toDouble());
    }
    
    if (id == DataID::plot2DDefaultFilterSettings)
        if (sender)
        {
            auto mob = MobileServer::GetIt();
            auto scan = ScanPlot::GetIt();
            if (mob && scan)
            {
                int fitOrder = FindCheckedItemIn(scan->Plot->TwoDPlot->PlotControl->AGFitOrders);
                mob->setPropIdVar(DataID::plot2DDefaultFilterSettings, fitOrder);
            }
        }
    
    if (id == DataID::needRectSpeed && sender)
    {
        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
        {
            mob->setPropIdVar(DataID::rectMaxSpeed, rectSlFbp->GetMaximum());
            mob->setPropIdVar(DataID::rectSpeed, rectSlFbp->Value());
        }
    }
    
    if (id == DataID::rectTipPush)
    {
        rectSlFbi->setValue(val.toDouble());
    }
    
    if (id == DataID::rectTipDrive)
    {
        rectAmplitude->setValue(val.toDouble());
    }
    
    if (id == DataID::needRectTipDrive && sender)
    {
        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
            mob->setPropIdVar(DataID::rectTipDrive, rectAmplitude->Value());
    }
    
    if (id == DataID::needRectTipPush && sender)
    {
        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
            mob->setPropIdVar(DataID::rectTipPush, rectSlFbi->Value());
    }
    
    if (id == DataID::rectSensitivity)
    {
        rectSlVelocity->setValue(val.toDouble());
    }
    
    if (id == DataID::smartLandingChanged)
    {
        smartLanding->animateClick();
    }
    
    if (id == DataID::approachSOff)
    {
        smartLandingBtn1->animateClick();
    }
    
    if (id == DataID::approachSOn)
    {
        smartLandingBtn2->animateClick();
    }
    
    if (id == DataID::plot2DSelectLine)
    {
        Plot->TwoDPlot->overlayButtons->control->colorSettings->CBLineSelector->setCurrentIndex(val.toInt());
    }
    
    if (id == DataID::plot2DLineFontSettings)
    {
        QList<QVariant> list = val.toList();
        QColor lineColor = list.at(0).value<QColor>();
        int lineWidth = list.at(1).toInt();
        int font = list.at(2).toInt();
        //Plot->TwoDPlot->overlayButtons->CBLineColor->setCurrentIndex(lineColor);
        Plot->TwoDPlot->overlayButtons->control->colorSettings->CBLineWidth->setCurrentIndex(lineWidth);
        Plot->TwoDPlot->overlayButtons->control->colorSettings->CBFont->setCurrentIndex(font);
        
        Plot->TwoDPlot->setCurveColor(lineColor);
    }
    
    if (id == DataID::rectTipDriveAutoControll)
    {
        auto btn = dynamic_cast<QPushButton*>(rectAmplitude->centralWidget);
        if (val.isNull())
        {
            auto mob = MobileServer::GetIt();
            if (mob && mob->need_send())
                mob->setPropIdVar(DataID::rectTipDriveAutoControll, btn->isChecked());
        }
        else
        {
            bool checked = val.toBool();
            btn->setChecked(checked);
            btn->setText(checked ? tr("auto") : tr("mnl"));
        }
    }
    
    if (id == DataID::ChannelCurrentIds_update_send && sender)
    {
        CBChannel[0]->setCurrentIndex(CBChannel[0]->currentIndex());
    }
    
    if (id == DataID::ScanOnOff)
    {
        if (val.isNull())
        {
            auto mob = MobileServer::GetIt();
            if (mob && mob->need_send())
            {
                bool saveToData = false;
                QList<QVariant> list;
                list << m_BStart[0]->on_off << saveToData;
                mob->setPropIdVar(DataID::ScanOnOff, list);
            }
        }
        else
        {
            m_BStart[0]->change_state_man_with_sig(val.toBool());
            for (auto btn:m_BStart)
                btn->change_state_man(val.toBool());
        }
    }
    
    if (id == DataID::ScanNxNy) {
        int i = val.toInt();
        i += 3;
        
        CBPointsOX->blockSignals(true);
        CBPointsOY->blockSignals(true);
        CBPointsOXext->blockSignals(true);
        CBPointsOYext->blockSignals(true);
        CBPointsOX->setCurrentIndex(i);
        CBPointsOY->setCurrentIndex(i);
        CBPointsOXext->setCurrentIndex(i);
        CBPointsOYext->setCurrentIndex(i);
        CBPointsOX->blockSignals(false);
        CBPointsOY->blockSignals(false);
        CBPointsOXext->blockSignals(false);
        CBPointsOYext->blockSignals(false);
        check_nx_ny();
        sl_ext_mode();
    }
    if (id == DataID::ScanSaveCh) {
        if (val.toInt() == 4) {
            Plot->TwoDPlot->PlotControl->BSave->animateClick();
        } else {
            Plot->SetActivePlot(val.toInt());
            Plot->PlotControl->BSave->animateClick();
        }
    }
    
    if (id == DataID::PlotScalesChanged) {
        QVariantList lst = val.toList();
        plot::TScalesChangedEvent e(Range(), Range(), (plot::TScalesChangedEvent::TReason)lst.at(0).toInt());
        e.XLimits[0] = lst.at(1).toDouble();
        e.XLimits[1] = lst.at(2).toDouble();
        e.YLimits[0] = lst.at(3).toDouble();
        e.YLimits[1] = lst.at(4).toDouble();
        Plot->SetPlotsXScales(e.XLimits[0], e.XLimits[1]);
        Plot->SetPlotsYScales(e.YLimits[0], e.YLimits[1]);
        eventFilter(this, &e);
    }
}
