#include "spmmodule.h"

#include "hardware/EG3000.h"
#include "scanplot.h"
#include "IExternalScan.h"
#include "motors.hpp"
#include "oscilloscope.h"
#include "autocalib.hpp"
#include "modules/datamodule.h"
#include "modules/eg3000module.h"
#include "modules/scanmodule.h"
#include "modules/motorsmodule.h"
#include "modules/oscilloscopemodule.h"
#include "xyzcalibrations.hpp"
#include "spm_motors.hpp"
#include "deflect_widget.hpp"
#include "dacscontrol.h"

void SPMModule::connected(const QLatin1String name, I_MODULE *module)
{
    if (name == EG3000Module::name()) {
        EG3000_CONTROL *m = (EG3000_CONTROL *)module->basecast();
        
        m->rThread.readers.add(this);
        m->wThread.writers.add(this);
        
        connect(&m->rThread, SIGNAL(capBits(int,long)), this, SLOT(capBits(int,long)));
    }
    if (name == DataModule::name()) {
        DataPlot *m = (DataPlot *)module->basecast();
        
        connect(m->Container, SIGNAL(restoreAttributes(plot::TMetaStorage)),
                this, SLOT(restoreAttributes(plot::TMetaStorage)));
        
        connect(this, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)),
                m->Container, SLOT(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)));
    }
    if (name == ScanModule::name()) {
        ScanPlot *m = (ScanPlot *)module->basecast();
        
        m->Plot->TwoDPlotSPM = MyPlScan;
        m->Plot->TwoDPlotSPM->showPlotControl = false;
        //m->Plot->ControlLayout->addWidget(MyPlScan->PlotControl);
        MyPlScan->PlotControl->hide();
        connect(MyPlScan->Pure2dPlot, SIGNAL(doubleClicked()), m->Plot, SLOT(plot2dDoubleClicked()));
        connect(MyPlScan->Pure2dPlot, SIGNAL(ShowPlotControl()), m->Plot, SLOT(Show2dPlotControl()));
        
        IExternalScan *sc;
        m->addExternalScan(MainWindow::tr("JM frce crv"), sc = getNewJMIExtScan(this));
        if (auto data = DataPlot::GetIt())
            connect(sc, SIGNAL(AddDataToContainer(ccd::TSetOfSpectrum*,plot::TMetaStorage)),
                    data->Container, SLOT(AddDataToContainer(ccd::TSetOfSpectrum*,plot::TMetaStorage)));
        m->spmMonitorGBHlay->addWidget(LaserSpotWindow, 5);
        m->LaserSpotWindow = LaserSpotWindow;
        //connect(LaserSpotWindow, SIGNAL(doubleClicked()), m, SLOT(doubleClickedLaserSpotWindow()));
        
        connect(BFeedbackOnOff, SIGNAL(state_changed(bool)), m, SLOT(feedBackClickedInSPM(bool)));
        connect(m->rectSlFbi->centralWidget, SIGNAL(clicked(bool)), BFeedbackOnOff, SLOT(change_state_man_with_sig(bool)));
        connect(DCfbP, SIGNAL(newValue(double)), m, SLOT(DCfbPNewValue(double)));
        dynamic_cast<QPushButton*>(m->rectSlFbi->centralWidget)->setChecked(BFeedbackOnOff->isOn);
        m->spmMonitorFeedback->change_state_man(BFeedbackOnOff->isOn);
        connect(m->deviceSchemeWid->oscilator->cbChannel, SIGNAL(currentIndexChanged(int)), CB_fr_Channel, SLOT(setCurrentIndex(int)));
        
        // auto freq scan buttons
        connect(LaserSpot->probeCheckScanBtn, SIGNAL(clicked()), this, SLOT(s_FreqScanStartAuto()));
        connect(LaserSpotWindow->probeCheckScanBtn, SIGNAL(clicked()), this, SLOT(s_FreqScanStartAuto()));
    }
    if (name == MotorsModule::name()) {
        Motors *m = (Motors *)module->basecast();
        motors = m;
        
        m->addLandingSett(OptionsForMotor);
        m->addShuttleSett(OptionsForShuttle->GrBUp, 0);
        m->addShuttleSett(OptionsForShuttle->GrBMap, 1);
        
        active_shuttle_motors = getLandingMotors();
        
        connect(m, SIGNAL(mland_stopped()), this, SLOT(slot_mland_stopped()));
        connect(m->ShuttleType, SIGNAL(currentIndexChanged(int)), OptionsForShuttle, SLOT(HideAdditionShuttleButtons(int)));
        OptionsForShuttle->HideAdditionShuttleButtons(m->ShuttleType->currentIndex());
    }
    if (name == OscilloscopeModule::name()) {
        Oscilloscope *m = (Oscilloscope *)module->basecast();
        
        connect(m->acf, SIGNAL(calibInProgress(bool)), this, SLOT(s_setFBmasked(bool)));
        
        DoubleChanger *Head_Dac = new DoubleChanger(-10, 10);
        Head_Dac->setProperty("id", DataID::SPM_Vtip);
        linkDoubleChangers(Vtip, Head_Dac);
        QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
        if (!settingsGui.value("gui/Ratis_xyz_stage_mode", false).toBool())
            m->DacsWidget->insertChannel(tr("V Tip"), tr("V"), Head_Dac, 0);
        
        connect(this, SIGNAL(sig_osc_start()), m, SLOT(sl_osc_start()));
        connect(this, SIGNAL(sig_osc_stop()), m, SLOT(sl_osc_stop()));
        connect(this, SIGNAL(sig_osc_channel(int)), m, SLOT(setChA(int)));
        connect(this, SIGNAL(sig_osc_channelB(int)), m, SLOT(setChB(int)));
        connect(m->ePeriod, SIGNAL(newValue(int)), ePeriod, SLOT(setValue(int)));
        connect(ePeriod, SIGNAL(newValue(int)), m->ePeriod, SLOT(setValue(int)));
        connect(m, SIGNAL(oscRefreshed()), this, SLOT(osc_refresh()));
        
        
        connect(m->DacsWidget, SIGNAL(zUpdate(double)), NS_Z_spectr, SLOT(setValue(double)));
        
        connect(this, SIGNAL(sig_Zpos(double)), m->DacsWidget, SLOT(Go_to_Z(double)));
        connect(this, SIGNAL(sig_Z2pos(double)), m->DacsWidget, SLOT(Go_to_Z2(double)));
        
        connect(m->CalibWidget, SIGNAL(sig_XYZ_range(double,double,double)), this, SLOT(sl_set_XYZ_range(double, double, double)));
        connect(m->CalibWidget, SIGNAL(sig_XYZ_range2(double,double,double)), this, SLOT(sl_set_XYZ_range2(double, double, double)));
        
        connect(this, SIGNAL(input_calib_updated()), m->CalibWidget, SIGNAL(input_calib_updated()));
        connect(m->CalibWidget, SIGNAL(input_calib_updated()), this, SLOT(sl_setPlotMode()));
        
        connect(m, SIGNAL(newOscMAGData(QList<double>, int, int)), LaserSpot, SLOT(newOscMAGData(QList<double>, int, int)));
        connect(m, SIGNAL(newOscMAGData(QList<double>, int, int)), LaserSpotWindow, SLOT(newOscMAGData(QList<double>, int, int)));
        
        connect(m, SIGNAL(newOscHeightRMS(double, QString)), LaserSpot, SLOT(newOscHeightRMS(double, QString)));
        connect(m, SIGNAL(newOscHeightRMS(double, QString)), LaserSpotWindow, SLOT(newOscHeightRMS(double, QString)));
        
        linkDoubleChangers(DCfbSetPoint, m->DacsWidget->setPoint);
        //connect(DCfbSetPoint, SIGNAL(newValue(double)), m->DacsWidget->setPoint, SLOT(setValue(double))); //sync value change on setValue
    }
}

void SPMModule::initialSend(MobileInstance *client)
{
    QStringList chnames;
    for (int i = 0; i < fbs->count(); i++)
        chnames.append(fbs->itemText(i));
    client->serv->setPropIdVarS(client->socket, DataID::SPM_FBch_list, chnames);
}
void SPMModule::setValue(int id, QVariant val, QObject *sender)
{
    if (id == DataID::Deflect_send)
        if (sender)
            sender->setProperty("deflect_need", val);
    
    if (id == DataID::Deflect_probeCheck)
        PBFreqStart->change_state_man_with_sig(true);
}

void SPMModule::setRunning(bool on)
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    if (on) {
        BLaserOnOff->change_state_man_with_sig(
            settings.value("state/spm/laser_on", true).toBool());
        
        bool head, stage;
        {
            QSettings settings("Param/gui.conf", QSettings::IniFormat);
            head = settings.value("display/oscilloscope/head", true).toBool();
            stage = settings.value("display/oscilloscope/stage", true).toBool();
        }
        int fb_en = 0;
        if (!head) fb_en = 2;
        if (!stage) fb_en = 1;
        if (fb_en != 0) {
            CBFBScanner->setEnabled(false);
            CBFBScanner->hide();
            disconnect(CBFBScanner, nullptr, LaserSpot, nullptr);
            disconnect(CBFBScanner, nullptr, LaserSpotWindow, nullptr);
            LaserSpot->setFbscanner(-1);
            LaserSpotWindow->setFbscanner(-1);
        }
        if (fb_en == 2) CBFBScanner->setCurrentIndex(1);
        if (fb_en == 0) {
            QSettings settings("Param/gui.conf", QSettings::IniFormat);
            CBFBScanner->setCurrentIndex(settings.value("state/spm/fb_scanner", 0).toInt());
        }
    } else {
        settings.setValue("state/spm/laser_on", BLaserOnOff->isOn);
        settings.setValue("state/spm/fb_scanner", CBFBScanner->currentIndex());
    }
}
