#include "videomodule.h"

#include "oscilloscope.h"
#include "remote_nst_dataid.hpp"
#include "controls.h"
#include "options/options.h"
#include "dacscontrol.h"
#include "nstslider.h"
#include "modules/oscilloscopemodule.h"
#include "modules/optionsmodule.h"
#include "xyzcalibrations.hpp"

void VideoModule::connected(const QLatin1String name, I_MODULE *module)
{
    if (name == OscilloscopeModule::name()) {
        Oscilloscope *m = (Oscilloscope *)module->basecast();
        
        linkDoubleChangers(m->DacsWidget->Zobj, zPos);
        connect(m->DacsWidget->Zobj->slider, SIGNAL(valueChanged(double)), zPos, SLOT(setValue(double))); //sync value change on setValue
        
        connect(m->CalibWidget, SIGNAL(sig_XYZ_range(double,double,double)), this, SLOT(sl_set_XYZ_range(double, double, double)));
        connect(m->CalibWidget, SIGNAL(sig_XYZ_range2(double,double,double)), this, SLOT(sl_set_XYZ_range2(double, double, double)));
    }
    if (name == OptionsModule::name()) {
        options *m = (options *)module->basecast();
        
        m->addNewTab(mainWidget, tr("Video"), 3);
    }
}

void VideoModule::setValue(int id, QVariant val, QObject *sender)
{
    //if (inst_idx != 0)
    //    return;
    
    if (id == DataID::Video_send)
        if (sender)
            sender->setProperty("video_need", val);
    
    if (id == DataID::VideoOnToggle)
        m_BStart->animateClick();
    if (id == DataID::VideoXYjoy)
        goXYjoy(val.toPointF());
    if (id == DataID::VideoZjoy)
        goZjoy(val.toDouble());
}
