#include "motors.hpp"

#include <QSettings>
#include <QCheckBox>
#include <QGroupBox>
#include <QComboBox>
#include <QRadioButton>
#include <QTimer>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QApplication>
#include <QToolButton>
#include <QMenu>
#include <QDebug>
#include <QTableWidget>
#include <QHeaderView>
#include <QCloseEvent>
#include "controls.h"
#include "edit.hpp"
#include "tools.hpp"
#include "spm_motors.hpp"
#include "options_window.h"
#include "USBVideo/videoplayer.h"
#include "scanplot.h"
#include "gui/status.hpp"
#include "gui/mslider.hpp"
#include "remote_nst_dataid.hpp"
#include "mobile/remote_mobile.hpp"
#include "gui/ttabwidget.hpp"
#include <QTextBlock>
#include "ars3000.h"
#include "pumpsstage.h"

//see also MotorRoles enum
const int Motors::numMaxMotors;
const int Motors::axes_names_count;
const char* Motors::axes_names[axes_names_count] =
    { QT_TR_NOOP("-"), QT_TR_NOOP("X Head"), QT_TR_NOOP("Y Head"), QT_TR_NOOP("Z Landing"),
        QT_TR_NOOP("X Stage"), QT_TR_NOOP("Y Stage"), QT_TR_NOOP("Z Focus"),
        QT_TR_NOOP("X3"), QT_TR_NOOP("Y3"), QT_TR_NOOP("Z3"),
        QT_TR_NOOP("X Piezo M"), QT_TR_NOOP("Y Piezo M"), QT_TR_NOOP("Z Piezo M"),
        QT_TR_NOOP("Unused"),
        QT_TR_NOOP("ND Filter"), QT_TR_NOOP("Beam Exp"), QT_TR_NOOP("Edge Filter"),
        QT_TR_NOOP("Slit Monochr."), QT_TR_NOOP("Slit PMT"), QT_TR_NOOP("Microsc. Trin"),
        QT_TR_NOOP("Obj X Sl Monochr."), QT_TR_NOOP("Obj Y Sl Monochr."),
        QT_TR_NOOP("Obj X Sl PMT"), QT_TR_NOOP("Obj Y Sl PMT"), };

void Motors::initShuttleGB_GUI(QHBoxLayout *hMainLay)
{
    QVBoxLayout *shutLandLay = new QVBoxLayout();
    hMainLay->addLayout(shutLandLay, 0);
    QGroupBox *shuttleGB = new QGroupBox(tr("Shuttle"));
    ShuttleLay = new QVBoxLayout();
    shuttleGB->setLayout(ShuttleLay);
    
    QHBoxLayout *incShuttleTypeLay = new QHBoxLayout();
    QGroupBox *shutTypeGB = new QGroupBox(tr("Shuttle type"));
    QVBoxLayout *ShuttleTypeLay = new QVBoxLayout(shutTypeGB);
    incShuttleTypeLay->addWidget(shutTypeGB, 0, Qt::AlignTop);
    ShuttleType = new QComboBox();
    ShuttleTypeLay->addWidget(ShuttleType, 0);
    ShuttleType->addItem(tr("Shuttle Pro V2"));
    ShuttleType->addItem(tr("Xpress Shuttle"));
    connect(ShuttleType, SIGNAL(currentIndexChanged(int)), this, SLOT(save_shuttleType()));
    
    ShuttleLay->addWidget(shutTypeGB);
    shutLandLay->addWidget(shuttleGB, 1);
}

void Motors::initMotorTestGB_GUI(QVBoxLayout *leftLay, const QString buttonsStyleSheet)
{
    QHBoxLayout *gbMTLay = new QHBoxLayout;
    QHBoxLayout *inGbMTLay = new QHBoxLayout;
    QVBoxLayout *slayb = new QVBoxLayout;
    QGroupBox *groupMotorTest = new QGroupBox(tr("Motor test"));
    gbMTLay->addWidget(groupMotorTest);
    leftLay->addLayout(gbMTLay);
    groupMotorTest->setLayout(inGbMTLay);
    inGbMTLay->addLayout(slayb);
    
    QHBoxLayout *tuningButsLay = new QHBoxLayout();
    
    QPushButton *pb;
    pb = new QPushButton(makeHugeIcon(":/SPMTuning_window/FastUp.png"), QString());
    connect(pb, SIGNAL(clicked()), this, SLOT(goUpE()));
    tuningButsLay->addWidget(pb);
    pb->setStyleSheet(buttonsStyleSheet);
    pb->setToolTip(tr("Fast up"));
    pb = new QPushButton(makeHugeIcon(":/SPMTuning_window/Up.png"), QString());
    connect(pb, SIGNAL(clicked()), this, SLOT(goUp()));
    tuningButsLay->addWidget(pb);
    pb->setStyleSheet(buttonsStyleSheet);
    pb->setToolTip(tr("Up"));
    pb = new QPushButton(makeHugeIcon(":/SPMTuning_window/approach/landing_done.png"), QString());
    pb->setShortcut(QKeySequence(Qt::Key_Escape));
    connect(pb, SIGNAL(clicked()), this, SLOT(goStop()));
    tuningButsLay->addWidget(pb);
    pb->setStyleSheet(buttonsStyleSheet);
    pb->setToolTip(tr("Stop"));
    pb = new QPushButton(makeHugeIcon(":/SPMTuning_window/Down.png"), QString());
    connect(pb, SIGNAL(clicked()), this, SLOT(goDown()));
    tuningButsLay->addWidget(pb);
    pb->setStyleSheet(buttonsStyleSheet);
    pb->setToolTip(tr("Down"));
    pb = new QPushButton(makeHugeIcon(":/SPMTuning_window/FastDown.png"), QString());
    connect(pb, SIGNAL(clicked()), this, SLOT(goDownE()));
    tuningButsLay->addWidget(pb);
    pb->setStyleSheet(buttonsStyleSheet);
    pb->setToolTip(tr("Fast down"));
    slayb->addLayout(tuningButsLay);
    
    slayb->addWidget(new QLabel(tr("Speed (")+QObject::tr("micro")+tr("m/s)")));
    slayb->addWidget(Espeed = new Edit(1, 10000));
    slayb->addWidget(new QLabel(tr("Distance (")+QObject::tr("micro")+tr("m)")));
    slayb->addWidget(Edist = new Edit(1, 100000));
    Espeed->setImmediateMode();
    Edist->setImmediateMode();
    connect(Espeed, SIGNAL(newValue()), this, SLOT(save_params()));
    connect(Edist, SIGNAL(newValue()), this, SLOT(save_params()));
    
    slayb->addWidget(new QLabel(tr("Backslash speed(mm/s)") + ": "));
    slayb->addWidget(EBLspeed = new Edit(0.1, 999, 1));
    connect(EBLspeed, SIGNAL(newValue()), this, SLOT(setBLspeed()));
    
    QVBoxLayout *sliderLay = new QVBoxLayout;
    MotorSlider *Mslider = new MotorSlider(true, 7);
    connect(Mslider, SIGNAL(newPos(int)), this, SLOT(goShuttleInt(int)));
    sliderLay->addWidget(Mslider, 1000);
    inGbMTLay->addLayout(sliderLay);
}

void Motors::initMotorTableGB_header_GUI(QVBoxLayout *inGbMTableLay, const QString buttonsStyleSheet)
{
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    
    //Num motors edit and Save device button
    QHBoxLayout *Lheader = new QHBoxLayout;
    inGbMTableLay->addLayout(Lheader);
    
    Lheader->addWidget(new QLabel(tr("Active motors: ")));
    edNumMotors = new Edit(1, 16);
    edNumMotors->setImmediateMode();
    edNumMotors->setValue(settings_double(settings, "motors_count", 8));
    edNumMotors->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    connect(edNumMotors, SIGNAL(newValue()), this, SLOT(save_params()));
    connect(edNumMotors, SIGNAL(newValue()), this, SLOT(init()));
    Lheader->addWidget(edNumMotors);

#ifdef _NEW_MOTORS_CHANGES
    motorsDebug = new QLabel*[3];
    for (int i=0; i<3; i++)
    {
        motorsDebugNumMot[i] = -1;
        motorsDebug[i] = new QLabel("");
        motorsDebug[i]->setStyleSheet("QLabel {"
                                      "border-style: solid;"
                                      "border-width: 1px;"
                                      "border-color: black; "
                                      "}");
        Lheader->addWidget(motorsDebug[i]);
    }
    motorsDebug[0]->setText(tr("UART1 = none"));
    motorsDebug[1]->setText(tr("UART2 = none"));
    motorsDebug[2]->setText(tr("UART3 = none"));
#endif
    
    connect(EBLspeed, SIGNAL(newValue()), this, SLOT(save_params()));
    
    Lheader->addStretch(1);
    
    BSave = new QPushButton(makeHugeIcon(":/SPMTuning_window/savePribor.png"), QString());
    connect(BSave, SIGNAL(clicked(bool)), this, SLOT(save_device()));
    BSave->hide();
    Lheader->addWidget(BSave, 0, Qt::AlignRight);
    BSave->setStyleSheet(buttonsStyleSheet);
    BSave->setToolTip(tr("Save to device"));
    
    BClearDevice = new QPushButton(makeHugeIcon(":/SPMTuning_window/clearMemory.png"), QString());
    connect(BClearDevice, SIGNAL(clicked(bool)), this, SLOT(clear_device()));
    BClearDevice->hide();
    BClearDevice->setToolTip(tr("Clear device mem"));
    Lheader->addWidget(BClearDevice, 0, Qt::AlignRight);
    BClearDevice->setStyleSheet(buttonsStyleSheet);
}

void Motors::initMotorTableGB_GUI(QVBoxLayout *leftLay, const QString buttonsStyleSheet)
{
    QHBoxLayout *inGbMTableLay = new QHBoxLayout;
    QGroupBox *groupMotorTable = new QGroupBox(tr("Motor table"));
    groupMotorTable->setLayout(inGbMTableLay);
    leftLay->addWidget(groupMotorTable, 1);
    QVBoxLayout *leftInGB = new QVBoxLayout;
    inGbMTableLay->addLayout(leftInGB);
    
    initMotorTableGB_header_GUI(leftInGB, buttonsStyleSheet);
    
    QStringList headList;
    headList.append(tr(""));
    headList.append(tr("Role"));
    headList.append(tr("Motor#"));
    headList.append(tr("Limit switch mode"));
    headList.append(tr("Direction"));
    headList.append(tr("Min speed (")+QObject::tr("micro")+tr("m/s)"));
    headList.append(tr("Max speed (")+QObject::tr("micro")+tr("m/s)"));
    headList.append(tr("Backlash(mm)"));
    headList.append(tr("Thread pitch(mm)"));
    headList.append(tr("Steps/revolution"));
    headList.append(tr("Status"));
    
    TableMotors = new QTableWidget(16, headList.count());
    leftInGB->addWidget(TableMotors);
    
    TableMotors->verticalHeader()->setVisible(false);
    TableMotors->setDragEnabled(false);
    TableMotors->setFocusPolicy(Qt::NoFocus);
    TableMotors->horizontalHeader()->setClickable(false);
    TableMotors->horizontalHeader()->setMovable(false);
    TableMotors->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
    TableMotors->setSortingEnabled(false);
    TableMotors->setAlternatingRowColors(true);
    TableMotors->setHorizontalHeaderLabels(headList);
    
    for (int i = 0; i < 16; i++)
    {
        QString stridx = QString::number(i+1);
        PBselector[i] = new QRadioButton(stridx);
        LErole[i] = new QComboBox();
        for (int u = 0; u < axes_names_count; u++)
            LErole[i]->addItem(tr(axes_names[u]), (MotorRoles)u);
        LErole[i]->setStyleSheet(DEFAULT_COMBO_BOX_STYLE);
        Eidx[i] = new Edit(1, 16);
        CBinv[i] = new QCheckBox();
        CBdir[i] = new QCheckBox();
        EMinSpeed[i] = new Edit(1, 10000);
        EMaxSpeed[i] = new Edit(1, 10000);
        EBLdist[i] = new Edit(0, 999, 2);
        EBLdist[i]->setSpecialValueText(tr("none"));
        Epitch[i] = new Edit(0.01, 9, 2);
        Estepsrev[i] = new Edit(1, 99000);
        SWStatus[i] = new StatusWidget(20);
        SWStatus[i]->set(StatusWidget::Square, Qt::black);
        
        TableMotors->setCellWidget(i, 0, PBselector[i]);
        TableMotors->setCellWidget(i, 1, LErole[i]);
        TableMotors->setCellWidget(i, 2, Eidx[i]);
        TableMotors->setCellWidget(i, 3, centerWidget(CBinv[i]));
        TableMotors->setCellWidget(i, 4, centerWidget(CBdir[i]));
        TableMotors->setCellWidget(i, 5, EMinSpeed[i]);
        TableMotors->setCellWidget(i, 6, EMaxSpeed[i]);
        TableMotors->setCellWidget(i, 7, EBLdist[i]);
        TableMotors->setCellWidget(i, 8, Epitch[i]);
        TableMotors->setCellWidget(i, 9, Estepsrev[i]);
        TableMotors->setCellWidget(i, 10, centerWidget(SWStatus[i]));
        
        FillMUI(i);
        
        connect(PBselector[i], SIGNAL(clicked()), this, SLOT(SetCurrent()));
        connect(LErole[i], SIGNAL(currentIndexChanged(int)), this, SLOT(motorRoleChanged()));
        connect(LErole[i], SIGNAL(currentIndexChanged(int)), this, SLOT(save_current()));
        connect(Eidx[i], SIGNAL(newValue()), this, SLOT(save_current()));
        connect(CBinv[i], SIGNAL(clicked()), this, SLOT(save_current()));
        connect(CBdir[i], SIGNAL(clicked()), this, SLOT(save_current()));
        connect(EMinSpeed[i], SIGNAL(newValue()), this, SLOT(save_current()));
        connect(EMaxSpeed[i], SIGNAL(newValue()), this, SLOT(save_current()));
        connect(EBLdist[i], SIGNAL(newValue()), this, SLOT(save_current()));
        connect(Epitch[i], SIGNAL(newValue()), this, SLOT(save_current()));
        connect(Estepsrev[i], SIGNAL(newValue()), this, SLOT(save_current()));
        connect(Eidx[i], SIGNAL(newValue()), this, SLOT(init()));
        connect(CBinv[i], SIGNAL(clicked()), this, SLOT(init()));
        connect(CBdir[i], SIGNAL(clicked()), this, SLOT(init()));
        connect(EBLdist[i], SIGNAL(newValue()), this, SLOT(init()));
    }
    ShowHideRows();
    
    int minWidth = TableMotors->horizontalHeader()->length()+
                   TableMotors->verticalHeader()->width();
    TableMotors->setMinimumWidth(minWidth);
    
    inGbMTableLay->addWidget(ars3000);
    inGbMTableLay->addWidget(pumpStage);
}

void Motors::ShowHideRows()
{
    for (int r = 0; r < TableMotors->rowCount(); r++)
    {
        bool hide = (r >= edNumMotors->value());
        TableMotors->setRowHidden(r, hide);
        if (hide)
        {
            LErole[r]->setCurrentIndex(0);
            CBinv[r]->setChecked(true);
            CBdir[r]->setChecked(false);
            if (PBselector[r]->isChecked())
                PBselector[r]->setChecked(false);
            EMinSpeed[r]->setValue(1);
            EMinSpeed[r]->setNewValue();
            EMaxSpeed[r]->setValue(800);
            EMaxSpeed[r]->setNewValue();
            EBLdist[r]->setValue(0);
            EBLdist[r]->setNewValue();
            //Eidx[r]->setValue(0);
            //Eidx[r]->setNewValue();
        }
    }
}

void Motors::addLandingSett(QWidget *w)
{
    LandingLay->addWidget(w);
}

void Motors::addShuttleSett(QWidget *w, int stretch)
{
    ShuttleLay->addWidget(w, stretch);
}

void Motors::usbStatusChanged(int ixd)
{
#ifdef _NEW_MOTORS_CHANGES
    if (/*ixd == EDok || */ixd == EDno)
    {
        motorsDebug[0]->setText(tr("UART1 = none"));
        motorsDebug[1]->setText(tr("UART2 = none"));
        motorsDebug[2]->setText(tr("UART3 = none"));
        motorsDebugNumMot[0] = -1;
        motorsDebugNumMot[1] = -1;
        motorsDebugNumMot[2] = -1;
    }
#endif
}
