#include "options.h"
#include <QWidget>
#include <QPointer>
#include <QSettings>
#include <QCheckBox>
#include <QGroupBox>
#include <QComboBox>
#include <QRadioButton>
#include <QTimer>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QApplication>
#include <QToolButton>
#include <QMenu>
#include <QDebug>
#include <QTableWidget>
#include <QHeaderView>
#include <QTabWidget>
#include "gui/ttabwidget.hpp"
#include "options_window.h"
#include "modules.hpp"
#include "modules/optionsmodule.h"

options::options(QWidget *parent) : QWidget(parent)
{
    setWindowTitle(tr("Options"));

    QHBoxLayout *mlay = new QHBoxLayout(this);

    TTabs = new TTabWidget;
    mlay->addWidget(TTabs);
    connect(TTabs, SIGNAL(currentChanged(int)), this, SLOT(tabChangedId(int)));
    
    additionLayOverTabs = new QHBoxLayout(TTabs);
    TTabs->installEventFilter(this);

    optWin = new TOptionsWindow(this);
}

void options::addNewTab(QWidget *w, QString name)
{
    TTabs->addTab(w, name);
}

void options::addNewTab(QWidget *w, QString name, int order)
{
    TTabs->insertTab(order, w, name);
}

void options::tabChangedId(int id)
{
    emit tabChanged(TTabs->tabText(id));
}

bool options::eventFilter(QObject *object, QEvent *event)
{
    if (event->type() == QEvent::Resize)
    {
        additionLayOverTabs->setGeometry(TTabs->tabBar()->geometry());
    }
    return QWidget::eventFilter(object, event);
}

options* options::GetIt() noexcept
{
    auto opt = ModuleManager::instance->find1VoidPtr(OptionsModule::name());
    return static_cast<options*>(opt);
}
