#include "options_window.h"
#include "mainwindow.h"
#include "ccd/ccdplot.h"
#include "scanplot.h"
#include "spmtunning.h"
#include "autocalib.hpp"
#include "gui/status.hpp"
#include "gui/ttabwidget.hpp"
#include "options/options.h"
#include "dataplot.h"
#include <QStyledItemDelegate>
#include <QVBoxLayout>
#include <QFormLayout>
#include <QCheckBox>
#include <QSettings>
#include <QTreeWidget>
#include <QHeaderView>
#include <QFontDialog>
#include <QFileDialog>
#include <QToolBar>
#include <QApplication>
#include "container/datalist.hpp"
#include "gui/rectslider.h"
#include "widgets/threedoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"

void TOptionsWindow::selectCustomTR()
{
    if (Ctranslation->currentIndex() != 2)
        return;

    QString f = QFileDialog::getOpenFileName(this, tr("Select custom translation"), QString(), "*.qm", nullptr, QFileDialog::DontUseNativeDialog);
    if (f.isEmpty())
        f = qApp->property("translation").toString();

    if (f.endsWith(":/translation_base"))
        Ctranslation->setCurrentIndex(0);
    else if (f.endsWith(":/translation_russian"))
        Ctranslation->setCurrentIndex(1);
    else {
        Ctranslation->setCurrentIndex(2);
        Ctranslation->setItemData(2, f);
    }

    Apply(true);
}

void TOptionsWindow::saveScanAllFilterSettings()
{
    QSettings settings("Param/scan.conf", QSettings::IniFormat);
    settings.beginGroup("all_plot_filter");
    settings.setValue("enable_by_def", cbByDefEnableFilters->isChecked());
    settings.endGroup();
    settings.setValue("show_help_buttons", cbShowHelpButtons->isChecked());
    settings.setValue("fade_animation_time", edFadeAnimationTimeMS->value());
    plot::ThreeDOverlayButtons::fadingAnimationMS = edFadeAnimationTimeMS->value();
    plot::TwoDOverlayButtons::fadingAnimationMS = edFadeAnimationTimeMS->value();
}

void TOptionsWindow::saveRectSliderMouseWheelPower(int val)
{
    QSettings settings("Param/scan.conf", QSettings::IniFormat);
    settings.setValue("RectSliderMouseWheelPowerForPercents", edRectSliderMouseWheelPowerForPercents->value());
    settings.setValue("RectSliderMouseWheelPowerForValues", edRectSliderMouseWheelPowerForValues->value());

    RectSlider::setWheelPowerForPercents(edRectSliderMouseWheelPowerForPercents->value()/100.0);
    RectSlider::setWheelPowerForValues(edRectSliderMouseWheelPowerForValues->value()/100.0);
}

TOptionsWindow::TOptionsWindow(QWidget *parent) : QWidget(parent, Qt::Dialog)
{
    optionsParent = (options *)parent;
    QSettings settings2("Param/gui.conf", QSettings::IniFormat);
    bool ratisMode = settings2.value("gui/Ratis_xyz_stage_mode", false).toBool();
    initGeneralGUI();
    
    initHardwareOptions(optionsParent, ratisMode);

    //connect(Bedit, SIGNAL(clicked()), configWidget, SLOT(show()));
    //connect(Bedit, SIGNAL(clicked()), Bok, SLOT(click()));

    Show();

    connect(Ctranslation, SIGNAL(currentIndexChanged(int)), this, SLOT(Apply()));
    connect(maxScanPoints, SIGNAL(currentIndexChanged(int)), this, SLOT(Apply()));
    connect(szbox, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(as_time, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(maximumMegabytesInMainContainer, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(chUseSound, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chAutoSendToData, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBgwy_imp, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBremote_net, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBremote_log, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBremote_guest, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBremote_pwd, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(remote_port, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(remote_pwd, SIGNAL(textChanged(const QString &)), this, SLOT(Apply()));
    connect(chDataModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chsSpectroscopyModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chScanModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chSpmModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chOscilloscopeModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chVideoModule1, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chVideoModule2, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chLithographyModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chAboutModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShuttleStatusModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chHeadScannerModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chStageScannerModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chZ3ScannerModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chRatisXYZStageMode, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowBwChan, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowFz, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShow4D, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowLiftMod, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowContScan, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowDataRightMenu, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chUnlockToolbars, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chOpticInitOnStart, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chHideDidntFindedSensorsErrorMessages, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chIgnoreScanSetPointError, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowSecondChannelTab, SIGNAL(toggled(bool)), this, SLOT(Apply()));
#ifdef _OLD_EXTRA_CONTROL
    connect(chShowExtraControl, SIGNAL(toggled(bool)), this, SLOT(Apply()));
#endif
    connect(chShowDeviceControl, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(edRectSliderMouseWheelPowerForPercents, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(edRectSliderMouseWheelPowerForValues, SIGNAL(newValue()), this, SLOT(Apply()));
    #ifdef _SCAN_SLICE
    connect(edScanZoomSlice, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(chEnableScanSlice, SIGNAL(toggled(bool)), this, SLOT(Apply()));
#endif
    connect(chShowScanSlider, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowScanRightMenu, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowSpectrRightMenu, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chMoveXYZToZeroOff, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(edZeroWaitPerChannel, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(chScanSampleControl, SIGNAL(toggled(bool)), this, SLOT(Apply()));
}

void TOptionsWindow::initGeneralGUI()
{
    QSettings settings2("Param/gui.conf", QSettings::IniFormat);
    bool ratisMode = settings2.value("gui/Ratis_xyz_stage_mode", false).toBool();
    QWidget *optWid1 = new QWidget();
    QVBoxLayout *Mainlayout = new QVBoxLayout(optWid1);
    QHBoxLayout *Mainlayout2 = new QHBoxLayout();
    Mainlayout->addLayout(Mainlayout2, 1);
    Mainlayout->addWidget(new QLabel("* - " + tr("Requires program restart")), 0, Qt::AlignBottom);
    QVBoxLayout *plotsLay = new QVBoxLayout();
    Mainlayout2->addLayout(plotsLay, 1);

    if (!ratisMode)
        optionsParent->addNewTab(optWid1, tr("General"), 0);

//----Environment settings-------

    QPushButton *Bfont = new QPushButton(tr("Select font"));
    QPushButton *Bdata = new QPushButton(tr("Select"));
    connect(Bfont, SIGNAL(clicked()), this, SLOT(BFont()));
    connect(Bdata, SIGNAL(clicked()), this, SLOT(BData()));
    szbox = new Edit(10, 500);
    szbox->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    as_time = new Edit(0, 500);
    as_time->setSpecialValueText(tr("Disabled"));
    as_time->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    chUseSound = new QCheckBox();
    chAutoSendToData = new QCheckBox();
    
    maximumMegabytesInMainContainer = new Edit(10, 99999);
    maximumMegabytesInMainContainer->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    
    maxScanPoints = new QComboBox();
    ScanPlot::Set_CBPoints(maxScanPoints, false);
    for (int i=0; i<5; i++) //remove first 5 items
        maxScanPoints->removeItem(0);
    
    Ctranslation = new QComboBox();
    Ctranslation->addItem(tr("English"), QString(":/translation_base"));
    Ctranslation->addItem(tr("Russian"), QString(":/translation_russian"));
    Ctranslation->addItem(tr("Custom..."), QString("translation_russian.qm"));
    connect(Ctranslation, SIGNAL(activated(int)), this, SLOT(selectCustomTR()));

    auto *environmentSettingsGB = new QGroupBox(tr("Environment settings"));
    auto *layEnvironmentSettings = new QVBoxLayout(environmentSettingsGB);
    plotsLay->addWidget(environmentSettingsGB, 0);
    QFormLayout *BoplayoutFormLay = makeFormLayout();

    layEnvironmentSettings->addLayout(BoplayoutFormLay, 1);
    BoplayoutFormLay->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
    BoplayoutFormLay->setVerticalSpacing(10);
    BoplayoutFormLay->addRow(tr("Language *"), Ctranslation);
    BoplayoutFormLay->addRow(tr("Buttons size *"), szbox);
    BoplayoutFormLay->addRow(tr("Font *"), Bfont);
    BoplayoutFormLay->addRow(tr("Data directory"), Bdata);
    BoplayoutFormLay->addRow(tr("Data autosave time (min)"), as_time);
    BoplayoutFormLay->addRow(tr("Maximum megabytes in main container"), maximumMegabytesInMainContainer);
    BoplayoutFormLay->addRow(tr("Use sound notifications"), chUseSound);
    BoplayoutFormLay->addRow(tr("Enable Auto-send to Data from Scan"), chAutoSendToData);
    BoplayoutFormLay->addRow(tr("Max scan points *"), maxScanPoints);

//-------------------------------------

//--------common plot

    QGroupBox *commonPlotSettingsGB = new QGroupBox(tr("Common plot settings"));
    QVBoxLayout *commonPlotSettingsLay = new QVBoxLayout(commonPlotSettingsGB);
    QGridLayout *commonPlotSettingsGL = new QGridLayout();
    commonPlotSettingsLay->addLayout(commonPlotSettingsGL);
    plotsLay->addWidget(commonPlotSettingsGB, 0);

    cbByDefEnableFilters = new QCheckBox(tr("Turn on by default scan filters"));
    commonPlotSettingsLay->addWidget(cbByDefEnableFilters, 1);
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        settings.beginGroup("all_plot_filter");
        cbByDefEnableFilters->setChecked(settings.value("enable_by_def", false).toBool());
        settings.endGroup();
    }
    connect(cbByDefEnableFilters, SIGNAL(toggled(bool)), this, SLOT(saveScanAllFilterSettings()));

    cbShowHelpButtons = new QCheckBox(tr("Show help buttons"));
    commonPlotSettingsLay->addWidget(cbShowHelpButtons, 1);
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        cbShowHelpButtons->setChecked(settings.value("show_help_buttons", true).toBool());
    }
    connect(cbShowHelpButtons, SIGNAL(toggled(bool)), this, SLOT(saveScanAllFilterSettings()));

    edFadeAnimationTimeMS = new Edit(0, 9999, 0);
    auto fadeLay = new QHBoxLayout();
    fadeLay->addWidget(new QLabel(tr("Fade animation time(ms)")));
    fadeLay->addWidget(edFadeAnimationTimeMS);
    commonPlotSettingsLay->addLayout(fadeLay);
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        int time = settings.value("fade_animation_time", 1000).toInt();
        edFadeAnimationTimeMS->setValue(time);
        plot::ThreeDOverlayButtons::fadingAnimationMS = time;
        plot::TwoDOverlayButtons::fadingAnimationMS = time;
    }
    connect(edFadeAnimationTimeMS, SIGNAL(newValue()), this, SLOT(saveScanAllFilterSettings()));
//----------------------------------------

//--------2D plot

    QGroupBox *twoDPlotSettingsGB = new QGroupBox(tr("2D plot settings"));
    QVBoxLayout *twoDPlotSettingsLay = new QVBoxLayout(twoDPlotSettingsGB);
    QGridLayout *twoDPlotSettingsGL = new QGridLayout();
    twoDPlotSettingsLay->addLayout(twoDPlotSettingsGL);
    plotsLay->addWidget(twoDPlotSettingsGB, 0);

    chShowSpectrRightMenu = new QCheckBox(tr("Show spectroscopy right menu") + " *");
    twoDPlotSettingsLay->addWidget(chShowSpectrRightMenu, 0);

//----------------------------------------

//----3D Plot settings
    QGroupBox *plotSettingsGB = new QGroupBox(tr("3D plot settings"));
    QVBoxLayout *plotSettingsLay = new QVBoxLayout(plotSettingsGB);
    plotsLay->addWidget(plotSettingsGB, 1);

    chShowScanRightMenu = new QCheckBox(tr("Show scan right menu") + " *");
    chShowScanSlider = new QCheckBox(tr("Show scan slider") + " *");

    #ifdef _SCAN_SLICE
    edScanZoomSlice = new Edit(1, 99);
    edScanZoomSlice->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    chEnableScanSlice = new QCheckBox(tr("Enable scan bounds slice"));

    QHBoxLayout *scanSliceLay = new QHBoxLayout();
    scanSliceLay->addWidget(new QLabel(tr("Scan bounds slice %")));
    scanSliceLay->addWidget(edScanZoomSlice);
#endif

    plotSettingsLay->addWidget(chShowScanRightMenu, 1);
    plotSettingsLay->addWidget(chShowScanSlider, 1);
    #ifdef _SCAN_SLICE
    plotSettingsLay->addWidget(chEnableScanSlice, 1);
    plotSettingsLay->addLayout(scanSliceLay, 1);
#endif
    
//----------------------------------------

    
    
    QVBoxLayout *secondLay = new QVBoxLayout();
    Mainlayout2->addLayout(secondLay, 1);

//------FUnctionality

    chOpticInitOnStart = new QCheckBox(tr("Enable Optics initialization on start"));

    chUnlockToolbars = new QCheckBox(tr("Unlock toolbars"));
    chUnlockToolbars->hide();
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        if (!settings.value("display/spectroscopy", false).toBool())
            chOpticInitOnStart->hide();
    }
    chShowBwChan = new QCheckBox(tr("Show Bw channel") + " *");
    chShowFz = new QCheckBox(tr("Show F[z]") + " *");
    chShow4D = new QCheckBox(tr("Show 4D") + " *");
    chShowLiftMod = new QCheckBox(tr("Show lift mod") + " *");
    chShowContScan = new QCheckBox(tr("Show continues scan") + " *");
    chShowDataRightMenu = new QCheckBox(tr("Show data right menu") + " *");
    chHideDidntFindedSensorsErrorMessages = new QCheckBox(tr("Hide didn't finded sensors error messages"));
    chIgnoreScanSetPointError = new QCheckBox(tr("Ignore \'Didn't finded setpoint\' errors"));

    edRectSliderMouseWheelPowerForPercents = new Edit(1, 10000);
    edRectSliderMouseWheelPowerForPercents->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    edRectSliderMouseWheelPowerForValues = new Edit(1, 10000);
    edRectSliderMouseWheelPowerForValues->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    QGridLayout *glOtherScan = new QGridLayout();
    glOtherScan->addWidget(new QLabel(tr("Mouse rect slider velocity for percent values")), 0, 0);
    glOtherScan->addWidget(edRectSliderMouseWheelPowerForPercents, 0, 1);
    glOtherScan->addWidget(new QLabel(tr("Mouse rect slider velocity for unit values")), 1, 0);
    glOtherScan->addWidget(edRectSliderMouseWheelPowerForValues, 1, 1);
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        edRectSliderMouseWheelPowerForPercents->setValue(settings.value("RectSliderMouseWheelPowerForPercents", 35).toInt());
        edRectSliderMouseWheelPowerForValues->setValue(settings.value("RectSliderMouseWheelPowerForValues", 65).toInt());

        RectSlider::setWheelPowerForPercents(edRectSliderMouseWheelPowerForPercents->value()/100.0);
        RectSlider::setWheelPowerForValues(edRectSliderMouseWheelPowerForValues->value()/100.0);
    }
    connect(edRectSliderMouseWheelPowerForPercents, SIGNAL(newValue(int)), this, SLOT(saveRectSliderMouseWheelPower(int)));
    connect(edRectSliderMouseWheelPowerForValues, SIGNAL(newValue(int)), this, SLOT(saveRectSliderMouseWheelPower(int)));

    chShowSecondChannelTab = new QCheckBox(tr("Show second channel tab") + " *");
    #ifdef _OLD_EXTRA_CONTROL
    chShowExtraControl = new QCheckBox(tr("Show extra control") + " *");
#endif
    chShowDeviceControl = new QCheckBox(tr("Show device control") + " *");

    QGroupBox *functionalityGB = new QGroupBox(tr("Functionality"));
    QVBoxLayout *functionalityLay = new QVBoxLayout(functionalityGB);
    secondLay->addWidget(functionalityGB, 1);

    functionalityLay->addWidget(chShowBwChan, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowFz, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShow4D, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowLiftMod, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowContScan, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowDataRightMenu, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chUnlockToolbars, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chOpticInitOnStart, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chHideDidntFindedSensorsErrorMessages, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chIgnoreScanSetPointError, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowSecondChannelTab, 1, Qt::AlignVCenter);
    #ifdef _OLD_EXTRA_CONTROL
    functionalityLay->addWidget(chShowExtraControl, 1, Qt::AlignVCenter);
#endif
    functionalityLay->addWidget(chShowDeviceControl, 1, Qt::AlignVCenter);
    functionalityLay->addLayout(glOtherScan, 1);

//--------------------------------

//------Network

    CBgwy_imp = new QCheckBox(tr("Enable Gwyddion import server") + " *");
    CBremote_net = new QCheckBox(tr("Enable Nspec Light server") + " *");
    CBremote_log = new QCheckBox(tr("Enable Nspec Light access logging"));
    CBremote_guest = new QCheckBox(tr("Nspec Light is read-only"));
    CBremote_pwd = new QCheckBox(tr("Nspec Light needs password"));
    remote_pwd = new QLineEdit;
    remote_pwd->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    remote_pwd->setPlaceholderText(tr("empty"));
    remote_pwd->setFixedWidth(140);
    remote_port = new Edit(1, 65535);
    remote_port->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);

    QGroupBox *NetworkGB = new QGroupBox(tr("Network"));
    QVBoxLayout *networkLay = new QVBoxLayout();
    QFormLayout *networkFormLay = makeFormLayout();
    networkFormLay->addRow(tr("Nspec Light password"), remote_pwd);
    networkFormLay->addRow(tr("Nspec Light port *"), remote_port);
    NetworkGB->setLayout(networkLay);
    secondLay->addWidget(NetworkGB, 1);

    networkLay->addWidget(CBgwy_imp, 1, Qt::AlignVCenter);
    networkLay->addWidget(CBremote_net, 1, Qt::AlignVCenter);
    networkLay->addWidget(CBremote_log, 1, Qt::AlignVCenter);
    networkLay->addWidget(CBremote_guest, 1, Qt::AlignVCenter);
    networkLay->addWidget(CBremote_pwd, 1, Qt::AlignVCenter);
    networkLay->addLayout(networkFormLay, 1);


//-------------------------

    QVBoxLayout *thirdLay = new QVBoxLayout();
    Mainlayout2->addLayout(thirdLay, 1);

//-----Modules------------


    chDataModule = new QCheckBox(tr("Data") + " *");
    chsSpectroscopyModule = new QCheckBox(tr("Spectroscopy") + " *");
    chScanModule = new QCheckBox(tr("Scan") + " *");
    chSpmModule = new QCheckBox(tr("SPM") + " *");
    chOscilloscopeModule = new QCheckBox(tr("Oscilloscope") + " *");
    chLithographyModule = new QCheckBox(tr("Lithography") + " *");
    chVideoModule1 = new QCheckBox(tr("Video") + " *");
    chVideoModule2 = new QCheckBox(tr("Video") + " 2" " *");
    chAboutModule = new QCheckBox(tr("About button") + " *");
    chShuttleStatusModule = new QCheckBox(tr("Shuttle Status") + " *");
    chHeadScannerModule = new QCheckBox(tr("Head scanner") + " *");
    chStageScannerModule = new QCheckBox(tr("Stage scanner") + " *");
    chZ3ScannerModule = new QCheckBox(tr("Z3 scanner") + " *");
    chRatisXYZStageMode = new QCheckBox(tr("Ratis XYZ stage mode") + " *");
    chScanSampleControl = new QCheckBox(tr("Sample control") + " *");

    QGroupBox *ModulesGB = new QGroupBox(tr("Modules"));
    QVBoxLayout *Boplayout2 = new QVBoxLayout();
    ModulesGB->setLayout(Boplayout2);
    thirdLay->addWidget(ModulesGB, 1);

    Boplayout2->addWidget(chDataModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chsSpectroscopyModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chScanModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chSpmModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chOscilloscopeModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chVideoModule1, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chVideoModule2, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chLithographyModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chAboutModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chShuttleStatusModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chHeadScannerModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chStageScannerModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chZ3ScannerModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chRatisXYZStageMode, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chScanSampleControl, 1, Qt::AlignVCenter);

//----------------------------------

//-----sofware-of-functionality-----

    auto lMoveXYZToZeroOff = new QLabel(tr("Move X,Y,Z to 0"));
    chMoveXYZToZeroOff = new QCheckBox();

    auto lZeroWaitPerChannel = new QLabel(tr("Wait ms after change channel to zero"));
    edZeroWaitPerChannel = new Edit(1, 10000);
    edZeroWaitPerChannel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);

    QGroupBox *softwareFuncGB = new QGroupBox(tr("Software of functionality"));
    QGridLayout *glSoftwareFunc = new QGridLayout(softwareFuncGB);
    thirdLay->addWidget(softwareFuncGB, 0);

    glSoftwareFunc->addWidget(lMoveXYZToZeroOff, 0, 0);
    glSoftwareFunc->addWidget(chMoveXYZToZeroOff, 0, 1);

    glSoftwareFunc->addWidget(lZeroWaitPerChannel, 1, 0);
    glSoftwareFunc->addWidget(edZeroWaitPerChannel, 1, 1);

//----------------------------------
}

void TOptionsWindow::BFont()
{
    bool ok;
    QFont font_tmp = QFontDialog::getFont(&ok, font, this);

    if (ok) {
        font = font_tmp;
        Apply(true);
    }
}
void TOptionsWindow::BData()
{
    QString newdir = QFileDialog::getExistingDirectory(this, tr("Default data directory"), datadir,
                                                       QFileDialog::ShowDirsOnly | QFileDialog::DontUseNativeDialog);

    if (!newdir.isEmpty())
    {
        datadir = newdir;
        Apply();
    }
}

bool checkSetConfig(QSettings &settings, const QString &name, const QVariant &def_v, const QVariant &new_v)
{
    if (settings.value(name, def_v) != new_v) {
        if (new_v == def_v)
            settings.remove(name);
        else
            settings.setValue(name, new_v);
        return true;
    }
    return false;
}

void TOptionsWindow::Apply(bool definitlyRestart)
{
    bool restart = definitlyRestart;

    QSettings settings("Param/gui.conf", QSettings::IniFormat);

    restart = checkSetConfig(settings, "net/gwy_import_srv", true, CBgwy_imp->isChecked()) || restart;
    restart = checkSetConfig(settings, "net/srv_on", false, CBremote_net->isChecked()) || restart;
    settings.setValue("net/srv_guest", CBremote_guest->isChecked());
    settings.setValue("net/srv_log", CBremote_log->isChecked());
    settings.setValue("net/srv_pwd_on", CBremote_pwd->isChecked());

    settings.setValue("net/srv_pwd", remote_pwd->text());
    restart = checkSetConfig(settings, "net/srv_port", 33331, remote_port->value()) || restart;

    restart = checkSetConfig(settings, "gui/Ratis_xyz_stage_mode", false, chRatisXYZStageMode->isChecked()) || restart;
    if (chRatisXYZStageMode->isChecked())
    {
        chDataModule->setChecked(false);
        chsSpectroscopyModule->setChecked(false);
        chSpmModule->setChecked(false);
        chVideoModule1->setChecked(false);
        chVideoModule2->setChecked(false);
        chLithographyModule->setChecked(false);
        chAboutModule->setChecked(false);
        chShuttleStatusModule->setChecked(false);
        chZ3ScannerModule->setChecked(false);
        chHeadScannerModule->setChecked(false);
    }

    //TODO table/loop
    restart = checkSetConfig(settings, "display/data", true, chDataModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/spectroscopy", false, chsSpectroscopyModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/scan", true, chScanModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/spm", true, chSpmModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope", true, chOscilloscopeModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/video", true, chVideoModule1->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/video2", false, chVideoModule2->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/lithography", false, chLithographyModule->isChecked()) || restart;

    restart = checkSetConfig(settings, "display/oscilloscope/head", true, chHeadScannerModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope/stage", true, chStageScannerModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope/z3", false, chZ3ScannerModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/about", true, chAboutModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/shuttle_status", true, chShuttleStatusModule->isChecked()) || restart;

    restart = checkSetConfig(settings, "gui/show_bw_chan", true, chShowBwChan->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_fz", true, chShowFz->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_4d", true, chShow4D->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_lift_mod", true, chShowLiftMod->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_cont_scan", true, chShowContScan->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_second_chan", true, chShowSecondChannelTab->isChecked()) || restart;
    #ifdef _OLD_EXTRA_CONTROL
    restart = checkSetConfig(settings, "gui/show_extra_control", true, chShowExtraControl->isChecked()) || restart;
#endif
    restart = checkSetConfig(settings, "gui/show_device_control", true, chShowDeviceControl->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_scan_slider", true, chShowScanSlider->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_scan_right_menu", true, chShowScanRightMenu->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_data_right_menu", true, chShowDataRightMenu->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_spectr_right_menu", true, chShowSpectrRightMenu->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/sample_control", false, chScanSampleControl->isChecked()) || restart;

    settings.setValue("scan/autosend_data", chAutoSendToData->isChecked());
    settings.setValue("autosave_time", as_time->value());
    settings.setValue("maximumMegabytesInMainContainer", maximumMegabytesInMainContainer->value());
    TContainer::maxMegabytesInMainContainer = maximumMegabytesInMainContainer->value();
    settings.setValue("hide_didnt_finded_sensors_error_messages", chHideDidntFindedSensorsErrorMessages->isChecked());
    settings.setValue("ignore_scan_setpoint_errors", chIgnoreScanSetPointError->isChecked());
    

    #ifdef _SCAN_SLICE
    settings.setValue("scan_zoom_slice_enabled", chEnableScanSlice->isChecked());
    settings.setValue("scan_zoom_slice", edScanZoomSlice->value());
#endif

    settings.setValue("firmware/setXYZto0", chMoveXYZToZeroOff->isChecked());
    settings.setValue("firmware/waitXYZto0", edZeroWaitPerChannel->value());
    
    emit onSettingsChanged(CSTF_AUTOSENDING_SAVE);
    
    //todo REFACTOR. move that code away.
    //TODO reloadConfig() signal
    if (auto scan = ScanPlot::GetIt()) {
        scan->Autosend = chAutoSendToData->isChecked();

        //scan->scan_slice();
    }
    //todo REFACTOR. move that code away.
    if (auto spm = SPMTunning::GetIt()) {
        spm->AutoSend = chAutoSendToData->isChecked();
    }
    //todo REFACTOR. move that code away.
    if (auto ccd = CCDPlot::GetIt()) {
        ccd->AutoSend = chAutoSendToData->isChecked();
    }
    //todo REFACTOR. move that code away.
    if (auto data = DataPlot::GetIt()) {
        data->Container->SetAutoSaveTime(as_time->value()*60*1000);
    }

    if (checkSetConfig(settings, "gui_unlock", false, chUnlockToolbars->isChecked()))
    {
        auto mainWindow = MainWindow::GetIt();
        foreach (QToolBar *t, mainWindow->findChildren<QToolBar *>())
            t->setMovable(settings.value("gui_unlock", false).toBool());
    }
    settings.setValue("sound_notification", chUseSound->isChecked());

    if (font != qApp->font()) {
        settings.setValue("font", font.toString());
        restart = true;
    }
    if (datadir != settings.value("data_dir", TContainer::getDataDir()).toString())
        settings.setValue("data_dir", datadir);
    
    if (maxScanPoints->currentIndex() != settings.value("maxScanPoints", 0).toInt())
    {
        settings.setValue("maxScanPoints", maxScanPoints->currentIndex());
        restart = true;
    }

    if (Ctranslation->itemData(Ctranslation->currentIndex()) !=
            settings.value("translation", QString(":/translation_base")).toString())
    {
        settings.setValue("translation", Ctranslation->itemData(Ctranslation->currentIndex()));
        restart = true;
    }

    int Bsize = szbox->value();
    if (Bsize != qApp->property("buttonsize").toInt()) {
        settings.setValue("buttonsize", Bsize);
        restart = true;
    }

    if (restart) {
        new Notification(tr("Settings") + ": " + tr("Please restart application to apply changes."), Notification::Good, 10);
        doBeep();
    }

    hide();
}

void TOptionsWindow::Show()
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);

    CBgwy_imp->setChecked(settings.value("net/gwy_import_srv", true).toBool());
    CBremote_net->setChecked(settings.value("net/srv_on", false).toBool());
    CBremote_guest->setChecked(settings.value("net/srv_guest", true).toBool());
    CBremote_log->setChecked(settings.value("net/srv_log", true).toBool());
    CBremote_pwd->setChecked(settings.value("net/srv_pwd_on", false).toBool());

    remote_pwd->setText(settings.value("net/srv_pwd", "").toString());
    remote_port->setValue(settings.value("net/srv_port", 33331).toInt());

    chDataModule->setChecked(settings.value("display/data", true).toBool());
    chsSpectroscopyModule->setChecked(settings.value("display/spectroscopy", false).toBool());
    chScanModule->setChecked(settings.value("display/scan", true).toBool());
    chSpmModule->setChecked(settings.value("display/spm", true).toBool());
    chOscilloscopeModule->setChecked(settings.value("display/oscilloscope", true).toBool());
    chVideoModule1->setChecked(settings.value("display/video", true).toBool());
    chVideoModule2->setChecked(settings.value("display/video2", false).toBool());
    chLithographyModule->setChecked(settings.value("display/lithography", false).toBool());

    chHeadScannerModule->setChecked(settings.value("display/oscilloscope/head", true).toBool());
    chStageScannerModule->setChecked(settings.value("display/oscilloscope/stage", true).toBool());
    chZ3ScannerModule->setChecked(settings.value("display/oscilloscope/z3", false).toBool());
    chAboutModule->setChecked(settings.value("display/about", true).toBool());
    chShuttleStatusModule->setChecked(settings.value("display/shuttle_status", true).toBool());

    chAutoSendToData->setChecked(settings.value("scan/autosend_data", true).toBool());
    as_time->setValue(settings.value("autosave_time", 30).toInt());
    maximumMegabytesInMainContainer->setValue(settings.value("maximumMegabytesInMainContainer", 2000).toInt());
    maxScanPoints->setCurrentIndex(settings.value("maxScanPoints", 0).toInt());
    chOpticInitOnStart->setChecked(settings.value("autoinit_ccd", false).toBool());

    chUseSound->setChecked(settings.value("sound_notification", true).toBool());
    chUnlockToolbars->setChecked(true);

    chShowBwChan->setChecked(settings.value("gui/show_bw_chan", true).toBool());
    chShowFz->setChecked(settings.value("gui/show_fz", true).toBool());
    chShow4D->setChecked(settings.value("gui/show_4d", true).toBool());
    chShowLiftMod->setChecked(settings.value("gui/show_lift_mod", true).toBool());
    chShowContScan->setChecked(settings.value("gui/show_cont_scan", true).toBool());
    chShowSecondChannelTab->setChecked(settings.value("gui/show_second_chan", true).toBool());
    #ifdef _OLD_EXTRA_CONTROL
    chShowExtraControl->setChecked(settings.value("gui/show_extra_control", true).toBool());
#endif
    chShowDeviceControl->setChecked(settings.value("gui/show_device_control", true).toBool());
    chShowScanSlider->setChecked(settings.value("gui/show_scan_slider", true).toBool());
    chShowScanRightMenu->setChecked(settings.value("gui/show_scan_right_menu", true).toBool());
    chShowDataRightMenu->setChecked(settings.value("gui/show_data_right_menu", true).toBool());
    chShowSpectrRightMenu->setChecked(settings.value("gui/show_spectr_right_menu", true).toBool());
    chRatisXYZStageMode->setChecked(settings.value("gui/Ratis_xyz_stage_mode", false).toBool());
    chHideDidntFindedSensorsErrorMessages->setChecked(settings.value("hide_didnt_finded_sensors_error_messages", false).toBool());
    chIgnoreScanSetPointError->setChecked(settings.value("ignore_scan_setpoint_errors", false).toBool());
    chMoveXYZToZeroOff->setChecked(settings.value("firmware/setXYZto0", false).toBool());
    #ifdef _SCAN_SLICE
    edScanZoomSlice->setValue(settings.value("scan_zoom_slice", 10).toDouble());
    chEnableScanSlice->setChecked(settings.value("scan_zoom_slice_enabled", false).toBool());
#endif
    edZeroWaitPerChannel->setValue(settings.value("firmware/waitXYZto0", 200).toInt());
    chScanSampleControl->setChecked(settings.value("gui/sample_control", false).toBool());

    font = qApp->font();
    datadir = settings.value("data_dir", TContainer::getDataDir()).toString();

    szbox->setValue(qApp->property("buttonsize").toInt());

    QString transl = qApp->property("translation").toString();

    if (transl.endsWith(":/translation_base"))
        Ctranslation->setCurrentIndex(0);
    else if (transl.endsWith(":/translation_russian"))
        Ctranslation->setCurrentIndex(1);
    else {
        Ctranslation->setCurrentIndex(2);
        Ctranslation->setItemData(2, transl);
    }
}
