#pragma once

#include <QWidget>
#include "edit.hpp"
//#include "3d/plot_3dplot.h"
#include "options/options.h"
#include "hardware/EG3000.h"

namespace plot {
    class T3dPlot;
    class T3dPlotControl;
}

class QCheckBox;
class QTreeWidget;
class QComboBox;
class QTreeWidgetItem;
class QSettings;

enum changedSettingsTypeFlag
{
    CSTF_AUTOSENDING_SAVE =              1 << 0,
    CSTF_DEVICE_CONFIG_CONTROLLER_CONF = 1 << 1,
};

class TOptionsWindow: public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT
public:
    TOptionsWindow(QWidget *parent);

    QCheckBox *chShowScanRightMenu;
    QCheckBox *chShowDataRightMenu;
    QCheckBox *chShowSpectrRightMenu;
    QCheckBox *chRatisXYZStageMode; //mode when everything is hidded except oscilloscope
    QCheckBox *chHideDidntFindedSensorsErrorMessages;
    QCheckBox *chIgnoreScanSetPointError;

    #ifdef _SCAN_SLICE
    QCheckBox *chEnableScanSlice;
    Edit *edScanZoomSlice;
#endif

    QCheckBox *chMoveXYZToZeroOff;
    Edit *edZeroWaitPerChannel;
    
public: signals:
    void onSettingsChanged(changedSettingsTypeFlag type);
    
public slots:
    void selectCustomTR();
    void Show();
    void Apply(bool definitlyRestart = false);
    void BFont();
    void BData();
    void configEdited(QTreeWidgetItem *item, int column);
    void editorResize();
    void saveRectSliderMouseWheelPower(int);
    void saveScanAllFilterSettings();
    
private slots:
    void devGlobalConfigSettingsChanged(changedSettingsTypeFlag type);
    void devGlobalConfigLoad();
    
private:
    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) override;
    bool sendData(const void *handle) override;
    void resetWriter() override;
    void setItemBoolValue(QTreeWidgetItem* item, bool value);
    
    QTreeWidgetItem* addConfig(QSettings &settings, const QString &file, const QString &path, const QVariant &default_val, const QString &comment = "");
    QTreeWidgetItem* addConfig(QSettings &settings, const QString &file, const QString &path, const QString &comment = "");
    QTreeWidgetItem* addConfig(const QString &file, const QString &path, const QVariant &default_val, const QString &comment = "");
    QTreeWidgetItem* addConfig(const QString &file, const QString &path, const QString &comment = "");

    void initGeneralGUI();
    void initHardwareOptions(options *optionsParent, bool ratisMode);
    void initHardwareOptions_emulator();
    void initHardwareOptions_ccd();
    void initHardwareOptions_ars();
    void initHardwareOptions_controller();
    void initHardwareOptions_motor();
    void initHardwareOptions_shuttle();
    void initHardwareOptions_laser();
    void initHardwareOptions_analog();
    void initHardwareOptions_landing();

    options *optionsParent;

    QTreeWidget *editor;
    QMap<QString, QTreeWidgetItem *> items;

    QComboBox *Ctranslation;
    Edit *szbox;
    Edit *as_time;
    Edit *maximumMegabytesInMainContainer;
    QComboBox *maxScanPoints;

    QFont font;
    QString datadir;

    QCheckBox *CBgwy_imp;
    QCheckBox *CBremote_net;
    QCheckBox *CBremote_pwd;
    QCheckBox *CBremote_log;
    QCheckBox *CBremote_guest;

    Edit *remote_port;
    QLineEdit *remote_pwd;

    QCheckBox *chDataModule;
    QCheckBox *chsSpectroscopyModule;
    QCheckBox *chScanModule;
    QCheckBox *chSpmModule;
    QCheckBox *chOscilloscopeModule;
    QCheckBox *chLithographyModule;
    QCheckBox *chVideoModule1;
    QCheckBox *chVideoModule2;

    QCheckBox *chHeadScannerModule;
    QCheckBox *chStageScannerModule;
    QCheckBox *chZ3ScannerModule;
    QCheckBox *chAboutModule;
    QCheckBox *chShuttleStatusModule;

    QCheckBox *chAutoSendToData;
    QCheckBox *chOpticInitOnStart;
    QCheckBox *chUseSound;
    QCheckBox *chUnlockToolbars;


    QCheckBox *chShowBwChan;
    QCheckBox *chShowFz;
    QCheckBox *chShow4D;
    QCheckBox *chShowLiftMod;
    QCheckBox *chShowContScan;
    QCheckBox *chShowSecondChannelTab;
    #ifdef _OLD_EXTRA_CONTROL
    QCheckBox *chShowExtraControl;
#endif
    QCheckBox *chShowDeviceControl;
    QCheckBox *chShowScanSlider;
    QCheckBox *cbByDefEnableFilters;
    QCheckBox *cbShowHelpButtons;
    QCheckBox *chScanSampleControl;
    Edit *edFadeAnimationTimeMS;

    Edit *edRectSliderMouseWheelPowerForPercents;
    Edit *edRectSliderMouseWheelPowerForValues;
    
    QList<QTreeWidgetItem*> devGlobalConfigEditorItemList;
    
    TMessageFlag sendGlobalConfig;
    TMessageFlag sendGetGlobalConfig;
};
