#include "options_window.h"

#include <QTreeView>
#include <QTreeWidget>
#include <QStyledItemDelegate>
#include <QVBoxLayout>
#include <QFormLayout>
#include <QCheckBox>
#include <QSettings>
#include <QTreeWidget>
#include <QHeaderView>
#include <QFontDialog>
#include <QFileDialog>
#include <QToolBar>
#include <QApplication>
#include <QLabel>
#include "i_eg3000.h"

class EditorDelegate: public QStyledItemDelegate
{
public:
    EditorDelegate(QObject *parent) : QStyledItemDelegate(parent) {}
    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const override
    {/*
        if (index.data().toString() == "false" || index.data().toString() == "true") {
            QCheckBox *box = new QCheckBox(parent);
            box->setAutoFillBackground(true);
            return box;
        }*/
        return index.column() != 1 ? nullptr : QStyledItemDelegate::createEditor(parent, option, index);
    }/*
    void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
    {
        QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
        if (!edit)
            return QStyledItemDelegate::setModelData(editor, model, index);

    model->setData(index, edit->isChecked(), Qt::EditRole);
}
void setEditorData(QWidget *editor, const QModelIndex &index) const
{
    QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
    if (!edit)
        return QStyledItemDelegate::setEditorData(editor, index);
    
    edit->setChecked(index.model()->data(index, Qt::EditRole).toBool());
}
void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
    if (!edit)
        return QStyledItemDelegate::updateEditorGeometry(editor, option, index);
    
    edit->setGeometry(option.rect);
}*/
};

void TOptionsWindow::initHardwareOptions(options *optionsParent, bool ratisMode)
{
    editor = new QTreeWidget();
    editor->setItemDelegate(new EditorDelegate(this));
    
    QWidget *configWidget = new QWidget;
    QVBoxLayout *cl1 = new QVBoxLayout(configWidget);
    if (!ratisMode)
        optionsParent->addNewTab(configWidget, tr("Hardware settings"), 1);
    
    cl1->addWidget(editor);
    
    QHBoxLayout *cwL = new QHBoxLayout;
    cwL->addWidget(new QLabel(tr("* - requires restart of program/device to apply")));
    cl1->addLayout(cwL);
    
    QStringList header;
    header.append(tr("File")+"/"+tr("Variable"));
    header.append(tr("Value"));
    header.append(tr("Comment"));
    editor->setColumnCount(3);
    editor->setHeaderLabels(header);
    int size = qApp->font().pixelSize();
    if (size < 1) size = 1;
    editor->header()->setMinimumHeight(size);
    
    initHardwareOptions_emulator();
    initHardwareOptions_ccd();
    initHardwareOptions_ars();
    initHardwareOptions_controller();
    initHardwareOptions_motor();
    initHardwareOptions_shuttle();
    initHardwareOptions_laser();
    initHardwareOptions_analog();
    initHardwareOptions_landing();
    
    //editor->header()->resizeSections(QHeaderView::ResizeToContents);
    
    connect(editor, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(configEdited(QTreeWidgetItem*,int)));
    connect(editor, SIGNAL(collapsed(QModelIndex)), this, SLOT(editorResize()));
    connect(editor, SIGNAL(expanded(QModelIndex)), this, SLOT(editorResize()));
    
    connect(this, SIGNAL(onSettingsChanged(changedSettingsTypeFlag)),
            this, SLOT(devGlobalConfigSettingsChanged(changedSettingsTypeFlag)));
}

void TOptionsWindow::initHardwareOptions_emulator()
{
#ifdef USE_EG3K_EMULATOR
    addConfig("Emulator.conf", "usb", false, "* " + tr("Use EG3000 emulator"));
#endif
#ifdef USE_CCD_EMULATOR
    addConfig("Emulator.conf", "ccd", false, "* " + tr("Use CCD emulator"));
#endif
    addConfig("Emulator.conf", "auto_tip_landing", false, "* " + tr("Use tip landing simulator"));
    addConfig("Emulator.conf", "sensor_simulator", false, "* " + tr("Enable sensor simulator option"));
}

void TOptionsWindow::initHardwareOptions_ccd()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    addConfig(settings, "ccd.conf", "/", tr("Device selection, parameters"));
    addConfig(settings, "ccd.conf", "scale_reverse", true, tr("CCD x axis mirror"));
    addConfig(settings, "ccd.conf", "use_pmt", false, "* " + tr("Use internal PMTs"));
    addConfig(settings, "ccd.conf", "use_ormins", false, "* " + tr("Use Ormins CCD"));
    addConfig(settings, "ccd.conf", "use_sol_ccd", false, "* " + tr("Use Sol CCD"));
    addConfig(settings, "ccd.conf", "use_ars", true, "* " + tr("Use ARS3000 confocal module"));
    addConfig(settings, "ccd.conf", "use_m266", false, "* " + tr("Use M266 instead of M833 spectrograph"));
    addConfig(settings, "ccd.conf", "use_shamrock", false, "* " + tr("Use Shamrock spectrograph"));
    addConfig(settings, "ccd.conf", "use_solart2", false, "* " + tr("Use SolarT2 spectrograph"));
    addConfig(settings, "ccd.conf", "baseline_before_intensity", false, "* ");
    addConfig(settings, "ccd.conf", "andor_fast", false, tr("Enable fast Andor mode"));
    addConfig(settings, "ccd.conf", "shutter_open", 30, tr("opening time, ms"));
    addConfig(settings, "ccd.conf", "shutter_close", 30, tr("closing time, ms"));
}

void TOptionsWindow::initHardwareOptions_ars()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_ars", true).toBool())
    {
        QSettings settings("Param/ars.conf", QSettings::IniFormat);
        addConfig(settings, "ars.conf", "motor_k/", tr("Distance correction factor from phys to virtual(global), mm/mm"));
        const char *mnames[6] = { QT_TR_NOOP("laser"),
                                 QT_TR_NOOP("slit0/in"),
                                 QT_TR_NOOP("slit1/nd_filter") };
        for (int i = 0; i < 6; i++)
            addConfig(settings, "ars.conf", "motor_k/"+QString::number(i), 1., tr(mnames[i]));
        addConfig(settings, "ars.conf", "laser_select_pos/", tr("Motor pos for each laser, mm"));
        for (int i = 0; i < 3; i++)
            addConfig(settings, "ars.conf", "laser_select_pos/"+QString::number(i), i*10);
        addConfig(settings, "ars.conf", "slit_k/", tr("Slit correction factor from width to motor dist, mm/mm"));
        for (int i = 0; i < 2; i++)
            addConfig(settings, "ars.conf", "slit_k/"+QString::number(i), 1);
        addConfig(settings, "ars.conf", "slit_zeropos/", tr("Slit dist from limit switch to zero pos, mm"));
        for (int i = 0; i < 2; i++)
            addConfig(settings, "ars.conf", "slit_zeropos/"+QString::number(i), 0.); //fuck msvc
    }
}

void TOptionsWindow::initHardwareOptions_controller()
{
    QSettings settings("Param/controller.conf", QSettings::IniFormat);
    addConfig(settings, "controller.conf", "/", tr("Device selection, limits, autocalib parameters, etc."));
    addConfig(settings, "controller.conf", "use_eg4000", true, tr("Enable libusb - EG4000 and later"));
    addConfig(settings, "controller.conf", "csb_speed_settings_idx", 0, "* " + tr("sensor rate index in rate table"));
    addConfig(settings, "controller.conf", "double_c_output", 0, "* " + tr("C output with two sensors on channel. 0=C1-C2, 1=C1, 2=C2"));
    addConfig(settings, "controller.conf", "limit_scan_rate", 10, tr("Maximum scan rate, Hz"));
    addConfig(settings, "controller.conf", "sensors_autocalib_gain", 100, tr("Gain to use in calibration process"));
    addConfig(settings, "controller.conf", "sensors_autocalib_reserve", 0.025, tr("% of overcalibration, 1.0 = 100%"));
    QString ctrps = tr("Center position with account for nonlinearity");
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_x_h", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_y_h", 0.48, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_z_h", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_x_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_y_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_z_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_optical_range_correction", 1.1, tr("divider for optical sensor ranges"));
    addConfig(settings, "controller.conf", "auto_i_divisor", 500, tr("I = P/divisor - when I is auto-adjusted"));
    addConfig(settings, "controller.conf", "pi_autocalib_p_scale", 0.45, tr("Multiplier for P"));
    addConfig(settings, "controller.conf", "pi_autocalib_i_scale", 10, tr("Divisor for I"));
    addConfig(settings, "controller.conf", "pi_autocalib_spm_p_scale", 0.44, tr("Multiplier for P"));
    addConfig(settings, "controller.conf", "pi_autocalib_spm_i_scale", 2, tr("Divisor for I"));
    addConfig(settings, "controller.conf", "hvsplit/z2_k", 0, tr("scale from slow to fast. 0 means disabled"));
    addConfig(settings, "controller.conf", "hvsplit/z2_freq", 1, tr("lpf freq khz"));
    addConfig(settings, "controller.conf", "mult_spm_pid_head", 1, tr("Multiply factor for head SPM P/I/D"));
    addConfig(settings, "controller.conf", "mult_spm_pid_stage", 1, tr("Multiply factor for stage SPM P/I/D"));
    addConfig(settings, "controller.conf", "autocalib_pid", false, tr("Enable PID autotuning on start"));
    addConfig(settings, "controller.conf", "autocalib_sensors", false, tr("Enable Sensors autocalibration on start"));
    
    devGlobalConfigEditorItemList.append(
        addConfig(settings, "controller.conf", "aib_present", false, "* " + tr("Enables AIB read from ADC HR 1/2")));
    devGlobalConfigEditorItemList.append(
        addConfig(settings, "controller.conf", "direct_sensors/direct_sensors", false, "* " + tr("Fast counter in lockin")));
    devGlobalConfigEditorItemList.append(
        addConfig(settings, "controller.conf", "direct_sensors/dir_sens_swap_XY", false, "* " + tr("Swap XY input pins")));
    devGlobalConfigEditorItemList.append(
        addConfig(settings, "controller.conf", "direct_sensors/dir_sens_invert_X", false, "* " + tr("Invert X")));
    devGlobalConfigEditorItemList.append(
        addConfig(settings, "controller.conf", "direct_sensors/dir_sens_invert_Y", false, "* " + tr("Invert Y")));
    devGlobalConfigEditorItemList.append(
        addConfig(settings, "controller.conf", "direct_sensors/dir_sens_invert_Z", false, "* " + tr("Invert Z")));
}

void TOptionsWindow::initHardwareOptions_motor()
{
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    addConfig(settings, "motor.conf", "csb_motors_addr", -1, tr("-1, 0 - leave default, 1-head, 2-stage, 3-z3, 5-motors"));

#ifdef _NEW_MOTORS_CHANGES
    addConfig(settings, "motor.conf", "csb_motors_count", 255, "* " + tr("255 - default/auto, 0.. - count for 1st channel"));
    addConfig(settings, "motor.conf", "csb_motors_count2", 0, "* " + tr("count for 2nd channel when not default/auto"));
    addConfig(settings, "motor.conf", "csb_motors_count3", 0, "* " + tr("count for 3rd channel when not default/auto"));
#else
    addConfig(settings, "motor.conf", "csb_motors_count", -1, "* " + tr("-1 - default/auto, 0.. - count"));
#endif
}

void TOptionsWindow::initHardwareOptions_shuttle()
{
    addConfig("shuttle.conf", "shuttle_xpress", false, "* " + tr("Enable small model buttons"));
    addConfig("shuttle.conf", "piezo/delay", 50, "* " + tr("Delay between steps, ms"));
}

void TOptionsWindow::initHardwareOptions_laser()
{
#ifdef _ENABLE_NST_LASERS
    QSettings settings("Param/laser.conf", QSettings::IniFormat);
    if (settings.allKeys().size())
        addConfig(settings, "laser.conf", "/", tr("See readme.txt"));
    
    foreach (QString key, settings.allKeys())
        if (!key.contains("nstLaser") && key != "useCoboltLasers")
            addConfig(settings, "laser.conf", key);
    
    addConfig(settings, "laser.conf", "useCoboltLasers", false, "* " + tr("Use Cobolt lasers"));
    addConfig(settings, "laser.conf", "nstLaserAlpha", false, "* " + tr("NST laser Aplha 785"));
    addConfig(settings, "laser.conf", "nstLaserBeta", false, "* " + tr("NST laser Beta 532"));
    addConfig(settings, "laser.conf", "nstLaserGamma", false, "* " + tr("NST laser Gamma 405"));
    addConfig(settings, "laser.conf", "nstLaserDelta", false, "* " + tr("NST laser Delta 633"));
    addConfig(settings, "laser.conf", "nstLaserEtta", false, "* " + tr("NST laser Etta Light Source"));
#endif
}

void TOptionsWindow::initHardwareOptions_analog()
{
    QSettings settings("Param/analog.conf", QSettings::IniFormat);
    addConfig(settings, "analog.conf", "/", tr("Analog inputs/outputs calibrations"));
    QString smin = tr("Min/max voltage for ");
    QString sadc = tr("ADC");
    QString sdac = tr("DAC");
    QStringList l, c;
    l << "1" << "1mag" << "2" << "2mag" << "3" << "4" << "5" << "1hr" << "2hr";
    c << "1/DFL" << "1-MAG" << "2/LFL/I_TIP" << "2-MAG" << "3/INT" << "4" << "5" << "AUX 1" << "AUX 2";
    addConfig(settings, "analog.conf", "adc/", smin + sadc);
    for (int i = 0; i < l.size() && i < c.size(); i++) {
        addConfig(settings, "analog.conf", "adc/v" + l.at(i) + "_min", -10, c.at(i));
        addConfig(settings, "analog.conf", "adc/v" + l.at(i) + "_max", 10, c.at(i));
    }
    
    addConfig(settings, "analog.conf", "dac/", smin + sdac);
    addConfig(settings, "analog.conf", "dac/vdrive_min", 0, "* " + tr("Drive"));
    addConfig(settings, "analog.conf", "dac/vdrive_max", 10, "* " + tr("Drive"));
    
    l.clear(); c.clear();
    l << "tip" << "1" << "2" << "3hr" << "4hr";
    c << tr("Tip") << "1" << "2" << "AUX 1" << "AUX 2";
    for (int i = 0; i < l.size() && i < c.size(); i++) {
        addConfig(settings, "analog.conf", "dac/v" + l.at(i) + "_min", -10, "* " + c.at(i));
        addConfig(settings, "analog.conf", "dac/v" + l.at(i) + "_max", 10, "* " + c.at(i));
    }
    
    l.clear(); c.clear();
    l << "hvX1" << "hvY1" << "hvZ1" << "svX2" << "svY2" << "svZ2";
    c << tr("head") + " X1" << tr("head") + " Y1" << tr("head") + " Z1"
      << tr("stage") + " X2" << tr("stage") + " Y2" << tr("stage") + " Z2";
    for (int i = 0; i < l.size() && i < c.size(); i++) {
        addConfig(settings, "analog.conf", "dac/" + l.at(i) + "_min", -15., "* " + c.at(i));
        addConfig(settings, "analog.conf", "dac/" + l.at(i) + "_max", 165., "* " + c.at(i));
    }
    
    addConfig(settings, "analog.conf", "dac/z3_min", -15., "* " + tr("Z3"));
    addConfig(settings, "analog.conf", "dac/z3_max", 165., "* " + tr("Z3"));
    
    addConfig(settings, "analog.conf", "pmt/", tr("Min/max voltage for PMT"));
    addConfig(settings, "analog.conf", "pmt/hv1_umax", 1000, "* " + tr("HV1 Umax"));
    addConfig(settings, "analog.conf", "pmt/dacaux1_min", -10, "* " + tr("DAC AUX 1 min"));
    addConfig(settings, "analog.conf", "pmt/dacaux1_max", 10, "* " + tr("DAC AUX 1 max"));
    addConfig(settings, "analog.conf", "pmt/hv2_umax", 1000, "* " + tr("HV2 Umax"));
    addConfig(settings, "analog.conf", "pmt/dacaux2_min", -10, "* " + tr("DAC AUX 2 min"));
    addConfig(settings, "analog.conf", "pmt/dacaux2_max", 10, "* " + tr("DAC AUX 2 max"));
}

void TOptionsWindow::initHardwareOptions_landing()
{
    QSettings settings("Param/landing.conf", QSettings::IniFormat);
    addConfig(settings, "landing.conf", "/", tr("Tip monitor settings"));
    addConfig(settings, "landing.conf", "max_tip_drive_amplitude", 3.0, tr("Maximum tip drive amplitude"));
    addConfig(settings, "landing.conf", "auto_land_free_amplitude", 2.0, tr("Auto landing free amplitude"));
    addConfig(settings, "landing.conf", "auto_land_step_size", 0.1, tr("Auto landing step size"));
    addConfig(settings, "landing.conf", "auto_land_step_wait_ms", 50, tr("Auto landing wait step ms"));
    addConfig(settings, "landing.conf", "auto_land_stage_sens_calib", true, tr("Auto landing enable stage - \"Auto sens calibration\""));
    addConfig(settings, "landing.conf", "auto_land_stage_frequency_scan", true, tr("Auto landing enable stage - \"Frequency scan\""));
}

QTreeWidgetItem* TOptionsWindow::addConfig(const QString &file, const QString &path, const QString &comment)
{
    return addConfig(file, path, QVariant(), comment);
}

QTreeWidgetItem* TOptionsWindow::addConfig(const QString &file, const QString &path, const QVariant &default_val, const QString &comment)
{
    QSettings settings("Param/" + file, QSettings::IniFormat);
    return addConfig(settings, file, path, default_val, comment);
}

QTreeWidgetItem* TOptionsWindow::addConfig(QSettings &settings, const QString &file, const QString &path, const QString &comment)
{
    return addConfig(settings, file, path, QVariant(), comment);
}

QTreeWidgetItem* TOptionsWindow::addConfig(QSettings &settings, const QString &file, const QString &path, const QVariant &default_val, const QString &comment)
{
#ifdef NDEBUG
    //In debug empty key(path) causing an error
    QVariant value = settings.value(path, default_val);
#else
    QVariant value = (path == "/") ? QVariant() : settings.value(path, default_val);
#endif
    
    QStringList pathparts = path.split("/");
    QString lastpart = pathparts.last();
    pathparts.removeLast();
    pathparts.prepend(file);
    
    QTreeWidgetItem *root;
    QString acc;
    while (!pathparts.isEmpty()) {
        if (!items.contains(acc + pathparts.first())) {
            if (acc.isEmpty())
                root = new QTreeWidgetItem(editor, QStringList() << file);
            else
                root = new QTreeWidgetItem(root, QStringList() << pathparts.first());
            
            items.insert(acc + pathparts.first(), root);
        } else
            root = items.value(acc + pathparts.first());
        
        acc += pathparts.first();
        pathparts.removeFirst();
    }
    if (lastpart.isEmpty()) {
        root->setText(2, comment);
        return nullptr;
    }
    
    editor->setFont(qApp->font());
    QTreeWidgetItem *item = new QTreeWidgetItem(root, QStringList() << lastpart << value.toString() << comment);
    item->setData(0, Qt::UserRole, file);
    item->setData(0, Qt::UserRole+1, path);
    item->setData(0, Qt::UserRole+3, default_val);
    
    if (value.toString() == "false" || value.toString() == "true") {
        item->setCheckState(1, value.toBool() ? Qt::Checked : Qt::Unchecked);
        item->setData(1, Qt::DisplayRole, QVariant());
        item->setData(0, Qt::UserRole+2, true);
    } else
        item->setFlags(item->flags() | Qt::ItemIsEditable);
    
    return item;
}

void TOptionsWindow::editorResize()
{
    editor->header()->resizeSections(QHeaderView::ResizeToContents);
}

void TOptionsWindow::configEdited(QTreeWidgetItem *item, int)
{
    auto confName = item->data(0, Qt::UserRole).toString();
    QSettings settings("Param/" + confName, QSettings::IniFormat);
    QString path = item->data(0, Qt::UserRole+1).toString();
    QVariant default_val = item->data(0, Qt::UserRole+3);
    if (!item->data(0, Qt::UserRole+2).toBool()) { // !bool
        if (item->text(1).isEmpty() || item->text(1) == default_val.toString()) {
            item->setText(1, default_val.toString());
            settings.remove(path);
        } else
            settings.setValue(path, item->text(1));
    } else {
        if ((item->checkState(1) == Qt::Checked) == default_val.toBool())
            settings.remove(path);
        else
            settings.setValue(path, item->checkState(1) == Qt::Checked);
    }
    
    if (devGlobalConfigEditorItemList.contains(item))
        emit onSettingsChanged(CSTF_DEVICE_CONFIG_CONTROLLER_CONF);
}

void TOptionsWindow::setItemBoolValue(QTreeWidgetItem* item, bool value)
{
    item->setCheckState(1, value ? Qt::Checked : Qt::Unchecked);
    item->setData(1, Qt::DisplayRole, QVariant());
    item->setData(0, Qt::UserRole+2, true);
}

void TOptionsWindow::devGlobalConfigSettingsChanged(changedSettingsTypeFlag type)
{
    sendGlobalConfig = true;
}

void TOptionsWindow::devGlobalConfigLoad()
{
    QSettings settings("Param/controller.conf", QSettings::IniFormat);
    bool AIBPresent = settings.value("aib_present", false).toInt();
    bool directSensors = settings.value("direct_sensors/direct_sensors", false).toInt();
    bool swapXY = settings.value("direct_sensors/dir_sens_swap_XY", false).toInt();
    
    bool invX = settings.value("direct_sensors/dir_sens_invert_X", false).toInt();
    bool invY = settings.value("direct_sensors/dir_sens_invert_Y", false).toInt();
    bool invZ = settings.value("direct_sensors/dir_sens_invert_Z", false).toInt();
    
    editor->blockSignals(true);
    setItemBoolValue(devGlobalConfigEditorItemList.at(0), AIBPresent);
    setItemBoolValue(devGlobalConfigEditorItemList.at(1), directSensors);
    setItemBoolValue(devGlobalConfigEditorItemList.at(2), swapXY);
    setItemBoolValue(devGlobalConfigEditorItemList.at(3), invX);
    setItemBoolValue(devGlobalConfigEditorItemList.at(4), invY);
    setItemBoolValue(devGlobalConfigEditorItemList.at(5), invZ);
    editor->blockSignals(false);
}

bool TOptionsWindow::processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD )
{
    if (addr != ADR_CPU)
        return false;
    
    if (sub_addr == CPU_GLOBAL_CONFIG)
    {
        EG_Config config;
        memcpy(&config, data, sizeof(EG_Config));
        
        bool invX = config.DirSens.InvertXYZ & (1 << 0); // BIT_0 - X
        bool invY = config.DirSens.InvertXYZ & (1 << 1); // BIT_1 - Y
        bool invZ = config.DirSens.InvertXYZ & (1 << 2); // BIT_2 - Z
        
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        settings.setValue("aib_present", (bool)config.AIBPresent);
        settings.setValue("direct_sensors/direct_sensors", (bool)config.DirSens.DirectSensors);
        settings.setValue("direct_sensors/dir_sens_swap_XY", (bool)config.DirSens.SwapXY);
        settings.setValue("direct_sensors/dir_sens_invert_X", invX);
        settings.setValue("direct_sensors/dir_sens_invert_Y", invY);
        settings.setValue("direct_sensors/dir_sens_invert_Z", invZ);
        
        QMetaObject::invokeMethod(this, "devGlobalConfigLoad");
        
        return true;
    }
    
    return false;
}

bool TOptionsWindow::sendData(const void *handle)
{
    bool wrote = false;
    
    if (sendGlobalConfig.claimMessage())
    {
        wrote = true;
        EG_Config config;
        
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        config.AIBPresent = settings.value("aib_present", false).toInt();
        config.DirSens.DirectSensors = settings.value("direct_sensors/direct_sensors", false).toInt();
        config.DirSens.SwapXY = settings.value("direct_sensors/dir_sens_swap_XY", false).toInt();
        
        bool invX = settings.value("direct_sensors/dir_sens_invert_X", false).toInt();
        bool invY = settings.value("direct_sensors/dir_sens_invert_Y", false).toInt();
        bool invZ = settings.value("direct_sensors/dir_sens_invert_Z", false).toInt();
        config.DirSens.InvertXYZ = 0;
        if (invX)
            config.DirSens.InvertXYZ += (1 << 0); // BIT_0 - X
        if (invY)
            config.DirSens.InvertXYZ += (1 << 1); // BIT_1 - Y
        if (invZ)
            config.DirSens.InvertXYZ += (1 << 2); // BIT_2 - Z
        
        eg3000_send_packet(handle, ADR_CPU, CPU_GLOBAL_CONFIG, &config, sizeof(EG_Config));
    }
    
    if (sendGetGlobalConfig.claimMessage())
    {
        wrote = true;
        eg3000_send_empty_packet(handle, ADR_CPU, CPU_GET_GLOBAL_CONFIG);
    }
    
    return wrote;
}

void TOptionsWindow::resetWriter()
{
    sendGlobalConfig = false;
    sendGetGlobalConfig = true;
}
