#include "oscilloscope.h"

#include <QSettings>
#include <QMessageBox>
#include <QCloseEvent>
#include <QHBoxLayout>
#include <QStackedLayout>
#include <QCheckBox>
#include <QTimer>
#include "remote_nst_dataid.hpp"
#include "gui/status.hpp"
#include "autocalib.hpp"
#include "xyzcalibrations.hpp"
#include <QDebug>
#include <QtGlobal>
#include "modules.hpp"
#include "scanplot.h"
#include <qwt_plot_canvas.h>
#include "mobile/remote_mobile.hpp"
#include <QVariant>
#include "singleapp.h"
#include "hardware/EG3000.h"
#include "spmtunning.h"
#include "options_window.h"
#include "qwt_plot_layout.h"
#include "2d/plot_2dplot.h"
#include "2d/plot_2dpure.h"
#include "datalistoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"
#include "2d/ovcontrol_2d.h"
#include "hardware/eg3000_calibrations.h"
#include "modules/oscilloscopemodule.h"
#include "dacscontrol.h"

QVector<QPointer<QComboBox> > cb_channels_list;
void changeCBname(int srcidx, QString name)
{
    for (int i = 0; i < cb_channels_list.size(); i++)
        if (QComboBox *box = cb_channels_list.at(i))
            for (int i = 0; i < box->count(); i++)
                if (box->itemData(i) == srcidx) {
                    box->setItemText(i, name);
                    break;
                }
}

void fillCBchannels_(QComboBox *box, bool want_xyz)
{
    cb_channels_list << box;

    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    bool spm = settings.value("display/spm", true).toBool();
    bool head = settings.value("display/oscilloscope/head", true).toBool();
    bool stage = settings.value("display/oscilloscope/stage", true).toBool();
    bool z3 = settings.value("display/oscilloscope/z3", false).toBool();
    bool hasADDACS1234 = false;
    {
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        hasADDACS1234 = !settings.value("use_eg4000", true).toBool();
    }

    spm = true;

    box->addItem(QObject::tr("OFF"), srcNone);
    if (spm) {
        box->addItem(QObject::tr("Height"), srcHeight);
        box->addItem(QObject::tr("DFL"), srcDFL);
        box->addItem(QObject::tr("LFL")/*+"/"+QObject::tr("ITip")*/, srcLFL);
        box->addItem(QObject::tr("MAG"), srcMag1);
        box->addItem(QObject::tr("Phase"), srcPhase1);
    }

    if (want_xyz || true) {
        if (head) {
            QString scanner;
            if (stage || z3)
                scanner = " " + QObject::tr("head");
            box->addItem(QObject::tr("Ux") + scanner, srcDACX);
            box->addItem(QObject::tr("Uy") + scanner, srcDACY);
            box->addItem(QObject::tr("Uz") + scanner, srcDACZ);
        }
        if (stage) {
            QString scanner;
            if (head || z3)
                scanner = " " + QObject::tr("stage");
            box->addItem(QObject::tr("Ux") + scanner, srcDACX2);
            box->addItem(QObject::tr("Uy") + scanner, srcDACY2);
            box->addItem(QObject::tr("Uz") + scanner, srcDACZ2);
        }
        if (z3) {
            QString scanner;
            if (head || stage)
                scanner = " " + QObject::tr("z3");
            box->addItem(QObject::tr("Uz") + scanner, srcDACZ3);
        }

        if (head) {
            QString scanner;
            if (stage || z3)
                scanner = " " + QObject::tr("head");
            box->addItem(QObject::tr("X Sensor") + scanner, srcCX);
            box->addItem(QObject::tr("Y Sensor") + scanner, srcCY);
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ);
        }
        if (stage) {
            QString scanner;
            if (head || z3)
                scanner = " " + QObject::tr("stage");
            box->addItem(QObject::tr("X Sensor") + scanner, srcCX2);
            box->addItem(QObject::tr("Y Sensor") + scanner, srcCY2);
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ2);
        }
        if (z3) {
            QString scanner;
            if (head || stage)
                scanner = " " + QObject::tr("z3");
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ3);
        }
    } else {
        if (head) {
            QString scanner;
            if (stage)
                scanner = " " + QObject::tr("head");
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ);
        }
        if (stage) {
            QString scanner;
            if (head)
                scanner = " " + QObject::tr("stage");
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ2);
        }
    }
    bool RatisMode = false;
    {
        QSettings settings2("Param/gui.conf", QSettings::IniFormat);
        RatisMode = settings2.value("gui/Ratis_xyz_stage_mode", false).toBool();
    }

    if (!RatisMode)
    {
        if (true || !spm) {
            box->addItem(QObject::tr("ADC 1"), srcADC1);
            box->addItem(QObject::tr("ADC 2"), srcADC2);
            if (spm) box->addItem(QObject::tr("ADC 3"), srcADC3);
        }
        if (hasADDACS1234) {
            box->addItem(QObject::tr("ADC 4/PMT"), srcADC4);
            box->addItem(QObject::tr("ADC 5"), srcADC5);
        }

        QSettings settings("Param/ccd.conf", QSettings::IniFormat);
        if (!settings.value("use_pmt", false).toBool()) {
            box->addItem(QObject::tr("ADC AUX 1"), srcADCHR1);
            box->addItem(QObject::tr("ADC AUX 2"), srcADCHR2);
        } else {
            box->addItem(QObject::tr("PMT 1(INT)"), srcADCHR1);
            box->addItem(QObject::tr("PMT 2(EXT)"), srcADCHR2);
        }
    }

    if (spm) {
        box->addItem(QObject::tr("Tip V"), srcVTip);
        box->addItem(QObject::tr("MAG(LFL)"), srcMag2);
        box->addItem(QObject::tr("DInt"), srcIntensity);
        box->addItem(QObject::tr("MAG Sin"), srcMagSin1);
        box->addItem(QObject::tr("MAG Cos"), srcMagCos1);
    }
}

void Oscilloscope::device_status(int code)
{
    QString text;
    int type = Notification::Warn;

    switch (code) {
    default:
        return;
    case 0:
        text = tr("Controller has no firmware.") + " " + tr("Program it from oscilloscope window.");
        type = Notification::Bad;
        break;
    case 1:
        text = tr("Controller load timeout.") + " " + tr("Reboot and program it.");
        type = Notification::Bad;
        break;
    case 2:
        text = tr("Waiting for device to program.");
        break;
    case 3:
        text = tr("Programming. Please wait. Do not remove power from the device.");
        break;
    case 4:
        text = tr("Programming succesfully completed.");
        type = Notification::Good;
        break;
    };

    destructStatus(device_status_notice);
    device_status_notice = new Notification(text, (Notification::Tint)type, 60);
    doBeep();
}

Oscilloscope::Oscilloscope(QWidget *parent) : QWidget(parent)
{
    getDataCh = 0;
    bshowrelative = false;
    for (int i = 0; i < numChanels; i++) {
        k_m_nm[i] = 1;
        osc_data_idx[i] = srcNone;
    }
    osc_on = false;
    last_osc_ch = 0;

    refreshRateLimit.invalidate();

    time_dilation = 1;
    oscRate = 4000;
    oscMaxLength = 10 * oscRate;
    current_t_count = 0;
    last_updated_idx = -1;

    connect(this, SIGNAL(newOscData_(QByteArray)), this, SLOT(newOscData(QByteArray)));

    plot::ResizeOwner(curveData, numChanels);
    curveData[0]->Pen.setColor(Qt::red);
    curveData[1]->Pen.setColor(QColor(0, 170, 0)); //Qt::green
    curveData[2]->Pen.setColor(Qt::blue);
    curveData[3]->Pen.setColor(Qt::black);
    for (int i = 0; i < numChanels; i++)
        curveData[i]->setXCalib(0, time_dilation / oscRate);

    acf = new AutoCalib();
    acf->osc = this;

    setWindowTitle(tr("Oscilloscope - NSpec"));
    GUICreate();
    connect(CalibWidget, SIGNAL(sigSetpointError(int)), acf, SLOT(s_pidError(int)));

    connect(MyPl, SIGNAL(CurveColourChanged(QColor, int)), this, SLOT(on_MyPl_CurveColourChanged(QColor, int)));
    for (int i = 0; i < numChanels; i++)
        connect(CBChannel[i], SIGNAL(currentIndexChanged(int)), this, SLOT(updateRelativeState()));
    connect(MyPl, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), SLOT(AddDataToContainer_slot(plot::TCurvesDataOwner,plot::TMetaStorage)));

    MyPl->Pure2dPlot->singleAutoScale = true;
    MyPl->Pure2dPlot->SourceData = plot::getUnLock(curveData);
    MyPl->Refresh();
    MyPl->PlotControl->BAutoScale->setChecked(false);

#ifdef USE_EG3K_EMULATOR
    QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
    bool useUsbEmulator = settings.value("usb", false).toBool();
    
    if (useUsbEmulator)
    {
        QTimer *t = new QTimer;
        connect(t, SIGNAL(timeout()), this, SLOT(inject_test_packet()));
        t->start(64/4);
    }
#endif
}

void Oscilloscope::AddDataToContainer_slot(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    attributes.insert("Name", MyPl->Pure2dPlot->Title);
    attributes.insert("Source", "Oscilloscope");

    for (int i = 0; i < numChanels; i++) {
        if (CBChannel[i]->currentIndex() <= 0)
            continue;
        attributes.insert("Channel " + QString::number(i), CBChannel[i]->currentText());
        attributes.insert("Average", LEdAverage[i]->text());
        attributes.insert("RMS", LEdStDev[i]->text());
    }

    emit AddDataToContainer(src, attributes);
}


void Oscilloscope::closeEvent(QCloseEvent *event)
{

}

void Oscilloscope::setGBChannelSelected(int id, bool selected)
{
    QString style = "QGroupBox::title {subcontrol-position: topcenter;}";
    if (selected)
        style += "QGroupBox {border: 2px solid " + LEdColour[id]->m_Colour.name() + ";}";
    GrBChannel[id]->setStyleSheet(style);
    LChnLabelName[id]->setStyleSheet("font-weight: bold; color: " + LEdColour[id]->m_Colour.name() + ";");
}

void Oscilloscope::onSettingsTabChanged(const QString tabName)
{
    bool show = (tabName == this->tabName);
    btnShowHideRightMenuBar->setVisible(show);
}

void Oscilloscope::setInitialStateOscilloscope()
{
    setChannel(0, srcMag1);
    
    osc_Start(true);
    setRelative(false);
}

void Oscilloscope::setChannel(int channel, EG_DataSources src)
{
    CBChannel[channel]->setCurrentIndex(
        qMax(0, CBChannel[channel]->findData(src)));
}

void Oscilloscope::sett_showhide()
{
    CalibWidget->setVisible(!CalibWidget->isVisibleTo(this));
    btnShowHideRightMenuBar->setText(CalibWidget->isVisibleTo(this) ? ">>" : "<<");
}
void Oscilloscope::calib_plot_change(bool val)
{
    DataListOverlayButtons::setButtonStyle(BToCalib, val);
    int size = DataListOverlayButtons::UpButtonSize*0.92;
    BToCalib->setIconSize(QSize(size, size));
    OscStackedWidget->setCurrentIndex(val ? 1 : 0);
}


void Oscilloscope::s_setT(int t)
{
    MyPl->Pure2dPlot->singleAutoScaleXLimit = t;
    oscMaxLength = t * oscRate;
    MyPl->Pure2dPlot->SetAllXScales(0, t);
    MyPl->Refresh();
}


void Oscilloscope::inject_test_packet()
{
#ifdef USE_EG3K_EMULATOR
    if (!osc_on)
        return;

    int idx = last_updated_idx;
    do idx++;
    while (idx < numChanels && !osc_data_idx[idx]);
    if (idx > 3)
        idx = 0;

    EG_OscData Osc_data;
    Osc_data.Channel = idx;
    Osc_data.Size = 256;
    Osc_data.Source = osc_data_idx[idx];
    for (int i = 0; i < 256; i++)
        Osc_data.Data[i] = 30000 + idx * 10000
                + rand() % (5 + i);

    processPacket(ADR_CPU, CPU_OSCDATA, &Osc_data, 0);
#endif
}

bool Oscilloscope::processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD )
{
    if (addr != ADR_CPU)
        return false;

    if (sub_addr == CPU_TIMEDATA) {
        DWORD *buf = (DWORD *)data;
        time_dilation = qBound(0.1, double(buf[0]) / buf[1], 10.);
        return false;
    }
    if (sub_addr == CPU_OSCDATA) {
        EG_OscData *Osc_data = (EG_OscData *)data;

        int dataSize = sizeof(EG_OscData) - (OSC_LINE_SIZE - Osc_data->Size)*sizeof(WORD);
        emit newOscData_(QByteArray((char *)Osc_data, dataSize));

        return true;
    }

    return false;
}
#include <i_eg3000.h>
bool Oscilloscope::sendData(const void *handle)
{
    bool wrote = false;

    if (sendOscChannels.claimMessage()) {
        wrote = true;
        eg3000_osc_set_data((void *)handle, osc_data_idx[0], osc_data_idx[1], osc_data_idx[2], osc_data_idx[3]);
    }
    if (sendOscOnOff.claimMessage()) {
        wrote = true;
        eg3000_osc_startstop((void *)handle, osc_on);
    }
    if (doGetData.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_GET_DATA, getDataCh);
    }

    return wrote;
}
void Oscilloscope::resetWriter()
{
    sendOscChannels = true;
    sendOscOnOff = true;
    doGetData = false;

    QMetaObject::invokeMethod(this, "reset");
}

void Oscilloscope::reset()
{
    time_dilation = 1;

    updateRelativeState();
    osc_Start(osc_on);
    MyPl->Pure2dPlot->update(); //likely to happen inside RefreshInc(), manual bailout
}

void Oscilloscope::get_curr_data(int ch)
{
    getDataCh = ch;
    doGetData = true;
}

void Oscilloscope::newOscData(QByteArray data)
{
    const EG_OscData *sdata = (const EG_OscData *)data.constData();

    if (sdata->Channel >= 4 && sdata->Channel < 8)
        lsbData = data;
    if (sdata->Channel >= 4 || !osc_data_idx[sdata->Channel])
        return;

    if (sdata->Channel <= last_updated_idx) {
        current_t_count += OscLineLength;
        if (current_t_count + OscLineLength/(double)4 > oscMaxLength / time_dilation
                && osc_on) //prevent reset in case we want to see prev data
            current_t_count = 0;
    }
    last_updated_idx = sdata->Channel;
    if (current_t_count == 0)
        curveData[sdata->Channel]->YSeq.clear();

    if (!curveData[sdata->Channel]->YSeq.size())
        curveData[sdata->Channel]->setXCalib(double(current_t_count) / oscRate, time_dilation / oscRate);

    const EG_OscData *sdata2 = (const EG_OscData *)lsbData.constData();
    if (lsbData.isEmpty() || sdata2->Channel % 4 != sdata->Channel)
        sdata2 = nullptr;

    BYTE source = sdata->Source;
    BYTE sourceNonRelative = source;
    if (bshowrelative)
        source = srcNone; //default common calib

    static QVector<double> oscDataVectorNonRelative(OscLineLength);
    for (int i = 0; i < OscLineLength; i++)
    {
        double dt = input_calib(source, sdata->Data[i] + (sdata2 ? ((double)sdata2->Data[i]) / 0x10000 : 0));
        curveData[sdata->Channel]->AddY(k_m_nm[sdata->Channel] * dt);
        double dtNonRelative = input_calib(sourceNonRelative, sdata->Data[i] + (sdata2 ? ((double)sdata2->Data[i]) / 0x10000 : 0));
        oscDataVectorNonRelative[i] = dtNonRelative;
    }

    QList<double> oscDataVectorCopy;
    QList<QVariant> oscDataListVariant;
    for (int i=0; i<oscDataVectorNonRelative.size(); i++)
    {
        oscDataVectorCopy.append(oscDataVectorNonRelative.at(i));
        oscDataListVariant.append(oscDataVectorNonRelative.at(i));
    }
    if (sourceNonRelative == srcMag1)
    {
        emit newOscMAGData(oscDataVectorCopy, oscMaxLength/oscRate, OscLineLength);
        if (auto mob = MobileServer::GetIt()) {
            if (mob->any_need_data("deflect_need"))
            {
                auto list = QVariantList();
                list << QVariant(oscDataListVariant);
                list << oscMaxLength/oscRate << OscLineLength;
                mob->sendData(DataID::Deflect_oscMagData, list);
            }
        }
    }

    lsbData.clear();

    for (int i = sdata->Channel+1; i < numChanels; i++)
        if (osc_data_idx[i])
            return;

    GUIRefresh();
}

void Oscilloscope::sl_osc_start()
{
    //MyPl->SetAutoScalingEnabled(false);
    if (!start->isOn)
        osc_Start(true);
}
void Oscilloscope::sl_osc_stop()
{
    MyPl->SetAutoScalingEnabled(false);
    osc_Start(false);
}
void Oscilloscope::osc_Start(bool st)
{
    if (osc_on == st)
        return;
    
    start->change_state_man(st);
    MyPl->overlayButtons->control->BFitData->setEnabled(!st);
    if (st) {
        if (MyPl->PlotControl->BMarkers->isChecked())
            MyPl->PlotControl->BMarkers->click();

        current_t_count = 0;
        last_updated_idx = -1;
        for (int i = 0; i < curveData.size(); i++)
            curveData[i]->YSeq.clear();

        lsbData.clear();

        MyPl->Refresh();
    }
    refreshRateLimit.invalidate();

    osc_on = st;
    sendOscOnOff = true;
}
void Oscilloscope::on_MyPl_CurveColourChanged(QColor c, int N)
{
    if (N < 0)
        for (int i = 0; i < numChanels; i++)
            on_MyPl_CurveColourChanged(c, i);
    else
        LEdColour[N]->SetColour(c);
}
bool Oscilloscope::eventFilter(QObject *Object, QEvent *Event)
{
    if (Event->type() == QEvent::MouseButtonRelease)
    {
        auto mouseEv = static_cast<QMouseEvent*>(Event);
        int gbId = 0;
        if (Object == GrBChannel[0]) gbId = 0;
        else if (Object == GrBChannel[1]) gbId = 1;
        else if (Object == GrBChannel[2]) gbId = 2;
        else if (Object == GrBChannel[3]) gbId = 3;
        else if (mouseEv->button() == Qt::RightButton)
        {
            for (int i = 0; i < numChanels; i++)
                if (Object == CBChannel[i])
                {
                    CBChannel[i]->setCurrentIndex(0);
                    gbId = i;
                    break;
                }
        }

        for (int i=0; i<numChanels; i++)
            setGBChannelSelected(i, i == gbId);
    }
    if (Event->type() == QEvent::FocusIn)
        for (int i = 0; i < numChanels; i++)
            if (Object == CBChannel[i]) {
                last_osc_ch = i;
                blockSignals(true);
                updateRelativeState();
                blockSignals(false);
                break;
            }

    return QWidget::eventFilter(Object, Event);
}
void Oscilloscope::updateRelativeState()
{
    setRelative(BShowRelative->isChecked());
}

void Oscilloscope::setRelative(bool on)
{
    bshowrelative = on;
    
    BShowRelative->blockSignals(true);
    BShowRelative->setChecked(on);
    BShowRelative->blockSignals(false);
    if (bshowrelative)
        BShowRelative->setIcon(makeHugeIcon(":/plot/relativeScaleAU.png"));
    else
        BShowRelative->setIcon(makeHugeIcon(":/plot/relativeScale.png"));
    
    MyPl->overlayButtons->setButtonActive(BShowRelative, bshowrelative);
    auto channelSender = static_cast<QComboBox*>(sender());
    for (int i = 0; i < numChanels; i++)
    {
        int ch = osc_data_idx[i] = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();
        
        LEdAverage[i]->setVisible(ch != srcNone);
        LAverage[i]->setVisible(ch != srcNone);
        LEdStDev[i]->setVisible(ch != srcNone);
        LStDev[i]->setVisible(ch != srcNone);
        
        if (channelSender)
            setGBChannelSelected(i, CBChannel[i] == channelSender);
        
        if (ch == srcNone) {
            curveData[i]->YSeq.clear();
            continue;
        }
        if (bshowrelative)
            ch = srcNone; //default common calib
        
        double min = input_calib(ch, 0),
               max = input_calib(ch, 65535);
        if (max < min)
            qSwap(min, max);
        
        plot::TSIUnit tunit;
        tunit.SetSIUnit(input_units(ch));
        tunit.UpdateLegend(min, max);
        k_m_nm[i] = 1. / tunit.Multiplier;
        min = min / tunit.Multiplier;
        max = max / tunit.Multiplier;
        
        if (i != last_osc_ch)
            continue;
        
        QString name = CBChannel[i]->currentText();
        auto legendUnit = tunit.Legend;
        if (bshowrelative)
            legendUnit = tr("A.U.");
        MyPl->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(name, legendUnit);
        double adjustBounds = qAbs(qMax((1.1 * min - min), (1.1 * max - max)));
        MyPl->Pure2dPlot->SetAllYScales(min - adjustBounds, max + adjustBounds, false);
    }
    
    sendOscChannels = true;
    
    MyPl->Refresh();
    MyPl->overlayButtons->control->B2DRestoreScale->click();
}

void Oscilloscope::setChA(int src)
{
    CBChannel[0]->setCurrentIndex(CBChannel[0]->findData(src));
}

void Oscilloscope::setChB(int src)
{
    CBChannel[1]->setCurrentIndex(CBChannel[1]->findData(src));
}

void Oscilloscope::colorClick()
{
    TRectWidget *w = qobject_cast<TRectWidget *>(sender());
    if (!w) return;

    int active = MyPl->Pure2dPlot->ActiveCurve;
    int idx = -1;
    for (int i = 0; i < numChanels; i++)
        if (w == LEdColour[i]) {
            idx = i;
            break;
        }
    if (idx == -1)
        return;

    MyPl->Pure2dPlot->ActiveCurve = idx;
    MyPl->PlotControl->AGCurveColours->actions().last()->trigger();
    //MyPl->on_CurveColour_changed();

    MyPl->Pure2dPlot->ActiveCurve = active;
}

Oscilloscope* Oscilloscope::GetIt() noexcept
{
    auto osc = ModuleManager::instance->find1VoidPtr(OscilloscopeModule::name());
    return static_cast<Oscilloscope*>(osc);
}
