#pragma once

#include <QWidget>
#include <QElapsedTimer>
#include <QPointer>
#include "controls.h"
#include "edit.hpp"
#include "plot.h"
#ifndef _ADROID
#include "eg3000/defs.h"
#endif
#include "tools.hpp"

class TRectWidget: public QWidget
{
    Q_OBJECT
signals:
    void clicked();

public:
    QColor m_Colour;
    void SetColour(const QColor &c) { m_Colour = c; update(); }

    TRectWidget(const QColor &c = Qt::black): m_Colour(c) { setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed); setMinimumSize(5, 7); }

    void paintEvent(QPaintEvent *) override { QPainter p(this); p.fillRect(rect(), m_Colour); }
    void mouseDoubleClickEvent(QMouseEvent *) override { emit clicked(); }
};

void fillCBchannels_(QComboBox *box, bool want_xyz = true);
void changeCBname(int srcidx, QString name);


class XYZCalibrations;
class DacsControl;
class AutoCalibPlot;
class AutoCalib;
class Notification;
class Oscilloscope: public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT
    
public:
    static const int numChanels = 4;
public:
    Oscilloscope(QWidget *parent = nullptr);
    static Oscilloscope* GetIt() noexcept;

    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) override;
    bool sendData(const void *handle) override;
    void resetWriter() override;

    void get_curr_data(int ch);
    void GUICreate();
    void setChannel(int channel, EG_DataSources src);
    void setInitialStateOscilloscope();
    void setRelative(bool on);
    
private:
    TMessageFlag sendOscChannels, sendOscOnOff;
    TMessageFlag doGetData;
    int getDataCh;    
    
private slots:
    void s_setT(int t);
    void updateRelativeState();
    void GUIRefresh();
    void on_MyPl_CurveColourChanged(QColor, int);
    void colorClick();
    void sett_showhide();
    void calib_plot_change(bool);
    void setGBChannelSelected(int id, bool selected);
    void onSettingsTabChanged(const QString);
    
public slots:
    void osc_Start(bool start);
    void sl_osc_start();
    void sl_osc_stop();
    void setChA(int);
    void setChB(int);
    void newOscData(QByteArray data);
    void reset();
    void inject_test_packet();
    void device_status(int code);
    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void guiPlotResized(QSize newSize);
    void guiPlotSetVisible(bool visible);

public: signals:
    void newOscData_(QByteArray);
    void oscRefreshed();
    void newOscMAGData(QList<double>, int, int);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);
    void newOscHeightRMS(double, QString);

public:
    void closeEvent(QCloseEvent *event) override;
    
    bool osc_on;
    const QString tabName = tr("Oscilloscope");
    
    Edit *ePeriod;
    
    QPushButton *BToCalib;
    TwoStateButton *PBCalibMode;
    
    XYZCalibrations *CalibWidget;
    DacsControl *DacsWidget;
    AutoCalibPlot *acp;
    AutoCalib *acf;

    QComboBox *CBChannel[numChanels];
    TRectWidget *LEdColour[numChanels];
    QLabel *LAverage[numChanels];
    QLabel *LStDev[numChanels];

    QLineEdit *LEdAverage[numChanels];
    QLineEdit *LEdStDev[numChanels];

    double k_m_nm[numChanels];

    plot::TCurvesDataOwner curveData;

    bool eventFilter(QObject *Object, QEvent *Event) override;

    QWidget *mainWidget;
    QPushButton *btnShowHideRightMenuBar;
    plot::T2dPlot *MyPl;
    
private:
    QPointer<Notification> device_status_notice;
    
    TwoStateButton *start;
    QLabel *SBTLabel;
    QPushButton *BShowRelative;
    bool bshowrelative;
    QStackedWidget *OscStackedWidget;
    QGroupBox *GrBChannel[numChanels];
    QLabel *LChnLabelName[numChanels];
    int osc_data_idx[numChanels];
    int last_osc_ch;
    
    long current_t_count;
    int last_updated_idx;
    int oscMaxLength;
    const int OscLineLength = 256;
    
    int oscRate;
    double time_dilation;
    QElapsedTimer refreshRateLimit;
    QByteArray lsbData;
};



