#pragma once

#include <plot.h>
#include "controls.h"
#include "edit.hpp"
#include <defs.h>
#include "tools.hpp"
#include <QPointer>
#include <QElapsedTimer>
#include "USBVideo/videoplayer.h"
#include "deflect_widget.hpp"
#include "gui/rectslider.h"
#include <QPointer>
#include "status.hpp"
#include "gui/t3dchooser.h"
#include "gui/threestatebutton.h"
#include "gui/devicescheme.h"
#include "gui/hoverbutton.h"
#include "gui/manystatebutton.h"
#include "options/options.h"
#include "gui/swaptwostatebutton.h"
#include "plot/3d/plot_3dplot.h"
#include "plot/widgets/threedoverlaybuttons.h"
#include "gui/single/rectslsensitivityadditwindow.h"
#include "gui/single/rectsltippushadditwindow.h"
#include "gui/single/rectsltipdriveadditwindow.h"
#include "gui/single/rectslspeedadditwindow.h"
#include "gui/hoverbuttonsdirections.h"
#include "motors.hpp"

class QCheckBox;
class IExternalScan;
class SPMTunning;
IExternalScan *getNewJMIExtScan(SPMTunning *);

class QwtThermo;
class StatusBar;
class TTabWidget;
class ScanPlot: public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT

public: signals:
    void sig_XYPosition(double x, double y);
    void sig_XYPosition2(double x, double y);

    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

    //litho
    void SendBackground(const QImage &Img);
    void SendRect(const QRectF &Rect);

    void recalibrateSensors(int);
    
    void nstLaserEttaCurrentChanged(double percent);
    void motorArs3000MoveToPosition(Motors::MotorRoles role, int confLine);

private:
    void GUICreate();
    void sendXYsync(double x, double y, int sc);
    QString getLegendCurrPoint();
    QString getLegendNameChannel(int i);
    const double velocityMulfactorValues[6] = { 0.1, 1., 5., 10., 50., 100. };
    bool isEndPartScanShouldBeCalled = false;

public:
    void resizeEvent();
    static const int numScanChannel = 4;

    BYTE scan_on_off;
    float scan_lm_h;
    double Step_X;
    double Step_Y;
    BYTE scan_dir_st_point;
    BYTE scan_ch[numScanChannel];

    bool LmEn, LmEn_ch[numScanChannel];
    double LmHeight;

    bool scan_precise_mode;
    int scanner_scan;
    int ccdNX,ccdNY;

    QComboBox *comboVideoProfile;
    QComboBox *comboMaterialThin;
    rectSLSensitivityAdditWindow *rectSLSensitivityAdditWindowWid;
    rectSLTipPushAdditWindow *rectSLTipPushAdditWindowWid;
    rectSLTipDriveAdditWindow *rectSLTipDriveAdditWindowWid;
    rectSLSpeedAdditWindow *rectSLSpeedAdditWindowWid;
    
protected slots:
    void onARS3000ConfLineChanged(Motors::MotorRoles role, int confLine);
    
private slots:
    void video_motorRefractionChangedBtn(bool on);
    void video_laserEttaEnableButton();
    void video_onLaserEttaTurnOnOff(bool on);
    void video_onLaserEttaClicked();
    void endPartScan();
    void on2DFilterSettingsChanged(int fitOrder);
    void video_onHelpBtn_clicked();
    void reset_scan_test();
    void injectTestData();
    void scan_first_delay(double time);
    void scan_line_delay(double time);
    void scan_point_delay(double time);
    void spm_pid_precision(double delta);
    void scan_fast_return(bool);
    void scan_sync_mode(int);
    void scan_lm_height(double);
    void autoFocusEnded(bool);
    void materialThinChangedSave(int);
    void currentLineFontSettingsChanged(int n);
    void currentLineColorChanged(QColor, int);
    void lowVoltageModeChanged(bool);
    void onFilter_changed(bool pressed);
    void onFilter2D_changed(bool pressed);
    void setAutoTipPush(bool);
    void setFBoffon(bool);
    void multSpmPidStageChanged(int);
    void SignalsTabChanged(int);
    void splitterMoved(int pos, int index);
    void DCfbPPercNewValue(double);
    void DCfbPNewValue_emit(double val);
    void spm_SmartLandingButtonChangedMan(int oldSt, int newSt);
    void spm_SmartLandingButtonChanged(int oldSt, int newSt);
    void cancel_clicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void cancelApply_clicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void on2DBackButton_clicked();
    bool sl_fast_scan_correction(int ch);
    void sl_motor_level();
    void make_screen_videocam(bool start);
    void BSendToGraphEditor_clicked();
    void FillDataAttributes();
    void FillDataAttributes2();
    void slAutosaveData();
    void sl_syncXY();
    void sl_Update_XYZ_pos();
    void sendXYPos();
    void onLEdDeltaX_editingFinished();
    void onLEdDeltaY_editingFinished();
    void slUpdZscanner(double zpos);
    void updateDirection();
    void onBApply_clicked();
    void scan_start_stop(bool);
    void s_stopScan();
    void sl_stop_scan();
    bool removeOldData(int ch);
    void sl_update_progress(QByteArray data);
    void sl_scan_refresh_ch(int n);
    void sl_ext_scan_acquired();
    void sl_ext_scan_update();
    void sl_ext_scan_refresh();
    void sl_dacscan(bool);
    void sl_dacscan_advance();
    bool is_dacscan();
    void LmOnOff(bool value);
    void sl_CB_filters_update();
    void updateScan2DPlotVisible();
    void update2DPlotScanMode();
    void sl_section_legend(int);
    void sl_set_section_enabled(bool);
    void receiveCameraImage(QImage img);
    void plotsChanged();
    void Plot3DTabChanged(int tabId);
    void autoFocusPrec_ChangedState(bool state);
    void autoFocusFast_ChangedState(bool state);
    void motorXStageLeft(bool pressed);
    void motorXStageRight(bool pressed);
    void motorYStageTop(bool pressed);
    void motorYStageBottom(bool pressed);
    void autoFocusUpDownBtnWaiting();
    void autoFocusUpDownBtn_man(int oldSt, int newSt);
    void autoFocusUpDownEnded(bool sucsess);
    void zLandButtonSysOn(bool pressed);
    void zLandButtonSysOff(bool pressed);
    void zLandButtonSysOn_motorEvent(int mZLanding, bool atEnd, bool active);
    void zLandButtonSysOnHeadLanding_motorEvent(int mRealId, bool atEnd, bool active);
    void zLandButtonSysOffHeadLanding_motorEvent(int mRealId, bool atEnd, bool active);
    void zLandButtonSysOff_motorEvent(int mZLanding, bool atEnd, bool active);
    void scan_direction_changed(swapDir::Direction);
    void plotFontSize_changed();
    void onSendToDataVideo_clicked();
    void onSendToData_clicked();
    void on4DSendToData_clicked();
    void on4DClear_clicked();
    void cancelZLandingSysOn();
    void scanContChanged(bool);
    void startScanAgainFunc();
    
#ifndef _OLD_EXTRA_CONTROL
    void showExtraControlOnPlot();
#endif
    
public slots:
    void startMAFPrcs();
    void startPAFFast();
    void startPAF();
    void autoFocus2Ended(bool);
    void autoFocus2_ChangedState(bool);
    void setLowVoltageMode(bool pressed);
    void setScannerLowVoltageMode(int);
    void updateVideoChannel();
    void FeedBackChangedUpdateMag(bool);
    void DCfbSetPointChanged();
    void DCfbSetPointChanged(double value);
    void rectSliderSlFbiChanged(double);
    void doubleClickedLaserSpotWindow();
    void reset();
    void DCfbPNewValue(double);
    void capBits(int, long);
    void s_scanError();
    void recalibDone(bool);
    void sl_ScanCorrection();
    void Send2dPlotData();
    void Send3dPlotData1();
    void Send3dPlotData2();
    void Send3dPlotData3();
    void Send3dPlotData4();
    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slot(plot::T3dDataOwner, plot::TMetaStorage);
    void FillContainer(int plotId, plot::T3dDataOwner *src, plot::TMetaStorage *attributes);
    void restoreAttributes(const plot::TMetaStorage&);
    void sl_setCurrentX(double v);
    void sl_setCurrentY(double v);
    void sl_setCurrentX2(double v);
    void sl_setCurrentY2(double v);
    void sl_set_XYZ_range(double, double, double);
    void sl_set_XYZ_range2(double, double, double);
    void sl_set_XYZ_range_sync();
    void check_nx_ny();
    void check_speed();
    void acquireNextPoint();
    void sl_ext_mode();
    void setScanner(int sc);
    void sl_CB_update();
    void SetSectionUnits();
    void on_BSave_clicked();
    void feedBackClickedInSPM(bool val);
    void selectByRect_clicked();
    void tipDriveNewValue_FromSlider(double);
    void tipDriveNewValue(double val);
    void ChannelResizedAllPlots(QSize newSize);
    void ChannelSetVisibleGUIAllPlots(bool visible, plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void ChannelResized(int n, QSize newSize);
    void ChannelSetVisibleGUI(int n, bool visible, plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void videoChannelResized(QSize newSize);
    void videoChannelSetVisibleGUI(bool visible);
    void autoFocus_ChangedState(bool on);
    void motorZFocusUp(bool pressed);
    void motorZFocusDown(bool pressed);
    void autoFocusUpDownBtn(int oldSt, int newSt);
    void fontSize_changed();
    void onSaveImage_clicked();
    void ShowRightMenu(bool show);
    void on4DSwitchSpectra_clicked(bool pressed);
    void on4DSelectCapturedSpectr_clicked(bool pressed);
    void on4DSelectConvolutionFunc_clicked(bool pressed);
    void onVideoCBChannelChanged(int index);
#ifdef _SCAN_SLICE
    Range* getScanSliceRangeX();
    Range* getScanSliceRangeY();
    void scan_slice();
#endif

public:
    Q_SIGNAL void sync_scan_x(double);
    Q_SIGNAL void sync_scan_y(double);
    Q_SIGNAL void sync_scan_x2(double);
    Q_SIGNAL void sync_scan_y2(double);
    
public:
    ScanPlot(QWidget *parent = nullptr);
    ~ScanPlot() override;
    static ScanPlot* GetIt() noexcept;
    void addExternalScan(const QString &name, IExternalScan *iface);
    bool eventFilter(QObject *Object, QEvent *Event) override;
    bool sendData(const void *handle) override;
    void resetWriter() override;
#ifdef _OLD_EXTRA_CONTROL
    void extraControlTabCreated();
#endif
    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) override;
    void Fill_Data_Array(plot::T2dArray& Array, const EG_ScanData *Scan_Data, const EG_ScanData *lsb_Scan_Data = nullptr, plot::TCurveData *Section = nullptr, bool raw = false);
    static void Set_CBPoints(QComboBox *box, bool useLimit = true);
    void updateVideoProfiles(VideoPlayer *v, int currProfId = 0);
    
    double oldMag = 5.0;
    bool Autosend;
    QComboBox *CBChannel[numScanChannel];
    QCheckBox *CBSec[numScanChannel];
    Edit *LEdDeltaX;
    unsigned short int Nx_points, Ny_points;
    bool scan_direction;
    QComboBox *CBPointsOYext;
    QComboBox *CBPointsOXext;
    IExternalScan *currExtScan;
    TwoStateButton *PBSpectr_start_stop;
    QComboBox *CB_Channel;
    Edit *ev_Speed_spectr;
    QComboBox *CB_Points_spectr;
    Edit *ev_Z1_spectr;
    Edit *ev_Z2_spectr;
    QComboBox *CBExternalMode;
    DoubleChanger *driveInt;
    QComboBox *pointSync;
    QComboBox *CBPointsOY;
    QComboBox *CBPointsOX;
    plot::T3dPlot *Plot;
    swapTwoStateButton *m_BStart[numScanChannel];
    TwoStateButton *scanCont[numScanChannel];
    TwoStateButton *BMakeScreen;
    TwoStateButton *BAutoFocus;
    TwoStateButton *BAutoFocus2;
    QPushButton *videoHelpBtn;
    QScrollArea *videoHelpScrollArea;
    bool videoHelpShowing = false;
    static const int videoChannelId = 3;
    TwoStateButton *videoChannelAutoFocusButtonPrec;
    TwoStateButton *videoChannelAutoFocusButtonFast;
    TwoStateButton *videoChannelAutoFocusButtonPAF;
    QComboBox *videoChannelMagnification;
    bool twodPlotFilterScanMode = true;
    QPushButton *smartLandingBtn1;
    QPushButton *smartLandingBtn2;
    ThreeStateButton *smartLanding;
    TwoStateButton *spmMonitorFeedback;
    QHBoxLayout *spmMonitorGBHlay;
    DeflectWid *LaserSpotWindow;
    RectSlider *rectSlFbp;
    RectSlider *rectSlVelocity;
    RectSlider *rectSlFbi;
    RectSlider *rectAmplitude;
    deviceScheme *deviceSchemeWid;
    //ManyStateButton *videoChannelAutoFocusUpDownBtn;
#ifdef _OLD_EXTRA_CONTROL
    DoubleChanger *firstDelay;
    DoubleChanger *lineDelay;
    DoubleChanger *pointDelay;
#else
    QLabel *LExScanFirstPDelay;
    Edit *EdExScanFirstPDelay;
    QLabel *LExLineFirstPDelay;
    Edit *EdExLineFirstPDelay;
    QLabel *LExFastBackward;
    QLabel *LExSync;
    QLabel *LExSyncPointDelay;
    Edit *EdExSyncPointDelay;
    QLabel *LExScanType;
    QLabel *LExScanner;
    QLabel *LExLowVoltage;
    QCheckBox *CBExLowVoltage;
    
    QList<QLabel*> extraControlsLabels;
    QList<QWidget*> extraControlsRightPart;
    
    colorRect2 *ExtraBackground;
#endif
    
private:
    void GUICreate_Plot();
    void GUICreate_TipMonitor();
    void GUICreate_TipMonitor_Approach(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_TipMonitor_Speed(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_TipMonitor_Sensitivity(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_TipMonitor_TipPush(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_TipMonitor_TipDrive(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_Signals(QHBoxLayout *topLay);
    void GUICreate_Signals_Channels();
    void GUICreate_Signals_Fz();
    void GUICreate_Signals_4D();
    void GUICreate_Signals_LiftMode();
    void GUICreate_Signals_ContScan();
    void GUICreate_Signals_ChannelChooser(QHBoxLayout *channelTabGBlay);
    void GUICreate_SampleControl(QHBoxLayout *topLay);
    void GUICreate_ExtraControl();
    void GUICreate_SPMMonitor();
    void GUICreate_DeviceControl();

    void GUISetSizeTipMonitor(int width);
    int GUIMinimumWidthTipMonitor();

    void resizeEvent(QResizeEvent *e) override;
    void GUIResizeSPM();
    
    void zLandClearZLandingMotorsStopped();
    int zLandCountStoppedZLandingMotorsStopped();
    
    QByteArray lsbData;
    EG_ScanData *ScanDataPart[8];
    EG_ScanData *ScanDataPart_lsb[8];
    double xLeft, xRight, yTop, yBottom;
    QWidget *BoxExternal;
    TwoStateButton *BCCDXYLock;
    QPushButton *BMotorLevel;
#ifdef _OLD_EXTRA_CONTROL
    QWidget *BoxDelays;
    QWidget *BoxSync;
    QGroupBox *scanTabGB;
    QLabel *LScanner;
#endif
    
    bool GUIInited = false;
    QGroupBox *tipMonitorGB;
    QGroupBox *signalsGB;
    int minimumLeftPanelWidth = 100;
    int minimumRectSliderSize = 10;
    int minimumTipMonitorHeight = 10;
    int minimumSignalsHeight = 10;
    
    int ccdMode;
    
    WORD m_spm_pid_precision;
    TMessageFlag updateSPMPrecision;
    WORD m_scan_first_delay;
    TMessageFlag updateFirstDelay;
    WORD m_scan_line_delay;
    TMessageFlag updateLineDelay;
    WORD m_scan_point_delay;
    TMessageFlag updatePointDelay;
    bool m_scan_fast_return;
    TMessageFlag updateFastReturn;
    int m_scan_sync_mode;
    TMessageFlag updateSyncMode;

    TMessageFlag sendSpeed, sendLMheight, sendConfig, sendExtConfig, sendStartStop, sendExtStartStop, sendExtReady;
    
    QVector<QWidget *> scanControlWidgets; //disabled on scan
    QPointer<StatusBar> scan_bar;
    QPointer<StatusBar> ccd_scan_bar;
    bool scanFailed;

    Range *copyX = nullptr;
    Range *copyMaxX = nullptr;
    
    bool ratisMode = false;
    QTimer *AutosaveTimer;
    QTimer *RefreshTimer4d;
    QElapsedTimer refresh_3d_limit_timer[numScanChannel];
    qint64 refresh_limit_last_elapsed[numScanChannel];
    bool refresh_limit_need_math_update[numScanChannel];
    int channelsActive;
    unsigned int Ch_activated;
    
    //QGroupBox *GrBDACscan;
    QComboBox *CBDACscanCh;
    Edit *EDACcount;
    Edit *EDACstep;
    double DACinitialValue;
    int DACcurrentNum;
    bool show4dMenu = true;

    QColor lastSelectedLineColor = Qt::black;
    
    bool sectionEnabled;

    bool l2k_controller, l2k_controller_v22;
    bool is_optical_scanner[3];

    double requested_scan_speed; //to hw
    double user_scan_speed;    
    
    Edit *LEdX;
    Edit *LEdY;
    Edit *LEdDeltaY;
    QLabel *LDeltaX;
    QLabel *LDeltaY;
    double x, y, x2, y2;
    double Xmax, Ymax;
    double Xmax2, Ymax2;
    QwtThermo *Zposition;
    QLabel *ZposValue;
    
    int line_number[4];
    QwtPlotMarker *scanLineInd[4];
    bool ready[4];
    int nlcupd[4];
    double stPointX, stPointY;

    QVector<IExternalScan *> externalScans;
    QVector<QVector<QWidget *> *> externalScanWidgets; //List of all widgets in scan widgets
    
    QVBoxLayout **lay4D;
    QGroupBox *gb4DButtons;
    QVBoxLayout *lay4DButtons;

    BYTE fast_scan_correction;
    bool fast_scan_CLZ_disabled;

    TTabWidget *tabSignals;
    const int pos4DTab = 3;
    const int posFzTab = 2;
    QString fzTabName = "";
    QWidget *fzWidget;
    QHBoxLayout *fzHMainLayout;
    
    const int numZLandingMotors = 3;
    QList<bool> zLandingMotorsStopped;
    QPointer<StatusBar> zLandingStatusBar;
    
    DoubleChanger *spmPrecision;
    
    QCheckBox *fastReturn;
    QComboBox *CBScanner;
    QLabel *LPointsOX;
    QLabel *LPointsOY;
    
    QComboBox *CBStartPos; //todo remove that button, that replaced by button on plot
    plot::TMetaStorage scan_attributes;
    plot::T3dDataOwner *dataScan;
    plot::T3dDataOwner *dataScan_src;
    plot::T2dArray *dataScan_X;
    plot::T2dArray *dataScan_Xb;
    plot::T2dArray *dataScan_Y;
    plot::T2dArray *dataScan_Yb;
    
    plot::T3dDataOwner scanData[2][numScanChannel];
    plot::TCurvesDataOwner Section_data;

    QLabel *lb_ccd_scan;
    QLabel *lb_ccd_scan_sm;

    QCheckBox *cbLiftMode;
    QCheckBox *LmSwitch[numScanChannel];
    QLabel *LblLiftMode;
    Edit *LEd_LFmodeHeight;
    
    QCheckBox *cbContScan;

    QToolButton *FastScanCorrection;
    TwoStateButton *BPreciseScan;
    TwoStateButton *BXYLock;
    
    QPushButton *BSendToData[numScanChannel];
    QToolButton *TBFontSize;
    QActionGroup *AGFontSizes;

    QPushButton *BVideoSaveImage;
    QPushButton *BVideoSendToData;
    QPushButton *BVideoLaserEtta;

    QVBoxLayout *LeftLay;
    
    plot::ImageItem *ritem;
    
    bool showingVideoFrame;
    double videoChannelAxisScaleXMax[2];
    double videoChannelAxisScaleYMax[2];
    QString plotLegends[numScanChannel];
    bool plotPointChanged;
    
    bool videoChannelAutoFocusPrevCameraState = false;
    QList<QPushButton*> allVideoBtnsList;
    TwoStateButton *videoChannelPlayCamera;
    QLabel *videoChannelMagnificationLabel;
    QLabel *videoChannelMaterialThinLabel;
    QComboBox *videoChannelMaterialThin;
    int videoChannelAutoFocusPrevMode = -1;
    int videoChannelAutoFocusUpDownPrevMode = -1;
    
    int videoHelpImgWidth = 1;
    int videoHelpImgHeight = 1;
    
    TwoStateButton *lowVoltageModeButton[numScanChannel];

    HoverButtonsDirectionsHalf *hbMotorZFocus;
    HoverButtonsDirectionsFull *hbdMotorStage;
    
    int zLandingOff_stage = 0;
    bool zLandingWaitsMAF = false;
    
    QPushButton *increaseLaserSpotWindow;
    QGroupBox *spmMonitorGB;
    
    QHBoxLayout *mainLay;
    bool isLaserSpotWindowIncreased;
    QPointer<Notification> selection_notification;

    t3dChooser *plotChooser;
    
    QVBoxLayout *ExtraLay;
    QHBoxLayout *lay_inExtHor;
    QScrollArea *scrollAreaSchemeWid;
    
    bool sendingData = false;
    bool startScanAgain = false;
    QLabel *videoLaserEttaSliderMin;
    QLabel *videoLaserEttaSliderMax;

protected:
    TwoStateButton2 *sampleControlActivateBtn;
    TwoStateButton2 *moveStagePumpsBtn;
    TwoStateButton *VideoLaserEttaTurnOn;
    NSTSlider *VideoLaserEttaSetCurrent;
    bool useNSTLaserEtta = false;
    TwoStateButton *videoMotorRefraction;
};
