#include "scanplot.h"

#include "mainwindow.h"
#include "spmtunning.h"
#include "oscilloscope.h"
#include "modules.hpp"
#include "qwt_plot_layout.h"
#include "motors.hpp"
#include "3d/ovcontrol_3d.h"
#include "nstslider.h"
#include "spm_motors.hpp"

void ScanPlot::updateVideoProfiles(VideoPlayer *v, int currProfId)
{
    comboVideoProfile->blockSignals(true);
    videoChannelMagnification->blockSignals(true);
    comboVideoProfile->clear();
    videoChannelMagnification->clear();
    QList<QString> *list = v->getVideoProfiles();
    for (int i = 0; i < list->size(); i++)
    {
        comboVideoProfile->addItem(list->value(i));
        videoChannelMagnification->addItem(list->value(i));
    }
    comboVideoProfile->setCurrentIndex(currProfId);
    videoChannelMagnification->setCurrentIndex(currProfId);
    comboVideoProfile->blockSignals(false);
    videoChannelMagnification->blockSignals(false);

    updateVideoChannel();
}

void ScanPlot::onSendToDataVideo_clicked()
{
    Plot->SetActivePlot(videoChannelId);
    auto v = VideoPlayer::GetIt();
    if (v)
        v->on_BToData_clicked();

}

void ScanPlot::onVideoCBChannelChanged(int index)
{
    bool showVideo = (index == 0);

    Plot->ThreeDPlots[videoChannelId]->overlayButtons->MouseEvent(new QEvent(QEvent::Leave));
    Plot->ThreeDPlots[videoChannelId]->overlayButtons->replacedGUIButtons = showVideo;
    Plot->ThreeDPlots[videoChannelId]->stopMouseEvent = showVideo;
    Plot->ThreeDPlots[videoChannelId]->overlayButtons->setVisibleIfUnderMouse();
}

void ScanPlot::videoChannelSetVisibleGUI(bool visible)
{
//    if (visible && n != -1)
//        Plot->SetActivePlot(n);
    
    auto overlay = Plot->ThreeDPlots[videoChannelId]->overlayButtons;
    
    if (visible && overlay->blockShowing3DPlotButtons)
        return;
    
    overlay->setButtonVisible(lowVoltageModeButton[videoChannelId], false);
    //lowVoltageModeButton[videoChannelId]->setVisible(false);
    if (visible)
    {
        overlay->updateResize();
    }
    
    videoHelpScrollArea->setVisible(visible && (overlay->visibilityMode == plot::ThreeDOverlayButtons::svm_help));
    //overlay->BBackButton->setVisible(visible && videoHelpShowing);
    //overlay->BBackButton->setParent(Plot->ThreeDPlots[videoChannelId]);
    
    bool showBtns = visible && (overlay->visibilityMode == plot::ThreeDOverlayButtons::svm_none);
    
    overlay->setButtonVisible(overlay->BBackButton, visible && (overlay->visibilityMode != plot::ThreeDOverlayButtons::svm_none));
    
    overlay->setButtonVisible(videoHelpBtn, showBtns && overlay->control->showHelpButton && overlay->control->helpSetted);
    videoHelpBtn->setIconSize(QSize(videoHelpBtn->width(), videoHelpBtn->height()));

    auto v = VideoPlayer::GetIt();
    overlay->setButtonVisible(videoChannelAutoFocusButtonPrec, v && (showBtns && v->mafPrcsEnabled->isChecked()));
    overlay->setButtonVisible(videoChannelAutoFocusButtonFast, v && (showBtns && v->mafFastEnabled->isChecked()));
    overlay->setButtonVisible(videoChannelAutoFocusButtonPAF, v && (showBtns && v->pafEnabled->isChecked()));
    overlay->setButtonVisible(videoChannelPlayCamera, showBtns);
    //videoChannelMagnificationLabel->setVisible(showBtns);
    //videoChannelMagnification->setVisible(showBtns);
    overlay->setButtonVisible(videoChannelMagnificationLabel, showBtns);
    overlay->setButtonVisible(videoChannelMagnification, showBtns);
    overlay->setButtonVisible(videoMotorRefraction, showBtns);
    
    overlay->setButtonVisible(hbMotorZFocus, showBtns);
    overlay->setButtonVisible(hbdMotorStage, showBtns);
    
    overlay->setButtonVisible(BVideoSaveImage, showBtns);
    overlay->setButtonVisible(BVideoSendToData, showBtns);

//    if (videoChannelAutoFocusUpDownBtn->isEnabled())
//        videoChannelAutoFocusUpDownBtn->setVisible(visible);
    
    if (useNSTLaserEtta)
    {
        overlay->setButtonVisible(BVideoLaserEtta, showBtns);
        
        bool showLaser = visible && (overlay->visibilityMode == plot::ThreeDOverlayButtons::svm_laser_etta);
        overlay->setButtonVisible(VideoLaserEttaSetCurrent, showLaser);
        overlay->setButtonVisible(VideoLaserEttaTurnOn, showLaser);
        overlay->setButtonVisible(videoLaserEttaSliderMin, showLaser);
        overlay->setButtonVisible(videoLaserEttaSliderMax, showLaser);
    }
}
void ScanPlot::videoChannelResized(QSize newSize)
{
    auto v = VideoPlayer::GetIt();
    if (!v) return;
    
    const int maxUpBtnsInLine = 5;
    auto plot = Plot->ThreeDPlots[videoChannelId];
    auto overlay = plot->overlayButtons;
    int leftX = overlay->buttonRect.x();
    int width = overlay->buttonRect.width();
    int height = overlay->buttonRect.height();
    int minBtnsDistBetw = width*0.007;
    if (minBtnsDistBetw == 0)
        minBtnsDistBetw = 1;
    int upY = overlay->buttonRect.y();
    int videoBtnsSize = (newSize.width() - leftX)*0.2;
    int upNum = 1;
    if (v->mafPrcsEnabled->isChecked())
        upNum++;
    if (v->mafFastEnabled->isChecked())
        upNum++;
    if (v->pafEnabled->isChecked())
        upNum++;
    
    int origUpNum = upNum;
    
    upNum++;
    if (overlay->control->showHelpButton)
        origUpNum = upNum;
    
    int UpButtonSize = width/upNum - (upNum)*minBtnsDistBetw;
    int minUpButtonSize = width/4 - 4*minBtnsDistBetw;
    int maxUpButtonSize = width/5 - 5*minBtnsDistBetw;
    if (UpButtonSize < minUpButtonSize) UpButtonSize = minUpButtonSize;
    if (UpButtonSize > maxUpButtonSize) UpButtonSize = maxUpButtonSize;
    
    int btnsDistBetw = (width - UpButtonSize*upNum)/(upNum - 1);
    btnsDistBetw *= 0.95;
    if (btnsDistBetw < 10) btnsDistBetw = 10;
    for (int i = 0; i<allVideoBtnsList.size(); i++)
    {
        allVideoBtnsList.at(i)->setFixedSize(UpButtonSize, UpButtonSize);
        allVideoBtnsList.at(i)->setIconSize(QSize(UpButtonSize, UpButtonSize));
    }
    
    overlay->BBackButton->setFixedSize(UpButtonSize, UpButtonSize);
    overlay->BBackButton->setIconSize(QSize(UpButtonSize, UpButtonSize));
    
    int cbSize = (newSize.height() - upY)*0.05*1.14;
    if (cbSize < 0) cbSize = 0;
    if (cbSize > 9999) cbSize = 9999;
    videoChannelMagnification->setFixedSize(videoBtnsSize, cbSize);
    
    int downY = overlay->buttonRect.y() + overlay->buttonRect.height() -
                videoChannelMagnification->height();
    
    videoHelpBtn->setFixedSize(UpButtonSize, UpButtonSize);
    videoHelpBtn->setIconSize(QSize(UpButtonSize, UpButtonSize));
    
    int helpScrollAreaHeightY = overlay->BBackButton->pos().y() + overlay->BBackButton->height();
    int helpScrollAreaHeight = overlay->buttonRect.height() - overlay->BBackButton->height();
    
    videoHelpScrollArea->move(overlay->buttonRect.x(), helpScrollAreaHeightY);
    int helpScrollAreaWidth = newSize.width() - overlay->buttonRect.x() - overlay->btnsDistBetw;
    helpScrollAreaWidth = qMin(helpScrollAreaWidth, videoHelpImgWidth);
    helpScrollAreaHeight = qMin(helpScrollAreaHeight, videoHelpImgHeight);
    videoHelpScrollArea->setFixedSize(helpScrollAreaWidth, helpScrollAreaHeight);
    
    int motorBtnsSize = videoBtnsSize*0.5;
    hbMotorZFocus->setSize(motorBtnsSize);
    hbdMotorStage->setSize(motorBtnsSize);
    auto font = QFont(videoChannelMagnificationLabel->font().family(), cbSize*0.45);
    QFontMetrics fontMetric(font);
    font.setBold(true);
    videoChannelMagnificationLabel->setScaledContents(true);
    videoChannelMagnificationLabel->setFont(font);
    videoChannelMagnificationLabel->adjustSize();
    
    videoChannelMagnification->setFont(font);
    auto view = videoChannelMagnification->view();
    view->setFont(font);
    
    auto fontBig = QFont(videoChannelMagnificationLabel->font().family(), motorBtnsSize*0.45);
    hbMotorZFocus->setFont(fontBig);
    hbdMotorStage->setFont(fontBig);
    
    int btnX = leftX;
    int btnY = upY;
    videoChannelPlayCamera->move(btnX, btnY);
    {
        int btnY2 = btnY + UpButtonSize + btnsDistBetw;
        if (useNSTLaserEtta)
        {
            BVideoLaserEtta->move(btnX, btnY2);
            btnY2 += UpButtonSize + btnsDistBetw;
        }
        videoMotorRefraction->move(btnX, btnY2);
    }
    if (v->mafPrcsEnabled->isChecked())
    {
        btnX += UpButtonSize + btnsDistBetw;
        videoChannelAutoFocusButtonPrec->move(btnX, btnY);
    }
    if (v->mafFastEnabled->isChecked())
    {
        btnX += UpButtonSize + btnsDistBetw;
        videoChannelAutoFocusButtonFast->move(btnX, btnY);
    }
    if (v->pafEnabled->isChecked())
    {
        btnX += UpButtonSize + btnsDistBetw;
        videoChannelAutoFocusButtonPAF->move(btnX, btnY);
    }
    //    if (videoChannelAutoFocusUpDownBtn->isEnabled())
    //    {
    //        btnX += UpButtonSize + btnsDistBetw;
    //        videoChannelAutoFocusUpDownBtn->move(btnX, btnY);
    //    }
    
    if (overlay->control->showHelpButton)
    {
        btnX += UpButtonSize + btnsDistBetw;
        videoHelpBtn->move(btnX, btnY);
    }
    
    if (origUpNum <= 4)
        btnX += UpButtonSize + btnsDistBetw;
    if (origUpNum >= maxUpBtnsInLine)
        btnY += UpButtonSize + btnsDistBetw;
    BVideoSaveImage->move(btnX, btnY);
    if (origUpNum >= 3)
        btnY += UpButtonSize + btnsDistBetw;
    else
        btnX += UpButtonSize + btnsDistBetw;
    BVideoSendToData->move(btnX, btnY);
    
    btnX = newSize.width() - motorBtnsSize*1.2;
    btnY = BVideoSendToData->y() + UpButtonSize + 20;
    if (btnY < height*0.4)
        btnY = height*0.4;
    hbMotorZFocus->move(btnX, btnY);
    
    QPoint motorXYCenter(leftX, btnY);
    hbdMotorStage->move(motorXYCenter);
    
    int xPos = leftX;
    videoChannelMagnificationLabel->move(xPos, downY + videoChannelMagnification->height()*0.5 - videoChannelMagnificationLabel->height()*0.5);
    xPos += videoChannelMagnificationLabel->width() + 10;
    videoChannelMagnification->move(xPos, downY);
    
    if (useNSTLaserEtta)
    {
        xPos = leftX + width - UpButtonSize - btnsDistBetw;
        btnY = height - UpButtonSize + btnsDistBetw*0.5;
        VideoLaserEttaTurnOn->move(xPos, btnY);
        
        int dcWidth = width - UpButtonSize - leftX + btnsDistBetw;
        if (dcWidth < 1)
            dcWidth = 1;
        xPos = leftX;
        int y = downY;
        VideoLaserEttaSetCurrent->move(xPos, y);
        VideoLaserEttaSetCurrent->setFixedWidth(dcWidth);
        
        videoLaserEttaSliderMin->setFont(font);
        videoLaserEttaSliderMax->setFont(font);
        
        
        y -= VideoLaserEttaSetCurrent->height();
        videoLaserEttaSliderMin->move(xPos, y);
        videoLaserEttaSliderMax->move(xPos + dcWidth - fontMetric.width(videoLaserEttaSliderMax->text())*0.5, y);
    }
}

void ScanPlot::autoFocusEnded(bool sucsess)
{
    auto v = VideoPlayer::GetIt();
    disconnect(v, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusEnded(bool)));
    BAutoFocus->change_state_man(false);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        mob->setPropIdVar(DataID::videoMafPrecise, false);
        mob->setPropIdVar(DataID::videoMafFast, false);
    }
    videoChannelAutoFocusButtonPrec->change_state_man(false);
    videoChannelAutoFocusButtonFast->change_state_man(false);
    videoChannelAutoFocusButtonFast->setEnabled(true);
    videoChannelAutoFocusButtonPrec->setEnabled(true);
    videoChannelAutoFocusButtonPAF->setEnabled(true);
    BAutoFocus->setEnabled(true);
    BAutoFocus2->setEnabled(true);
    if (zLandingWaitsMAF)
    {
        zLandingOff_stage = 1;
        auto m = Motors::GetIt();
        auto mZLanding = m->getRoleMotor(Motors::MotorRoles::RoleZ1Land);
        connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), this, SLOT(zLandButtonSysOn_motorEvent(int, bool, bool)));
        zLandButtonSysOn_motorEvent(mZLanding, true, false);
    }
    if (sucsess)
    {
        {
            if (v->mafPaf->isChecked())
                BAutoFocus2->change_state_man_with_sig(true);
        }
    }
    if (videoChannelAutoFocusPrevMode != -1)
        v->autoFocusMode->setCurrentIndex(videoChannelAutoFocusPrevMode);
    videoChannelAutoFocusPrevMode = -1;
    auto spm = SPMTunning::GetIt();
    if (spm->OptionsForMotor->sysOnMAF)
        spm->BLaserOnOff->change_state_man_with_sig(false);
}
void ScanPlot::autoFocus2Ended(bool sucsess)
{
    BAutoFocus2->change_state_man(false);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::videoPaf, false);
    videoChannelAutoFocusButtonPAF->change_state_man(false);
    videoChannelAutoFocusButtonFast->setEnabled(true);
    videoChannelAutoFocusButtonPAF->setEnabled(true);
    videoChannelAutoFocusButtonPrec->setEnabled(true);
    BAutoFocus->setEnabled(true);
    BAutoFocus2->setEnabled(true);
    BMakeScreen->change_state_man_with_sig(videoChannelAutoFocusPrevCameraState);
    auto v = VideoPlayer::GetIt();
    if (v->mafPaf->isChecked())
    {
//        videoChannelAutoFocusUpDownBtn->setEnabled(true);
//        videoChannelAutoFocusUpDownBtn->setVisible(videoChannelAutoFocusButtonPrec->isVisible());
//        videoChannelAutoFocusUpDownBtn->change_state_man(1);
    }
}
void ScanPlot::autoFocus_ChangedState(bool on)
{
    auto video = VideoPlayer::GetIt();

    if (video->autoFocusScanVariant == 0)
        videoChannelAutoFocusButtonPrec->setEnabled(!on);
    else
        videoChannelAutoFocusButtonFast->setEnabled(!on);

    if (on)
    {
        videoChannelAutoFocusPrevCameraState = BMakeScreen->isOn;
        video->BAutoFocus_clicked(on);
        connect(video, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusEnded(bool)));
    }
    else {
        video->BAutoFocus_clicked(on);
        BMakeScreen->change_state_man_with_sig(videoChannelAutoFocusPrevCameraState);
    }
}
void ScanPlot::autoFocusUpDownBtn_man(int oldSt, int newSt)
{
    if ((oldSt == 2 && newSt == 3) || (oldSt == 4 && newSt == 1))
    {
        videoChannelAutoFocusButtonPrec->setEnabled(true);
        videoChannelAutoFocusButtonFast->setEnabled(true);
        videoChannelAutoFocusButtonPAF->setEnabled(true);
        BAutoFocus2->setEnabled(true);
        BAutoFocus->setEnabled(true);

        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
        {
            mob->setPropIdVar(DataID::videoFocusUpDown, newSt);
        }
    }
}
void ScanPlot::autoFocusUpDownBtnWaiting()
{
    SleepExport::msleep_gui(1000);
    auto motors = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto m = Motors::GetIt();
    auto mZfocus = motors->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
    connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
    v->autoFocusStages(mZfocus, false, false);
}
void ScanPlot::autoFocusUpDownBtn(int oldSt, int newSt)
{
    auto v = VideoPlayer::GetIt();
    auto m = Motors::GetIt();
    auto mZfocus = m->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
    auto spm = SPMTunning::GetIt();

    if (oldSt == 1 && newSt == 2)
    {
        spm->BLaserOnOff->change_state_man_with_sig(true);
        videoChannelAutoFocusButtonPrec->setEnabled(false);
        videoChannelAutoFocusButtonFast->setEnabled(false);
        videoChannelAutoFocusButtonPAF->setEnabled(false);
        BAutoFocus2->setEnabled(false);
        BAutoFocus->setEnabled(false);

        videoChannelAutoFocusUpDownPrevMode = v->autoFocusMode->currentIndex();
        v->autoFocusMode->setCurrentIndex(0);
        disconnect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
        v->autoFocusStage = 1;
        v->autoFocusMagicMultiplyCurrentId = 1;
        v->autoFocusDirection = 1;
        v->autoFocusStopped = false;
        v->isInterrupted = false;
        //m->goStop(mZfocus);
        connect(v, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusUpDownEnded(bool)));
        //connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
        m->goDown(mZfocus, m->getMotorMaxSpeed(mZfocus), v->autoFocusExitDistance);
        autoFocusUpDownBtnWaiting();
    }
    else if ((oldSt == 2 && newSt == 3) || (oldSt == 4 && newSt == 1))
    { // stopping
        spm->BLaserOnOff->change_state_man_with_sig(false);
        //videoChannelAutoFocusUpDownBtn->setEnabled(false);
        //videoChannelAutoFocusUpDownBtn->setVisible(false);
        //videoChannelAutoFocusUpDownBtn->change_state_man(1);
        v->isInterrupted = true;
        v->autoFocusStage = -10;
        v->BAutoFocus_clicked(false);
    }
    else if (oldSt == 3 && newSt == 4)
    {
        spm->BLaserOnOff->change_state_man_with_sig(true);
        videoChannelAutoFocusButtonPrec->setEnabled(false);
        videoChannelAutoFocusButtonFast->setEnabled(false);
        videoChannelAutoFocusButtonPAF->setEnabled(false);
        BAutoFocus2->setEnabled(false);
        BAutoFocus->setEnabled(false);

        disconnect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
        v->autoFocusMode->setCurrentIndex(0);
        v->autoFocusStage = 1;
        v->autoFocusMagicMultiplyCurrentId = 0;
        v->autoFocusDirection = 0;
        v->autoFocusStopped = false;
        v->isInterrupted = false;
        //m->goStop(mZfocus);
        connect(v, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusUpDownEnded(bool)));
        //connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
        m->goUp(mZfocus, m->getMotorMaxSpeed(mZfocus), v->autoFocusExitDistance);
        autoFocusUpDownBtnWaiting();
    }

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        mob->setPropIdVar(DataID::videoFocusUpDown, newSt);
    }
}
void ScanPlot::autoFocusUpDownEnded(bool sucsess)
{
    auto v = VideoPlayer::GetIt();
    disconnect(v, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusUpDownEnded(bool)));
    BAutoFocus->change_state_man(false);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        mob->setPropIdVar(DataID::videoMafPrecise, false);
        mob->setPropIdVar(DataID::videoMafFast, false);
    }
    videoChannelAutoFocusButtonPrec->change_state_man(false);
    videoChannelAutoFocusButtonFast->change_state_man(false);
    videoChannelAutoFocusButtonFast->setEnabled(true);
    videoChannelAutoFocusButtonPAF->setEnabled(true);
    BAutoFocus->setEnabled(true);
    BAutoFocus2->setEnabled(true);
    if (videoChannelAutoFocusUpDownPrevMode != -1)
        v->autoFocusMode->setCurrentIndex(videoChannelAutoFocusUpDownPrevMode);
    videoChannelAutoFocusUpDownPrevMode = -1;
    auto spm = SPMTunning::GetIt();
    spm->BLaserOnOff->change_state_man_with_sig(false);
    //if (sucsess)
        //videoChannelAutoFocusUpDownBtn->change_state_man(videoChannelAutoFocusUpDownBtn->state + 1);
    //else
        //autoFocusUpDownBtn(2, 3);
}
void ScanPlot::autoFocusPrec_ChangedState(bool state)
{
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::videoMafPrecise, state);
    videoChannelAutoFocusButtonFast->setEnabled(!state);
    videoChannelAutoFocusButtonPAF->setEnabled(!state);
    BAutoFocus2->setEnabled(!state);
    if (state)
    {
        auto v = VideoPlayer::GetIt();
        v->autoFocusDirection = 0;
//        if (videoChannelAutoFocusUpDownBtn->isVisible())
//        {
//            videoChannelAutoFocusUpDownBtn->setEnabled(false);
//            videoChannelAutoFocusUpDownBtn->setVisible(false);
//        }
        if (!videoChannelPlayCamera->isOn)
            videoChannelPlayCamera->change_state_man_with_sig(true);
        videoChannelAutoFocusPrevMode = v->autoFocusScanVariant;
        v->autoFocusMode->setCurrentIndex(1);
    }
    BAutoFocus->change_state_man_with_sig(state);
}
void ScanPlot::autoFocusFast_ChangedState(bool state)
{
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::videoMafFast, state);
    videoChannelAutoFocusButtonPrec->setEnabled(!state);
    videoChannelAutoFocusButtonPAF->setEnabled(!state);
    BAutoFocus2->setEnabled(!state);
    if (state)
    {
        auto v = VideoPlayer::GetIt();
        v->autoFocusDirection = 0;
//        if (videoChannelAutoFocusUpDownBtn->isVisible())
//        {
//            videoChannelAutoFocusUpDownBtn->setEnabled(false);
//            videoChannelAutoFocusUpDownBtn->setVisible(false);
//        }
        if (!videoChannelPlayCamera->isOn)
            videoChannelPlayCamera->change_state_man_with_sig(true);
        videoChannelAutoFocusPrevMode = v->autoFocusScanVariant;
        v->autoFocusMode->setCurrentIndex(0);
    }
    BAutoFocus->change_state_man_with_sig(state);
}
void ScanPlot::autoFocus2_ChangedState(bool on)
{
    videoChannelAutoFocusButtonPAF->change_state_man(on);
    BAutoFocus2->change_state_man(on);
    videoChannelAutoFocusButtonFast->setEnabled(!on);
    videoChannelAutoFocusButtonPrec->setEnabled(!on);
    BAutoFocus->setEnabled(!on);
    auto v = VideoPlayer::GetIt();
    if (on)
    {
//        if (videoChannelAutoFocusUpDownBtn->isVisible())
//        {
//            videoChannelAutoFocusUpDownBtn->setEnabled(false);
//            videoChannelAutoFocusUpDownBtn->setVisible(false);
//        }
        videoChannelAutoFocusPrevCameraState = BMakeScreen->isOn;
        v->BAutoFocus->click();
    }
    else {
        v->cancelOperation();
    }

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::videoPaf, on);
}

void ScanPlot::motorXStageLeft(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mXstage = m->getRoleMotor(Motors::MotorRoles::RoleStageX);
    if (pressed)
        m->goUpE(mXstage, v->GetCurrentVxyPos());
    else
        m->goStop(mXstage);
}
void ScanPlot::motorXStageRight(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mXstage = m->getRoleMotor(Motors::MotorRoles::RoleStageX);
    if (pressed)
        m->goDownE(mXstage, v->GetCurrentVxyPos());
    else
        m->goStop(mXstage);
}
void ScanPlot::motorYStageTop(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mYstage = m->getRoleMotor(Motors::MotorRoles::RoleStageY);
    if (pressed)
        m->goDownE(mYstage, v->GetCurrentVxyPos());
    else
        m->goStop(mYstage);
}
void ScanPlot::motorYStageBottom(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mYstage = m->getRoleMotor(Motors::MotorRoles::RoleStageY);
    if (pressed)
        m->goUpE(mYstage, v->GetCurrentVxyPos());
    else
        m->goStop(mYstage);
}
void ScanPlot::motorZFocusUp(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mZfocus = m->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
    if (pressed)
        m->goDownE(mZfocus, v->GetCurrentZFocusSpeed());
    else
        m->goStop(mZfocus);
}
void ScanPlot::motorZFocusDown(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mZfocus = m->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
    if (pressed)
        m->goUpE(mZfocus, v->GetCurrentZFocusSpeed());
    else
        m->goStop(mZfocus);
}

void ScanPlot::receiveCameraImage(QImage img)
{
	auto plot = Plot->ThreeDPlots.at(videoChannelId);
	plot->SetCurrentXScales(0, videoChannelAxisScaleXMax[1]);
	plot->SetCurrentYScales(0, videoChannelAxisScaleYMax[1]);
    ritem->image = img;
	plot->Replot();
}

void ScanPlot::startMAFPrcs()
{
    videoChannelAutoFocusButtonPrec->change_state_man_with_sig(true);
    delete sender();
}

void ScanPlot::startPAFFast()
{
    videoChannelAutoFocusButtonFast->change_state_man_with_sig(true);
    delete sender();
}

void ScanPlot::startPAF()
{
    videoChannelAutoFocusButtonPAF->change_state_man_with_sig(true);
    delete sender();
}

void ScanPlot::updateVideoChannel()
{
    auto video = VideoPlayer::GetIt();
    if (!video)
        return;

    auto videoPlot = Plot->ThreeDPlots.at(videoChannelId);
    if (showingVideoFrame)
    {
        videoChannelAxisScaleXMax[1] = video->sl_get_rangeX() * 1e-6;
        videoChannelAxisScaleYMax[1] = video->sl_get_rangeY() * 1e-6;
        videoPlot->SetCurrentXScales(0, videoChannelAxisScaleXMax[1]);
        videoPlot->SetCurrentYScales(0, videoChannelAxisScaleYMax[1]);
        plotLegends[videoChannelId] = comboVideoProfile->currentText();
        Plot->SetTitle(plotLegends[videoChannelId], videoChannelId);
    }
    else
    {
        videoPlot->BFixXIsChecked = videoPlot->BFixYIsChecked = false;
        videoPlot->SetCurrentXScales(0, videoChannelAxisScaleXMax[0]);
        videoPlot->SetCurrentYScales(0, videoChannelAxisScaleYMax[0]);
        plotLegends[videoChannelId] = CBChannel[videoChannelId]->currentText();
        Plot->SetTitle(plotLegends[videoChannelId] + getLegendCurrPoint(), videoChannelId);
    }

    videoPlot->Replot();

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        QList<QVariant> list;
        list << video->sl_get_rangeX()
             << video->sl_get_rangeY();

        QStringList sList;
        for (int i=0; i<videoChannelMagnification->count(); i++)
            sList << videoChannelMagnification->itemText(i);
        list << sList
             << videoChannelMagnification->currentIndex();

        list << video->mafPrcsEnabled->isChecked()
             << video->mafFastEnabled->isChecked()
             << video->pafEnabled->isChecked();

        mob->setPropIdVar(DataID::videoChangedMagnification, list);
    }
}
void ScanPlot::make_screen_videocam(bool start)
{
    auto video = VideoPlayer::GetIt();
    if (!video)
        return;

    showingVideoFrame = start;

    if (start)
    {
        connect(video, SIGNAL(ImgReceivedAfter(QImage)), this, SLOT(receiveCameraImage(QImage)));

        ritem->attach(Plot->ThreeDPlots.at(videoChannelId)->QPlot);
        if(!video->StartStop(true))
        {
            BMakeScreen->change_state_man(false);
            make_screen_videocam(false);
        }
    }
    else
    {
        video->StartStop(false);
        disconnect(video, SIGNAL(ImgReceivedAfter(QImage)), this, SLOT(receiveCameraImage(QImage)));
        //ritem->detach();
    }

    videoChannelPlayCamera->change_state_man(start);
    BMakeScreen->change_state_man(start);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::VideoOn, start);

    updateVideoChannel();
}

void ScanPlot::video_onHelpBtn_clicked()
{
    videoHelpShowing = true;
    auto plot = Plot->ThreeDPlots[videoChannelId];
    Plot->SetActivePlot(videoChannelId);
    auto overlay = plot->overlayButtons;
    overlay->BBackButton->setParent(plot);
    overlay->setBackButtonVisible(true);
    overlay->setVisibilityMode(plot::ThreeDOverlayButtons::svm_help);
    overlay->setVisibleMenu(true);
}

void ScanPlot::video_onLaserEttaClicked()
{
    auto plot = Plot->ThreeDPlots[videoChannelId];
    Plot->SetActivePlot(videoChannelId);
    auto overlay = plot->overlayButtons;
    
    overlay->setVisibilityMode(plot::ThreeDOverlayButtons::svm_laser_etta);
    overlay->setBackButtonVisible(true);
    overlay->setVisibleMenu(true);
}

void ScanPlot::video_onLaserEttaTurnOnOff(bool on)
{
    if (!on)
    {
        VideoLaserEttaTurnOn->setEnabled(false);
        QTimer::singleShot(3000, this, SLOT(video_laserEttaEnableButton()));
    }
}

void ScanPlot::video_laserEttaEnableButton()
{
    VideoLaserEttaTurnOn->setEnabled(true);
}

void ScanPlot::video_motorRefractionChangedBtn(bool on)
{
    emit motorArs3000MoveToPosition(Motors::MicroscTrin, !on);
}

void ScanPlot::onARS3000ConfLineChanged(Motors::MotorRoles role, int confLine)
{
    if (role == Motors::MicroscTrin)
    {
        videoMotorRefraction->change_state_man(!confLine);
    }
}
