#include "singleapp.h"

#include <QDebug>

QByteArray singleappGet()
{
    auto *sa = SingleApp::GetIt();
    return sa->getMessage();
}
bool singleappSet(const QByteArray ba)
{
    auto *sa = SingleApp::GetIt();
    return sa->setMessage(ba);
}

SingleApp::SingleApp()
{
    singleapp_sharedmem.setKey(UNIQUE_KEY);
    isSecondInstance = singleapp_sharedmem.attach();
    if (!isSecondInstance)
    {
        if (!singleapp_sharedmem.create(1024))
        {
            qDebug() << "Unable to create single instance.";
            isSecondInstance = true;
            return;
        }
        
        QByteArray msg("0\0"); // default value to note that no message is available.
        setMessage(msg);
    }
}

SingleApp::~SingleApp()
{
    if (singleapp_sharedmem.isAttached())
        singleapp_sharedmem.detach();
}

bool SingleApp::setMessage(const QByteArray ba)
{
    if (!singleapp_sharedmem.lock())
        return false;
    
    memcpy((char *)singleapp_sharedmem.data(), ba.constData(),
           qMin(singleapp_sharedmem.size(), ba.size()));
    
    return singleapp_sharedmem.unlock();
}

QByteArray SingleApp::getMessage()
{
    if (!singleapp_sharedmem.lock())
        return QByteArray();
    
    QByteArray byteArray = QByteArray((char*)singleapp_sharedmem.constData(), singleapp_sharedmem.size());
    
    singleapp_sharedmem.unlock();
    return byteArray;
}

const QLatin1String SingleApp::name()
{
    return QLatin1String("singleapp");
}

void *SingleApp::basecast(const QLatin1String name)
{ return name.latin1() != nullptr ? nullptr : static_cast<SingleApp *>(this); }

void SingleApp::setRunning(bool on)
{
    if (!on && singleapp_sharedmem.isAttached())
        singleapp_sharedmem.detach();
}

SingleApp* SingleApp::GetIt() noexcept
{
    auto sa = ModuleManager::instance->find1VoidPtr("singleapp");
    return static_cast<SingleApp*>(sa);
}
