#include "tools.hpp"

#include <QSettings>
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
#include <QWidget>
#include <QWindow>
#endif
#include <QDebug>

void doBeep()
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    if (settings.value("sound_notification", true).toBool())
        qApp->beep();

#if (QT_VERSION >= QT_VERSION_CHECK(5, 1, 0))
    if (qApp->thread() == QThread::currentThread()) {
        foreach (QWidget *w, qApp->topLevelWidgets()) {
            if (w->windowHandle() && w->windowHandle()->isVisible() && !w->windowTitle().isNull()) {
                w->windowHandle()->alert(0);
                break;
            }
        }
    }
#endif
}

double settings_double(QSettings &sett, const QString &path, double def)
{
    bool ok = false;
    double val = sett.value(path, def).toDouble(&ok);
    return ok ? val : def;
}

QWidget *centerWidget(QWidget *w)
{
    auto newWid = new QWidget();
    auto lay = new QHBoxLayout(newWid);
    lay->addWidget(w);
    lay->setAlignment(Qt::AlignCenter);
    lay->setContentsMargins(0,0,0,0);
    
    return newWid;
}

TNode::~TNode()
{
    if (container)
        container->removeNode(this);
}


#ifdef _WIN32
#pragma comment(lib, "setupapi.lib")
#include <Windows.h>
#include <SetupAPI.h>

HDEVINFO getDevInfoForClass(PCWSTR str, DWORD& dwCount)
{
    SetupDiClassGuidsFromNameW( str, 0, 0, &dwCount );
    
    if(dwCount)
    {
        GUID* pGuids = new GUID[dwCount];
        
        BOOL success = SetupDiClassGuidsFromNameW( str, pGuids, dwCount, &dwCount );
        
        HDEVINFO hDevInfoSet = SetupDiGetClassDevsW( pGuids, NULL, NULL, DIGCF_PRESENT);
        
        delete [] pGuids;
        
        return hDevInfoSet;
    }
    else
    {
        return NULL;
    }
}
bool getDeviceRegistryString(HDEVINFO hDevInfoSet,SP_DEVINFO_DATA &devInfo,DWORD propertyType,QString& propValue)
{
    DWORD dwType = 0;
    DWORD requiredSize=0;
    propValue="";
    BOOL result=SetupDiGetDeviceRegistryPropertyW( hDevInfoSet, &devInfo, propertyType, &dwType, NULL, NULL, &requiredSize);
    if ((result==ERROR_INVALID_DATA) || ((dwType!=REG_MULTI_SZ)&&(dwType!=REG_SZ)) || (requiredSize==0) )
    {
        return false;
        //throw std::exception(__FILE__":"__LINE__" "__FUNCDNAME__":  Error reading registry info");
    }
    size_t strSize=requiredSize/sizeof(wchar_t)+1;
    wchar_t* requestedData = new wchar_t[strSize];// буфер
    result=SetupDiGetDeviceRegistryPropertyW( hDevInfoSet, &devInfo, propertyType, &dwType,reinterpret_cast<PBYTE>(requestedData), requiredSize, &requiredSize);
    if(result==TRUE )
    {
        propValue=QString::fromWCharArray(requestedData,wcslen(requestedData));
    }
    else
    {
        qDebug()<<QString("WinDeviceHelper::getDeviceRegistryString: SetupDiGetDeviceRegistryPropertyW failed with error %1").arg(GetLastError());
    }
    delete[]requestedData;
    return (result==TRUE );
}
#endif

bool isAndorTurnedOn()
{
#ifdef _WIN32
    QSettings settings("Param/ARS3000.conf", QSettings::IniFormat);
    settings.beginGroup("monocromator");
    const QString deviceID = settings.value("shamrokMonochromatorDeviceID", "VID_0403&PID_D493&REV_0400").toString();
    settings.endGroup();
    
    DWORD dwGuids = 0;
    PCWSTR str = L"usb";
    HDEVINFO hDevInfoSet = getDevInfoForClass(str, dwGuids);
    if(dwGuids)
    {
        int nIndex = 0;
        
        SP_DEVINFO_DATA devInfo;
        devInfo.cbSize = sizeof( SP_DEVINFO_DATA );
        
        while( SetupDiEnumDeviceInfo( hDevInfoSet, nIndex, &devInfo ) && ( nIndex != -1 ) )
        {
            QString dataStr;
            if(getDeviceRegistryString(hDevInfoSet,devInfo,SPDRP_HARDWAREID,dataStr))
            {
                dataStr=dataStr.toUpper();
                if (dataStr.contains(deviceID.toUpper()))
                    return true;
            }
            nIndex++;
        }
    }
    
    return false;
#else
    return true;
#endif
}

